@@ -112,6 +112,7 @@ def create_manifest(
     print(f'Building manifest of files:  "{outpath}"...', end="", flush=True)
     with outpath.open("w", encoding="utf-8") as outfile:
         main_files_to_manifest(blender_srcdir, outfile)
+        assets_to_manifest(blender_srcdir, outfile)
         submodules_to_manifest(blender_srcdir, version, outfile)
 
         if packages_dir:
@@ -131,17 +132,27 @@ def submodules_to_manifest(
     skip_addon_contrib = version.is_release()
     assert not blender_srcdir.is_absolute()
 
-    for line in git_command("-C", blender_srcdir, "submodule"):
-        submodule = line.split()[1]
-
+    for submodule in ("scripts/addons", "scripts/addons_contrib"):
         # Don't use native slashes as GIT for MS-Windows outputs forward slashes.
-        if skip_addon_contrib and submodule == "release/scripts/addons_contrib":
+        if skip_addon_contrib and submodule == "scripts/addons_contrib":
             continue
 
         for path in git_ls_files(blender_srcdir / submodule):
             print(path, file=outfile)
 
 
+def assets_to_manifest(blender_srcdir: Path, outfile: TextIO) -> None:
+    assert not blender_srcdir.is_absolute()
+
+    assets_dir = blender_srcdir.parent / "lib" / "assets"
+    for path in assets_dir.glob("*"):
+        if path.name == "working":
+            continue
+        if path.name in SKIP_NAMES:
+            continue
+        print(path, file=outfile)
+
+
 def packages_to_manifest(outfile: TextIO, packages_dir: Path) -> None:
     for path in packages_dir.glob("*"):
         if not path.is_file():
@@ -172,7 +183,9 @@ def create_tarball(
     command += [
         "--transform",
         f"s,^{blender_srcdir.name}/,blender-{version}/,g",
-        "--use-compress-program=xz -9",
+        "--transform",
+        f"s,^lib/assets/,blender-{version}/release/datafiles/assets/,g",
+        "--use-compress-program=xz -1",
         "--create",
         f"--file={tarball}",
         f"--files-from={manifest}",
