@@ -246,22 +246,25 @@ static void fill_generic_attribute(BL::Mesh &b_mesh,
         if (polys_num == 0) {
           return;
         }
-        const MPoly *polys = static_cast<const MPoly *>(b_mesh.polygons[0].ptr.data);
+        const int *poly_offsets = static_cast<const int *>(b_mesh.polygons[0].ptr.data);
         for (int i = 0; i < polys_num; i++) {
-          const MPoly &b_poly = polys[i];
-          for (int j = 0; j < b_poly.totloop; j++) {
-            *data = get_value_at_index(b_poly.loopstart + j);
+          const int poly_start = poly_offsets[i];
+          const int poly_size = poly_offsets[i + 1] - poly_start;
+          for (int j = 0; j < poly_size; j++) {
+            *data = get_value_at_index(poly_start + j);
             data++;
           }
         }
       }
       else {
-        for (BL::MeshLoopTriangle &t : b_mesh.loop_triangles) {
-          const int index = t.index() * 3;
-          BL::Array<int, 3> loops = t.loops();
-          data[index] = get_value_at_index(loops[0]);
-          data[index + 1] = get_value_at_index(loops[1]);
-          data[index + 2] = get_value_at_index(loops[2]);
+        const int tris_num = b_mesh.loop_triangles.length();
+        const MLoopTri *looptris = static_cast<const MLoopTri *>(
+            b_mesh.loop_triangles[0].ptr.data);
+        for (int i = 0; i < tris_num; i++) {
+          const MLoopTri &tri = looptris[i];
+          data[i * 3 + 0] = get_value_at_index(tri.tri[0]);
+          data[i * 3 + 1] = get_value_at_index(tri.tri[1]);
+          data[i * 3 + 2] = get_value_at_index(tri.tri[2]);
         }
       }
       break;
@@ -277,7 +280,7 @@ static void fill_generic_attribute(BL::Mesh &b_mesh,
         assert(0);
       }
       else {
-        const MEdge *edges = static_cast<const MEdge *>(b_mesh.edges[0].ptr.data);
+        const int2 *edges = static_cast<const int2 *>(b_mesh.edges[0].ptr.data);
         const size_t verts_num = b_mesh.vertices.length();
         vector<int> count(verts_num, 0);
 
@@ -285,11 +288,11 @@ static void fill_generic_attribute(BL::Mesh &b_mesh,
         for (int i = 0; i < edges_num; i++) {
           TypeInCycles value = get_value_at_index(i);
 
-          const MEdge &b_edge = edges[i];
-          data[b_edge.v1] += value;
-          data[b_edge.v2] += value;
-          count[b_edge.v1]++;
-          count[b_edge.v2]++;
+          const int2 &b_edge = edges[i];
+          data[b_edge[0]] += value;
+          data[b_edge[1]] += value;
+          count[b_edge[0]]++;
+          count[b_edge[1]]++;
         }
 
         for (size_t i = 0; i < verts_num; i++) {
@@ -315,8 +318,11 @@ static void fill_generic_attribute(BL::Mesh &b_mesh,
         }
       }
       else {
-        for (BL::MeshLoopTriangle &t : b_mesh.loop_triangles) {
-          data[t.index()] = get_value_at_index(t.polygon_index());
+        const int tris_num = b_mesh.loop_triangles.length();
+        const int *looptri_polys = static_cast<const int *>(
+            b_mesh.loop_triangle_polygons[0].ptr.data);
+        for (int i = 0; i < tris_num; i++) {
+          data[i] = get_value_at_index(looptri_polys[i]);
         }
       }
       break;
@@ -331,7 +337,8 @@ static void fill_generic_attribute(BL::Mesh &b_mesh,
 static void attr_create_motion(Mesh *mesh, BL::Attribute &b_attribute, const float motion_scale)
 {
   if (!(b_attribute.domain() == BL::Attribute::domain_POINT) &&
-      (b_attribute.data_type() == BL::Attribute::data_type_FLOAT_VECTOR)) {
+      (b_attribute.data_type() == BL::Attribute::data_type_FLOAT_VECTOR))
+  {
     return;
   }
 
@@ -378,7 +385,8 @@ static void attr_create_generic(Scene *scene,
     }
 
     if (!(mesh->need_attribute(scene, name) ||
-          (is_render_color && mesh->need_attribute(scene, ATTR_STD_VERTEX_COLOR)))) {
+          (is_render_color && mesh->need_attribute(scene, ATTR_STD_VERTEX_COLOR))))
+    {
       continue;
     }
     if (attributes.find(name)) {
@@ -412,43 +420,58 @@ static void attr_create_generic(Scene *scene,
     switch (b_data_type) {
       case BL::Attribute::data_type_FLOAT: {
         BL::FloatAttribute b_float_attribute{b_attribute};
+        if (b_float_attribute.data.length() == 0) {
+          continue;
+        }
+        const float *src = static_cast<const float *>(b_float_attribute.data[0].ptr.data);
         Attribute *attr = attributes.add(name, TypeFloat, element);
         float *data = attr->data_float();
-        fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-          return b_float_attribute.data[i].value();
-        });
+        fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) { return src[i]; });
         break;
       }
       case BL::Attribute::data_type_BOOLEAN: {
         BL::BoolAttribute b_bool_attribute{b_attribute};
+        if (b_bool_attribute.data.length() == 0) {
+          continue;
+        }
+        const bool *src = static_cast<const bool *>(b_bool_attribute.data[0].ptr.data);
         Attribute *attr = attributes.add(name, TypeFloat, element);
         float *data = attr->data_float();
-        fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-          return (float)b_bool_attribute.data[i].value();
-        });
+        fill_generic_attribute(
+            b_mesh, data, b_domain, subdivision, [&](int i) { return (float)src[i]; });
         break;
       }
       case BL::Attribute::data_type_INT: {
         BL::IntAttribute b_int_attribute{b_attribute};
+        if (b_int_attribute.data.length() == 0) {
+          continue;
+        }
+        const int *src = static_cast<const int *>(b_int_attribute.data[0].ptr.data);
         Attribute *attr = attributes.add(name, TypeFloat, element);
         float *data = attr->data_float();
-        fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-          return (float)b_int_attribute.data[i].value();
-        });
+        fill_generic_attribute(
+            b_mesh, data, b_domain, subdivision, [&](int i) { return (float)src[i]; });
         break;
       }
       case BL::Attribute::data_type_FLOAT_VECTOR: {
         BL::FloatVectorAttribute b_vector_attribute{b_attribute};
+        if (b_vector_attribute.data.length() == 0) {
+          continue;
+        }
+        const float(*src)[3] = static_cast<const float(*)[3]>(b_vector_attribute.data[0].ptr.data);
         Attribute *attr = attributes.add(name, TypeVector, element);
         float3 *data = attr->data_float3();
         fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-          BL::Array<float, 3> v = b_vector_attribute.data[i].vector();
-          return make_float3(v[0], v[1], v[2]);
+          return make_float3(src[i][0], src[i][1], src[i][2]);
         });
         break;
       }
       case BL::Attribute::data_type_BYTE_COLOR: {
         BL::ByteColorAttribute b_color_attribute{b_attribute};
+        if (b_color_attribute.data.length() == 0) {
+          continue;
+        }
+        const uchar(*src)[4] = static_cast<const uchar(*)[4]>(b_color_attribute.data[0].ptr.data);
 
         if (element == ATTR_ELEMENT_CORNER) {
           element = ATTR_ELEMENT_CORNER_BYTE;
@@ -462,21 +485,26 @@ static void attr_create_generic(Scene *scene,
           uchar4 *data = attr->data_uchar4();
           fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
             /* Compress/encode vertex color using the sRGB curve. */
-            const float4 c = get_float4(b_color_attribute.data[i].color());
-            return color_float4_to_uchar4(color_linear_to_srgb_v4(c));
+            return make_uchar4(src[i][0], src[i][1], src[i][2], src[i][3]);
           });
         }
         else {
           float4 *data = attr->data_float4();
           fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-            BL::Array<float, 4> v = b_color_attribute.data[i].color();
-            return make_float4(v[0], v[1], v[2], v[3]);
+            return make_float4(color_srgb_to_linear(byte_to_float(src[i][0])),
+                               color_srgb_to_linear(byte_to_float(src[i][1])),
+                               color_srgb_to_linear(byte_to_float(src[i][2])),
+                               color_srgb_to_linear(byte_to_float(src[i][3])));
           });
         }
         break;
       }
       case BL::Attribute::data_type_FLOAT_COLOR: {
         BL::FloatColorAttribute b_color_attribute{b_attribute};
+        if (b_color_attribute.data.length() == 0) {
+          continue;
+        }
+        const float(*src)[4] = static_cast<const float(*)[4]>(b_color_attribute.data[0].ptr.data);
 
         Attribute *attr = attributes.add(name, TypeRGBA, element);
         if (is_render_color) {
@@ -485,18 +513,33 @@ static void attr_create_generic(Scene *scene,
 
         float4 *data = attr->data_float4();
         fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-          BL::Array<float, 4> v = b_color_attribute.data[i].color();
-          return make_float4(v[0], v[1], v[2], v[3]);
+          return make_float4(src[i][0], src[i][1], src[i][2], src[i][3]);
         });
         break;
       }
       case BL::Attribute::data_type_FLOAT2: {
         BL::Float2Attribute b_float2_attribute{b_attribute};
+        if (b_float2_attribute.data.length() == 0) {
+          continue;
+        }
+        const float(*src)[2] = static_cast<const float(*)[2]>(b_float2_attribute.data[0].ptr.data);
         Attribute *attr = attributes.add(name, TypeFloat2, element);
         float2 *data = attr->data_float2();
         fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
-          BL::Array<float, 2> v = b_float2_attribute.data[i].vector();
-          return make_float2(v[0], v[1]);
+          return make_float2(src[i][0], src[i][1]);
+        });
+        break;
+      }
+      case BL::Attribute::data_type_INT32_2D: {
+        BL::Int2Attribute b_int2_attribute{b_attribute};
+        if (b_int2_attribute.data.length() == 0) {
+          continue;
+        }
+        const int2 *src = static_cast<const int2 *>(b_int2_attribute.data[0].ptr.data);
+        Attribute *attr = attributes.add(name, TypeFloat2, element);
+        float2 *data = attr->data_float2();
+        fill_generic_attribute(b_mesh, data, b_domain, subdivision, [&](int i) {
+          return make_float2(float(src[i][0]), float(src[i][1]));
         });
         break;
       }
@@ -511,6 +554,9 @@ static void attr_create_generic(Scene *scene,
 static void attr_create_uv_map(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh)
 {
   if (!b_mesh.uv_layers.empty()) {
+    const int tris_num = b_mesh.loop_triangles.length();
+    const MLoopTri *looptris = static_cast<const MLoopTri *>(b_mesh.loop_triangles[0].ptr.data);
+
     for (BL::MeshUVLoopLayer &l : b_mesh.uv_layers) {
       const bool active_render = l.active_render();
       AttributeStandard uv_std = (active_render) ? ATTR_STD_UV : ATTR_STD_NONE;
@@ -538,14 +584,13 @@ static void attr_create_uv_map(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh)
           uv_attr = mesh->attributes.add(uv_name, TypeFloat2, ATTR_ELEMENT_CORNER);
         }
 
+        const float(*b_uv_map)[2] = static_cast<const float(*)[2]>(l.uv[0].ptr.data);
         float2 *fdata = uv_attr->data_float2();
-
-        for (BL::MeshLoopTriangle &t : b_mesh.loop_triangles) {
-          int3 li = get_int3(t.loops());
-          fdata[0] = get_float2(l.data[li[0]].uv());
-          fdata[1] = get_float2(l.data[li[1]].uv());
-          fdata[2] = get_float2(l.data[li[2]].uv());
-          fdata += 3;
+        for (int i = 0; i < tris_num; i++) {
+          const MLoopTri &tri = looptris[i];
+          fdata[i * 3 + 0] = make_float2(b_uv_map[tri.tri[0]][0], b_uv_map[tri.tri[0]][1]);
+          fdata[i * 3 + 1] = make_float2(b_uv_map[tri.tri[1]][0], b_uv_map[tri.tri[1]][1]);
+          fdata[i * 3 + 2] = make_float2(b_uv_map[tri.tri[2]][0], b_uv_map[tri.tri[2]][1]);
         }
       }
 
@@ -578,7 +623,7 @@ static void attr_create_subd_uv_map(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh,
   if (polys_num == 0) {
     return;
   }
-  const MPoly *polys = static_cast<const MPoly *>(b_mesh.polygons[0].ptr.data);
+  const int *poly_offsets = static_cast<const int *>(b_mesh.polygons[0].ptr.data);
 
   if (!b_mesh.uv_layers.empty()) {
     BL::Mesh::uv_layers_iterator l;
@@ -614,9 +659,10 @@ static void attr_create_subd_uv_map(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh,
         float2 *fdata = uv_attr->data_float2();
 
         for (int i = 0; i < polys_num; i++) {
-          const MPoly &b_poly = polys[i];
-          for (int j = 0; j < b_poly.totloop; j++) {
-            *(fdata++) = get_float2(l->data[b_poly.loopstart + j].uv());
+          const int poly_start = poly_offsets[i];
+          const int poly_size = poly_offsets[i + 1] - poly_start;
+          for (int j = 0; j < poly_size; j++) {
+            *(fdata++) = get_float2(l->data[poly_start + j].uv());
           }
         }
       }
@@ -649,9 +695,7 @@ static void attr_create_subd_uv_map(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh,
 /* Compare vertices by sum of their coordinates. */
 class VertexAverageComparator {
  public:
-  VertexAverageComparator(const array<float3> &verts) : verts_(verts)
-  {
-  }
+  VertexAverageComparator(const array<float3> &verts) : verts_(verts) {}
 
   bool operator()(const int &vert_idx_a, const int &vert_idx_b)
   {
@@ -679,7 +723,7 @@ static void attr_create_pointiness(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh, b
   if (num_verts == 0) {
     return;
   }
-  const MVert *verts = static_cast<const MVert *>(b_mesh.vertices[0].ptr.data);
+  const float(*positions)[3] = static_cast<const float(*)[3]>(b_mesh.vertices[0].ptr.data);
 
   /* STEP 1: Find out duplicated vertices and point duplicates to a single
    *         original vertex.
@@ -699,13 +743,15 @@ static void attr_create_pointiness(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh, b
     const float3 &vert_co = mesh->get_verts()[vert_index];
     bool found = false;
     for (int other_sorted_vert_index = sorted_vert_index + 1; other_sorted_vert_index < num_verts;
-         ++other_sorted_vert_index) {
+         ++other_sorted_vert_index)
+    {
       const int other_vert_index = sorted_vert_indeices[other_sorted_vert_index];
       const float3 &other_vert_co = mesh->get_verts()[other_vert_index];
       /* We are too far away now, we wouldn't have duplicate. */
       if ((other_vert_co.x + other_vert_co.y + other_vert_co.z) -
               (vert_co.x + vert_co.y + vert_co.z) >
-          3 * FLT_EPSILON) {
+          3 * FLT_EPSILON)
+      {
         break;
       }
       /* Found duplicate. */
@@ -754,21 +800,19 @@ static void attr_create_pointiness(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh, b
   EdgeMap visited_edges;
   memset(&counter[0], 0, sizeof(int) * counter.size());
 
-  const MEdge *edges = static_cast<MEdge *>(b_mesh.edges[0].ptr.data);
+  const int2 *edges = static_cast<int2 *>(b_mesh.edges[0].ptr.data);
   const int edges_num = b_mesh.edges.length();
 
   for (int i = 0; i < edges_num; i++) {
-    const MEdge &b_edge = edges[i];
-    const int v0 = vert_orig_index[b_edge.v1];
-    const int v1 = vert_orig_index[b_edge.v2];
+    const int2 &b_edge = edges[i];
+    const int v0 = vert_orig_index[b_edge[0]];
+    const int v1 = vert_orig_index[b_edge[1]];
     if (visited_edges.exists(v0, v1)) {
       continue;
     }
     visited_edges.insert(v0, v1);
-    const MVert &b_vert_0 = verts[v0];
-    const MVert &b_vert_1 = verts[v1];
-    float3 co0 = make_float3(b_vert_0.co[0], b_vert_0.co[1], b_vert_0.co[2]);
-    float3 co1 = make_float3(b_vert_1.co[0], b_vert_1.co[1], b_vert_1.co[2]);
+    float3 co0 = make_float3(positions[v0][0], positions[v0][1], positions[v0][2]);
+    float3 co1 = make_float3(positions[v1][0], positions[v1][1], positions[v1][2]);
     float3 edge = normalize(co1 - co0);
     edge_accum[v0] += edge;
     edge_accum[v1] += -edge;
@@ -798,9 +842,9 @@ static void attr_create_pointiness(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh, b
   memset(&counter[0], 0, sizeof(int) * counter.size());
   visited_edges.clear();
   for (int i = 0; i < edges_num; i++) {
-    const MEdge &b_edge = edges[i];
-    const int v0 = vert_orig_index[b_edge.v1];
-    const int v1 = vert_orig_index[b_edge.v2];
+    const int2 &b_edge = edges[i];
+    const int v0 = vert_orig_index[b_edge[0]];
+    const int v1 = vert_orig_index[b_edge[1]];
     if (visited_edges.exists(v0, v1)) {
       continue;
     }
@@ -820,6 +864,27 @@ static void attr_create_pointiness(Scene *scene, Mesh *mesh, BL::Mesh &b_mesh, b
   }
 }
 
+static const int *find_corner_vert_attribute(BL::Mesh b_mesh)
+{
+  for (BL::Attribute &b_attribute : b_mesh.attributes) {
+    if (b_attribute.domain() != BL::Attribute::domain_CORNER) {
+      continue;
+    }
+    if (b_attribute.data_type() != BL::Attribute::data_type_INT) {
+      continue;
+    }
+    if (b_attribute.name() != ".corner_vert") {
+      continue;
+    }
+    BL::IntAttribute b_int_attribute{b_attribute};
+    if (b_int_attribute.data.length() == 0) {
+      return nullptr;
+    }
+    return static_cast<const int *>(b_int_attribute.data[0].ptr.data);
+  }
+  return nullptr;
+}
+
 /* The Random Per Island attribute is a random float associated with each
  * connected component (island) of the mesh. The attribute is computed by
  * first classifying the vertices into different sets using a Disjoint Set
@@ -839,7 +904,6 @@ static void attr_create_random_per_island(Scene *scene,
     return;
   }
 
-  const int polys_num = b_mesh.polygons.length();
   int number_of_vertices = b_mesh.vertices.length();
   if (number_of_vertices == 0) {
     return;
@@ -847,11 +911,12 @@ static void attr_create_random_per_island(Scene *scene,
 
   DisjointSet vertices_sets(number_of_vertices);
 
-  const MEdge *edges = static_cast<MEdge *>(b_mesh.edges[0].ptr.data);
+  const int2 *edges = static_cast<int2 *>(b_mesh.edges[0].ptr.data);
   const int edges_num = b_mesh.edges.length();
+  const int *corner_verts = find_corner_vert_attribute(b_mesh);
 
   for (int i = 0; i < edges_num; i++) {
-    vertices_sets.join(edges[i].v1, edges[i].v2);
+    vertices_sets.join(edges[i][0], edges[i][1]);
   }
 
   AttributeSet &attributes = (subdivision) ? mesh->subd_attributes : mesh->attributes;
@@ -859,18 +924,22 @@ static void attr_create_random_per_island(Scene *scene,
   float *data = attribute->data_float();
 
   if (!subdivision) {
-    for (BL::MeshLoopTriangle &t : b_mesh.loop_triangles) {
-      data[t.index()] = hash_uint_to_float(vertices_sets.find(t.vertices()[0]));
+    const int tris_num = b_mesh.loop_triangles.length();
+    if (tris_num != 0) {
+      const MLoopTri *looptris = static_cast<const MLoopTri *>(b_mesh.loop_triangles[0].ptr.data);
+      for (int i = 0; i < tris_num; i++) {
+        const int vert = corner_verts[looptris[i].tri[0]];
+        data[i] = hash_uint_to_float(vertices_sets.find(vert));
+      }
     }
   }
   else {
+    const int polys_num = b_mesh.polygons.length();
     if (polys_num != 0) {
-      const MPoly *polys = static_cast<const MPoly *>(b_mesh.polygons[0].ptr.data);
-      const MLoop *loops = static_cast<const MLoop *>(b_mesh.loops[0].ptr.data);
+      const int *poly_offsets = static_cast<const int *>(b_mesh.polygons[0].ptr.data);
       for (int i = 0; i < polys_num; i++) {
-        const MPoly &b_poly = polys[i];
-        const MLoop &b_loop = loops[b_poly.loopstart];
-        data[i] = hash_uint_to_float(vertices_sets.find(b_loop.v));
+        const int vert = corner_verts[poly_offsets[i]];
+        data[i] = hash_uint_to_float(vertices_sets.find(vert));
       }
     }
   }
@@ -878,7 +947,7 @@ static void attr_create_random_per_island(Scene *scene,
 
 /* Create Mesh */
 
-static std::optional<BL::IntAttribute> find_material_index_attribute(BL::Mesh b_mesh)
+static const int *find_material_index_attribute(BL::Mesh b_mesh)
 {
   for (BL::Attribute &b_attribute : b_mesh.attributes) {
     if (b_attribute.domain() != BL::Attribute::domain_FACE) {
@@ -890,9 +959,34 @@ static std::optional<BL::IntAttribute> find_material_index_attribute(BL::Mesh b_
     if (b_attribute.name() != "material_index") {
       continue;
     }
-    return BL::IntAttribute{b_attribute};
+    BL::IntAttribute b_int_attribute{b_attribute};
+    if (b_int_attribute.data.length() == 0) {
+      return nullptr;
+    }
+    return static_cast<const int *>(b_int_attribute.data[0].ptr.data);
   }
-  return std::nullopt;
+  return nullptr;
+}
+
+static const bool *find_sharp_face_attribute(BL::Mesh b_mesh)
+{
+  for (BL::Attribute &b_attribute : b_mesh.attributes) {
+    if (b_attribute.domain() != BL::Attribute::domain_FACE) {
+      continue;
+    }
+    if (b_attribute.data_type() != BL::Attribute::data_type_BOOLEAN) {
+      continue;
+    }
+    if (b_attribute.name() != "sharp_face") {
+      continue;
+    }
+    BL::IntAttribute b_int_attribute{b_attribute};
+    if (b_int_attribute.data.length() == 0) {
+      return nullptr;
+    }
+    return static_cast<const bool *>(b_int_attribute.data[0].ptr.data);
+  }
+  return nullptr;
 }
 
 static void create_mesh(Scene *scene,
@@ -904,13 +998,10 @@ static void create_mesh(Scene *scene,
                         const bool subdivision = false,
                         const bool subdivide_uvs = true)
 {
-  /* count vertices and faces */
-  int numverts = b_mesh.vertices.length();
+  const int numverts = b_mesh.vertices.length();
   const int polys_num = b_mesh.polygons.length();
   int numfaces = (!subdivision) ? b_mesh.loop_triangles.length() : b_mesh.polygons.length();
-  int numtris = 0;
-  int numcorners = 0;
-  int numngons = 0;
+  const int numcorners = b_mesh.loops.length();
   bool use_loop_normals = b_mesh.use_auto_smooth() &&
                           (mesh->get_subdivision_type() != Mesh::SUBDIVISION_CATMULL_CLARK);
 
@@ -919,38 +1010,45 @@ static void create_mesh(Scene *scene,
     return;
   }
 
-  const MVert *verts = static_cast<const MVert *>(b_mesh.vertices[0].ptr.data);
+  const float(*positions)[3] = static_cast<const float(*)[3]>(b_mesh.vertices[0].ptr.data);
+  const int *corner_verts = find_corner_vert_attribute(b_mesh);
+  const int *material_indices = find_material_index_attribute(b_mesh);
+  const bool *sharp_faces = find_sharp_face_attribute(b_mesh);
+  const float(*corner_normals)[3] = nullptr;
+  if (use_loop_normals) {
+    corner_normals = static_cast<const float(*)[3]>(b_mesh.corner_normals[0].ptr.data);
+  }
 
+  int numngons = 0;
+  int numtris = 0;
   if (!subdivision) {
     numtris = numfaces;
   }
   else {
-    const MPoly *polys = static_cast<const MPoly *>(b_mesh.polygons[0].ptr.data);
+    const int *poly_offsets = static_cast<const int *>(b_mesh.polygons[0].ptr.data);
     for (int i = 0; i < polys_num; i++) {
-      const MPoly &b_poly = polys[i];
-      numngons += (b_poly.totloop == 4) ? 0 : 1;
-      numcorners += b_poly.totloop;
+      const int poly_start = poly_offsets[i];
+      const int poly_size = poly_offsets[i + 1] - poly_start;
+      numngons += (poly_size == 4) ? 0 : 1;
     }
   }
 
   /* allocate memory */
   if (subdivision) {
-    mesh->reserve_subd_faces(numfaces, numngons, numcorners);
+    mesh->resize_subd_faces(numfaces, numngons, numcorners);
   }
+  mesh->resize_mesh(numverts, numtris);
 
-  mesh->reserve_mesh(numverts, numtris);
-
-  /* create vertex coordinates and normals */
+  float3 *verts = mesh->get_verts().data();
   for (int i = 0; i < numverts; i++) {
-    const MVert &b_vert = verts[i];
-    mesh->add_vertex(make_float3(b_vert.co[0], b_vert.co[1], b_vert.co[2]));
+    verts[i] = make_float3(positions[i][0], positions[i][1], positions[i][2]);
   }
 
   AttributeSet &attributes = (subdivision) ? mesh->subd_attributes : mesh->attributes;
   Attribute *attr_N = attributes.add(ATTR_STD_VERTEX_NORMAL);
   float3 *N = attr_N->data_float3();
 
-  if (subdivision || !use_loop_normals) {
+  if (subdivision || !(use_loop_normals && corner_normals)) {
     const float(*b_vert_normals)[3] = static_cast<const float(*)[3]>(
         b_mesh.vertex_normals[0].ptr.data);
     for (int i = 0; i < numverts; i++) {
@@ -978,61 +1076,109 @@ static void create_mesh(Scene *scene,
     }
   }
 
-  std::optional<BL::IntAttribute> material_indices = find_material_index_attribute(b_mesh);
-  auto get_material_index = [&](const int poly_index) -> int {
-    if (material_indices) {
-      return clamp(material_indices->data[poly_index].value(), 0, used_shaders.size() - 1);
-    }
-    return 0;
+  auto clamp_material_index = [&](const int material_index) -> int {
+    return clamp(material_index, 0, used_shaders.size() - 1);
   };
 
   /* create faces */
-  const MPoly *polys = static_cast<const MPoly *>(b_mesh.polygons[0].ptr.data);
   if (!subdivision) {
-    for (BL::MeshLoopTriangle &t : b_mesh.loop_triangles) {
-      const int poly_index = t.polygon_index();
-      const MPoly &b_poly = polys[poly_index];
-      int3 vi = get_int3(t.vertices());
+    int *triangles = mesh->get_triangles().data();
+    bool *smooth = mesh->get_smooth().data();
+    int *shader = mesh->get_shader().data();
 
-      int shader = get_material_index(poly_index);
-      bool smooth = (b_poly.flag & ME_SMOOTH) || use_loop_normals;
+    const MLoopTri *looptris = static_cast<const MLoopTri *>(b_mesh.loop_triangles[0].ptr.data);
+    for (int i = 0; i < numtris; i++) {
+      const MLoopTri &tri = looptris[i];
+      triangles[i * 3 + 0] = corner_verts[tri.tri[0]];
+      triangles[i * 3 + 1] = corner_verts[tri.tri[1]];
+      triangles[i * 3 + 2] = corner_verts[tri.tri[2]];
+    }
 
-      if (use_loop_normals) {
-        BL::Array<float, 9> loop_normals = t.split_normals();
+    if (material_indices) {
+      const int *looptri_polys = static_cast<const int *>(
+          b_mesh.loop_triangle_polygons[0].ptr.data);
+      for (int i = 0; i < numtris; i++) {
+        shader[i] = clamp_material_index(material_indices[looptri_polys[i]]);
+      }
+    }
+    else {
+      std::fill(shader, shader + numtris, 0);
+    }
+
+    if (sharp_faces && !(use_loop_normals && corner_normals)) {
+      const int *looptri_polys = static_cast<const int *>(
+          b_mesh.loop_triangle_polygons[0].ptr.data);
+      for (int i = 0; i < numtris; i++) {
+        smooth[i] = !sharp_faces[looptri_polys[i]];
+      }
+    }
+    else {
+      std::fill(smooth, smooth + numtris, true);
+    }
+
+    if (use_loop_normals && corner_normals) {
+      for (int i = 0; i < numtris; i++) {
+        const MLoopTri &tri = looptris[i];
         for (int i = 0; i < 3; i++) {
-          N[vi[i]] = make_float3(
-              loop_normals[i * 3], loop_normals[i * 3 + 1], loop_normals[i * 3 + 2]);
+          const int corner = tri.tri[i];
+          const int vert = corner_verts[corner];
+          const float *normal = corner_normals[corner];
+          N[vert] = make_float3(normal[0], normal[1], normal[2]);
         }
       }
-
-      /* Create triangles.
-       *
-       * NOTE: Autosmooth is already taken care about.
-       */
-      mesh->add_triangle(vi[0], vi[1], vi[2], shader, smooth);
     }
+
+    mesh->tag_triangles_modified();
+    mesh->tag_shader_modified();
+    mesh->tag_smooth_modified();
   }
   else {
-    vector<int> vi;
+    int *subd_start_corner = mesh->get_subd_start_corner().data();
+    int *subd_num_corners = mesh->get_subd_num_corners().data();
+    int *subd_shader = mesh->get_subd_shader().data();
+    bool *subd_smooth = mesh->get_subd_smooth().data();
+    int *subd_ptex_offset = mesh->get_subd_ptex_offset().data();
+    int *subd_face_corners = mesh->get_subd_face_corners().data();
 
-    const MLoop *loops = static_cast<const MLoop *>(b_mesh.loops[0].ptr.data);
-
-    for (int i = 0; i < numfaces; i++) {
-      const MPoly &b_poly = polys[i];
-      int n = b_poly.totloop;
-      int shader = get_material_index(i);
-      bool smooth = (b_poly.flag & ME_SMOOTH) || use_loop_normals;
-
-      vi.resize(n);
-      for (int i = 0; i < n; i++) {
-        /* NOTE: Autosmooth is already taken care about. */
-
-        vi[i] = loops[b_poly.loopstart + i].v;
+    if (sharp_faces && !use_loop_normals) {
+      for (int i = 0; i < numfaces; i++) {
+        subd_smooth[i] = !sharp_faces[i];
       }
-
-      /* create subd faces */
-      mesh->add_subd_face(&vi[0], n, shader, smooth);
     }
+    else {
+      std::fill(subd_smooth, subd_smooth + numfaces, true);
+    }
+
+    if (material_indices) {
+      for (int i = 0; i < numfaces; i++) {
+        subd_shader[i] = clamp_material_index(material_indices[i]);
+      }
+    }
+    else {
+      std::fill(subd_shader, subd_shader + numfaces, 0);
+    }
+
+    std::copy(corner_verts, corner_verts + numcorners, subd_face_corners);
+
+    const int *poly_offsets = static_cast<const int *>(b_mesh.polygons[0].ptr.data);
+    int ptex_offset = 0;
+    for (int i = 0; i < numfaces; i++) {
+      const int poly_start = poly_offsets[i];
+      const int poly_size = poly_offsets[i + 1] - poly_start;
+
+      subd_start_corner[i] = poly_start;
+      subd_num_corners[i] = poly_size;
+      subd_ptex_offset[i] = ptex_offset;
+      const int num_ptex = (poly_size == 4) ? 1 : poly_size;
+      ptex_offset += num_ptex;
+    }
+
+    mesh->tag_subd_face_corners_modified();
+    mesh->tag_subd_start_corner_modified();
+    mesh->tag_subd_num_corners_modified();
+    mesh->tag_subd_shader_modified();
+    mesh->tag_subd_smooth_modified();
+    mesh->tag_subd_ptex_offset_modified();
   }
 
   /* Create all needed attributes.
@@ -1083,31 +1229,32 @@ static void create_subd_mesh(Scene *scene,
   const int edges_num = b_mesh.edges.length();
 
   if (edges_num != 0 && b_mesh.edge_creases.length() > 0) {
-    BL::MeshEdgeCreaseLayer creases = b_mesh.edge_creases[0];
+    const float *creases = static_cast<const float *>(b_mesh.edge_creases[0].data[0].ptr.data);
 
     size_t num_creases = 0;
     for (int i = 0; i < edges_num; i++) {
-      if (creases.data[i].value() != 0.0f) {
+      if (creases[i] != 0.0f) {
         num_creases++;
       }
     }
 
     mesh->reserve_subd_creases(num_creases);
 
-    const MEdge *edges = static_cast<MEdge *>(b_mesh.edges[0].ptr.data);
+    const int2 *edges = static_cast<int2 *>(b_mesh.edges[0].ptr.data);
     for (int i = 0; i < edges_num; i++) {
-      const float crease = creases.data[i].value();
+      const float crease = creases[i];
       if (crease != 0.0f) {
-        const MEdge &b_edge = edges[i];
-        mesh->add_edge_crease(b_edge.v1, b_edge.v2, crease);
+        const int2 &b_edge = edges[i];
+        mesh->add_edge_crease(b_edge[0], b_edge[1], crease);
       }
     }
   }
 
-  for (BL::MeshVertexCreaseLayer &c : b_mesh.vertex_creases) {
-    for (int i = 0; i < c.data.length(); ++i) {
-      if (c.data[i].value() != 0.0f) {
-        mesh->add_vertex_crease(i, c.data[i].value());
+  for (BL::MeshVertexCreaseLayer &layer : b_mesh.vertex_creases) {
+    const float *creases = static_cast<const float *>(layer.data[0].ptr.data);
+    for (int i = 0; i < layer.data.length(); ++i) {
+      if (creases[i] != 0.0f) {
+        mesh->add_vertex_crease(i, creases[i]);
       }
     }
   }
@@ -1184,7 +1331,8 @@ void BlenderSync::sync_mesh(BL::Depsgraph b_depsgraph, BObjectInfo &b_ob_info, M
   for (const SocketType &socket : new_mesh.type->inputs) {
     /* Those sockets are updated in sync_object, so do not modify them. */
     if (socket.name == "use_motion_blur" || socket.name == "motion_steps" ||
-        socket.name == "used_shaders") {
+        socket.name == "used_shaders")
+    {
       continue;
     }
     mesh->set_value(socket, new_mesh, socket);
@@ -1252,14 +1400,13 @@ void BlenderSync::sync_mesh_motion(BL::Depsgraph b_depsgraph,
     float3 *mP = attr_mP->data_float3() + motion_step * numverts;
     float3 *mN = (attr_mN) ? attr_mN->data_float3() + motion_step * numverts : NULL;
 
-    const MVert *verts = static_cast<const MVert *>(b_mesh.vertices[0].ptr.data);
+    const float(*positions)[3] = static_cast<const float(*)[3]>(b_mesh.vertices[0].ptr.data);
 
     /* NOTE: We don't copy more that existing amount of vertices to prevent
      * possible memory corruption.
      */
     for (int i = 0; i < std::min<size_t>(b_verts_num, numverts); i++) {
-      const MVert &b_vert = verts[i];
-      mP[i] = make_float3(b_vert.co[0], b_vert.co[1], b_vert.co[2]);
+      mP[i] = make_float3(positions[i][0], positions[i][1], positions[i][2]);
     }
     if (mN) {
       const float(*b_vert_normals)[3] = static_cast<const float(*)[3]>(
