@@ -202,7 +202,8 @@ void BlenderSession::reset_session(BL::BlendData &b_data, BL::Depsgraph &b_depsg
       b_scene, background, use_developer_ui);
 
   if (scene->params.modified(scene_params) || session->params.modified(session_params) ||
-      !this->b_render.use_persistent_data()) {
+      !this->b_render.use_persistent_data())
+  {
     /* if scene or session parameters changed, it's easier to simply re-create
      * them rather than trying to distinguish which settings need to be updated
      */
@@ -376,8 +377,8 @@ void BlenderSession::render(BL::Depsgraph &b_depsgraph_)
   }
 
   int view_index = 0;
-  for (b_rr.views.begin(b_view_iter); b_view_iter != b_rr.views.end();
-       ++b_view_iter, ++view_index) {
+  for (b_rr.views.begin(b_view_iter); b_view_iter != b_rr.views.end(); ++b_view_iter, ++view_index)
+  {
     b_rview_name = b_view_iter->name();
 
     buffer_params.layer = b_view_layer.name();
@@ -404,7 +405,7 @@ void BlenderSession::render(BL::Depsgraph &b_depsgraph_)
      * point we know that we've got everything to render current view layer.
      */
     /* At the moment we only free if we are not doing multi-view
-     * (or if we are rendering the last view). See T58142/D4239 for discussion.
+     * (or if we are rendering the last view). See #58142/D4239 for discussion.
      */
     if (view_index == num_views - 1) {
       free_blender_memory_if_possible();
@@ -514,6 +515,7 @@ void BlenderSession::render_frame_finish()
 static bool bake_setup_pass(Scene *scene, const string &bake_type_str, const int bake_filter)
 {
   Integrator *integrator = scene->integrator;
+  Film *film = scene->film;
   const char *bake_type = bake_type_str.c_str();
 
   PassType type = PASS_NONE;
@@ -541,13 +543,29 @@ static bool bake_setup_pass(Scene *scene, const string &bake_type_str, const int
   else if (strcmp(bake_type, "ENVIRONMENT") == 0) {
     type = PASS_BACKGROUND;
   }
-  /* AO passes. */
+  /* AO pass. */
   else if (strcmp(bake_type, "AO") == 0) {
     type = PASS_AO;
   }
+  /* Shadow pass. */
+  else if (strcmp(bake_type, "SHADOW") == 0) {
+    /* Bake as combined pass, together with marking the object as a shadow catcher. */
+    type = PASS_SHADOW_CATCHER;
+    film->set_use_approximate_shadow_catcher(true);
+
+    use_direct_light = true;
+    use_indirect_light = true;
+    include_albedo = true;
+
+    integrator->set_use_diffuse(true);
+    integrator->set_use_glossy(true);
+    integrator->set_use_transmission(true);
+    integrator->set_use_emission(true);
+  }
   /* Combined pass. */
   else if (strcmp(bake_type, "COMBINED") == 0) {
     type = PASS_COMBINED;
+    film->set_use_approximate_shadow_catcher(true);
 
     use_direct_light = (bake_filter & BL::BakeSettings::pass_filter_DIRECT) != 0;
     use_indirect_light = (bake_filter & BL::BakeSettings::pass_filter_INDIRECT) != 0;
@@ -559,15 +577,11 @@ static bool bake_setup_pass(Scene *scene, const string &bake_type_str, const int
                                      0);
     integrator->set_use_emission((bake_filter & BL::BakeSettings::pass_filter_EMIT) != 0);
   }
-  /* Shadow pass. */
-  else if (strcmp(bake_type, "SHADOW") == 0) {
-    type = PASS_SHADOW;
-    use_direct_light = true;
-  }
   /* Light component passes. */
   else if (strcmp(bake_type, "DIFFUSE") == 0) {
     if ((bake_filter & BL::BakeSettings::pass_filter_DIRECT) &&
-        bake_filter & BL::BakeSettings::pass_filter_INDIRECT) {
+        bake_filter & BL::BakeSettings::pass_filter_INDIRECT)
+    {
       type = PASS_DIFFUSE;
       use_direct_light = true;
       use_indirect_light = true;
@@ -588,7 +602,8 @@ static bool bake_setup_pass(Scene *scene, const string &bake_type_str, const int
   }
   else if (strcmp(bake_type, "GLOSSY") == 0) {
     if ((bake_filter & BL::BakeSettings::pass_filter_DIRECT) &&
-        bake_filter & BL::BakeSettings::pass_filter_INDIRECT) {
+        bake_filter & BL::BakeSettings::pass_filter_INDIRECT)
+    {
       type = PASS_GLOSSY;
       use_direct_light = true;
       use_indirect_light = true;
@@ -609,7 +624,8 @@ static bool bake_setup_pass(Scene *scene, const string &bake_type_str, const int
   }
   else if (strcmp(bake_type, "TRANSMISSION") == 0) {
     if ((bake_filter & BL::BakeSettings::pass_filter_DIRECT) &&
-        bake_filter & BL::BakeSettings::pass_filter_INDIRECT) {
+        bake_filter & BL::BakeSettings::pass_filter_INDIRECT)
+    {
       type = PASS_TRANSMISSION;
       use_direct_light = true;
       use_indirect_light = true;
@@ -684,17 +700,23 @@ void BlenderSession::bake(BL::Depsgraph &b_depsgraph_,
 
   /* Object might have been disabled for rendering or excluded in some
    * other way, in that case Blender will report a warning afterwards. */
-  bool object_found = false;
+  Object *bake_object = nullptr;
   if (!session->progress.get_cancel()) {
     foreach (Object *ob, scene->objects) {
       if (ob->name == b_object.name()) {
-        object_found = true;
+        bake_object = ob;
         break;
       }
     }
   }
 
-  if (object_found && !session->progress.get_cancel()) {
+  /* For the shadow pass, temporarily mark the object as a shadow catcher. */
+  const bool was_shadow_catcher = (bake_object) ? bake_object->get_is_shadow_catcher() : false;
+  if (bake_object && bake_type == "SHADOW") {
+    bake_object->set_is_shadow_catcher(true);
+  }
+
+  if (bake_object && !session->progress.get_cancel()) {
     /* Get session and buffer parameters. */
     const SessionParams session_params = BlenderSync::get_session_params(
         b_engine, b_userpref, b_scene, background);
@@ -715,10 +737,15 @@ void BlenderSession::bake(BL::Depsgraph &b_depsgraph_,
   }
 
   /* Perform bake. Check cancel to avoid crash with incomplete scene data. */
-  if (object_found && !session->progress.get_cancel()) {
+  if (bake_object && !session->progress.get_cancel()) {
     session->start();
     session->wait();
   }
+
+  /* Restore object state. */
+  if (bake_object) {
+    bake_object->set_is_shadow_catcher(was_shadow_catcher);
+  }
 }
 
 void BlenderSession::synchronize(BL::Depsgraph &b_depsgraph_)
@@ -1066,7 +1093,7 @@ void BlenderSession::ensure_display_driver_if_needed()
   unique_ptr<BlenderDisplayDriver> display_driver = make_unique<BlenderDisplayDriver>(
       b_engine, b_scene, background);
   display_driver_ = display_driver.get();
-  session->set_display_driver(move(display_driver));
+  session->set_display_driver(std::move(display_driver));
 }
 
 CCL_NAMESPACE_END
