@@ -26,7 +26,7 @@
 
 CCL_NAMESPACE_BEGIN
 
-typedef map<void *, ShaderInput *> PtrInputMap;
+typedef unordered_multimap<void *, ShaderInput *> PtrInputMap;
 typedef map<void *, ShaderOutput *> PtrOutputMap;
 typedef map<string, ConvertNode *> ProxyMap;
 
@@ -61,6 +61,12 @@ static DisplacementMethod get_displacement_method(PointerRNA &ptr)
       ptr, "displacement_method", DISPLACE_NUM_METHODS, DISPLACE_BUMP);
 }
 
+static EmissionSampling get_emission_sampling(PointerRNA &ptr)
+{
+  return (EmissionSampling)get_enum(
+      ptr, "emission_sampling", EMISSION_SAMPLING_NUM, EMISSION_SAMPLING_AUTO);
+}
+
 static int validate_enum_value(int value, int num_values, int default_value)
 {
   if (value >= num_values) {
@@ -975,22 +981,8 @@ static ShaderNode *add_node(Scene *scene,
     sky->set_sun_disc(b_sky_node.sun_disc());
     sky->set_sun_size(b_sky_node.sun_size());
     sky->set_sun_intensity(b_sky_node.sun_intensity());
-    /* Patch sun position to be able to animate daylight cycle while keeping the shading code
-     * simple. */
-    float sun_rotation = b_sky_node.sun_rotation();
-    /* Wrap into [-2PI..2PI] range. */
-    float sun_elevation = fmodf(b_sky_node.sun_elevation(), M_2PI_F);
-    /* Wrap into [-PI..PI] range. */
-    if (fabsf(sun_elevation) >= M_PI_F) {
-      sun_elevation -= copysignf(2.0f, sun_elevation) * M_PI_F;
-    }
-    /* Wrap into [-PI/2..PI/2] range while keeping the same absolute position. */
-    if (sun_elevation >= M_PI_2_F || sun_elevation <= -M_PI_2_F) {
-      sun_elevation = copysignf(M_PI_F, sun_elevation) - sun_elevation;
-      sun_rotation += M_PI_F;
-    }
-    sky->set_sun_elevation(sun_elevation);
-    sky->set_sun_rotation(sun_rotation);
+    sky->set_sun_elevation(b_sky_node.sun_elevation());
+    sky->set_sun_rotation(b_sky_node.sun_rotation());
     sky->set_altitude(b_sky_node.altitude());
     sky->set_air_density(b_sky_node.air_density());
     sky->set_dust_density(b_sky_node.dust_density());
@@ -1245,14 +1237,17 @@ static void add_nodes(Scene *scene,
 
         ConvertNode *proxy = graph->create_node<ConvertNode>(to_socket_type, to_socket_type, true);
 
-        input_map[b_link.from_socket().ptr.data] = proxy->inputs[0];
+        /* Muted nodes can result in multiple Cycles input sockets mapping to the same Blender
+         * input socket, so this needs to be a multimap. */
+        input_map.emplace(b_link.from_socket().ptr.data, proxy->inputs[0]);
         output_map[b_link.to_socket().ptr.data] = proxy->outputs[0];
 
         graph->add(proxy);
       }
     }
     else if (b_node.is_a(&RNA_ShaderNodeGroup) || b_node.is_a(&RNA_NodeCustomGroup) ||
-             b_node.is_a(&RNA_ShaderNodeCustomGroup)) {
+             b_node.is_a(&RNA_ShaderNodeCustomGroup))
+    {
 
       BL::ShaderNodeTree b_group_ntree(PointerRNA_NULL);
       if (b_node.is_a(&RNA_ShaderNodeGroup))
@@ -1280,7 +1275,7 @@ static void add_nodes(Scene *scene,
         /* register the proxy node for internal binding */
         group_proxy_input_map[b_input.identifier()] = proxy;
 
-        input_map[b_input.ptr.data] = proxy->inputs[0];
+        input_map.emplace(b_input.ptr.data, proxy->inputs[0]);
 
         set_default_value(proxy->inputs[0], b_input, b_data, b_ntree);
       }
@@ -1332,7 +1327,7 @@ static void add_nodes(Scene *scene,
           if (proxy_it != proxy_output_map.end()) {
             ConvertNode *proxy = proxy_it->second;
 
-            input_map[b_input.ptr.data] = proxy->inputs[0];
+            input_map.emplace(b_input.ptr.data, proxy->inputs[0]);
 
             set_default_value(proxy->inputs[0], b_input, b_data, b_ntree);
           }
@@ -1363,7 +1358,7 @@ static void add_nodes(Scene *scene,
             /* XXX should not happen, report error? */
             continue;
           }
-          input_map[b_input.ptr.data] = input;
+          input_map.emplace(b_input.ptr.data, input);
 
           set_default_value(input, b_input, b_data, b_ntree);
         }
@@ -1388,27 +1383,31 @@ static void add_nodes(Scene *scene,
     /* Ignore invalid links to avoid unwanted cycles created in graph.
      * Also ignore links with unavailable sockets. */
     if (!(b_link.is_valid() && b_link.from_socket().enabled() && b_link.to_socket().enabled()) ||
-        b_link.is_muted()) {
+        b_link.is_muted())
+    {
       continue;
     }
     /* get blender link data */
     BL::NodeSocket b_from_sock = b_link.from_socket();
     BL::NodeSocket b_to_sock = b_link.to_socket();
 
-    ShaderOutput *output = 0;
-    ShaderInput *input = 0;
-
+    ShaderOutput *output = nullptr;
     PtrOutputMap::iterator output_it = output_map.find(b_from_sock.ptr.data);
     if (output_it != output_map.end())
       output = output_it->second;
-    PtrInputMap::iterator input_it = input_map.find(b_to_sock.ptr.data);
-    if (input_it != input_map.end())
-      input = input_it->second;
 
-    /* either node may be NULL when the node was not exported, typically
+    /* either socket may be NULL when the node was not exported, typically
      * because the node type is not supported */
-    if (output && input)
-      graph->connect(output, input);
+    if (output != nullptr) {
+      ShaderOutput *output = output_it->second;
+      auto inputs = input_map.equal_range(b_to_sock.ptr.data);
+      for (PtrInputMap::iterator input_it = inputs.first; input_it != inputs.second; ++input_it) {
+        ShaderInput *input = input_it->second;
+        if (input != nullptr) {
+          graph->connect(output, input);
+        }
+      }
+    }
   }
 }
 
@@ -1534,7 +1533,8 @@ void BlenderSync::sync_materials(BL::Depsgraph &b_depsgraph, bool update_all)
 
     /* test if we need to sync */
     if (shader_map.add_or_update(&shader, b_mat) || update_all ||
-        scene_attr_needs_recalc(shader, b_depsgraph)) {
+        scene_attr_needs_recalc(shader, b_depsgraph))
+    {
       ShaderGraph *graph = new ShaderGraph();
 
       shader->name = b_mat.name().c_str();
@@ -1559,7 +1559,7 @@ void BlenderSync::sync_materials(BL::Depsgraph &b_depsgraph, bool update_all)
 
       /* settings */
       PointerRNA cmat = RNA_pointer_get(&b_mat.ptr, "cycles");
-      shader->set_use_mis(get_boolean(cmat, "sample_as_light"));
+      shader->set_emission_sampling_method(get_emission_sampling(cmat));
       shader->set_use_transparent_shadow(get_boolean(cmat, "use_transparent_shadow"));
       shader->set_heterogeneous_volume(!get_boolean(cmat, "homogeneous_volume"));
       shader->set_volume_sampling_method(get_volume_sampling(cmat));
@@ -1617,12 +1617,14 @@ void BlenderSync::sync_world(BL::Depsgraph &b_depsgraph, BL::SpaceView3D &b_v3d,
 
   if (world_recalc || update_all || b_world.ptr.data != world_map ||
       viewport_parameters.shader_modified(new_viewport_parameters) ||
-      scene_attr_needs_recalc(shader, b_depsgraph)) {
+      scene_attr_needs_recalc(shader, b_depsgraph))
+  {
     ShaderGraph *graph = new ShaderGraph();
 
     /* create nodes */
     if (new_viewport_parameters.use_scene_world && b_world && b_world.use_nodes() &&
-        b_world.node_tree()) {
+        b_world.node_tree())
+    {
       BL::ShaderNodeTree b_ntree(b_world.node_tree());
 
       add_nodes(scene, b_engine, b_data, b_depsgraph, b_scene, graph, b_ntree);
@@ -1784,7 +1786,8 @@ void BlenderSync::sync_lights(BL::Depsgraph &b_depsgraph, bool update_all)
 
     /* test if we need to sync */
     if (shader_map.add_or_update(&shader, b_light) || update_all ||
-        scene_attr_needs_recalc(shader, b_depsgraph)) {
+        scene_attr_needs_recalc(shader, b_depsgraph))
+    {
       ShaderGraph *graph = new ShaderGraph();
 
       /* create nodes */
