@@ -35,7 +35,8 @@ class BlenderSmokeLoader : public ImageLoader {
     }
 
     if (attribute == ATTR_STD_VOLUME_DENSITY || attribute == ATTR_STD_VOLUME_FLAME ||
-        attribute == ATTR_STD_VOLUME_HEAT || attribute == ATTR_STD_VOLUME_TEMPERATURE) {
+        attribute == ATTR_STD_VOLUME_HEAT || attribute == ATTR_STD_VOLUME_TEMPERATURE)
+    {
       metadata.type = IMAGE_DATA_TYPE_FLOAT;
       metadata.channels = 1;
     }
@@ -315,24 +316,29 @@ static void sync_volume_object(BL::BlendData &b_data,
       std = ATTR_STD_VOLUME_TEMPERATURE;
     }
     else if (name == Attribute::standard_name(ATTR_STD_VOLUME_VELOCITY) ||
-             name == b_volume.velocity_grid()) {
+             name == b_volume.velocity_grid())
+    {
       std = ATTR_STD_VOLUME_VELOCITY;
     }
     else if (name == Attribute::standard_name(ATTR_STD_VOLUME_VELOCITY_X) ||
-             name == b_volume.velocity_x_grid()) {
+             name == b_volume.velocity_x_grid())
+    {
       std = ATTR_STD_VOLUME_VELOCITY_X;
     }
     else if (name == Attribute::standard_name(ATTR_STD_VOLUME_VELOCITY_Y) ||
-             name == b_volume.velocity_y_grid()) {
+             name == b_volume.velocity_y_grid())
+    {
       std = ATTR_STD_VOLUME_VELOCITY_Y;
     }
     else if (name == Attribute::standard_name(ATTR_STD_VOLUME_VELOCITY_Z) ||
-             name == b_volume.velocity_z_grid()) {
+             name == b_volume.velocity_z_grid())
+    {
       std = ATTR_STD_VOLUME_VELOCITY_Z;
     }
 
     if ((std != ATTR_STD_NONE && volume->need_attribute(scene, std)) ||
-        volume->need_attribute(scene, name)) {
+        volume->need_attribute(scene, name))
+    {
       Attribute *attr = (std != ATTR_STD_NONE) ?
                             volume->attributes.add(std) :
                             volume->attributes.add(name, TypeDesc::TypeFloat, ATTR_ELEMENT_VOXEL);
