@@ -17,7 +17,11 @@
 
 #ifdef WITH_EMBREE
 
-#  include <embree3/rtcore_geometry.h>
+#  if EMBREE_MAJOR_VERSION >= 4
+#    include <embree4/rtcore_geometry.h>
+#  else
+#    include <embree3/rtcore_geometry.h>
+#  endif
 
 #  include "bvh/embree.h"
 
@@ -107,9 +111,13 @@ BVHEmbree::~BVHEmbree()
   }
 }
 
-void BVHEmbree::build(Progress &progress, Stats *stats, RTCDevice rtc_device_)
+void BVHEmbree::build(Progress &progress,
+                      Stats *stats,
+                      RTCDevice rtc_device_,
+                      const bool rtc_device_is_sycl_)
 {
   rtc_device = rtc_device_;
+  rtc_device_is_sycl = rtc_device_is_sycl_;
   assert(rtc_device);
 
   rtcSetDeviceErrorFunction(rtc_device, rtc_error_func, NULL);
@@ -128,7 +136,11 @@ void BVHEmbree::build(Progress &progress, Stats *stats, RTCDevice rtc_device_)
   scene = rtcNewScene(rtc_device);
   const RTCSceneFlags scene_flags = (dynamic ? RTC_SCENE_FLAG_DYNAMIC : RTC_SCENE_FLAG_NONE) |
                                     (compact ? RTC_SCENE_FLAG_COMPACT : RTC_SCENE_FLAG_NONE) |
-                                    RTC_SCENE_FLAG_ROBUST;
+                                    RTC_SCENE_FLAG_ROBUST
+#  if EMBREE_MAJOR_VERSION >= 4
+                                    | RTC_SCENE_FLAG_FILTER_FUNCTION_IN_ARGUMENTS
+#  endif
+      ;
   rtcSetSceneFlags(scene, scene_flags);
   build_quality = dynamic ? RTC_BUILD_QUALITY_LOW :
                             (params.use_spatial_split ? RTC_BUILD_QUALITY_HIGH :
@@ -226,6 +238,9 @@ void BVHEmbree::add_instance(Object *ob, int i)
 
   rtcSetGeometryUserData(geom_id, (void *)instance_bvh->scene);
   rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  if EMBREE_MAJOR_VERSION >= 4
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
+#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2);
@@ -254,27 +269,40 @@ void BVHEmbree::add_triangles(const Object *ob, const Mesh *mesh, int i)
   rtcSetGeometryBuildQuality(geom_id, build_quality);
   rtcSetGeometryTimeStepCount(geom_id, num_motion_steps);
 
-  unsigned *rtc_indices = (unsigned *)rtcSetNewGeometryBuffer(
-      geom_id, RTC_BUFFER_TYPE_INDEX, 0, RTC_FORMAT_UINT3, sizeof(int) * 3, num_triangles);
-  assert(rtc_indices);
-  if (!rtc_indices) {
-    VLOG_WARNING << "Embree could not create new geometry buffer for mesh " << mesh->name.c_str()
-                 << ".\n";
-    return;
+  const int *triangles = mesh->get_triangles().data();
+  if (!rtc_device_is_sycl) {
+    rtcSetSharedGeometryBuffer(geom_id,
+                               RTC_BUFFER_TYPE_INDEX,
+                               0,
+                               RTC_FORMAT_UINT3,
+                               triangles,
+                               0,
+                               sizeof(int) * 3,
+                               num_triangles);
   }
-  for (size_t j = 0; j < num_triangles; ++j) {
-    Mesh::Triangle t = mesh->get_triangle(j);
-    rtc_indices[j * 3] = t.v[0];
-    rtc_indices[j * 3 + 1] = t.v[1];
-    rtc_indices[j * 3 + 2] = t.v[2];
+  else {
+    /* NOTE(sirgienko): If the Embree device is a SYCL device, then Embree execution will
+     * happen on GPU, and we cannot use standard host pointers at this point. So instead
+     * of making a shared geometry buffer - a new Embree buffer will be created and data
+     * will be copied. */
+    int *triangles_buffer = (int *)rtcSetNewGeometryBuffer(
+        geom_id, RTC_BUFFER_TYPE_INDEX, 0, RTC_FORMAT_UINT3, sizeof(int) * 3, num_triangles);
+    assert(triangles_buffer);
+    if (triangles_buffer) {
+      static_assert(sizeof(int) == sizeof(uint));
+      std::memcpy(triangles_buffer, triangles, sizeof(int) * 3 * (num_triangles));
+    }
   }
-
   set_tri_vertex_buffer(geom_id, mesh, false);
 
   rtcSetGeometryUserData(geom_id, (void *)prim_offset);
+  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  if EMBREE_MAJOR_VERSION >= 4
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
+#  else
   rtcSetGeometryOccludedFilterFunction(geom_id, kernel_embree_filter_occluded_func);
   rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_intersection_func);
-  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2);
@@ -309,28 +337,77 @@ void BVHEmbree::set_tri_vertex_buffer(RTCGeometry geom_id, const Mesh *mesh, con
       verts = &attr_mP->data_float3()[t_ * num_verts];
     }
 
-    float *rtc_verts = (update) ?
-                           (float *)rtcGetGeometryBufferData(geom_id, RTC_BUFFER_TYPE_VERTEX, t) :
-                           (float *)rtcSetNewGeometryBuffer(geom_id,
-                                                            RTC_BUFFER_TYPE_VERTEX,
-                                                            t,
-                                                            RTC_FORMAT_FLOAT3,
-                                                            sizeof(float) * 3,
-                                                            num_verts + 1);
-
-    assert(rtc_verts);
-    if (rtc_verts) {
-      for (size_t j = 0; j < num_verts; ++j) {
-        rtc_verts[0] = verts[j].x;
-        rtc_verts[1] = verts[j].y;
-        rtc_verts[2] = verts[j].z;
-        rtc_verts += 3;
-      }
-    }
-
     if (update) {
       rtcUpdateGeometryBuffer(geom_id, RTC_BUFFER_TYPE_VERTEX, t);
     }
+    else {
+      if (!rtc_device_is_sycl) {
+        /* NOTE(sirgienko) Embree requires padding for VERTEX layout as last buffer element
+         * must be readable using 16-byte SSE load instructions. Because of this, we are
+         * artificially increasing shared buffer size by 1 - it shouldn't cause any memory
+         * access violation as this last element is not accessed directly since no triangle
+         * can reference it. */
+        rtcSetSharedGeometryBuffer(geom_id,
+                                   RTC_BUFFER_TYPE_VERTEX,
+                                   t,
+                                   RTC_FORMAT_FLOAT3,
+                                   verts,
+                                   0,
+                                   sizeof(float3),
+                                   num_verts + 1);
+      }
+      else {
+        /* NOTE(sirgienko): If the Embree device is a SYCL device, then Embree execution will
+         * happen on GPU, and we cannot use standard host pointers at this point. So instead
+         * of making a shared geometry buffer - a new Embree buffer will be created and data
+         * will be copied. */
+        /* As float3 is packed on GPU side, we map it to packed_float3. */
+        /* There is no need for additional padding in rtcSetNewGeometryBuffer since Embree 3.6:
+         * "Fixed automatic vertex buffer padding when using rtcSetNewGeometry API function". */
+        packed_float3 *verts_buffer = (packed_float3 *)rtcSetNewGeometryBuffer(
+            geom_id,
+            RTC_BUFFER_TYPE_VERTEX,
+            t,
+            RTC_FORMAT_FLOAT3,
+            sizeof(packed_float3),
+            num_verts);
+        assert(verts_buffer);
+        if (verts_buffer) {
+          for (size_t i = (size_t)0; i < num_verts; ++i) {
+            verts_buffer[i].x = verts[i].x;
+            verts_buffer[i].y = verts[i].y;
+            verts_buffer[i].z = verts[i].z;
+          }
+        }
+      }
+    }
+  }
+}
+
+/**
+ * Packs the hair motion curve data control variables (CVs) into float4s as [x y z radius]
+ */
+template<typename T>
+void pack_motion_verts(size_t num_curves,
+                       const Hair *hair,
+                       const T *verts,
+                       const float *curve_radius,
+                       float4 *rtc_verts)
+{
+  for (size_t j = 0; j < num_curves; ++j) {
+    Hair::Curve c = hair->get_curve(j);
+    int fk = c.first_key;
+    int k = 1;
+    for (; k < c.num_keys + 1; ++k, ++fk) {
+      rtc_verts[k].x = verts[fk].x;
+      rtc_verts[k].y = verts[fk].y;
+      rtc_verts[k].z = verts[fk].z;
+      rtc_verts[k].w = curve_radius[fk];
+    }
+    /* Duplicate Embree's Catmull-Rom spline CVs at the start and end of each curve. */
+    rtc_verts[0] = rtc_verts[1];
+    rtc_verts[k] = rtc_verts[k - 1];
+    rtc_verts += c.num_keys + 2;
   }
 }
 
@@ -360,15 +437,10 @@ void BVHEmbree::set_curve_vertex_buffer(RTCGeometry geom_id, const Hair *hair, c
   const int t_mid = (num_motion_steps - 1) / 2;
   const float *curve_radius = &hair->get_curve_radius()[0];
   for (int t = 0; t < num_motion_steps; ++t) {
-    const float3 *verts;
-    if (t == t_mid || attr_mP == NULL) {
-      verts = &hair->get_curve_keys()[0];
-    }
-    else {
-      int t_ = (t > t_mid) ? (t - 1) : t;
-      verts = &attr_mP->data_float3()[t_ * num_keys];
-    }
-
+    // As float4 and float3 are no longer interchangeable the 2 types need to be
+    // handled separately. Attributes are float4s where the radius is stored in w and
+    // the middle motion vector is from the mesh points which are stored float3s with
+    // the radius stored in another array.
     float4 *rtc_verts = (update) ? (float4 *)rtcGetGeometryBufferData(
                                        geom_id, RTC_BUFFER_TYPE_VERTEX, t) :
                                    (float4 *)rtcSetNewGeometryBuffer(geom_id,
@@ -381,18 +453,14 @@ void BVHEmbree::set_curve_vertex_buffer(RTCGeometry geom_id, const Hair *hair, c
     assert(rtc_verts);
     if (rtc_verts) {
       const size_t num_curves = hair->num_curves();
-      for (size_t j = 0; j < num_curves; ++j) {
-        Hair::Curve c = hair->get_curve(j);
-        int fk = c.first_key;
-        int k = 1;
-        for (; k < c.num_keys + 1; ++k, ++fk) {
-          rtc_verts[k] = float3_to_float4(verts[fk]);
-          rtc_verts[k].w = curve_radius[fk];
-        }
-        /* Duplicate Embree's Catmull-Rom spline CVs at the start and end of each curve. */
-        rtc_verts[0] = rtc_verts[1];
-        rtc_verts[k] = rtc_verts[k - 1];
-        rtc_verts += c.num_keys + 2;
+      if (t == t_mid || attr_mP == NULL) {
+        const float3 *verts = &hair->get_curve_keys()[0];
+        pack_motion_verts<float3>(num_curves, hair, verts, curve_radius, rtc_verts);
+      }
+      else {
+        int t_ = (t > t_mid) ? (t - 1) : t;
+        const float4 *verts = &attr_mP->data_float4()[t_ * num_keys];
+        pack_motion_verts<float4>(num_curves, hair, verts, curve_radius, rtc_verts);
       }
     }
 
@@ -421,15 +489,10 @@ void BVHEmbree::set_point_vertex_buffer(RTCGeometry geom_id,
   const int t_mid = (num_motion_steps - 1) / 2;
   const float *radius = pointcloud->get_radius().data();
   for (int t = 0; t < num_motion_steps; ++t) {
-    const float3 *verts;
-    if (t == t_mid || attr_mP == NULL) {
-      verts = pointcloud->get_points().data();
-    }
-    else {
-      int t_ = (t > t_mid) ? (t - 1) : t;
-      verts = &attr_mP->data_float3()[t_ * num_points];
-    }
-
+    // As float4 and float3 are no longer interchangeable the 2 types need to be
+    // handled separately. Attributes are float4s where the radius is stored in w and
+    // the middle motion vector is from the mesh points which are stored float3s with
+    // the radius stored in another array.
     float4 *rtc_verts = (update) ? (float4 *)rtcGetGeometryBufferData(
                                        geom_id, RTC_BUFFER_TYPE_VERTEX, t) :
                                    (float4 *)rtcSetNewGeometryBuffer(geom_id,
@@ -441,9 +504,21 @@ void BVHEmbree::set_point_vertex_buffer(RTCGeometry geom_id,
 
     assert(rtc_verts);
     if (rtc_verts) {
-      for (size_t j = 0; j < num_points; ++j) {
-        rtc_verts[j] = float3_to_float4(verts[j]);
-        rtc_verts[j].w = radius[j];
+      if (t == t_mid || attr_mP == NULL) {
+        /* Pack the motion points into a float4 as [x y z radius]. */
+        const float3 *verts = pointcloud->get_points().data();
+        for (size_t j = 0; j < num_points; ++j) {
+          rtc_verts[j].x = verts[j].x;
+          rtc_verts[j].y = verts[j].y;
+          rtc_verts[j].z = verts[j].z;
+          rtc_verts[j].w = radius[j];
+        }
+      }
+      else {
+        /* Motion blur is already packed as [x y z radius]. */
+        int t_ = (t > t_mid) ? (t - 1) : t;
+        const float4 *verts = &attr_mP->data_float4()[t_ * num_points];
+        memcpy(rtc_verts, verts, sizeof(float4) * num_points);
       }
     }
 
@@ -476,9 +551,13 @@ void BVHEmbree::add_points(const Object *ob, const PointCloud *pointcloud, int i
   set_point_vertex_buffer(geom_id, pointcloud, false);
 
   rtcSetGeometryUserData(geom_id, (void *)prim_offset);
+  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  if EMBREE_MAJOR_VERSION >= 4
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
+#  else
   rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_func_backface_cull);
   rtcSetGeometryOccludedFilterFunction(geom_id, kernel_embree_filter_occluded_func_backface_cull);
-  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2);
@@ -535,6 +614,10 @@ void BVHEmbree::add_curves(const Object *ob, const Hair *hair, int i)
   set_curve_vertex_buffer(geom_id, hair, false);
 
   rtcSetGeometryUserData(geom_id, (void *)prim_offset);
+  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  if EMBREE_MAJOR_VERSION >= 4
+  rtcSetGeometryEnableFilterFunctionFromArguments(geom_id, true);
+#  else
   if (hair->curve_shape == CURVE_RIBBON) {
     rtcSetGeometryIntersectFilterFunction(geom_id, kernel_embree_filter_intersection_func);
     rtcSetGeometryOccludedFilterFunction(geom_id, kernel_embree_filter_occluded_func);
@@ -544,7 +627,7 @@ void BVHEmbree::add_curves(const Object *ob, const Hair *hair, int i)
     rtcSetGeometryOccludedFilterFunction(geom_id,
                                          kernel_embree_filter_occluded_func_backface_cull);
   }
-  rtcSetGeometryMask(geom_id, ob->visibility_for_tracing());
+#  endif
 
   rtcCommitGeometry(geom_id);
   rtcAttachGeometryByID(scene, geom_id, i * 2 + 1);
