@@ -360,7 +360,7 @@ void BVHSpatialSplit::split_triangle_primitive(const Mesh *mesh,
 
     /* edge intersects the plane => insert intersection to both boxes. */
     if ((v0p < pos && v1p > pos) || (v0p > pos && v1p < pos)) {
-      float3 t = lerp(v0, v1, clamp((pos - v0p) / (v1p - v0p), 0.0f, 1.0f));
+      float3 t = mix(v0, v1, clamp((pos - v0p) / (v1p - v0p), 0.0f, 1.0f));
       left_bounds.grow(t);
       right_bounds.grow(t);
     }
@@ -408,7 +408,7 @@ void BVHSpatialSplit::split_curve_primitive(const Hair *hair,
 
   /* edge intersects the plane => insert intersection to both boxes. */
   if ((v0p < pos && v1p > pos) || (v0p > pos && v1p < pos)) {
-    float3 t = lerp(v0, v1, clamp((pos - v0p) / (v1p - v0p), 0.0f, 1.0f));
+    float3 t = mix(v0, v1, clamp((pos - v0p) / (v1p - v0p), 0.0f, 1.0f));
     left_bounds.grow(t);
     right_bounds.grow(t);
   }
@@ -425,6 +425,7 @@ void BVHSpatialSplit::split_point_primitive(const PointCloud *pointcloud,
   /* No real splitting support for points, assume they are small enough for it
    * not to matter. */
   float3 point = pointcloud->get_points()[prim_index];
+  float radius = pointcloud->get_radius()[prim_index];
 
   if (tfm != NULL) {
     point = transform_point(tfm, point);
@@ -432,11 +433,11 @@ void BVHSpatialSplit::split_point_primitive(const PointCloud *pointcloud,
   point = get_unaligned_point(point);
 
   if (point[dim] <= pos) {
-    left_bounds.grow(point);
+    left_bounds.grow(point, radius);
   }
 
   if (point[dim] >= pos) {
-    right_bounds.grow(point);
+    right_bounds.grow(point, radius);
   }
 }
 
