@@ -17,13 +17,10 @@ template<typename FunctionType> class CPUKernelFunction {
  public:
   CPUKernelFunction(FunctionType kernel_default,
                     FunctionType kernel_sse2,
-                    FunctionType kernel_sse3,
                     FunctionType kernel_sse41,
-                    FunctionType kernel_avx,
                     FunctionType kernel_avx2)
   {
-    kernel_info_ = get_best_kernel_info(
-        kernel_default, kernel_sse2, kernel_sse3, kernel_sse41, kernel_avx, kernel_avx2);
+    kernel_info_ = get_best_kernel_info(kernel_default, kernel_sse2, kernel_sse41, kernel_avx2);
   }
 
   template<typename... Args> inline auto operator()(Args... args) const
@@ -43,9 +40,7 @@ template<typename FunctionType> class CPUKernelFunction {
    * pointer. */
   class KernelInfo {
    public:
-    KernelInfo() : KernelInfo("", nullptr)
-    {
-    }
+    KernelInfo() : KernelInfo("", nullptr) {}
 
     /* TODO(sergey): Use string view, to have higher-level functionality (i.e. comparison) without
      * memory allocation. */
@@ -60,16 +55,12 @@ template<typename FunctionType> class CPUKernelFunction {
 
   KernelInfo get_best_kernel_info(FunctionType kernel_default,
                                   FunctionType kernel_sse2,
-                                  FunctionType kernel_sse3,
                                   FunctionType kernel_sse41,
-                                  FunctionType kernel_avx,
                                   FunctionType kernel_avx2)
   {
     /* Silence warnings about unused variables when compiling without some architectures. */
     (void)kernel_sse2;
-    (void)kernel_sse3;
     (void)kernel_sse41;
-    (void)kernel_avx;
     (void)kernel_avx2;
 
 #ifdef WITH_CYCLES_OPTIMIZED_KERNEL_AVX2
@@ -78,24 +69,12 @@ template<typename FunctionType> class CPUKernelFunction {
     }
 #endif
 
-#ifdef WITH_CYCLES_OPTIMIZED_KERNEL_AVX
-    if (DebugFlags().cpu.has_avx() && system_cpu_support_avx()) {
-      return KernelInfo("AVX", kernel_avx);
-    }
-#endif
-
 #ifdef WITH_CYCLES_OPTIMIZED_KERNEL_SSE41
     if (DebugFlags().cpu.has_sse41() && system_cpu_support_sse41()) {
       return KernelInfo("SSE4.1", kernel_sse41);
     }
 #endif
 
-#ifdef WITH_CYCLES_OPTIMIZED_KERNEL_SSE3
-    if (DebugFlags().cpu.has_sse3() && system_cpu_support_sse3()) {
-      return KernelInfo("SSE3", kernel_sse3);
-    }
-#endif
-
 #ifdef WITH_CYCLES_OPTIMIZED_KERNEL_SSE2
     if (DebugFlags().cpu.has_sse2() && system_cpu_support_sse2()) {
       return KernelInfo("SSE2", kernel_sse2);
