@@ -18,38 +18,24 @@ class DummyDevice : public Device {
     error_msg = info.error_msg;
   }
 
-  ~DummyDevice()
-  {
-  }
+  ~DummyDevice() {}
 
-  virtual BVHLayoutMask get_bvh_layout_mask() const override
+  virtual BVHLayoutMask get_bvh_layout_mask(uint /*kernel_features*/) const override
   {
     return 0;
   }
 
-  virtual void mem_alloc(device_memory &) override
-  {
-  }
+  virtual void mem_alloc(device_memory &) override {}
 
-  virtual void mem_copy_to(device_memory &) override
-  {
-  }
+  virtual void mem_copy_to(device_memory &) override {}
 
-  virtual void mem_copy_from(device_memory &, size_t, size_t, size_t, size_t) override
-  {
-  }
+  virtual void mem_copy_from(device_memory &, size_t, size_t, size_t, size_t) override {}
 
-  virtual void mem_zero(device_memory &) override
-  {
-  }
+  virtual void mem_zero(device_memory &) override {}
 
-  virtual void mem_free(device_memory &) override
-  {
-  }
+  virtual void mem_free(device_memory &) override {}
 
-  virtual void const_copy_to(const char *, void *, size_t) override
-  {
-  }
+  virtual void const_copy_to(const char *, void *, size_t) override {}
 };
 
 Device *device_dummy_create(const DeviceInfo &info, Stats &stats, Profiler &profiler)
