@@ -72,7 +72,8 @@ class MultiDevice : public Device {
       foreach (SubDevice &peer_sub, devices) {
         if (peer_sub.peer_island_index < 0 &&
             peer_sub.device->info.type == sub.device->info.type &&
-            peer_sub.device->check_peer_access(sub.device)) {
+            peer_sub.device->check_peer_access(sub.device))
+        {
           peer_sub.peer_island_index = sub.peer_island_index;
           peer_islands[sub.peer_island_index].push_back(&peer_sub);
         }
@@ -96,12 +97,13 @@ class MultiDevice : public Device {
     return error_msg;
   }
 
-  virtual BVHLayoutMask get_bvh_layout_mask() const override
+  virtual BVHLayoutMask get_bvh_layout_mask(uint kernel_features) const override
   {
     BVHLayoutMask bvh_layout_mask = BVH_LAYOUT_ALL;
     BVHLayoutMask bvh_layout_mask_all = BVH_LAYOUT_NONE;
     foreach (const SubDevice &sub_device, devices) {
-      BVHLayoutMask device_bvh_layout_mask = sub_device.device->get_bvh_layout_mask();
+      BVHLayoutMask device_bvh_layout_mask = sub_device.device->get_bvh_layout_mask(
+          kernel_features);
       bvh_layout_mask &= device_bvh_layout_mask;
       bvh_layout_mask_all |= device_bvh_layout_mask;
     }
@@ -116,6 +118,15 @@ class MultiDevice : public Device {
       return BVH_LAYOUT_MULTI_METAL;
     }
 
+    if (bvh_layout_mask == BVH_LAYOUT_HIPRT) {
+      return BVH_LAYOUT_MULTI_HIPRT;
+    }
+
+    /* With multiple oneAPI devices, every device needs its own acceleration structure */
+    if (bvh_layout_mask == BVH_LAYOUT_EMBREEGPU) {
+      return BVH_LAYOUT_MULTI_EMBREEGPU;
+    }
+
     /* When devices do not share a common BVH layout, fall back to creating one for each */
     const BVHLayoutMask BVH_LAYOUT_OPTIX_EMBREE = (BVH_LAYOUT_OPTIX | BVH_LAYOUT_EMBREE);
     if ((bvh_layout_mask_all & BVH_LAYOUT_OPTIX_EMBREE) == BVH_LAYOUT_OPTIX_EMBREE) {
@@ -125,6 +136,15 @@ class MultiDevice : public Device {
     if ((bvh_layout_mask_all & BVH_LAYOUT_METAL_EMBREE) == BVH_LAYOUT_METAL_EMBREE) {
       return BVH_LAYOUT_MULTI_METAL_EMBREE;
     }
+    const BVHLayoutMask BVH_LAYOUT_EMBREEGPU_EMBREE = (BVH_LAYOUT_EMBREEGPU | BVH_LAYOUT_EMBREE);
+    if ((bvh_layout_mask_all & BVH_LAYOUT_EMBREEGPU_EMBREE) == BVH_LAYOUT_EMBREEGPU_EMBREE) {
+      return BVH_LAYOUT_MULTI_EMBREEGPU_EMBREE;
+    }
+
+    const BVHLayoutMask BVH_LAYOUT_HIPRT_EMBREE = (BVH_LAYOUT_HIPRT | BVH_LAYOUT_EMBREE);
+    if ((bvh_layout_mask_all & BVH_LAYOUT_HIPRT_EMBREE) == BVH_LAYOUT_HIPRT_EMBREE) {
+      return BVH_LAYOUT_MULTI_HIPRT_EMBREE;
+    }
 
     return bvh_layout_mask;
   }
@@ -138,6 +158,15 @@ class MultiDevice : public Device {
     return true;
   }
 
+  bool load_osl_kernels() override
+  {
+    foreach (SubDevice &sub, devices)
+      if (!sub.device->load_osl_kernels())
+        return false;
+
+    return true;
+  }
+
   void build_bvh(BVH *bvh, Progress &progress, bool refit) override
   {
     /* Try to build and share a single acceleration structure, if possible */
@@ -148,8 +177,12 @@ class MultiDevice : public Device {
 
     assert(bvh->params.bvh_layout == BVH_LAYOUT_MULTI_OPTIX ||
            bvh->params.bvh_layout == BVH_LAYOUT_MULTI_METAL ||
+           bvh->params.bvh_layout == BVH_LAYOUT_MULTI_HIPRT ||
+           bvh->params.bvh_layout == BVH_LAYOUT_MULTI_EMBREEGPU ||
            bvh->params.bvh_layout == BVH_LAYOUT_MULTI_OPTIX_EMBREE ||
-           bvh->params.bvh_layout == BVH_LAYOUT_MULTI_METAL_EMBREE);
+           bvh->params.bvh_layout == BVH_LAYOUT_MULTI_METAL_EMBREE ||
+           bvh->params.bvh_layout == BVH_LAYOUT_MULTI_HIPRT_EMBREE ||
+           bvh->params.bvh_layout == BVH_LAYOUT_MULTI_EMBREEGPU_EMBREE);
 
     BVHMulti *const bvh_multi = static_cast<BVHMulti *>(bvh);
     bvh_multi->sub_bvhs.resize(devices.size());
@@ -174,17 +207,27 @@ class MultiDevice : public Device {
           params.bvh_layout = BVH_LAYOUT_OPTIX;
         else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_METAL)
           params.bvh_layout = BVH_LAYOUT_METAL;
+        else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_HIPRT)
+          params.bvh_layout = BVH_LAYOUT_HIPRT;
+        else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_EMBREEGPU)
+          params.bvh_layout = BVH_LAYOUT_EMBREEGPU;
         else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_OPTIX_EMBREE)
           params.bvh_layout = sub.device->info.type == DEVICE_OPTIX ? BVH_LAYOUT_OPTIX :
                                                                       BVH_LAYOUT_EMBREE;
         else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_METAL_EMBREE)
           params.bvh_layout = sub.device->info.type == DEVICE_METAL ? BVH_LAYOUT_METAL :
                                                                       BVH_LAYOUT_EMBREE;
-
+        else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_HIPRT_EMBREE)
+          params.bvh_layout = sub.device->info.type == DEVICE_HIP ? BVH_LAYOUT_HIPRT :
+                                                                      BVH_LAYOUT_EMBREE;
+        else if (bvh->params.bvh_layout == BVH_LAYOUT_MULTI_EMBREEGPU_EMBREE)
+          params.bvh_layout = sub.device->info.type == DEVICE_ONEAPI ? BVH_LAYOUT_EMBREEGPU :
+                                                                       BVH_LAYOUT_EMBREE;
         /* Skip building a bottom level acceleration structure for non-instanced geometry on Embree
          * (since they are put into the top level directly, see bvh_embree.cpp) */
         if (!params.top_level && params.bvh_layout == BVH_LAYOUT_EMBREE &&
-            !bvh->geometry[0]->is_instanced()) {
+            !bvh->geometry[0]->is_instanced())
+        {
           i++;
           continue;
         }
@@ -204,10 +247,12 @@ class MultiDevice : public Device {
 
   virtual void *get_cpu_osl_memory() override
   {
-    if (devices.size() > 1) {
+    /* Always return the OSL memory of the CPU device (this works since the constructor above
+     * guarantees that CPU devices are always added to the back). */
+    if (devices.size() > 1 && devices.back().device->info.type != DEVICE_CPU) {
       return NULL;
     }
-    return devices.front().device->get_cpu_osl_memory();
+    return devices.back().device->get_cpu_osl_memory();
   }
 
   bool is_resident(device_ptr key, Device *sub_device) override
@@ -228,8 +273,8 @@ class MultiDevice : public Device {
     SubDevice *owner_sub = &sub;
     if (owner_sub->ptr_map.find(key) == owner_sub->ptr_map.end()) {
       foreach (SubDevice *island_sub, peer_islands[sub.peer_island_index]) {
-        if (island_sub != owner_sub &&
-            island_sub->ptr_map.find(key) != island_sub->ptr_map.end()) {
+        if (island_sub != owner_sub && island_sub->ptr_map.find(key) != island_sub->ptr_map.end())
+        {
           owner_sub = island_sub;
         }
       }
@@ -245,7 +290,8 @@ class MultiDevice : public Device {
     SubDevice *owner_sub = island.front();
     foreach (SubDevice *island_sub, island) {
       if (key ? (island_sub->ptr_map.find(key) != island_sub->ptr_map.end()) :
-                (island_sub->device->stats.mem_used < owner_sub->device->stats.mem_used)) {
+                (island_sub->device->stats.mem_used < owner_sub->device->stats.mem_used))
+      {
         owner_sub = island_sub;
       }
     }
