@@ -14,7 +14,11 @@ class HdCyclesPlugin final : public PXR_NS::HdRendererPlugin {
   HdCyclesPlugin();
   ~HdCyclesPlugin() override;
 
+#if PXR_VERSION < 2302
   bool IsSupported() const override;
+#else
+  bool IsSupported(bool gpuEnabled) const override;
+#endif
 
   PXR_NS::HdRenderDelegate *CreateRenderDelegate() override;
   PXR_NS::HdRenderDelegate *CreateRenderDelegate(const PXR_NS::HdRenderSettingsMap &) override;
