@@ -142,9 +142,7 @@ class PathTraceWork {
 
 #ifdef WITH_PATH_GUIDING
   /* Initializes the per-thread guiding kernel data. */
-  virtual void guiding_init_kernel_globals(void *, void *, const bool)
-  {
-  }
+  virtual void guiding_init_kernel_globals(void *, void *, const bool) {}
 #endif
 
  protected:
