@@ -247,13 +247,17 @@ size_t CachedData::memory_used() const
 static M44d convert_yup_zup(const M44d &mtx, float scale_mult)
 {
   V3d scale, shear, rotation, translation;
-  extractSHRT(mtx,
-              scale,
-              shear,
-              rotation,
-              translation,
-              true,
-              IMATH_INTERNAL_NAMESPACE::Euler<double>::XZY);
+
+  if (!extractSHRT(mtx,
+                   scale,
+                   shear,
+                   rotation,
+                   translation,
+                   true,
+                   IMATH_INTERNAL_NAMESPACE::Euler<double>::XZY))
+  {
+    return mtx;
+  }
 
   M44d rot_mat, scale_mat, trans_mat;
   rot_mat.setEulerAngles(V3d(rotation.x, -rotation.z, rotation.y));
@@ -433,9 +437,7 @@ AlembicObject::AlembicObject() : Node(get_node_type())
   schema_type = INVALID;
 }
 
-AlembicObject::~AlembicObject()
-{
-}
+AlembicObject::~AlembicObject() {}
 
 void AlembicObject::set_object(Object *object_)
 {
@@ -609,6 +611,41 @@ void AlembicObject::load_data_in_cache(CachedData &cached_data,
   data_loaded = true;
 }
 
+void AlembicObject::load_data_in_cache(CachedData &cached_data,
+                                       AlembicProcedural *proc,
+                                       const IPointsSchema &schema,
+                                       Progress &progress)
+{
+  /* Only load data for the original Geometry. */
+  if (instance_of) {
+    return;
+  }
+
+  cached_data.clear();
+
+  PointsSchemaData data;
+  data.positions = schema.getPositionsProperty();
+  data.radiuses = schema.getWidthsParam();
+  data.velocities = schema.getVelocitiesProperty();
+  data.time_sampling = schema.getTimeSampling();
+  data.num_samples = schema.getNumSamples();
+  data.default_radius = proc->get_default_radius();
+  data.radius_scale = get_radius_scale();
+
+  read_geometry_data(proc, cached_data, data, progress);
+
+  if (progress.get_cancel()) {
+    return;
+  }
+
+  /* Use the schema as the base compound property to also be able to look for top level properties.
+   */
+  read_attributes(proc, cached_data, schema, {}, get_requested_attributes(), progress);
+
+  cached_data.invalidate_last_loaded_time(true);
+  data_loaded = true;
+}
+
 void AlembicObject::setup_transform_cache(CachedData &cached_data, float scale)
 {
   cached_data.transforms.clear();
@@ -805,7 +842,8 @@ void AlembicProcedural::generate(Scene *scene, Progress &progress)
 
     /* Check if the shaders were modified. */
     if (object->used_shaders_is_modified() && object->get_object() &&
-        object->get_object()->get_geometry()) {
+        object->get_object()->get_geometry())
+    {
       Geometry *geometry = object->get_object()->get_geometry();
       array<Node *> used_shaders = object->get_used_shaders();
       geometry->set_used_shaders(used_shaders);
@@ -907,7 +945,8 @@ void AlembicProcedural::generate(Scene *scene, Progress &progress)
 
     /* skip constant objects */
     if (object->is_constant() && !object->is_modified() && !object->need_shader_update &&
-        !scale_is_modified()) {
+        !scale_is_modified())
+    {
       continue;
     }
 
@@ -993,7 +1032,8 @@ void AlembicProcedural::load_objects(Progress &progress)
         geometry = scene_->create_node<PointCloud>();
       }
       else if (abc_object->schema_type == AlembicObject::POLY_MESH ||
-               abc_object->schema_type == AlembicObject::SUBD) {
+               abc_object->schema_type == AlembicObject::SUBD)
+      {
         geometry = scene_->create_node<Mesh>();
       }
       else {
@@ -1468,7 +1508,8 @@ void AlembicProcedural::build_caches(Progress &progress)
     }
     else if (object->schema_type == AlembicObject::CURVES) {
       if (!object->has_data_loaded() || default_radius_is_modified() ||
-          object->radius_scale_is_modified()) {
+          object->radius_scale_is_modified())
+      {
         ICurves curves(object->iobject, Alembic::Abc::kWrapExisting);
         ICurvesSchema schema = curves.getSchema();
         object->load_data_in_cache(object->get_cached_data(), this, schema, progress);
@@ -1476,7 +1517,8 @@ void AlembicProcedural::build_caches(Progress &progress)
     }
     else if (object->schema_type == AlembicObject::POINTS) {
       if (!object->has_data_loaded() || default_radius_is_modified() ||
-          object->radius_scale_is_modified()) {
+          object->radius_scale_is_modified())
+      {
         IPoints points(object->iobject, Alembic::Abc::kWrapExisting);
         IPointsSchema schema = points.getSchema();
         object->load_data_in_cache(object->get_cached_data(), this, schema, progress);
