@@ -28,7 +28,9 @@ class Hair : public Geometry {
                      const float3 *curve_keys,
                      const float *curve_radius,
                      BoundBox &bounds) const;
+    void bounds_grow(const int k, const float4 *keys, BoundBox &bounds) const;
     void bounds_grow(float4 keys[4], BoundBox &bounds) const;
+    void bounds_grow(float3 keys[4], BoundBox &bounds) const;
     void bounds_grow(const int k,
                      const float3 *curve_keys,
                      const float *curve_radius,
@@ -37,7 +39,7 @@ class Hair : public Geometry {
 
     void motion_keys(const float3 *curve_keys,
                      const float *curve_radius,
-                     const float3 *key_steps,
+                     const float4 *key_steps,
                      size_t num_curve_keys,
                      size_t num_steps,
                      float time,
@@ -46,7 +48,7 @@ class Hair : public Geometry {
                      float4 r_keys[2]) const;
     void cardinal_motion_keys(const float3 *curve_keys,
                               const float *curve_radius,
-                              const float3 *key_steps,
+                              const float4 *key_steps,
                               size_t num_curve_keys,
                               size_t num_steps,
                               float time,
@@ -58,7 +60,7 @@ class Hair : public Geometry {
 
     void keys_for_step(const float3 *curve_keys,
                        const float *curve_radius,
-                       const float3 *key_steps,
+                       const float4 *key_steps,
                        size_t num_curve_keys,
                        size_t num_steps,
                        size_t step,
@@ -67,7 +69,7 @@ class Hair : public Geometry {
                        float4 r_keys[2]) const;
     void cardinal_keys_for_step(const float3 *curve_keys,
                                 const float *curve_radius,
-                                const float3 *key_steps,
+                                const float4 *key_steps,
                                 size_t num_curve_keys,
                                 size_t num_steps,
                                 size_t step,
