@@ -72,6 +72,9 @@ struct ToNanoOp {
       catch (const std::exception &e) {
         VLOG_WARNING << "Error converting OpenVDB to NanoVDB grid: " << e.what();
       }
+      catch (...) {
+        VLOG_WARNING << "Error converting OpenVDB to NanoVDB grid: Unknown error";
+      }
       return true;
     }
     else {
@@ -87,13 +90,9 @@ VDBImageLoader::VDBImageLoader(openvdb::GridBase::ConstPtr grid_, const string &
 }
 #endif
 
-VDBImageLoader::VDBImageLoader(const string &grid_name) : grid_name(grid_name)
-{
-}
+VDBImageLoader::VDBImageLoader(const string &grid_name) : grid_name(grid_name) {}
 
-VDBImageLoader::~VDBImageLoader()
-{
-}
+VDBImageLoader::~VDBImageLoader() {}
 
 bool VDBImageLoader::load_metadata(const ImageDeviceFeatures &features, ImageMetaData &metadata)
 {
