@@ -7,6 +7,7 @@
 #include "scene/film.h"
 #include "scene/integrator.h"
 #include "scene/light.h"
+#include "scene/light_tree.h"
 #include "scene/mesh.h"
 #include "scene/object.h"
 #include "scene/scene.h"
@@ -23,6 +24,7 @@
 #include "util/path.h"
 #include "util/progress.h"
 #include "util/task.h"
+#include <stack>
 
 CCL_NAMESPACE_BEGIN
 
@@ -106,10 +108,11 @@ NODE_DEFINE(Light)
   SOCKET_FLOAT(sizeu, "Size U", 1.0f);
   SOCKET_VECTOR(axisv, "Axis V", zero_float3());
   SOCKET_FLOAT(sizev, "Size V", 1.0f);
-  SOCKET_BOOLEAN(round, "Round", false);
+  SOCKET_BOOLEAN(ellipse, "Ellipse", false);
   SOCKET_FLOAT(spread, "Spread", M_PI_F);
 
   SOCKET_INT(map_resolution, "Map Resolution", 0);
+  SOCKET_FLOAT(average_radiance, "Average Radiance", 0.0f);
 
   SOCKET_FLOAT(spot_angle, "Spot Angle", M_PI_4_F);
   SOCKET_FLOAT(spot_smooth, "Spot Smooth", 0.0f);
@@ -136,6 +139,8 @@ NODE_DEFINE(Light)
 
   SOCKET_STRING(lightgroup, "Light Group", ustring());
 
+  SOCKET_BOOLEAN(normalize, "Normalize", true);
+
   return type;
 }
 
@@ -162,7 +167,9 @@ bool Light::has_contribution(Scene *scene)
   if (light_type == LIGHT_BACKGROUND) {
     return true;
   }
-  return (shader) ? shader->has_surface_emission : scene->default_light->has_surface_emission;
+
+  const Shader *effective_shader = (shader) ? shader : scene->default_light;
+  return !is_zero(effective_shader->emission_estimate);
 }
 
 /* Light Manager */
@@ -228,96 +235,69 @@ void LightManager::test_enabled_lights(Scene *scene)
   }
 
   if (last_background_enabled != background_enabled ||
-      last_background_resolution != background_resolution) {
+      last_background_resolution != background_resolution)
+  {
     last_background_enabled = background_enabled;
     last_background_resolution = background_resolution;
     need_update_background = true;
   }
 }
 
-bool LightManager::object_usable_as_light(Object *object)
-{
-  Geometry *geom = object->get_geometry();
-  if (geom->geometry_type != Geometry::MESH && geom->geometry_type != Geometry::VOLUME) {
-    return false;
-  }
-  /* Skip objects with NaNs */
-  if (!object->bounds.valid()) {
-    return false;
-  }
-  /* Skip if we are not visible for BSDFs. */
-  if (!(object->get_visibility() & (PATH_RAY_DIFFUSE | PATH_RAY_GLOSSY | PATH_RAY_TRANSMIT))) {
-    return false;
-  }
-  /* Skip if we have no emission shaders. */
-  /* TODO(sergey): Ideally we want to avoid such duplicated loop, since it'll
-   * iterate all geometry shaders twice (when counting and when calculating
-   * triangle area.
-   */
-  foreach (Node *node, geom->get_used_shaders()) {
-    Shader *shader = static_cast<Shader *>(node);
-    if (shader->get_use_mis() && shader->has_surface_emission) {
-      return true;
-    }
-  }
-  return false;
-}
-
 void LightManager::device_update_distribution(Device *,
                                               DeviceScene *dscene,
                                               Scene *scene,
                                               Progress &progress)
 {
+  KernelIntegrator *kintegrator = &dscene->data.integrator;
+
+  /* Update CDF over lights. */
   progress.set_status("Updating Lights", "Computing distribution");
 
-  /* count */
-  size_t num_lights = 0;
-  size_t num_portals = 0;
-  size_t num_background_lights = 0;
+  /* Counts emissive triangles in the scene. */
   size_t num_triangles = 0;
 
-  bool background_mis = false;
-
-  foreach (Light *light, scene->lights) {
-    if (light->is_enabled) {
-      num_lights++;
-    }
-    if (light->is_portal) {
-      num_portals++;
-    }
-  }
-
   foreach (Object *object, scene->objects) {
     if (progress.get_cancel())
       return;
 
-    if (!object_usable_as_light(object)) {
+    if (!object->usable_as_light()) {
       continue;
     }
 
-    /* Count triangles. */
+    /* Count emissive triangles. */
     Mesh *mesh = static_cast<Mesh *>(object->get_geometry());
-    size_t mesh_num_triangles = mesh->num_triangles();
-    for (size_t i = 0; i < mesh_num_triangles; i++) {
+    int mesh_num_triangles = static_cast<int>(mesh->num_triangles());
+
+    for (int i = 0; i < mesh_num_triangles; i++) {
       int shader_index = mesh->get_shader()[i];
       Shader *shader = (shader_index < mesh->get_used_shaders().size()) ?
                            static_cast<Shader *>(mesh->get_used_shaders()[shader_index]) :
                            scene->default_surface;
 
-      if (shader->get_use_mis() && shader->has_surface_emission) {
+      if (shader->emission_sampling != EMISSION_SAMPLING_NONE) {
         num_triangles++;
       }
     }
   }
 
-  size_t num_distribution = num_triangles + num_lights;
-  VLOG_INFO << "Total " << num_distribution << " of light distribution primitives.";
+  const size_t num_lights = kintegrator->num_lights;
+  const size_t num_distribution = num_triangles + num_lights;
 
-  /* emission area */
+  /* Distribution size. */
+  kintegrator->num_distribution = num_distribution;
+
+  if (kintegrator->use_light_tree) {
+    dscene->light_distribution.free();
+    return;
+  }
+
+  VLOG_INFO << "Use light distribution with " << num_distribution << " emitters.";
+
+  /* Emission area. */
   KernelLightDistribution *distribution = dscene->light_distribution.alloc(num_distribution + 1);
   float totarea = 0.0f;
 
-  /* triangles */
+  /* Triangles. */
   size_t offset = 0;
   int j = 0;
 
@@ -325,7 +305,7 @@ void LightManager::device_update_distribution(Device *,
     if (progress.get_cancel())
       return;
 
-    if (!object_usable_as_light(object)) {
+    if (!object->usable_as_light()) {
       j++;
       continue;
     }
@@ -362,7 +342,7 @@ void LightManager::device_update_distribution(Device *,
                            static_cast<Shader *>(mesh->get_used_shaders()[shader_index]) :
                            scene->default_surface;
 
-      if (shader->get_use_mis() && shader->has_surface_emission) {
+      if (shader->emission_sampling != EMISSION_SAMPLING_NONE) {
         distribution[offset].totarea = totarea;
         distribution[offset].prim = i + mesh->prim_offset;
         distribution[offset].mesh_light.shader_flag = shader_flag;
@@ -390,9 +370,9 @@ void LightManager::device_update_distribution(Device *,
     j++;
   }
 
-  float trianglearea = totarea;
-  /* point lights */
-  bool use_lamp_mis = false;
+  const float trianglearea = totarea;
+
+  /* Lights. */
   int light_index = 0;
 
   if (num_lights > 0) {
@@ -403,24 +383,10 @@ void LightManager::device_update_distribution(Device *,
 
       distribution[offset].totarea = totarea;
       distribution[offset].prim = ~light_index;
-      distribution[offset].lamp.pad = 1.0f;
-      distribution[offset].lamp.size = light->size;
+      distribution[offset].mesh_light.object_id = OBJECT_NONE;
+      distribution[offset].mesh_light.shader_flag = 0;
       totarea += lightarea;
 
-      if (light->light_type == LIGHT_DISTANT) {
-        use_lamp_mis |= (light->angle > 0.0f && light->use_mis);
-      }
-      else if (light->light_type == LIGHT_POINT || light->light_type == LIGHT_SPOT) {
-        use_lamp_mis |= (light->size > 0.0f && light->use_mis);
-      }
-      else if (light->light_type == LIGHT_AREA) {
-        use_lamp_mis |= light->use_mis;
-      }
-      else if (light->light_type == LIGHT_BACKGROUND) {
-        num_background_lights++;
-        background_mis |= light->use_mis;
-      }
-
       light_index++;
       offset++;
     }
@@ -428,9 +394,9 @@ void LightManager::device_update_distribution(Device *,
 
   /* normalize cumulative distribution functions */
   distribution[num_distribution].totarea = totarea;
-  distribution[num_distribution].prim = 0.0f;
-  distribution[num_distribution].lamp.pad = 0.0f;
-  distribution[num_distribution].lamp.size = 0.0f;
+  distribution[num_distribution].prim = 0;
+  distribution[num_distribution].mesh_light.object_id = OBJECT_NONE;
+  distribution[num_distribution].mesh_light.shader_flag = 0;
 
   if (totarea > 0.0f) {
     for (size_t i = 0; i < num_distribution; i++)
@@ -441,82 +407,270 @@ void LightManager::device_update_distribution(Device *,
   if (progress.get_cancel())
     return;
 
-  /* update device */
-  KernelIntegrator *kintegrator = &dscene->data.integrator;
-  KernelBackground *kbackground = &dscene->data.background;
-  KernelFilm *kfilm = &dscene->data.film;
+  /* Update integrator state. */
   kintegrator->use_direct_light = (totarea > 0.0f);
 
-  if (kintegrator->use_direct_light) {
-    /* number of emissives */
-    kintegrator->num_distribution = num_distribution;
-
-    /* precompute pdfs */
-    kintegrator->pdf_triangles = 0.0f;
-    kintegrator->pdf_lights = 0.0f;
-
-    /* sample one, with 0.5 probability of light or triangle */
-    kintegrator->num_all_lights = num_lights;
-
-    if (trianglearea > 0.0f) {
-      kintegrator->pdf_triangles = 1.0f / trianglearea;
-      if (num_lights)
-        kintegrator->pdf_triangles *= 0.5f;
-    }
+  /* Precompute pdfs for distribution sampling.
+   * Sample one, with 0.5 probability of light or triangle. */
+  kintegrator->distribution_pdf_triangles = 0.0f;
+  kintegrator->distribution_pdf_lights = 0.0f;
 
+  if (trianglearea > 0.0f) {
+    kintegrator->distribution_pdf_triangles = 1.0f / trianglearea;
     if (num_lights) {
-      kintegrator->pdf_lights = 1.0f / num_lights;
-      if (trianglearea > 0.0f)
-        kintegrator->pdf_lights *= 0.5f;
+      kintegrator->distribution_pdf_triangles *= 0.5f;
     }
+  }
 
-    kintegrator->use_lamp_mis = use_lamp_mis;
+  if (num_lights) {
+    kintegrator->distribution_pdf_lights = 1.0f / num_lights;
+    if (trianglearea > 0.0f) {
+      kintegrator->distribution_pdf_lights *= 0.5f;
+    }
+  }
 
-    /* bit of an ugly hack to compensate for emitting triangles influencing
-     * amount of samples we get for this pass */
-    kfilm->pass_shadow_scale = 1.0f;
+  /* Copy distribution to device. */
+  dscene->light_distribution.copy_to_device();
+}
 
-    if (kintegrator->pdf_triangles != 0.0f)
-      kfilm->pass_shadow_scale /= 0.5f;
+/* Arguments for functions to convert the light tree to the kernel representation. */
+struct LightTreeFlatten {
+  const Scene *scene;
+  const LightTreeEmitter *emitters;
+  const uint *object_lookup_offset;
+  uint *light_array;
+  uint *mesh_array;
+  uint *triangle_array;
 
-    if (num_background_lights < num_lights)
-      kfilm->pass_shadow_scale /= (float)(num_lights - num_background_lights) / (float)num_lights;
+  /* Map from instance node to its node index. */
+  std::unordered_map<LightTreeNode *, int> instances;
+};
 
-    /* CDF */
-    dscene->light_distribution.copy_to_device();
+static void light_tree_node_copy_to_device(KernelLightTreeNode &knode,
+                                           const LightTreeNode &node,
+                                           const int child_index)
+{
+  /* Convert node to kernel representation. */
+  knode.energy = node.measure.energy;
 
-    /* Portals */
-    if (num_portals > 0) {
-      kbackground->portal_offset = light_index;
-      kbackground->num_portals = num_portals;
-      kbackground->portal_weight = 1.0f;
+  knode.bbox.min = node.measure.bbox.min;
+  knode.bbox.max = node.measure.bbox.max;
+
+  knode.bcone.axis = node.measure.bcone.axis;
+  knode.bcone.theta_o = node.measure.bcone.theta_o;
+  knode.bcone.theta_e = node.measure.bcone.theta_e;
+
+  knode.bit_trail = node.bit_trail;
+  knode.type = static_cast<LightTreeNodeType>(node.type);
+
+  if (node.is_leaf() || node.is_distant()) {
+    knode.num_emitters = node.get_leaf().num_emitters;
+    knode.leaf.first_emitter = node.get_leaf().first_emitter_index;
+  }
+  else if (node.is_inner()) {
+    knode.num_emitters = -1;
+    knode.inner.right_child = child_index;
+  }
+}
+
+static int light_tree_flatten(LightTreeFlatten &flatten,
+                              const LightTreeNode *node,
+                              KernelLightTreeNode *knodes,
+                              KernelLightTreeEmitter *kemitters,
+                              int &next_node_index);
+
+static void light_tree_leaf_emitters_copy_and_flatten(LightTreeFlatten &flatten,
+                                                      const LightTreeNode &node,
+                                                      KernelLightTreeNode *knodes,
+                                                      KernelLightTreeEmitter *kemitters,
+                                                      int &next_node_index)
+{
+  /* Convert emitters to kernel representation. */
+  for (int i = 0; i < node.get_leaf().num_emitters; i++) {
+    int emitter_index = i + node.get_leaf().first_emitter_index;
+    const LightTreeEmitter &emitter = flatten.emitters[emitter_index];
+    KernelLightTreeEmitter &kemitter = kemitters[emitter_index];
+
+    kemitter.energy = emitter.measure.energy;
+    kemitter.theta_o = emitter.measure.bcone.theta_o;
+    kemitter.theta_e = emitter.measure.bcone.theta_e;
+
+    if (emitter.is_triangle()) {
+      /* Triangle. */
+      int shader_flag = 0;
+      Object *object = flatten.scene->objects[emitter.object_id];
+      Mesh *mesh = static_cast<Mesh *>(object->get_geometry());
+      Shader *shader = static_cast<Shader *>(
+          mesh->get_used_shaders()[mesh->get_shader()[emitter.prim_id]]);
+
+      if (!(object->get_visibility() & PATH_RAY_CAMERA)) {
+        shader_flag |= SHADER_EXCLUDE_CAMERA;
+      }
+      if (!(object->get_visibility() & PATH_RAY_DIFFUSE)) {
+        shader_flag |= SHADER_EXCLUDE_DIFFUSE;
+      }
+      if (!(object->get_visibility() & PATH_RAY_GLOSSY)) {
+        shader_flag |= SHADER_EXCLUDE_GLOSSY;
+      }
+      if (!(object->get_visibility() & PATH_RAY_TRANSMIT)) {
+        shader_flag |= SHADER_EXCLUDE_TRANSMIT;
+      }
+      if (!(object->get_visibility() & PATH_RAY_VOLUME_SCATTER)) {
+        shader_flag |= SHADER_EXCLUDE_SCATTER;
+      }
+      if (!(object->get_is_shadow_catcher())) {
+        shader_flag |= SHADER_EXCLUDE_SHADOW_CATCHER;
+      }
+
+      kemitter.triangle.id = emitter.prim_id + mesh->prim_offset;
+      kemitter.mesh_light.shader_flag = shader_flag;
+      kemitter.mesh_light.object_id = emitter.object_id;
+      kemitter.triangle.emission_sampling = shader->emission_sampling;
+      flatten.triangle_array[emitter.prim_id + flatten.object_lookup_offset[emitter.object_id]] =
+          emitter_index;
+    }
+    else if (emitter.is_light()) {
+      /* Light object. */
+      kemitter.light.id = emitter.light_id;
+      kemitter.mesh_light.shader_flag = 0;
+      kemitter.mesh_light.object_id = OBJECT_NONE;
+      flatten.light_array[~emitter.light_id] = emitter_index;
     }
     else {
-      kbackground->num_portals = 0;
-      kbackground->portal_offset = 0;
-      kbackground->portal_weight = 0.0f;
-    }
+      /* Mesh instance. */
+      assert(emitter.is_mesh());
+      kemitter.mesh.object_id = emitter.object_id;
+      kemitter.mesh_light.shader_flag = 0;
+      kemitter.mesh_light.object_id = OBJECT_NONE;
+      flatten.mesh_array[emitter.object_id] = emitter_index;
 
-    /* Map */
-    kbackground->map_weight = background_mis ? 1.0f : 0.0f;
+      /* Create instance node. One instance node will be the same as the
+       * reference node, and for that it will recursively build the subtree. */
+      LightTreeNode *instance_node = emitter.root.get();
+      LightTreeNode *reference_node = instance_node->get_reference();
+
+      auto map_it = flatten.instances.find(reference_node);
+      if (map_it == flatten.instances.end()) {
+        if (instance_node != reference_node) {
+          /* Flatten the node with the subtree first so the subsequent instances know the index. */
+          std::swap(instance_node->type, reference_node->type);
+          std::swap(instance_node->variant_type, reference_node->variant_type);
+        }
+        instance_node->type &= ~LIGHT_TREE_INSTANCE;
+      }
+
+      kemitter.mesh.node_id = light_tree_flatten(
+          flatten, instance_node, knodes, kemitters, next_node_index);
+
+      KernelLightTreeNode &kinstance_node = knodes[kemitter.mesh.node_id];
+      kinstance_node.bit_trail = node.bit_trail;
+
+      if (map_it != flatten.instances.end()) {
+        kinstance_node.instance.reference = map_it->second;
+      }
+      else {
+        flatten.instances[reference_node] = kemitter.mesh.node_id;
+      }
+    }
+    kemitter.bit_trail = node.bit_trail;
+  }
+}
+
+static int light_tree_flatten(LightTreeFlatten &flatten,
+                              const LightTreeNode *node,
+                              KernelLightTreeNode *knodes,
+                              KernelLightTreeEmitter *kemitters,
+                              int &next_node_index)
+{
+  /* Convert both inner nodes and primitives to device representation. */
+  const int node_index = next_node_index++;
+  int child_index = -1;
+
+  if (node->is_leaf() || node->is_distant()) {
+    light_tree_leaf_emitters_copy_and_flatten(flatten, *node, knodes, kemitters, next_node_index);
+  }
+  else if (node->is_inner()) {
+    /* Nodes are stored in depth first order so that the left child node
+     * immediately follows the parent, and only the right child index needs
+     * to be stored. */
+    light_tree_flatten(flatten,
+                       node->get_inner().children[LightTree::left].get(),
+                       knodes,
+                       kemitters,
+                       next_node_index);
+    child_index = light_tree_flatten(flatten,
+                                     node->get_inner().children[LightTree::right].get(),
+                                     knodes,
+                                     kemitters,
+                                     next_node_index);
   }
   else {
-    dscene->light_distribution.free();
-
-    kintegrator->num_distribution = 0;
-    kintegrator->num_all_lights = 0;
-    kintegrator->pdf_triangles = 0.0f;
-    kintegrator->pdf_lights = 0.0f;
-    kintegrator->use_lamp_mis = false;
-
-    kbackground->num_portals = 0;
-    kbackground->portal_offset = 0;
-    kbackground->portal_weight = 0.0f;
-    kbackground->sun_weight = 0.0f;
-    kbackground->map_weight = 0.0f;
-
-    kfilm->pass_shadow_scale = 1.0f;
+    /* Instance node that is not inner or leaf, but just references another. */
+    assert(node->is_instance());
   }
+
+  light_tree_node_copy_to_device(knodes[node_index], *node, child_index);
+
+  return node_index;
+}
+
+void LightManager::device_update_tree(Device *,
+                                      DeviceScene *dscene,
+                                      Scene *scene,
+                                      Progress &progress)
+{
+  KernelIntegrator *kintegrator = &dscene->data.integrator;
+
+  if (!kintegrator->use_light_tree) {
+    return;
+  }
+
+  /* Update light tree. */
+  progress.set_status("Updating Lights", "Computing tree");
+
+  /* TODO: For now, we'll start with a smaller number of max lights in a node.
+   * More benchmarking is needed to determine what number works best. */
+  LightTree light_tree(scene, dscene, progress, 8);
+  LightTreeNode *root = light_tree.build(scene, dscene);
+  if (progress.get_cancel()) {
+    return;
+  }
+
+  /* Create arguments for recursive tree flatten. */
+  LightTreeFlatten flatten;
+  flatten.scene = scene;
+  flatten.emitters = light_tree.get_emitters();
+  flatten.object_lookup_offset = dscene->object_lookup_offset.data();
+  /* We want to create separate arrays corresponding to triangles and lights,
+   * which will be used to index back into the light tree for PDF calculations. */
+  flatten.light_array = dscene->light_to_tree.alloc(kintegrator->num_lights);
+  flatten.mesh_array = dscene->object_to_tree.alloc(scene->objects.size());
+  flatten.triangle_array = dscene->triangle_to_tree.alloc(light_tree.num_triangles);
+
+  /* Allocate emitters and nodes. */
+  const size_t num_emitters = light_tree.num_emitters();
+  KernelLightTreeEmitter *kemitters = dscene->light_tree_emitters.alloc(num_emitters);
+  KernelLightTreeNode *knodes = dscene->light_tree_nodes.alloc(light_tree.num_nodes);
+
+  /* Update integrator state. */
+  kintegrator->use_direct_light = num_emitters > 0;
+
+  /* Copy nodes and emitters. */
+  if (root) {
+    int next_node_index = 0;
+    light_tree_flatten(flatten, root, knodes, kemitters, next_node_index);
+  }
+
+  VLOG_INFO << "Use light tree with " << num_emitters << " emitters and " << light_tree.num_nodes
+            << " nodes.";
+
+  /* Copy arrays to device. */
+  dscene->light_tree_nodes.copy_to_device();
+  dscene->light_tree_emitters.copy_to_device();
+  dscene->light_to_tree.copy_to_device();
+  dscene->object_to_tree.copy_to_device();
+  dscene->object_lookup_offset.copy_to_device();
+  dscene->triangle_to_tree.copy_to_device();
 }
 
 static void background_cdf(
@@ -564,38 +718,41 @@ void LightManager::device_update_background(Device *device,
                                             Scene *scene,
                                             Progress &progress)
 {
+  KernelIntegrator *kintegrator = &dscene->data.integrator;
   KernelBackground *kbackground = &dscene->data.background;
   Light *background_light = NULL;
 
+  bool background_mis = false;
+
   /* find background light */
   foreach (Light *light, scene->lights) {
-    if (light->light_type == LIGHT_BACKGROUND) {
+    if (light->light_type == LIGHT_BACKGROUND && light->is_enabled) {
       background_light = light;
-      break;
+      background_mis |= light->use_mis;
     }
   }
 
+  kbackground->portal_weight = kintegrator->num_portals > 0 ? 1.0f : 0.0f;
+  kbackground->map_weight = background_mis ? 1.0f : 0.0f;
+  kbackground->sun_weight = 0.0f;
+
   /* no background light found, signal renderer to skip sampling */
   if (!background_light || !background_light->is_enabled) {
     kbackground->map_res_x = 0;
     kbackground->map_res_y = 0;
-    kbackground->map_weight = 0.0f;
-    kbackground->sun_weight = 0.0f;
     kbackground->use_mis = (kbackground->portal_weight > 0.0f);
     return;
   }
 
   progress.set_status("Updating Lights", "Importance map");
 
-  assert(dscene->data.integrator.use_direct_light);
-
   int2 environment_res = make_int2(0, 0);
   Shader *shader = scene->background->get_shader(scene);
   int num_suns = 0;
+  float sun_average_radiance = 0.0f;
   foreach (ShaderNode *node, shader->graph->nodes) {
     if (node->type == EnvironmentTextureNode::get_node_type()) {
       EnvironmentTextureNode *env = (EnvironmentTextureNode *)node;
-      ImageMetaData metadata;
       if (!env->handle.empty()) {
         ImageMetaData metadata = env->handle.metadata();
         environment_res.x = max(environment_res.x, (int)metadata.width);
@@ -612,7 +769,8 @@ void LightManager::device_update_background(Device *device,
         if (vec_in && vec_in->link && vec_in->link->parent) {
           ShaderNode *vec_src = vec_in->link->parent;
           if ((vec_src->type != TextureCoordinateNode::get_node_type()) ||
-              (vec_in->link != vec_src->output("Generated"))) {
+              (vec_in->link != vec_src->output("Generated")))
+          {
             environment_res.x = max(environment_res.x, 4096);
             environment_res.y = max(environment_res.y, 2048);
             continue;
@@ -621,7 +779,7 @@ void LightManager::device_update_background(Device *device,
 
         /* Determine sun direction from lat/long and texture mapping. */
         float latitude = sky->get_sun_elevation();
-        float longitude = M_2PI_F - sky->get_sun_rotation() + M_PI_2_F;
+        float longitude = sky->get_sun_rotation() + M_PI_2_F;
         float3 sun_direction = make_float3(
             cosf(latitude) * cosf(longitude), cosf(latitude) * sinf(longitude), sinf(latitude));
         Transform sky_transform = transform_inverse(sky->tex_mapping.compute_transform());
@@ -632,7 +790,9 @@ void LightManager::device_update_background(Device *device,
         kbackground->sun = make_float4(
             sun_direction.x, sun_direction.y, sun_direction.z, half_angle);
 
+        /* empirical value */
         kbackground->sun_weight = 4.0f;
+        sun_average_radiance = sky->get_sun_average_radiance();
         environment_res.x = max(environment_res.x, 512);
         environment_res.y = max(environment_res.y, 256);
         num_suns++;
@@ -641,7 +801,8 @@ void LightManager::device_update_background(Device *device,
   }
 
   /* If there's more than one sun, fall back to map sampling instead. */
-  if (num_suns != 1) {
+  kbackground->use_sun_guiding = (num_suns == 1);
+  if (!kbackground->use_sun_guiding) {
     kbackground->sun_weight = 0.0f;
     environment_res.x = max(environment_res.x, 4096);
     environment_res.y = max(environment_res.y, 2048);
@@ -701,6 +862,19 @@ void LightManager::device_update_background(Device *device,
   float cdf_total = marg_cdf[res.y - 1].y + marg_cdf[res.y - 1].x / res.y;
   marg_cdf[res.y].x = cdf_total;
 
+  float map_average_radiance = cdf_total * M_PI_2_F;
+  if (sun_average_radiance > 0.0f) {
+    /* The weighting here is just a heuristic that was empirically determined.
+     * The sun's average radiance is much higher than the map's average radiance,
+     * but we don't want to weight the background light too much because
+     * visibility is not accounted for anyway. */
+    background_light->set_average_radiance(0.8f * map_average_radiance +
+                                           0.2f * sun_average_radiance);
+  }
+  else {
+    background_light->set_average_radiance(map_average_radiance);
+  }
+
   if (cdf_total > 0.0f)
     for (int i = 1; i < res.y; i++)
       marg_cdf[i].y /= cdf_total;
@@ -714,27 +888,91 @@ void LightManager::device_update_background(Device *device,
   dscene->light_background_conditional_cdf.copy_to_device();
 }
 
-void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *scene)
+void LightManager::device_update_lights(Device *device, DeviceScene *dscene, Scene *scene)
 {
-  int num_scene_lights = scene->lights.size();
+  /* Counts lights in the scene. */
+  size_t num_lights = 0;
+  size_t num_portals = 0;
+  size_t num_background_lights = 0;
+  size_t num_distant_lights = 0;
+  bool use_light_mis = false;
 
-  int num_lights = 0;
   foreach (Light *light, scene->lights) {
-    if (light->is_enabled || light->is_portal) {
+    if (light->is_enabled) {
       num_lights++;
+
+      if (light->light_type == LIGHT_DISTANT) {
+        num_distant_lights++;
+      }
+      else if (light->light_type == LIGHT_POINT || light->light_type == LIGHT_SPOT) {
+        use_light_mis |= (light->size > 0.0f && light->use_mis);
+      }
+      else if (light->light_type == LIGHT_AREA) {
+        use_light_mis |= light->use_mis;
+      }
+      else if (light->light_type == LIGHT_BACKGROUND) {
+        num_distant_lights++;
+        num_background_lights++;
+      }
+    }
+    if (light->is_portal) {
+      num_portals++;
     }
   }
 
-  KernelLight *klights = dscene->lights.alloc(num_lights);
+  /* Update integrator settings. */
+  KernelIntegrator *kintegrator = &dscene->data.integrator;
+  kintegrator->use_light_tree = scene->integrator->get_use_light_tree() &&
+                                device->info.has_light_tree;
+  kintegrator->num_lights = num_lights;
+  kintegrator->num_distant_lights = num_distant_lights;
+  kintegrator->num_background_lights = num_background_lights;
+  kintegrator->use_light_mis = use_light_mis;
 
-  if (num_lights == 0) {
-    VLOG_WORK << "No effective light, ignoring points update.";
-    return;
-  }
+  kintegrator->num_portals = num_portals;
+  kintegrator->portal_offset = num_lights;
+
+  /* Create KernelLight for every portal and enabled light in the scene. */
+  KernelLight *klights = dscene->lights.alloc(num_lights + num_portals);
 
   int light_index = 0;
+  int portal_index = num_lights;
 
   foreach (Light *light, scene->lights) {
+    /* Consider moving portals update to their own function
+     * keeping this one more manageable. */
+    if (light->is_portal) {
+      assert(light->light_type == LIGHT_AREA);
+
+      float3 extentu = light->axisu * (light->sizeu * light->size);
+      float3 extentv = light->axisv * (light->sizev * light->size);
+
+      float len_u, len_v;
+      float3 axis_u = normalize_len(extentu, &len_u);
+      float3 axis_v = normalize_len(extentv, &len_v);
+      float area = len_u * len_v;
+      if (light->ellipse) {
+        area *= -M_PI_4_F;
+      }
+      float invarea = (area != 0.0f) ? 1.0f / area : 1.0f;
+      float3 dir = light->dir;
+
+      dir = safe_normalize(dir);
+
+      klights[portal_index].co = light->co;
+      klights[portal_index].area.axis_u = axis_u;
+      klights[portal_index].area.len_u = len_u;
+      klights[portal_index].area.axis_v = axis_v;
+      klights[portal_index].area.len_v = len_v;
+      klights[portal_index].area.invarea = invarea;
+      klights[portal_index].area.dir = dir;
+      klights[portal_index].tfm = light->tfm;
+      klights[portal_index].itfm = transform_inverse(light->tfm);
+
+      portal_index++;
+      continue;
+    }
+
     if (!light->is_enabled) {
       continue;
     }
@@ -776,15 +1014,13 @@ void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *sc
       shader_id &= ~SHADER_AREA_LIGHT;
 
       float radius = light->size;
-      float invarea = (radius > 0.0f) ? 1.0f / (M_PI_F * radius * radius) : 1.0f;
+      float invarea = (light->normalize && radius > 0.0f) ? 1.0f / (M_PI_F * radius * radius) :
+                                                            1.0f;
 
       if (light->use_mis && radius > 0.0f)
         shader_id |= SHADER_USE_MIS;
 
-      klights[light_index].co[0] = co.x;
-      klights[light_index].co[1] = co.y;
-      klights[light_index].co[2] = co.z;
-
+      klights[light_index].co = co;
       klights[light_index].spot.radius = radius;
       klights[light_index].spot.invarea = invarea;
     }
@@ -795,7 +1031,7 @@ void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *sc
       float radius = tanf(angle);
       float cosangle = cosf(angle);
       float area = M_PI_F * radius * radius;
-      float invarea = (area > 0.0f) ? 1.0f / area : 1.0f;
+      float invarea = (light->normalize && area > 0.0f) ? 1.0f / area : 1.0f;
       float3 dir = light->dir;
 
       dir = safe_normalize(dir);
@@ -803,10 +1039,7 @@ void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *sc
       if (light->use_mis && area > 0.0f)
         shader_id |= SHADER_USE_MIS;
 
-      klights[light_index].co[0] = dir.x;
-      klights[light_index].co[1] = dir.y;
-      klights[light_index].co[2] = dir.z;
-
+      klights[light_index].co = dir;
       klights[light_index].distant.invarea = invarea;
       klights[light_index].distant.radius = radius;
       klights[light_index].distant.cosangle = cosangle;
@@ -814,6 +1047,8 @@ void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *sc
     else if (light->light_type == LIGHT_BACKGROUND) {
       uint visibility = scene->background->get_visibility();
 
+      dscene->data.background.light_index = light_index;
+
       shader_id &= ~SHADER_AREA_LIGHT;
       shader_id |= SHADER_USE_MIS;
 
@@ -831,71 +1066,73 @@ void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *sc
       }
     }
     else if (light->light_type == LIGHT_AREA) {
-      float3 axisu = light->axisu * (light->sizeu * light->size);
-      float3 axisv = light->axisv * (light->sizev * light->size);
-      float area = len(axisu) * len(axisv);
-      if (light->round) {
+      float3 extentu = light->axisu * (light->sizeu * light->size);
+      float3 extentv = light->axisv * (light->sizev * light->size);
+
+      float len_u, len_v;
+      float3 axis_u = normalize_len(extentu, &len_u);
+      float3 axis_v = normalize_len(extentv, &len_v);
+      float area = len_u * len_v;
+      if (light->ellipse) {
         area *= -M_PI_4_F;
       }
-      float invarea = (area != 0.0f) ? 1.0f / area : 1.0f;
+      float invarea = (light->normalize && area != 0.0f) ? 1.0f / area : 1.0f;
       float3 dir = light->dir;
 
-      /* Convert from spread angle 0..180 to 90..0, clamping to a minimum
-       * angle to avoid excessive noise. */
-      const float min_spread_angle = 1.0f * M_PI_F / 180.0f;
-      const float spread_angle = 0.5f * (M_PI_F - max(light->spread, min_spread_angle));
+      /* Clamp angles in (0, 0.1) to 0.1 to prevent zero intensity due to floating-point precision
+       * issues, but still handles spread = 0 */
+      const float min_spread = 0.1f * M_PI_F / 180.0f;
+      const float half_spread = light->spread == 0 ? 0.0f : 0.5f * max(light->spread, min_spread);
+      const float tan_half_spread = light->spread == M_PI_F ? FLT_MAX : tanf(half_spread);
       /* Normalization computed using:
-       * integrate cos(x) * (1 - tan(x) * tan(a)) * sin(x) from x = 0 to pi/2 - a. */
-      const float tan_spread = tanf(spread_angle);
-      const float normalize_spread = 2.0f / (2.0f + (2.0f * spread_angle - M_PI_F) * tan_spread);
+       * integrate cos(x) * (1 - tan(x) / tan(a)) * sin(x) from x = 0 to a, a being half_spread.
+       * Divided by tan_half_spread to simplify the attenuation computation in `area.h`. */
+      const float normalize_spread = 1.0f / (tan_half_spread - half_spread);
 
       dir = safe_normalize(dir);
 
       if (light->use_mis && area != 0.0f)
         shader_id |= SHADER_USE_MIS;
 
-      klights[light_index].co[0] = co.x;
-      klights[light_index].co[1] = co.y;
-      klights[light_index].co[2] = co.z;
-
-      klights[light_index].area.axisu[0] = axisu.x;
-      klights[light_index].area.axisu[1] = axisu.y;
-      klights[light_index].area.axisu[2] = axisu.z;
-      klights[light_index].area.axisv[0] = axisv.x;
-      klights[light_index].area.axisv[1] = axisv.y;
-      klights[light_index].area.axisv[2] = axisv.z;
+      klights[light_index].co = co;
+      klights[light_index].area.axis_u = axis_u;
+      klights[light_index].area.len_u = len_u;
+      klights[light_index].area.axis_v = axis_v;
+      klights[light_index].area.len_v = len_v;
       klights[light_index].area.invarea = invarea;
-      klights[light_index].area.dir[0] = dir.x;
-      klights[light_index].area.dir[1] = dir.y;
-      klights[light_index].area.dir[2] = dir.z;
-      klights[light_index].area.tan_spread = tan_spread;
+      klights[light_index].area.dir = dir;
+      klights[light_index].area.tan_half_spread = tan_half_spread;
       klights[light_index].area.normalize_spread = normalize_spread;
     }
     else if (light->light_type == LIGHT_SPOT) {
       shader_id &= ~SHADER_AREA_LIGHT;
 
-      float radius = light->size;
-      float invarea = (radius > 0.0f) ? 1.0f / (M_PI_F * radius * radius) : 1.0f;
-      float spot_angle = cosf(light->spot_angle * 0.5f);
-      float spot_smooth = (1.0f - spot_angle) * light->spot_smooth;
-      float3 dir = light->dir;
+      float3 len;
+      float3 axis_u = normalize_len(light->axisu, &len.x);
+      float3 axis_v = normalize_len(light->axisv, &len.y);
+      float3 dir = normalize_len(light->dir, &len.z);
+      if (len.z == 0.0f) {
+        dir = zero_float3();
+      }
 
-      dir = safe_normalize(dir);
+      float radius = light->size;
+      float invarea = (light->normalize && radius > 0.0f) ? 1.0f / (M_PI_F * radius * radius) :
+                                                            1.0f;
+      float cos_half_spot_angle = cosf(light->spot_angle * 0.5f);
+      float spot_smooth = (1.0f - cos_half_spot_angle) * light->spot_smooth;
 
       if (light->use_mis && radius > 0.0f)
         shader_id |= SHADER_USE_MIS;
 
-      klights[light_index].co[0] = co.x;
-      klights[light_index].co[1] = co.y;
-      klights[light_index].co[2] = co.z;
-
+      klights[light_index].co = co;
+      klights[light_index].spot.axis_u = axis_u;
       klights[light_index].spot.radius = radius;
+      klights[light_index].spot.axis_v = axis_v;
       klights[light_index].spot.invarea = invarea;
-      klights[light_index].spot.spot_angle = spot_angle;
+      klights[light_index].spot.dir = dir;
+      klights[light_index].spot.cos_half_spot_angle = cos_half_spot_angle;
+      klights[light_index].spot.len = len;
       klights[light_index].spot.spot_smooth = spot_smooth;
-      klights[light_index].spot.dir[0] = dir.x;
-      klights[light_index].spot.dir[1] = dir.y;
-      klights[light_index].spot.dir[2] = dir.z;
     }
 
     klights[light_index].shader_id = shader_id;
@@ -918,49 +1155,7 @@ void LightManager::device_update_points(Device *, DeviceScene *dscene, Scene *sc
     light_index++;
   }
 
-  /* TODO(sergey): Consider moving portals update to their own function
-   * keeping this one more manageable.
-   */
-  foreach (Light *light, scene->lights) {
-    if (!light->is_portal)
-      continue;
-    assert(light->light_type == LIGHT_AREA);
-
-    float3 co = light->co;
-    float3 axisu = light->axisu * (light->sizeu * light->size);
-    float3 axisv = light->axisv * (light->sizev * light->size);
-    float area = len(axisu) * len(axisv);
-    if (light->round) {
-      area *= -M_PI_4_F;
-    }
-    float invarea = (area != 0.0f) ? 1.0f / area : 1.0f;
-    float3 dir = light->dir;
-
-    dir = safe_normalize(dir);
-
-    klights[light_index].co[0] = co.x;
-    klights[light_index].co[1] = co.y;
-    klights[light_index].co[2] = co.z;
-
-    klights[light_index].area.axisu[0] = axisu.x;
-    klights[light_index].area.axisu[1] = axisu.y;
-    klights[light_index].area.axisu[2] = axisu.z;
-    klights[light_index].area.axisv[0] = axisv.x;
-    klights[light_index].area.axisv[1] = axisv.y;
-    klights[light_index].area.axisv[2] = axisv.z;
-    klights[light_index].area.invarea = invarea;
-    klights[light_index].area.dir[0] = dir.x;
-    klights[light_index].area.dir[1] = dir.y;
-    klights[light_index].area.dir[2] = dir.z;
-    klights[light_index].tfm = light->tfm;
-    klights[light_index].itfm = transform_inverse(light->tfm);
-
-    light_index++;
-  }
-
-  VLOG_INFO << "Number of lights sent to the device: " << light_index;
-
-  VLOG_INFO << "Number of lights without contribution: " << num_scene_lights - light_index;
+  VLOG_INFO << "Number of lights sent to the device: " << num_lights;
 
   dscene->lights.copy_to_device();
 }
@@ -986,11 +1181,7 @@ void LightManager::device_update(Device *device,
 
   device_free(device, dscene, need_update_background);
 
-  device_update_points(device, dscene, scene);
-  if (progress.get_cancel())
-    return;
-
-  device_update_distribution(device, dscene, scene, progress);
+  device_update_lights(device, dscene, scene);
   if (progress.get_cancel())
     return;
 
@@ -1000,6 +1191,14 @@ void LightManager::device_update(Device *device,
       return;
   }
 
+  device_update_distribution(device, dscene, scene, progress);
+  if (progress.get_cancel())
+    return;
+
+  device_update_tree(device, dscene, scene, progress);
+  if (progress.get_cancel())
+    return;
+
   device_update_ies(dscene);
   if (progress.get_cancel())
     return;
@@ -1010,6 +1209,13 @@ void LightManager::device_update(Device *device,
 
 void LightManager::device_free(Device *, DeviceScene *dscene, const bool free_background)
 {
+  dscene->light_tree_nodes.free();
+  dscene->light_tree_emitters.free();
+  dscene->light_to_tree.free();
+  dscene->object_to_tree.free();
+  dscene->object_lookup_offset.free();
+  dscene->triangle_to_tree.free();
+
   dscene->light_distribution.free();
   dscene->lights.free();
   if (free_background) {
