@@ -21,7 +21,6 @@ CCL_NAMESPACE_BEGIN
 
 class Device;
 class DeviceScene;
-class Object;
 class Progress;
 class Scene;
 class Shader;
@@ -44,12 +43,13 @@ class Light : public Node {
   NODE_SOCKET_API(float, sizeu)
   NODE_SOCKET_API(float3, axisv)
   NODE_SOCKET_API(float, sizev)
-  NODE_SOCKET_API(bool, round)
+  NODE_SOCKET_API(bool, ellipse)
   NODE_SOCKET_API(float, spread)
 
   NODE_SOCKET_API(Transform, tfm)
 
   NODE_SOCKET_API(int, map_resolution)
+  NODE_SOCKET_API(float, average_radiance)
 
   NODE_SOCKET_API(float, spot_angle)
   NODE_SOCKET_API(float, spot_smooth)
@@ -73,12 +73,15 @@ class Light : public Node {
 
   NODE_SOCKET_API(ustring, lightgroup)
 
+  NODE_SOCKET_API(bool, normalize)
+
   void tag_update(Scene *scene);
 
   /* Check whether the light has contribution the scene. */
   bool has_contribution(Scene *scene);
 
   friend class LightManager;
+  friend class LightTree;
 };
 
 class LightManager {
@@ -127,20 +130,18 @@ class LightManager {
    */
   void test_enabled_lights(Scene *scene);
 
-  void device_update_points(Device *device, DeviceScene *dscene, Scene *scene);
+  void device_update_lights(Device *device, DeviceScene *dscene, Scene *scene);
   void device_update_distribution(Device *device,
                                   DeviceScene *dscene,
                                   Scene *scene,
                                   Progress &progress);
+  void device_update_tree(Device *device, DeviceScene *dscene, Scene *scene, Progress &progress);
   void device_update_background(Device *device,
                                 DeviceScene *dscene,
                                 Scene *scene,
                                 Progress &progress);
   void device_update_ies(DeviceScene *dscene);
 
-  /* Check whether light manager can use the object as a light-emissive. */
-  bool object_usable_as_light(Object *object);
-
   struct IESSlot {
     IESFile ies;
     uint hash;
