@@ -6,20 +6,16 @@
 
 #include "bvh/params.h"
 
+#include "scene/devicescene.h"
 #include "scene/film.h"
 #include "scene/image.h"
 #include "scene/shader.h"
 
-#include "device/device.h"
-#include "device/memory.h"
-
 #include "util/param.h"
 #include "util/string.h"
 #include "util/system.h"
 #include "util/texture.h"
 #include "util/thread.h"
-#include "util/types.h"
-#include "util/vector.h"
 
 CCL_NAMESPACE_BEGIN
 
@@ -54,84 +50,6 @@ class RenderStats;
 class SceneUpdateStats;
 class Volume;
 
-/* Scene Device Data */
-
-class DeviceScene {
- public:
-  /* BVH */
-  device_vector<int4> bvh_nodes;
-  device_vector<int4> bvh_leaf_nodes;
-  device_vector<int> object_node;
-  device_vector<int> prim_type;
-  device_vector<uint> prim_visibility;
-  device_vector<int> prim_index;
-  device_vector<int> prim_object;
-  device_vector<float2> prim_time;
-
-  /* mesh */
-  device_vector<packed_float3> tri_verts;
-  device_vector<uint> tri_shader;
-  device_vector<packed_float3> tri_vnormal;
-  device_vector<uint4> tri_vindex;
-  device_vector<uint> tri_patch;
-  device_vector<float2> tri_patch_uv;
-
-  device_vector<KernelCurve> curves;
-  device_vector<float4> curve_keys;
-  device_vector<KernelCurveSegment> curve_segments;
-
-  device_vector<uint> patches;
-
-  /* pointcloud */
-  device_vector<float4> points;
-  device_vector<uint> points_shader;
-
-  /* objects */
-  device_vector<KernelObject> objects;
-  device_vector<Transform> object_motion_pass;
-  device_vector<DecomposedTransform> object_motion;
-  device_vector<uint> object_flag;
-  device_vector<float> object_volume_step;
-  device_vector<uint> object_prim_offset;
-
-  /* cameras */
-  device_vector<DecomposedTransform> camera_motion;
-
-  /* attributes */
-  device_vector<uint4> attributes_map;
-  device_vector<float> attributes_float;
-  device_vector<float2> attributes_float2;
-  device_vector<packed_float3> attributes_float3;
-  device_vector<float4> attributes_float4;
-  device_vector<uchar4> attributes_uchar4;
-
-  /* lights */
-  device_vector<KernelLightDistribution> light_distribution;
-  device_vector<KernelLight> lights;
-  device_vector<float2> light_background_marginal_cdf;
-  device_vector<float2> light_background_conditional_cdf;
-
-  /* particles */
-  device_vector<KernelParticle> particles;
-
-  /* shaders */
-  device_vector<int4> svm_nodes;
-  device_vector<KernelShader> shaders;
-
-  /* lookup tables */
-  device_vector<float> lookup_table;
-
-  /* integrator */
-  device_vector<float> sample_pattern_lut;
-
-  /* ies lights */
-  device_vector<float> ies_lights;
-
-  KernelData data;
-
-  DeviceScene(Device *device);
-};
-
 /* Scene Parameters */
 
 class SceneParams {
@@ -261,7 +179,7 @@ class Scene : public NodeOwner {
   float motion_shutter_time();
 
   bool need_update();
-  bool need_reset();
+  bool need_reset(const bool check_camera = true);
 
   void reset();
   void device_free();
@@ -270,6 +188,7 @@ class Scene : public NodeOwner {
 
   void enable_update_stats();
 
+  bool load_kernels(Progress &progress);
   bool update(Progress &progress);
 
   bool has_shadow_catcher();
@@ -333,7 +252,6 @@ class Scene : public NodeOwner {
   uint loaded_kernel_features;
 
   void update_kernel_features();
-  bool load_kernels(Progress &progress, bool lock_scene = true);
 
   bool has_shadow_catcher_ = false;
   bool shadow_catcher_modified_ = true;
@@ -344,7 +262,7 @@ class Scene : public NodeOwner {
   /* Get maximum number of closures to be used in kernel. */
   int get_max_closure_count();
 
-  /* Get size of a volume stack needed to render this scene.  */
+  /* Get size of a volume stack needed to render this scene. */
   int get_volume_stack_size() const;
 
   template<typename T> void delete_node_impl(T *node)
