@@ -74,15 +74,15 @@ class ShaderInput {
   {
   }
 
-  ustring name()
+  ustring name() const
   {
     return socket_type.ui_name;
   }
-  int flags()
+  int flags() const
   {
     return socket_type.flags;
   }
-  SocketType::Type type()
+  SocketType::Type type() const
   {
     return socket_type.type;
   }
@@ -119,11 +119,11 @@ class ShaderOutput {
   {
   }
 
-  ustring name()
+  ustring name() const
   {
     return socket_type.ui_name;
   }
-  SocketType::Type type()
+  SocketType::Type type() const
   {
     return socket_type.type;
   }
@@ -160,15 +160,11 @@ class ShaderNode : public Node {
   virtual void compile(OSLCompiler &compiler) = 0;
 
   /* Expand node into additional nodes. */
-  virtual void expand(ShaderGraph * /* graph */)
-  {
-  }
+  virtual void expand(ShaderGraph * /* graph */) {}
 
   /* ** Node optimization ** */
   /* Check whether the node can be replaced with single constant. */
-  virtual void constant_fold(const ConstantFolder & /*folder*/)
-  {
-  }
+  virtual void constant_fold(const ConstantFolder & /*folder*/) {}
 
   /* Simplify settings used by artists to the ones which are simpler to
    * evaluate in the kernel but keep the final result unchanged.
