@@ -85,7 +85,7 @@ class DisplayDriver {
     int buffer_height = 0;
 
     /* OpenGL pixel buffer object. */
-    int opengl_pbo_id = 0;
+    int64_t opengl_pbo_id = 0;
 
     /* Clear the entire buffer before doing partial write to it. */
     bool need_clear = false;
