@@ -95,8 +95,8 @@ static void build_patch_map(PackedPatchTable &table,
   for (int array = 0; array < table.num_arrays; array++) {
     Far::ConstPatchParamArray params = patch_table->GetPatchParams(array);
 
-    for (int i = 0; i < patch_table->GetNumPatches(array);
-         i++, handle_index += PATCH_HANDLE_SIZE) {
+    for (int i = 0; i < patch_table->GetNumPatches(array); i++, handle_index += PATCH_HANDLE_SIZE)
+    {
       const Far::PatchParam &param = params[i];
       unsigned short depth = param.GetDepth();
 
