@@ -26,9 +26,11 @@ void util_cdf_evaluate(
     cdf[i + 1] = cdf[i] + fabsf(y);
   }
   /* Normalize the CDF. */
+  float fac = (cdf[resolution] == 0.0f) ? 0.0f : 1.0f / cdf[resolution];
   for (int i = 0; i <= resolution; i++) {
-    cdf[i] /= cdf[resolution];
+    cdf[i] *= fac;
   }
+  cdf[resolution] = 1.0f;
 }
 
 /* Invert pre-calculated CDF function. */
