@@ -1,22 +1,12 @@
-/*
- * Copyright 2011-2013 Blender Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- * http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
+/* SPDX-License-Identifier: Apache-2.0
+ * Copyright 2011-2022 Blender Foundation */
 
 #include "util/path.h"
+#include "util/algorithm.h"
+#include "util/map.h"
 #include "util/md5.h"
 #include "util/string.h"
+#include "util/vector.h"
 
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/strutil.h>
@@ -66,7 +56,6 @@ typedef struct stat path_stat_t;
 
 static string cached_path = "";
 static string cached_user_path = "";
-static string cached_temp_path = "";
 static string cached_xdg_cache_path = "";
 
 namespace {
@@ -91,9 +80,7 @@ class directory_iterator {
     const WIN32_FIND_DATAW &find_data_;
   };
 
-  directory_iterator() : path_info_("", find_data_), h_find_(INVALID_HANDLE_VALUE)
-  {
-  }
+  directory_iterator() : path_info_("", find_data_), h_find_(INVALID_HANDLE_VALUE) {}
 
   explicit directory_iterator(const string &path) : path_(path), path_info_(path, find_data_)
   {
@@ -174,9 +161,7 @@ class directory_iterator {
  public:
   class path_info {
    public:
-    explicit path_info(const string &path) : path_(path), entry_(NULL)
-    {
-    }
+    explicit path_info(const string &path) : path_(path), entry_(NULL) {}
 
     string path()
     {
@@ -193,9 +178,7 @@ class directory_iterator {
     const struct dirent *entry_;
   };
 
-  directory_iterator() : path_info_(""), name_list_(NULL), num_entries_(-1), cur_entry_(-1)
-  {
-  }
+  directory_iterator() : path_info_(""), name_list_(NULL), num_entries_(-1), cur_entry_(-1) {}
 
   explicit directory_iterator(const string &path) : path_(path), path_info_(path_), cur_entry_(0)
   {
@@ -253,7 +236,8 @@ class directory_iterator {
   bool skip_dots()
   {
     while (strcmp(name_list_[cur_entry_]->d_name, ".") == 0 ||
-           strcmp(name_list_[cur_entry_]->d_name, "..") == 0) {
+           strcmp(name_list_[cur_entry_]->d_name, "..") == 0)
+    {
       if (!step()) {
         return false;
       }
@@ -313,7 +297,7 @@ static char *path_specials(const string &sub)
   if (env_shader_path != NULL && sub == "shader") {
     return env_shader_path;
   }
-  else if (env_shader_path != NULL && sub == "source") {
+  else if (env_source_path != NULL && sub == "source") {
     return env_source_path;
   }
   return NULL;
@@ -336,11 +320,10 @@ static string path_xdg_cache_get()
 }
 #endif
 
-void path_init(const string &path, const string &user_path, const string &temp_path)
+void path_init(const string &path, const string &user_path)
 {
   cached_path = path;
   cached_user_path = user_path;
-  cached_temp_path = temp_path;
 
 #ifdef _MSC_VER
   // workaround for https://svn.boost.org/trac/boost/ticket/6320
@@ -384,15 +367,6 @@ string path_cache_get(const string &sub)
 #endif
 }
 
-string path_temp_get(const string &sub)
-{
-  if (cached_temp_path == "") {
-    cached_temp_path = Filesystem::temp_directory_path();
-  }
-
-  return path_join(cached_temp_path, sub);
-}
-
 #if defined(__linux__) || defined(__APPLE__)
 string path_xdg_home_get(const string &sub = "");
 #endif
@@ -505,12 +479,14 @@ static string path_unc_to_short(const string &path)
 {
   size_t len = path.size();
   if ((len > 3) && (path[0] == DIR_SEP) && (path[1] == DIR_SEP) && (path[2] == '?') &&
-      ((path[3] == DIR_SEP) || (path[3] == DIR_SEP_ALT))) {
+      ((path[3] == DIR_SEP) || (path[3] == DIR_SEP_ALT)))
+  {
     if ((len > 5) && (path[5] == ':')) {
       return path.substr(4, len - 4);
     }
     else if ((len > 7) && (path.substr(4, 3) == "UNC") &&
-             ((path[7] == DIR_SEP) || (path[7] == DIR_SEP_ALT))) {
+             ((path[7] == DIR_SEP) || (path[7] == DIR_SEP_ALT)))
+    {
       return "\\\\" + path.substr(8, len - 8);
     }
   }
@@ -541,7 +517,7 @@ static string path_make_compatible(const string &path)
   if ((path.size() >= 3) && (path[0] == DIR_SEP) && (path[1] == DIR_SEP)) {
     result = path_cleanup_unc(result);
   }
-  /* Make sure volume-only path ends up wit ha directory separator. */
+  /* Make sure volume-only path ends up wit a directory separator. */
   if (result.size() == 2 && result[1] == ':') {
     result += DIR_SEP;
   }
@@ -750,6 +726,166 @@ bool path_remove(const string &path)
   return remove(path.c_str()) == 0;
 }
 
+struct SourceReplaceState {
+  typedef map<string, string> ProcessedMapping;
+  /* Base director for all relative include headers. */
+  string base;
+  /* Result of processed files. */
+  ProcessedMapping processed_files;
+  /* Set of files containing #pragma once which have been included. */
+  set<string> pragma_onced;
+};
+
+static string path_source_replace_includes_recursive(const string &source,
+                                                     const string &source_filepath,
+                                                     SourceReplaceState *state);
+
+static string path_source_handle_preprocessor(const string &preprocessor_line,
+                                              const string &source_filepath,
+                                              SourceReplaceState *state)
+{
+  string result = preprocessor_line;
+
+  string rest_of_line = string_strip(preprocessor_line.substr(1));
+
+  if (0 == strncmp(rest_of_line.c_str(), "include", 7)) {
+    rest_of_line = string_strip(rest_of_line.substr(8));
+    if (rest_of_line[0] == '"') {
+      const size_t n_start = 1;
+      const size_t n_end = rest_of_line.find("\"", n_start);
+      const string filename = rest_of_line.substr(n_start, n_end - n_start);
+
+      string filepath = path_join(state->base, filename);
+      if (!path_exists(filepath)) {
+        filepath = path_join(path_dirname(source_filepath), filename);
+      }
+      string text;
+      if (path_read_text(filepath, text)) {
+        text = path_source_replace_includes_recursive(text, filepath, state);
+        /* Use line directives for better error messages. */
+        return "\n" + text + "\n";
+      }
+    }
+  }
+
+  return result;
+}
+
+/* Our own little c preprocessor that replaces #includes with the file
+ * contents, to work around issue of OpenCL drivers not supporting
+ * include paths with spaces in them.
+ */
+static string path_source_replace_includes_recursive(const string &_source,
+                                                     const string &source_filepath,
+                                                     SourceReplaceState *state)
+{
+  const string *psource = &_source;
+  string source_new;
+
+  auto pragma_once = _source.find("#pragma once");
+  if (pragma_once != string::npos) {
+    if (state->pragma_onced.find(source_filepath) != state->pragma_onced.end()) {
+      return "";
+    }
+    state->pragma_onced.insert(source_filepath);
+
+    //                                 "#pragma once"
+    //                                 "//prgma once"
+    source_new = _source;
+    memcpy(source_new.data() + pragma_once, "//pr", 4);
+    psource = &source_new;
+  }
+
+  /* Try to re-use processed file without spending time on replacing all
+   * include directives again.
+   */
+  SourceReplaceState::ProcessedMapping::iterator replaced_file = state->processed_files.find(
+      source_filepath);
+  if (replaced_file != state->processed_files.end()) {
+    return replaced_file->second;
+  }
+
+  const string &source = *psource;
+
+  /* Perform full file processing. */
+  string result = "";
+  const size_t source_length = source.length();
+  size_t index = 0;
+  /* Information about where we are in the source. */
+  size_t column_number = 1;
+  /* Currently gathered non-preprocessor token.
+   * Store as start/length rather than token itself to avoid overhead of
+   * memory re-allocations on each character concatenation.
+   */
+  size_t token_start = 0, token_length = 0;
+  /* Denotes whether we're inside of preprocessor line, together with
+   * preprocessor line itself.
+   *
+   * TODO(sergey): Investigate whether using token start/end position
+   * gives measurable speedup.
+   */
+  bool inside_preprocessor = false;
+  string preprocessor_line = "";
+  /* Actual loop over the whole source. */
+  while (index < source_length) {
+    char ch = source[index];
+
+    if (ch == '\n') {
+      if (inside_preprocessor) {
+        string block = path_source_handle_preprocessor(preprocessor_line, source_filepath, state);
+
+        if (!block.empty()) {
+          result += block;
+        }
+
+        /* Start gathering net part of the token. */
+        token_start = index;
+        token_length = 0;
+        inside_preprocessor = false;
+        preprocessor_line = "";
+      }
+      column_number = 0;
+    }
+    else if (ch == '#' && column_number == 1 && !inside_preprocessor) {
+      /* Append all possible non-preprocessor token to the result. */
+      if (token_length != 0) {
+        result.append(source, token_start, token_length);
+        token_start = index;
+        token_length = 0;
+      }
+      inside_preprocessor = true;
+    }
+
+    if (inside_preprocessor) {
+      preprocessor_line += ch;
+    }
+    else {
+      ++token_length;
+    }
+    ++index;
+    ++column_number;
+  }
+  /* Append possible tokens which happened before special events handled
+   * above.
+   */
+  if (token_length != 0) {
+    result.append(source, token_start, token_length);
+  }
+  if (inside_preprocessor) {
+    result += path_source_handle_preprocessor(preprocessor_line, source_filepath, state);
+  }
+  /* Store result for further reuse. */
+  state->processed_files[source_filepath] = result;
+  return result;
+}
+
+string path_source_replace_includes(const string &source, const string &path)
+{
+  SourceReplaceState state;
+  state.base = path;
+  return path_source_replace_includes_recursive(source, path, &state);
+}
+
 FILE *path_fopen(const string &path, const string &mode)
 {
 #ifdef _WIN32
@@ -761,19 +897,54 @@ FILE *path_fopen(const string &path, const string &mode)
 #endif
 }
 
-void path_cache_clear_except(const string &name, const set<string> &except)
+/* LRU Cache for Kernels */
+
+static void path_cache_kernel_mark_used(const string &path)
 {
-  string dir = path_user_get("cache");
+  std::time_t current_time = std::time(nullptr);
+  OIIO::Filesystem::last_write_time(path, current_time);
+}
 
-  if (path_exists(dir)) {
-    directory_iterator it(dir), it_end;
+bool path_cache_kernel_exists_and_mark_used(const string &path)
+{
+  if (path_exists(path)) {
+    path_cache_kernel_mark_used(path);
+    return true;
+  }
+  else {
+    return false;
+  }
+}
 
-    for (; it != it_end; ++it) {
-      string filename = path_filename(it->path());
+void path_cache_kernel_mark_added_and_clear_old(const string &new_path,
+                                                const size_t max_old_kernel_of_same_type)
+{
+  path_cache_kernel_mark_used(new_path);
 
-      if (string_startswith(filename, name.c_str()))
-        if (except.find(filename) == except.end())
-          path_remove(it->path());
+  string dir = path_dirname(new_path);
+  if (!path_exists(dir)) {
+    return;
+  }
+
+  /* Remove older kernels within the same directory. */
+  directory_iterator it(dir), it_end;
+  vector<pair<std::time_t, string>> same_kernel_types;
+
+  for (; it != it_end; ++it) {
+    const string &path = it->path();
+    if (path == new_path) {
+      continue;
+    }
+
+    std::time_t last_time = OIIO::Filesystem::last_write_time(path);
+    same_kernel_types.emplace_back(last_time, path);
+  }
+
+  if (same_kernel_types.size() > max_old_kernel_of_same_type) {
+    sort(same_kernel_types.begin(), same_kernel_types.end());
+
+    for (int i = 0; i < same_kernel_types.size() - max_old_kernel_of_same_type; i++) {
+      path_remove(same_kernel_types[i].second);
     }
   }
 }
