@@ -13,6 +13,7 @@
 #ifndef __FFMPEG_COMPAT_H__
 #define __FFMPEG_COMPAT_H__
 
+#include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 
 /* Check if our ffmpeg is new enough, avoids user complaints.
@@ -36,6 +37,25 @@
 #  define FFMPEG_INLINE static inline
 #endif
 
+#if (LIBAVFORMAT_VERSION_MAJOR < 59)
+/* For versions older than ffmpeg 5.0, use the old channel layout variables.
+ * We intend to only keep this  workaround for around two releases (3.5, 3.6).
+ * If it sticks around any longer, then we should consider refactoring this.
+ */
+#  define FFMPEG_USE_OLD_CHANNEL_VARS
+#endif
+
+/* AV_CODEC_CAP_AUTO_THREADS was renamed to AV_CODEC_CAP_OTHER_THREADS with
+ * upstream commit
+ * github.com/FFmpeg/FFmpeg/commit/7d09579190def3ef7562399489e628f3b65714ce
+ * (lavc 58.132.100) and removed with commit
+ * github.com/FFmpeg/FFmpeg/commit/10c9a0874cb361336237557391d306d26d43f137
+ * for ffmpeg 6.0.
+ */
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 132, 100)
+#  define AV_CODEC_CAP_OTHER_THREADS AV_CODEC_CAP_AUTO_THREADS
+#endif
+
 #if (LIBAVFORMAT_VERSION_MAJOR < 58) || \
     ((LIBAVFORMAT_VERSION_MAJOR == 58) && (LIBAVFORMAT_VERSION_MINOR < 76))
 #  define FFMPEG_USE_DURATION_WORKAROUND 1
@@ -194,8 +214,8 @@ void deinterlace_line_inplace(
 }
 
 /* deinterlacing : 2 temporal taps, 3 spatial taps linear filter. The
-   top field is copied as is, but the bottom field is deinterlaced
-   against the top field. */
+ * top field is copied as is, but the bottom field is deinterlaced
+ * against the top field. */
 FFMPEG_INLINE
 void deinterlace_bottom_field(
     uint8_t *dst, int dst_wrap, const uint8_t *src1, int src_wrap, int width, int height)
@@ -262,7 +282,8 @@ int av_image_deinterlace(
   if (pix_fmt != AV_PIX_FMT_YUV420P && pix_fmt != AV_PIX_FMT_YUVJ420P &&
       pix_fmt != AV_PIX_FMT_YUV422P && pix_fmt != AV_PIX_FMT_YUVJ422P &&
       pix_fmt != AV_PIX_FMT_YUV444P && pix_fmt != AV_PIX_FMT_YUV411P &&
-      pix_fmt != AV_PIX_FMT_GRAY8) {
+      pix_fmt != AV_PIX_FMT_GRAY8)
+  {
     return -1;
   }
   if ((width & 3) != 0 || (height & 3) != 0) {
