@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2013 Blender Foundation. All rights reserved. */
+ * Copyright 2013 Blender Foundation */
 
 /** \file
  * \ingroup GHOST
@@ -13,12 +13,14 @@
 #  pragma clang diagnostic ignored "-Wdeprecated-declarations"
 #endif
 
-#include "GHOST_ContextCGL.h"
+#include "GHOST_ContextCGL.hh"
 
 #include <Cocoa/Cocoa.h>
 #include <Metal/Metal.h>
 #include <QuartzCore/QuartzCore.h>
 
+#include <epoxy/gl.h>
+
 #include <cassert>
 #include <vector>
 
@@ -41,6 +43,7 @@ static void ghost_fatal_error_dialog(const char *msg)
 }
 
 NSOpenGLContext *GHOST_ContextCGL::s_sharedOpenGLContext = nil;
+MTLCommandQueue *GHOST_ContextCGL::s_sharedMetalCommandQueue = nil;
 int GHOST_ContextCGL::s_sharedCount = 0;
 
 GHOST_ContextCGL::GHOST_ContextCGL(bool stereoVisual,
@@ -52,7 +55,6 @@ GHOST_ContextCGL::GHOST_ContextCGL(bool stereoVisual,
       m_useMetalForRendering(type == GHOST_kDrawingContextTypeMetal),
       m_metalView(metalView),
       m_metalLayer(metalLayer),
-      m_metalCmdQueue(nil),
       m_metalRenderPipeline(nil),
       m_openGLView(openGLView),
       m_openGLContext(nil),
@@ -316,7 +318,7 @@ id<MTLTexture> GHOST_ContextCGL::metalOverlayTexture()
 
 MTLCommandQueue *GHOST_ContextCGL::metalCommandQueue()
 {
-  return m_metalCmdQueue;
+  return s_sharedMetalCommandQueue;
 }
 MTLDevice *GHOST_ContextCGL::metalDevice()
 {
@@ -401,8 +403,8 @@ GHOST_TSuccess GHOST_ContextCGL::initializeDrawingContext()
         }
       }
       const int max_ctx_attempts = increasedSamplerLimit ? 2 : 1;
-      for (int ctx_create_attempt = 0; ctx_create_attempt < max_ctx_attempts;
-           ctx_create_attempt++) {
+      for (int ctx_create_attempt = 0; ctx_create_attempt < max_ctx_attempts; ctx_create_attempt++)
+      {
 
         attribs.clear();
         attribs.reserve(40);
@@ -516,7 +518,7 @@ GHOST_TSuccess GHOST_ContextCGL::releaseNativeHandles()
 
 /* OpenGL on Metal
  *
- * Use Metal layer to avoid Viewport lagging on macOS, see T60043. */
+ * Use Metal layer to avoid Viewport lagging on macOS, see #60043. */
 
 static const MTLPixelFormat METAL_FRAMEBUFFERPIXEL_FORMAT = MTLPixelFormatBGRA8Unorm;
 static const OSType METAL_CORE_VIDEO_PIXEL_FORMAT = kCVPixelFormatType_32BGRA;
@@ -528,9 +530,14 @@ void GHOST_ContextCGL::metalInit()
     /* clang-format on */
     id<MTLDevice> device = m_metalLayer.device;
 
-    /* Create a command queue for blit/present operation. */
-    m_metalCmdQueue = (MTLCommandQueue *)[device newCommandQueue];
-    [m_metalCmdQueue retain];
+    /* Create a command queue for blit/present operation. Note: All context should share a single
+     * command queue to ensure correct ordering of work submitted from multiple contexts. */
+    if (s_sharedMetalCommandQueue == nil) {
+      s_sharedMetalCommandQueue = (MTLCommandQueue *)[device
+          newCommandQueueWithMaxCommandBufferCount:GHOST_ContextCGL::max_command_buffer_count];
+    }
+    /* Ensure active GHOSTContext retains a reference to the shared context. */
+    [s_sharedMetalCommandQueue retain];
 
     /* Create shaders for blit operation. */
     NSString *source = @R"msl(
@@ -613,9 +620,6 @@ void GHOST_ContextCGL::metalInit()
 
 void GHOST_ContextCGL::metalFree()
 {
-  if (m_metalCmdQueue) {
-    [m_metalCmdQueue release];
-  }
   if (m_metalRenderPipeline) {
     [m_metalRenderPipeline release];
   }
@@ -753,7 +757,8 @@ void GHOST_ContextCGL::metalUpdateFramebuffer()
     /* NOTE(Metal): Metal API Path. */
     if (m_defaultFramebufferMetalTexture[current_swapchain_index].texture &&
         m_defaultFramebufferMetalTexture[current_swapchain_index].texture.width == width &&
-        m_defaultFramebufferMetalTexture[current_swapchain_index].texture.height == height) {
+        m_defaultFramebufferMetalTexture[current_swapchain_index].texture.height == height)
+    {
       return;
     }
 
@@ -785,7 +790,7 @@ void GHOST_ContextCGL::metalUpdateFramebuffer()
         overlayTex;  //[(MTLTexture *)overlayTex retain];
 
     /* Clear texture on create */
-    id<MTLCommandBuffer> cmdBuffer = [m_metalCmdQueue commandBuffer];
+    id<MTLCommandBuffer> cmdBuffer = [s_sharedMetalCommandQueue commandBuffer];
     MTLRenderPassDescriptor *passDescriptor = [MTLRenderPassDescriptor renderPassDescriptor];
     {
       auto attachment = [passDescriptor.colorAttachments objectAtIndexedSubscript:0];
@@ -850,7 +855,7 @@ void GHOST_ContextCGL::metalSwapBuffers()
     }
 
     if (!m_useMetalForRendering) {
-      id<MTLCommandBuffer> cmdBuffer = [m_metalCmdQueue commandBuffer];
+      id<MTLCommandBuffer> cmdBuffer = [s_sharedMetalCommandQueue commandBuffer];
       {
         assert(m_defaultFramebufferMetalTexture[current_swapchain_index].texture != nil);
         id<MTLRenderCommandEncoder> enc = [cmdBuffer
@@ -892,7 +897,7 @@ void GHOST_ContextCGL::initClear()
 #if WITH_METAL
     // TODO (mg_gpusw_apple) this path is never taken, this is legacy left from inital integration
     // of metal and gl, the whole file should be cleaned up and stripped of the legacy path
-    id<MTLCommandBuffer> cmdBuffer = [m_metalCmdQueue commandBuffer];
+    id<MTLCommandBuffer> cmdBuffer = [s_sharedMetalCommandQueue commandBuffer];
     MTLRenderPassDescriptor *passDescriptor = [MTLRenderPassDescriptor renderPassDescriptor];
     {
       auto attachment = [passDescriptor.colorAttachments objectAtIndexedSubscript:0];
