@@ -1,4 +1,4 @@
-// Copyright 2021 Blender Foundation. All rights reserved.
+// Copyright 2021 Blender Foundation
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -18,9 +18,7 @@
 
 #include "internal/evaluator/eval_output_gpu.h"
 
-OpenSubdiv_EvaluatorCacheImpl::OpenSubdiv_EvaluatorCacheImpl()
-{
-}
+OpenSubdiv_EvaluatorCacheImpl::OpenSubdiv_EvaluatorCacheImpl() {}
 
 OpenSubdiv_EvaluatorCacheImpl::~OpenSubdiv_EvaluatorCacheImpl()
 {
