@@ -1,4 +1,4 @@
-// Copyright 2016 Blender Foundation. All rights reserved.
+// Copyright 2016 Blender Foundation
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -21,9 +21,7 @@
 namespace blender {
 namespace opensubdiv {
 
-TopologyRefinerImpl::TopologyRefinerImpl() : topology_refiner(nullptr)
-{
-}
+TopologyRefinerImpl::TopologyRefinerImpl() : topology_refiner(nullptr) {}
 
 TopologyRefinerImpl::~TopologyRefinerImpl()
 {
