@@ -33,12 +33,11 @@ struct PointerRNA;
 struct PropertyRNA;
 struct bAction;
 struct bActionGroup;
-struct bContext;
 
 /* Container for data required to do FCurve and Driver evaluation. */
 typedef struct AnimationEvalContext {
   /* For drivers, so that they have access to the dependency graph and the current view layer. See
-   * T77086. */
+   * #77086. */
   struct Depsgraph *depsgraph;
 
   /* FCurves and Drivers can be evaluated at a different time than the current scene time, for
