@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2006 Blender Foundation. All rights reserved. */
+ * Copyright 2006 Blender Foundation */
 
 /** \file
  * \ingroup bke
@@ -41,6 +41,7 @@ typedef enum eAttrDomainMask {
   ATTR_DOMAIN_MASK_CURVE = (1 << 4),
   ATTR_DOMAIN_MASK_ALL = (1 << 5) - 1
 } eAttrDomainMask;
+ENUM_OPERATORS(eAttrDomainMask, ATTR_DOMAIN_MASK_ALL);
 
 #define ATTR_DOMAIN_AS_MASK(domain) ((eAttrDomainMask)((1 << (int)(domain))))
 
@@ -56,8 +57,11 @@ bool BKE_attribute_allow_procedural_access(const char *attribute_name);
 /**
  * Create a new attribute layer.
  */
-struct CustomDataLayer *BKE_id_attribute_new(
-    struct ID *id, const char *name, int type, eAttrDomain domain, struct ReportList *reports);
+struct CustomDataLayer *BKE_id_attribute_new(struct ID *id,
+                                             const char *name,
+                                             eCustomDataType type,
+                                             eAttrDomain domain,
+                                             struct ReportList *reports);
 bool BKE_id_attribute_remove(struct ID *id, const char *name, struct ReportList *reports);
 
 /**
@@ -69,7 +73,7 @@ struct CustomDataLayer *BKE_id_attribute_duplicate(struct ID *id,
 
 struct CustomDataLayer *BKE_id_attribute_find(const struct ID *id,
                                               const char *name,
-                                              int type,
+                                              eCustomDataType type,
                                               eAttrDomain domain);
 
 struct CustomDataLayer *BKE_id_attribute_search(struct ID *id,
@@ -90,7 +94,7 @@ int BKE_id_attributes_length(const struct ID *id,
                              eCustomDataMask mask);
 
 struct CustomDataLayer *BKE_id_attributes_active_get(struct ID *id);
-void BKE_id_attributes_active_set(struct ID *id, struct CustomDataLayer *layer);
+void BKE_id_attributes_active_set(struct ID *id, const char *name);
 int *BKE_id_attributes_active_index_p(struct ID *id);
 
 CustomData *BKE_id_attributes_iterator_next_domain(struct ID *id, struct CustomDataLayer *layers);
@@ -105,39 +109,19 @@ int BKE_id_attribute_to_index(const struct ID *id,
                               eAttrDomainMask domain_mask,
                               eCustomDataMask layer_mask);
 
-struct CustomDataLayer *BKE_id_attribute_subset_active_get(const struct ID *id,
-                                                           int active_flag,
-                                                           eAttrDomainMask domain_mask,
-                                                           eCustomDataMask mask);
-void BKE_id_attribute_subset_active_set(struct ID *id,
-                                        struct CustomDataLayer *layer,
-                                        int active_flag,
-                                        eAttrDomainMask domain_mask,
-                                        eCustomDataMask mask);
+const char *BKE_id_attributes_active_color_name(const struct ID *id);
+const char *BKE_id_attributes_default_color_name(const struct ID *id);
+void BKE_id_attributes_active_color_set(struct ID *id, const char *name);
+void BKE_id_attributes_default_color_set(struct ID *id, const char *name);
 
-/**
- * Sets up a temporary ID with arbitrary CustomData domains. `r_id` will
- * be zero initialized with ID type id_type and any non-nullptr
- * CustomData parameter will be copied into the appropriate struct members.
- *
- * \param r_id: Pointer to storage sufficient for ID type-code id_type.
- */
-void BKE_id_attribute_copy_domains_temp(short id_type,
-                                        const struct CustomData *vdata,
-                                        const struct CustomData *edata,
-                                        const struct CustomData *ldata,
-                                        const struct CustomData *pdata,
-                                        const struct CustomData *cdata,
-                                        struct ID *r_id);
-
-struct CustomDataLayer *BKE_id_attributes_active_color_get(const struct ID *id);
-void BKE_id_attributes_active_color_set(struct ID *id, struct CustomDataLayer *active_layer);
-struct CustomDataLayer *BKE_id_attributes_render_color_get(const struct ID *id);
-void BKE_id_attributes_render_color_set(struct ID *id, struct CustomDataLayer *active_layer);
 struct CustomDataLayer *BKE_id_attributes_color_find(const struct ID *id, const char *name);
 
 bool BKE_id_attribute_calc_unique_name(struct ID *id, const char *name, char *outname);
 
+const char *BKE_uv_map_vert_select_name_get(const char *uv_map_name, char *buffer);
+const char *BKE_uv_map_edge_select_name_get(const char *uv_map_name, char *buffer);
+const char *BKE_uv_map_pin_name_get(const char *uv_map_name, char *buffer);
+
 #ifdef __cplusplus
 }
 #endif
