@@ -4,27 +4,25 @@
 
 /** \file
  * \ingroup bke
+ *
+ * This header encapsulates necessary code to build a BVH.
  */
 
-#include "BLI_bitmap.h"
 #include "BLI_kdopbvh.h"
 #include "BLI_threads.h"
 
 #ifdef __cplusplus
 #  include <mutex>
+
+#  include "BLI_bit_vector.hh"
 #endif
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/**
- * This header encapsulates necessary code to build a BVH
- */
-
 struct BMEditMesh;
 struct MFace;
-struct MVert;
 struct Mesh;
 struct PointCloud;
 
@@ -58,11 +56,10 @@ typedef struct BVHTreeFromMesh {
   BVHTree_RayCastCallback raycast_callback;
 
   /* Vertex array, so that callbacks have instant access to data. */
-  const struct MVert *vert;
-  const float (*vert_normals)[3];
-  const struct MEdge *edge;
+  const float (*vert_positions)[3];
+  const struct vec2i *edge;
   const struct MFace *face;
-  const struct MLoop *loop;
+  const int *corner_verts;
   const struct MLoopTri *looptri;
 
   /* Private data */
@@ -102,12 +99,14 @@ typedef enum BVHCacheType {
 BVHTree *bvhtree_from_editmesh_verts(
     BVHTreeFromEditMesh *data, struct BMEditMesh *em, float epsilon, int tree_type, int axis);
 
+#ifdef __cplusplus
+
 /**
  * Builds a BVH-tree where nodes are the vertices of the given `em`.
  */
 BVHTree *bvhtree_from_editmesh_verts_ex(BVHTreeFromEditMesh *data,
                                         struct BMEditMesh *em,
-                                        const BLI_bitmap *mask,
+                                        blender::BitSpan mask,
                                         int verts_num_active,
                                         float epsilon,
                                         int tree_type,
@@ -121,9 +120,9 @@ BVHTree *bvhtree_from_editmesh_verts_ex(BVHTreeFromEditMesh *data,
  * (else will be computed from mask).
  */
 BVHTree *bvhtree_from_mesh_verts_ex(struct BVHTreeFromMesh *data,
-                                    const struct MVert *vert,
+                                    const float (*vert_positions)[3],
                                     int verts_num,
-                                    const BLI_bitmap *verts_mask,
+                                    blender::BitSpan verts_mask,
                                     int verts_num_active,
                                     float epsilon,
                                     int tree_type,
@@ -137,12 +136,14 @@ BVHTree *bvhtree_from_editmesh_edges(
  */
 BVHTree *bvhtree_from_editmesh_edges_ex(BVHTreeFromEditMesh *data,
                                         struct BMEditMesh *em,
-                                        const BLI_bitmap *edges_mask,
+                                        blender::BitSpan edges_mask,
                                         int edges_num_active,
                                         float epsilon,
                                         int tree_type,
                                         int axis);
 
+#  ifdef __cplusplus
+
 /**
  * Builds a BVH-tree where nodes are the given edges.
  * \param vert, vert_allocated: if true, elem freeing will be done when freeing data.
@@ -151,16 +152,18 @@ BVHTree *bvhtree_from_editmesh_edges_ex(BVHTreeFromEditMesh *data,
  * \param edges_num_active: if >= 0, number of active edges to add to BVH-tree
  * (else will be computed from mask).
  */
-BVHTree *bvhtree_from_mesh_edges_ex(struct BVHTreeFromMesh *data,
-                                    const struct MVert *vert,
-                                    const struct MEdge *edge,
+BVHTree *bvhtree_from_mesh_edges_ex(BVHTreeFromMesh *data,
+                                    const float (*vert_positions)[3],
+                                    const blender::int2 *edge,
                                     int edges_num,
-                                    const BLI_bitmap *edges_mask,
+                                    blender::BitSpan edges_mask,
                                     int edges_num_active,
                                     float epsilon,
                                     int tree_type,
                                     int axis);
 
+#  endif
+
 BVHTree *bvhtree_from_editmesh_looptri(
     BVHTreeFromEditMesh *data, struct BMEditMesh *em, float epsilon, int tree_type, int axis);
 
@@ -169,7 +172,7 @@ BVHTree *bvhtree_from_editmesh_looptri(
  */
 BVHTree *bvhtree_from_editmesh_looptri_ex(BVHTreeFromEditMesh *data,
                                           struct BMEditMesh *em,
-                                          const BLI_bitmap *mask,
+                                          blender::BitSpan mask,
                                           int looptri_num_active,
                                           float epsilon,
                                           int tree_type,
@@ -179,16 +182,18 @@ BVHTree *bvhtree_from_editmesh_looptri_ex(BVHTreeFromEditMesh *data,
  * Builds a BVH-tree where nodes are the looptri faces of the given mesh.
  */
 BVHTree *bvhtree_from_mesh_looptri_ex(struct BVHTreeFromMesh *data,
-                                      const struct MVert *vert,
-                                      const struct MLoop *mloop,
+                                      const float (*vert_positions)[3],
+                                      const int *corner_verts,
                                       const struct MLoopTri *looptri,
                                       int looptri_num,
-                                      const BLI_bitmap *mask,
+                                      blender::BitSpan mask,
                                       int looptri_num_active,
                                       float epsilon,
                                       int tree_type,
                                       int axis);
 
+#endif
+
 /**
  * Builds or queries a BVH-cache for the cache BVH-tree of the request type.
  *
@@ -243,9 +248,11 @@ typedef struct BVHTreeFromPointCloud {
   const float (*coords)[3];
 } BVHTreeFromPointCloud;
 
-BVHTree *BKE_bvhtree_from_pointcloud_get(struct BVHTreeFromPointCloud *data,
-                                         const struct PointCloud *pointcloud,
-                                         int tree_type);
+#ifdef __cplusplus
+[[nodiscard]] BVHTree *BKE_bvhtree_from_pointcloud_get(BVHTreeFromPointCloud *data,
+                                                       const PointCloud *pointcloud,
+                                                       int tree_type);
+#endif
 
 void free_bvhtree_from_pointcloud(struct BVHTreeFromPointCloud *data);
 
