@@ -15,7 +15,6 @@ extern "C" {
 
 struct BoundBox;
 struct Curves;
-struct CustomDataLayer;
 struct Depsgraph;
 struct Main;
 struct Object;
@@ -29,7 +28,7 @@ bool BKE_curves_attribute_required(const struct Curves *curves, const char *name
 
 /* Depsgraph */
 
-struct Curves *BKE_curves_copy_for_eval(struct Curves *curves_src, bool reference);
+struct Curves *BKE_curves_copy_for_eval(struct Curves *curves_src);
 
 void BKE_curves_data_update(struct Depsgraph *depsgraph,
                             struct Scene *scene,
