@@ -139,6 +139,13 @@ typedef struct Global {
    * Typically Python drivers.
    */
   char autoexec_fail[200];
+
+  /**
+   * Has there been an opengl deprecation call detected when running on a none OpenGL backend.
+   */
+  bool opengl_deprecation_usage_detected;
+  const char *opengl_deprecation_usage_filename;
+  int opengl_deprecation_usage_lineno;
 } Global;
 
 /* **************** GLOBAL ********************* */
@@ -194,11 +201,12 @@ enum {
   G_DEBUG_IO = (1 << 17),                     /* IO Debugging (for Collada, ...). */
   G_DEBUG_GPU_FORCE_WORKAROUNDS = (1 << 18),  /* force gpu workarounds bypassing detections. */
   G_DEBUG_GPU_FORCE_DISABLE_SSBO = (1 << 19), /* force disabling usage of SSBO's */
-  G_DEBUG_XR = (1 << 20),                     /* XR/OpenXR messages */
-  G_DEBUG_XR_TIME = (1 << 21),                /* XR/OpenXR timing messages */
+  G_DEBUG_GPU_RENDERDOC = (1 << 20),          /* Enable RenderDoc integration. */
+  G_DEBUG_XR = (1 << 21),                     /* XR/OpenXR messages */
+  G_DEBUG_XR_TIME = (1 << 22),                /* XR/OpenXR timing messages */
 
-  G_DEBUG_GHOST = (1 << 22),  /* Debug GHOST module. */
-  G_DEBUG_WINTAB = (1 << 23), /* Debug Wintab. */
+  G_DEBUG_GHOST = (1 << 23),  /* Debug GHOST module. */
+  G_DEBUG_WINTAB = (1 << 24), /* Debug Wintab. */
 };
 
 #define G_DEBUG_ALL \
