@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2016 Blender Foundation. All rights reserved. */
+ * Copyright 2016 Blender Foundation */
 
 #pragma once
 
@@ -303,6 +303,27 @@ struct IDOverrideLibraryProperty *BKE_lib_override_library_property_get(
  */
 void BKE_lib_override_library_property_delete(struct IDOverrideLibrary *override,
                                               struct IDOverrideLibraryProperty *override_property);
+/**
+ * Delete a property override from the given ID \a override, if it exists.
+ *
+ * \return True when the property was found (and thus deleted), false if it wasn't found.
+ */
+bool BKE_lib_override_library_property_search_and_delete(struct IDOverrideLibrary *override,
+                                                         const char *rna_path);
+
+/** Change the RNA path of a library override on a property.
+ *
+ * No-op if the property override cannot be found.
+ *
+ * \param from_rna_path The RNA path of the property to change.
+ * \param to_rna_path The new RNA path. The library override system will copy the string to its own
+ * memory; the caller will retain ownership of the passed pointer.
+ * \return True if the property was found (and thus changed), false if it wasn't found.
+ */
+bool BKE_lib_override_library_property_rna_path_change(IDOverrideLibrary *liboverride,
+                                                       const char *old_rna_path,
+                                                       const char *new_rna_path);
+
 /**
  * Get the RNA-property matching the \a library_prop override property. Used for UI to query
  * additional data about the overridden property (e.g. UI name).
@@ -401,6 +422,8 @@ bool BKE_lib_override_library_status_check_reference(struct Main *bmain, struct
  * Compare local and reference data-blocks and create new override operations as needed,
  * or reset to reference values if overriding is not allowed.
  *
+ * \param r_report_flags: #eRNAOverrideMatchResult flags giving info about the result of this call.
+ *
  * \note Defining override operations is only mandatory before saving a `.blend` file on disk
  * (not for undo!).
  * Knowing that info at runtime is only useful for UI/UX feedback.
@@ -408,14 +431,41 @@ bool BKE_lib_override_library_status_check_reference(struct Main *bmain, struct
  * \note This is by far the biggest operation (the more time-consuming) of the three so far,
  * since it has to go over all properties in depth (all overridable ones at least).
  * Generating differential values and applying overrides are much cheaper.
- *
- * \return true if any library operation was created.
  */
-bool BKE_lib_override_library_operations_create(struct Main *bmain, struct ID *local);
+void BKE_lib_override_library_operations_create(struct Main *bmain,
+                                                struct ID *local,
+                                                int *r_report_flags);
 /**
  * Check all overrides from given \a bmain and create/update overriding operations as needed.
+ *
+ * \param r_report_flags: #eRNAOverrideMatchResult flags giving info about the result of this call.
  */
-bool BKE_lib_override_library_main_operations_create(struct Main *bmain, bool force_auto);
+void BKE_lib_override_library_main_operations_create(struct Main *bmain,
+                                                     bool force_auto,
+                                                     int *r_report_flags);
+
+/**
+ * Restore forbidden modified override properties to the values of their matching properties in the
+ * linked reference ID.
+ *
+ * \param r_report_flags: #eRNAOverrideMatchResult flags giving info about the result of this call.
+ *
+ * \note Typically used as part of BKE_lib_override_library_main_operations_create process, since
+ * modifying RNA properties from non-main threads is not safe.
+ */
+void BKE_lib_override_library_operations_restore(struct Main *bmain,
+                                                 struct ID *local,
+                                                 int *r_report_flags);
+/**
+ * Restore forbidden modified override properties to the values of their matching properties in the
+ * linked reference ID, for all liboverride IDs tagged as needing such process in given `bmain`.
+ *
+ * \param r_report_flags: #eRNAOverrideMatchResult flags giving info about the result of this call.
+ *
+ * \note Typically used as part of BKE_lib_override_library_main_operations_create process, since
+ * modifying RNA properties from non-main threads is not safe.
+ */
+void BKE_lib_override_library_main_operations_restore(struct Main *bmain, int *r_report_flags);
 
 /**
  * Reset all overrides in given \a id_root, while preserving ID relations.
@@ -476,6 +526,11 @@ void BKE_lib_override_library_main_update(struct Main *bmain);
  */
 bool BKE_lib_override_library_id_is_user_deletable(struct Main *bmain, struct ID *id);
 
+/**
+ * Debuging helper to show content of given liboverride data.
+ */
+void BKE_lib_override_debug_print(struct IDOverrideLibrary *override, const char *intro_txt);
+
 /* Storage (.blend file writing) part. */
 
 /* For now, we just use a temp main list. */
