@@ -23,16 +23,20 @@ typedef enum eMeshFairingDepth {
   MESH_FAIRING_DEPTH_TANGENCY = 2,
 } eMeshFairingDepth;
 
-/* affect_vertices is used to define the fairing area. Indexed by vertex index, set to true when
- * the vertex should be modified by fairing. */
+/**
+ * Affect_vertices is used to define the fairing area. Indexed by vertex index, set to true when
+ * the vertex should be modified by fairing.
+ */
 void BKE_bmesh_prefair_and_fair_verts(struct BMesh *bm,
                                       bool *affect_verts,
                                       eMeshFairingDepth depth);
 
-/* This function can optionally use the MVert coordinates of deform_mverts to read and write the
- * fairing result. When NULL, the function will use mesh->mverts directly. */
+/**
+ * This function can optionally use the vertex coordinates of deform_mverts to read and write the
+ * fairing result. When NULL, the function will use mesh positions directly.
+ */
 void BKE_mesh_prefair_and_fair_verts(struct Mesh *mesh,
-                                     struct MVert *deform_mverts,
+                                     float (*deform_vert_positions)[3],
                                      bool *affect_verts,
                                      eMeshFairingDepth depth);
 
