@@ -24,7 +24,6 @@ struct CCGVert;
 struct DMFlagMat;
 struct DerivedMesh;
 struct EdgeHash;
-struct MPoly;
 struct Mesh;
 struct MeshElemMap;
 struct Object;
@@ -96,7 +95,6 @@ typedef struct CCGDerivedMesh {
     struct CCGFace *face;
   } * faceMap;
 
-  short *edgeFlags;
   struct DMFlagMat *faceFlags;
 
   int *reverseFaceMap;
