@@ -93,7 +93,10 @@ void txt_sel_all(struct Text *text);
 void txt_sel_clear(struct Text *text);
 void txt_sel_line(struct Text *text);
 void txt_sel_set(struct Text *text, int startl, int startc, int endl, int endc);
-char *txt_sel_to_buf(struct Text *text, size_t *r_buf_strlen);
+char *txt_sel_to_buf(const struct Text *text, size_t *r_buf_strlen);
+/**
+ * \param in_buffer: UTF8 encoded text, invalid UTF8 byte-sequences are handled gracefully.
+ */
 void txt_insert_buf(struct Text *text, const char *in_buffer, int in_buffer_len)
     ATTR_NONNULL(1, 2);
 void txt_split_curline(struct Text *text);
@@ -103,9 +106,9 @@ bool txt_add_char(struct Text *text, unsigned int add);
 bool txt_add_raw_char(struct Text *text, unsigned int add);
 bool txt_replace_char(struct Text *text, unsigned int add);
 bool txt_unindent(struct Text *text);
-void txt_comment(struct Text *text);
 void txt_indent(struct Text *text);
-bool txt_uncomment(struct Text *text);
+void txt_comment(struct Text *text, const char *prefix);
+bool txt_uncomment(struct Text *text, const char *prefix);
 void txt_move_lines(struct Text *text, int direction);
 void txt_duplicate_line(struct Text *text);
 int txt_setcurr_tab_spaces(struct Text *text, int space);
