@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2011 Blender Foundation. All rights reserved. */
+ * Copyright 2011 Blender Foundation */
 
 #pragma once
 
@@ -67,20 +67,6 @@ void BKE_tracking_copy(struct MovieTracking *tracking_dst,
  */
 void BKE_tracking_settings_init(struct MovieTracking *tracking);
 
-/**
- * Get list base of active object's tracks.
- */
-struct ListBase *BKE_tracking_get_active_tracks(struct MovieTracking *tracking);
-/**
- * Get list base of active object's plane tracks.
- */
-struct ListBase *BKE_tracking_get_active_plane_tracks(struct MovieTracking *tracking);
-/**
- * Get reconstruction data of active object.
- */
-struct MovieTrackingReconstruction *BKE_tracking_get_active_reconstruction(
-    struct MovieTracking *tracking);
-
 /* Matrices for constraints and drawing. */
 
 /**
@@ -95,7 +81,7 @@ void BKE_tracking_get_camera_object_matrix(struct Object *camera_object, float m
  * \note frame number should be in clip space, not scene space.
  */
 void BKE_tracking_get_projection_matrix(struct MovieTracking *tracking,
-                                        struct MovieTrackingObject *object,
+                                        struct MovieTrackingObject *tracking_object,
                                         int framenr,
                                         int winx,
                                         int winy,
@@ -113,7 +99,7 @@ void BKE_tracking_clipboard_free(void);
  * Copy selected tracks from specified object to the clipboard.
  */
 void BKE_tracking_clipboard_copy_tracks(struct MovieTracking *tracking,
-                                        struct MovieTrackingObject *object);
+                                        struct MovieTrackingObject *tracking_object);
 /**
  * Check whether there are any tracks in the clipboard.
  */
@@ -124,7 +110,7 @@ bool BKE_tracking_clipboard_has_tracks(void);
  * Names of new tracks in object are guaranteed to be unique here.
  */
 void BKE_tracking_clipboard_paste_tracks(struct MovieTracking *tracking,
-                                         struct MovieTrackingObject *object);
+                                         struct MovieTrackingObject *tracking_object);
 
 /* **** Track **** */
 
@@ -248,19 +234,13 @@ void BKE_tracking_tracks_average(struct MovieTrackingTrack *dst_track,
                                  /*const*/ struct MovieTrackingTrack **src_tracks,
                                  int num_src_tracks);
 
-struct MovieTrackingTrack *BKE_tracking_track_get_named(struct MovieTracking *tracking,
-                                                        struct MovieTrackingObject *object,
-                                                        const char *name);
-struct MovieTrackingTrack *BKE_tracking_track_get_indexed(struct MovieTracking *tracking,
-                                                          int tracknr,
-                                                          struct ListBase **r_tracksbase);
-
-struct MovieTrackingTrack *BKE_tracking_track_get_active(struct MovieTracking *tracking);
+struct MovieTrackingTrack *BKE_tracking_track_get_for_selection_index(
+    struct MovieTracking *tracking, int selection_index, struct ListBase **r_tracksbase);
 
 float *BKE_tracking_track_get_mask(int frame_width,
                                    int frame_height,
-                                   struct MovieTrackingTrack *track,
-                                   struct MovieTrackingMarker *marker);
+                                   const struct MovieTrackingTrack *track,
+                                   const struct MovieTrackingMarker *marker);
 
 float BKE_tracking_track_get_weight_for_marker(struct MovieClip *clip,
                                                struct MovieTrackingTrack *track,
@@ -381,12 +361,6 @@ bool BKE_tracking_plane_track_has_marker_at_frame(struct MovieTrackingPlaneTrack
 bool BKE_tracking_plane_track_has_enabled_marker_at_frame(
     struct MovieTrackingPlaneTrack *plane_track, int framenr);
 
-struct MovieTrackingPlaneTrack *BKE_tracking_plane_track_get_named(
-    struct MovieTracking *tracking, struct MovieTrackingObject *object, const char *name);
-
-struct MovieTrackingPlaneTrack *BKE_tracking_plane_track_get_active(
-    struct MovieTracking *tracking);
-
 void BKE_tracking_plane_tracks_deselect_all(struct ListBase *plane_tracks_base);
 
 bool BKE_tracking_plane_track_has_point_track(struct MovieTrackingPlaneTrack *plane_track,
@@ -440,23 +414,26 @@ void BKE_tracking_plane_marker_get_subframe_corners(struct MovieTrackingPlaneTra
 struct MovieTrackingObject *BKE_tracking_object_add(struct MovieTracking *tracking,
                                                     const char *name);
 bool BKE_tracking_object_delete(struct MovieTracking *tracking,
-                                struct MovieTrackingObject *object);
+                                struct MovieTrackingObject *tracking_object);
 
 void BKE_tracking_object_unique_name(struct MovieTracking *tracking,
-                                     struct MovieTrackingObject *object);
+                                     struct MovieTrackingObject *tracking_object);
 
 struct MovieTrackingObject *BKE_tracking_object_get_named(struct MovieTracking *tracking,
                                                           const char *name);
 
-struct MovieTrackingObject *BKE_tracking_object_get_active(struct MovieTracking *tracking);
-struct MovieTrackingObject *BKE_tracking_object_get_camera(struct MovieTracking *tracking);
+struct MovieTrackingObject *BKE_tracking_object_get_active(const struct MovieTracking *tracking);
+struct MovieTrackingObject *BKE_tracking_object_get_camera(const struct MovieTracking *tracking);
 
-struct ListBase *BKE_tracking_object_get_tracks(struct MovieTracking *tracking,
-                                                struct MovieTrackingObject *object);
-struct ListBase *BKE_tracking_object_get_plane_tracks(struct MovieTracking *tracking,
-                                                      struct MovieTrackingObject *object);
-struct MovieTrackingReconstruction *BKE_tracking_object_get_reconstruction(
-    struct MovieTracking *tracking, struct MovieTrackingObject *object);
+/* Find point track with the given name in the tracking object.
+ * If such track does not exist NULL is returned. */
+struct MovieTrackingTrack *BKE_tracking_object_find_track_with_name(
+    struct MovieTrackingObject *tracking_object, const char *name);
+
+/* Find plane track with the given name in the tracking object.
+ * If such track does not exist NULL is returned. */
+struct MovieTrackingPlaneTrack *BKE_tracking_object_find_plane_track_with_name(
+    struct MovieTrackingObject *tracking_object, const char *name);
 
 /* --------------------------------------------------------------------
  * Camera.
@@ -474,12 +451,18 @@ void BKE_tracking_camera_to_blender(struct MovieTracking *tracking,
                                     int height);
 
 struct MovieReconstructedCamera *BKE_tracking_camera_get_reconstructed(
-    struct MovieTracking *tracking, struct MovieTrackingObject *object, int framenr);
+    struct MovieTracking *tracking, struct MovieTrackingObject *tracking_object, int framenr);
 void BKE_tracking_camera_get_reconstructed_interpolate(struct MovieTracking *tracking,
-                                                       struct MovieTrackingObject *object,
+                                                       struct MovieTrackingObject *tracking_object,
                                                        float framenr,
                                                        float mat[4][4]);
 
+/* Access the principal point in pixels space. */
+void BKE_tracking_camera_principal_point_pixel_get(struct MovieClip *clip,
+                                                   float r_principal_point_pixel[2]);
+void BKE_tracking_camera_principal_point_pixel_set(struct MovieClip *clip,
+                                                   const float principal_point_pixel[2]);
+
 /* --------------------------------------------------------------------
  * (Un)distortion.
  */
@@ -543,22 +526,22 @@ void BKE_tracking_max_distortion_delta_across_bound(struct MovieTracking *tracki
 
 struct ImBuf *BKE_tracking_sample_pattern(int frame_width,
                                           int frame_height,
-                                          struct ImBuf *search_ib,
-                                          struct MovieTrackingTrack *track,
-                                          struct MovieTrackingMarker *marker,
+                                          const struct ImBuf *search_ib,
+                                          const struct MovieTrackingTrack *track,
+                                          const struct MovieTrackingMarker *marker,
                                           bool from_anchor,
                                           bool use_mask,
                                           int num_samples_x,
                                           int num_samples_y,
                                           float pos[2]);
-struct ImBuf *BKE_tracking_get_pattern_imbuf(struct ImBuf *ibuf,
-                                             struct MovieTrackingTrack *track,
-                                             struct MovieTrackingMarker *marker,
+struct ImBuf *BKE_tracking_get_pattern_imbuf(const struct ImBuf *ibuf,
+                                             const struct MovieTrackingTrack *track,
+                                             const struct MovieTrackingMarker *marker,
                                              bool anchored,
                                              bool disable_channels);
-struct ImBuf *BKE_tracking_get_search_imbuf(struct ImBuf *ibuf,
-                                            struct MovieTrackingTrack *track,
-                                            struct MovieTrackingMarker *marker,
+struct ImBuf *BKE_tracking_get_search_imbuf(const struct ImBuf *ibuf,
+                                            const struct MovieTrackingTrack *track,
+                                            const struct MovieTrackingMarker *marker,
                                             bool anchored,
                                             bool disable_channels);
 
@@ -626,7 +609,7 @@ void BKE_tracking_homography_between_two_quads(/*const*/ float reference_corners
  * Perform early check on whether everything is fine to start reconstruction.
  */
 bool BKE_tracking_reconstruction_check(struct MovieTracking *tracking,
-                                       struct MovieTrackingObject *object,
+                                       struct MovieTrackingObject *tracking_object,
                                        char *error_msg,
                                        int error_size);
 
@@ -637,7 +620,7 @@ bool BKE_tracking_reconstruction_check(struct MovieTracking *tracking,
  */
 struct MovieReconstructContext *BKE_tracking_reconstruction_context_new(
     struct MovieClip *clip,
-    struct MovieTrackingObject *object,
+    struct MovieTrackingObject *tracking_object,
     int keyframe1,
     int keyframe2,
     int width,
@@ -657,8 +640,8 @@ void BKE_tracking_reconstruction_context_free(struct MovieReconstructContext *co
  * callback in libmv side and passing to an interface.
  */
 void BKE_tracking_reconstruction_solve(struct MovieReconstructContext *context,
-                                       short *stop,
-                                       short *do_update,
+                                       bool *stop,
+                                       bool *do_update,
                                        float *progress,
                                        char *stats_message,
                                        int message_size);
@@ -787,25 +770,17 @@ void BKE_tracking_dopesheet_update(struct MovieTracking *tracking);
  */
 struct MovieTrackingObject *BKE_tracking_find_object_for_track(
     const struct MovieTracking *tracking, const struct MovieTrackingTrack *track);
-struct ListBase *BKE_tracking_find_tracks_list_for_track(struct MovieTracking *tracking,
-                                                         const struct MovieTrackingTrack *track);
-
-/**
- * \note Returns NULL if the track comes from camera object,.
- */
 struct MovieTrackingObject *BKE_tracking_find_object_for_plane_track(
     const struct MovieTracking *tracking, const struct MovieTrackingPlaneTrack *plane_track);
-struct ListBase *BKE_tracking_find_tracks_list_for_plane_track(
-    struct MovieTracking *tracking, const struct MovieTrackingPlaneTrack *plane_track);
 
 void BKE_tracking_get_rna_path_for_track(const struct MovieTracking *tracking,
                                          const struct MovieTrackingTrack *track,
                                          char *rna_path,
-                                         size_t rna_path_len);
+                                         size_t rna_path_maxncpy);
 void BKE_tracking_get_rna_path_prefix_for_track(const struct MovieTracking *tracking,
                                                 const struct MovieTrackingTrack *track,
                                                 char *rna_path,
-                                                size_t rna_path_len);
+                                                size_t rna_path_maxncpy);
 void BKE_tracking_get_rna_path_for_plane_track(const struct MovieTracking *tracking,
                                                const struct MovieTrackingPlaneTrack *plane_track,
                                                char *rna_path,
@@ -814,7 +789,7 @@ void BKE_tracking_get_rna_path_prefix_for_plane_track(
     const struct MovieTracking *tracking,
     const struct MovieTrackingPlaneTrack *plane_track,
     char *rna_path,
-    size_t rna_path_len);
+    size_t rna_path_maxncpy);
 
 /* --------------------------------------------------------------------
  * Utility macros.
@@ -837,10 +812,6 @@ void BKE_tracking_get_rna_path_prefix_for_plane_track(
 #define PLANE_TRACK_VIEW_SELECTED(plane_track) \
   ((((plane_track)->flag & PLANE_TRACK_HIDDEN) == 0) && ((plane_track)->flag & SELECT))
 
-#define MARKER_VISIBLE(sc, track, marker) \
-  (((marker)->flag & MARKER_DISABLED) == 0 || ((sc)->flag & SC_HIDE_DISABLED) == 0 || \
-   ((sc)->clip->tracking.act_track == track))
-
 #ifdef __cplusplus
 }
 #endif
