@@ -28,20 +28,37 @@ typedef struct EditFontSelBox {
   float rot;
 } EditFontSelBox;
 
+/**
+ * Edit data for #Curve (a text curve, with an #Object::type of `OB_FONT`).
+ * */
 typedef struct EditFont {
+  /** Array of UTF32 code-points. */
   char32_t *textbuf;
+  /** Text style info (aligned with `textbuf`). */
   struct CharInfo *textbufinfo;
 
-  /* array of rectangles & rotation */
+  /** Array of rectangles & rotation. */
   float textcurs[4][2];
   EditFontSelBox *selboxes;
   int selboxes_len;
 
-  /* positional vars relative to the textbuf, textbufinfo (not utf8 bytes)
-   * a copy of these is kept in Curve, but use these in editmode */
-  int len, pos;
+  /* Positional vars relative to the `textbuf` (not utf8 bytes)
+   * a copy of these is kept in Curve, but use these in edit-mode. */
+
+  /** Length of `textbuf`. */
+  int len;
+  /** Cursor position of (aligned with `textbuf`). */
+  int pos;
+  /** Text selection start/end, see #BKE_vfont_select_get. */
   int selstart, selend;
 
+  /**
+   * Combined styles from #CharInfo.flag for the selected range selected
+   * (only including values from #CU_CHINFO_STYLE_ALL).
+   * A flag will be set only if ALL characters in the selected string have it.
+   */
+  int select_char_info_flag;
+
   /**
    * ID data is older than edit-mode data.
    * Set #Main.is_memfile_undo_flush_needed when enabling.
@@ -54,6 +71,10 @@ bool BKE_vfont_is_builtin(const struct VFont *vfont);
 void BKE_vfont_builtin_register(const void *mem, int size);
 
 void BKE_vfont_free_data(struct VFont *vfont);
+/**
+ * Return the built-in #VFont, without adding a user (the user-count may be zero).
+ * The caller is responsible for adding a user.
+ */
 struct VFont *BKE_vfont_builtin_get(void);
 struct VFont *BKE_vfont_load(struct Main *bmain, const char *filepath);
 struct VFont *BKE_vfont_load_exists_ex(struct Main *bmain, const char *filepath, bool *r_exists);
@@ -68,6 +89,9 @@ bool BKE_vfont_to_curve_ex(struct Object *ob,
                            bool *r_text_free,
                            struct CharTrans **r_chartransdata);
 bool BKE_vfont_to_curve_nubase(struct Object *ob, int mode, struct ListBase *r_nubase);
+
+int BKE_vfont_cursor_to_text_index(struct Object *ob, float cursor_location[2]);
+
 /**
  * \warning Expects to have access to evaluated data (i.e. passed object should be evaluated one).
  */
