@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2015 Blender Foundation. All rights reserved. */
+ * Copyright 2015 Blender Foundation */
 
 /** \file
  * \ingroup bke
@@ -594,7 +594,8 @@ static void splineik_evaluate_bone(
   if (ik_data->xzScaleMode != CONSTRAINT_SPLINEIK_XZS_NONE) {
     /* First, apply the original scale if enabled. */
     if (ik_data->xzScaleMode == CONSTRAINT_SPLINEIK_XZS_ORIGINAL ||
-        (ik_data->flag & CONSTRAINT_SPLINEIK_USE_ORIGINAL_SCALE) != 0) {
+        (ik_data->flag & CONSTRAINT_SPLINEIK_USE_ORIGINAL_SCALE) != 0)
+    {
       float scale;
 
       /* X-axis scale. */
@@ -925,7 +926,7 @@ static void pose_channel_flush_to_orig_if_needed(struct Depsgraph *depsgraph,
     return;
   }
   bPoseChannel *pchan_orig = pchan->orig_pchan;
-  /* TODO(sergey): Using BKE_pose_copy_pchan_result() introduces T70901, but why? */
+  /* TODO(sergey): Using BKE_pose_copy_pchan_result() introduces #70901, but why? */
   copy_m4_m4(pchan_orig->pose_mat, pchan->pose_mat);
   copy_m4_m4(pchan_orig->chan_mat, pchan->chan_mat);
   copy_v3_v3(pchan_orig->pose_head, pchan->pose_mat[3]);
