@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2006 Blender Foundation. All rights reserved. */
+ * Copyright 2006 Blender Foundation */
 
 /** \file
  * \ingroup bke
@@ -31,6 +31,7 @@
 #include "BKE_curves.hh"
 #include "BKE_customdata.h"
 #include "BKE_editmesh.h"
+#include "BKE_mesh.hh"
 #include "BKE_pointcloud.h"
 #include "BKE_report.h"
 
@@ -111,7 +112,7 @@ static std::optional<blender::bke::MutableAttributeAccessor> get_attribute_acces
     }
     case ID_CV: {
       Curves &curves_id = reinterpret_cast<Curves &>(id);
-      CurvesGeometry &curves = CurvesGeometry::wrap(curves_id.geometry);
+      CurvesGeometry &curves = curves_id.geometry.wrap();
       return curves.attributes_for_write();
     }
     default: {
@@ -140,11 +141,25 @@ bool BKE_attribute_allow_procedural_access(const char *attribute_name)
   return blender::bke::allow_procedural_attribute_access(attribute_name);
 }
 
+static bool bke_id_attribute_rename_if_exists(ID *id,
+                                              const char *old_name,
+                                              const char *new_name,
+                                              ReportList *reports)
+{
+  CustomDataLayer *layer = BKE_id_attribute_search(
+      id, old_name, CD_MASK_PROP_ALL, ATTR_DOMAIN_MASK_ALL);
+  if (layer == nullptr) {
+    return false;
+  }
+  return BKE_id_attribute_rename(id, old_name, new_name, reports);
+}
+
 bool BKE_id_attribute_rename(ID *id,
                              const char *old_name,
                              const char *new_name,
                              ReportList *reports)
 {
+  using namespace blender;
   if (BKE_id_attribute_required(id, old_name)) {
     BLI_assert_msg(0, "Required attribute name is not editable");
     return false;
@@ -153,8 +168,18 @@ bool BKE_id_attribute_rename(ID *id,
     BKE_report(reports, RPT_ERROR, "Attribute name can not be empty");
     return false;
   }
-  if (STREQ(old_name, new_name)) {
-    return false;
+
+  /* NOTE: Checking if the new name matches the old name only makes sense when the name
+   * is clamped to it's maximum length, otherwise assigning an over-long name multiple times
+   * will add `.001` suffix unnecessarily. */
+  {
+    const int new_name_maxncpy = CustomData_name_max_length_calc(new_name);
+    /* NOTE: A function that performs a clamped comparison without copying would be handy here. */
+    char new_name_clamped[MAX_CUSTOMDATA_LAYER_NAME];
+    BLI_strncpy_utf8(new_name_clamped, new_name, new_name_maxncpy);
+    if (STREQ(old_name, new_name_clamped)) {
+      return false;
+    }
   }
 
   CustomDataLayer *layer = BKE_id_attribute_search(
@@ -166,7 +191,33 @@ bool BKE_id_attribute_rename(ID *id,
 
   char result_name[MAX_CUSTOMDATA_LAYER_NAME];
   BKE_id_attribute_calc_unique_name(id, new_name, result_name);
-  BLI_strncpy_utf8(layer->name, result_name, sizeof(layer->name));
+
+  if (layer->type == CD_PROP_FLOAT2 && GS(id->name) == ID_ME) {
+    /* Rename UV sub-attributes. */
+    char buffer_src[MAX_CUSTOMDATA_LAYER_NAME];
+    char buffer_dst[MAX_CUSTOMDATA_LAYER_NAME];
+
+    bke_id_attribute_rename_if_exists(id,
+                                      BKE_uv_map_vert_select_name_get(layer->name, buffer_src),
+                                      BKE_uv_map_vert_select_name_get(result_name, buffer_dst),
+                                      reports);
+    bke_id_attribute_rename_if_exists(id,
+                                      BKE_uv_map_edge_select_name_get(layer->name, buffer_src),
+                                      BKE_uv_map_edge_select_name_get(result_name, buffer_dst),
+                                      reports);
+    bke_id_attribute_rename_if_exists(id,
+                                      BKE_uv_map_pin_name_get(layer->name, buffer_src),
+                                      BKE_uv_map_pin_name_get(result_name, buffer_dst),
+                                      reports);
+  }
+  if (StringRef(old_name) == BKE_id_attributes_active_color_name(id)) {
+    BKE_id_attributes_active_color_set(id, result_name);
+  }
+  if (StringRef(old_name) == BKE_id_attributes_default_color_name(id)) {
+    BKE_id_attributes_default_color_set(id, result_name);
+  }
+
+  STRNCPY_UTF8(layer->name, result_name);
 
   return true;
 }
@@ -203,22 +254,21 @@ static bool unique_name_cb(void *arg, const char *name)
 bool BKE_id_attribute_calc_unique_name(ID *id, const char *name, char *outname)
 {
   AttrUniqueData data{id};
+  const int name_maxncpy = CustomData_name_max_length_calc(name);
 
   /* Set default name if none specified.
    * NOTE: We only call IFACE_() if needed to avoid locale lookup overhead. */
-  if (!name || name[0] == '\0') {
-    BLI_strncpy(outname, IFACE_("Attribute"), MAX_CUSTOMDATA_LAYER_NAME);
-  }
-  else {
-    BLI_strncpy_utf8(outname, name, MAX_CUSTOMDATA_LAYER_NAME);
-  }
+  BLI_strncpy_utf8(outname, (name && name[0]) ? name : IFACE_("Attribute"), name_maxncpy);
 
-  return BLI_uniquename_cb(
-      unique_name_cb, &data, nullptr, '.', outname, MAX_CUSTOMDATA_LAYER_NAME);
+  const char *defname = ""; /* Dummy argument, never used as `name` is never zero length. */
+  return BLI_uniquename_cb(unique_name_cb, &data, defname, '.', outname, name_maxncpy);
 }
 
-CustomDataLayer *BKE_id_attribute_new(
-    ID *id, const char *name, const int type, const eAttrDomain domain, ReportList *reports)
+CustomDataLayer *BKE_id_attribute_new(ID *id,
+                                      const char *name,
+                                      const eCustomDataType type,
+                                      const eAttrDomain domain,
+                                      ReportList *reports)
 {
   using namespace blender::bke;
   DomainInfo info[ATTR_DOMAIN_NUM];
@@ -253,6 +303,24 @@ CustomDataLayer *BKE_id_attribute_new(
   return (index == -1) ? nullptr : &(customdata->layers[index]);
 }
 
+static void bke_id_attribute_copy_if_exists(ID *id, const char *srcname, const char *dstname)
+{
+  using namespace blender::bke;
+
+  std::optional<MutableAttributeAccessor> attributes = get_attribute_accessor_for_write(*id);
+  if (!attributes) {
+    return;
+  }
+
+  GAttributeReader src = attributes->lookup(srcname);
+  if (!src) {
+    return;
+  }
+
+  const eCustomDataType type = cpp_type_to_custom_data_type(src.varray.type());
+  attributes->add(dstname, src.domain, type, AttributeInitVArray(src.varray));
+}
+
 CustomDataLayer *BKE_id_attribute_duplicate(ID *id, const char *name, ReportList *reports)
 {
   using namespace blender::bke;
@@ -282,11 +350,48 @@ CustomDataLayer *BKE_id_attribute_duplicate(ID *id, const char *name, ReportList
   const eCustomDataType type = cpp_type_to_custom_data_type(src.varray.type());
   attributes->add(uniquename, src.domain, type, AttributeInitVArray(src.varray));
 
+  if (GS(id->name) == ID_ME && type == CD_PROP_FLOAT2) {
+    /* Duplicate UV sub-attributes. */
+    char buffer_src[MAX_CUSTOMDATA_LAYER_NAME];
+    char buffer_dst[MAX_CUSTOMDATA_LAYER_NAME];
+
+    bke_id_attribute_copy_if_exists(id,
+                                    BKE_uv_map_vert_select_name_get(name, buffer_src),
+                                    BKE_uv_map_vert_select_name_get(uniquename, buffer_dst));
+    bke_id_attribute_copy_if_exists(id,
+                                    BKE_uv_map_edge_select_name_get(name, buffer_src),
+                                    BKE_uv_map_edge_select_name_get(uniquename, buffer_dst));
+    bke_id_attribute_copy_if_exists(id,
+                                    BKE_uv_map_pin_name_get(name, buffer_src),
+                                    BKE_uv_map_pin_name_get(uniquename, buffer_dst));
+  }
+
   return BKE_id_attribute_search(id, uniquename, CD_MASK_PROP_ALL, ATTR_DOMAIN_MASK_ALL);
 }
 
+static int color_name_to_index(ID *id, const char *name)
+{
+  const CustomDataLayer *layer = BKE_id_attribute_search(
+      id, name, CD_MASK_COLOR_ALL, ATTR_DOMAIN_MASK_COLOR);
+  return BKE_id_attribute_to_index(id, layer, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
+}
+
+static int color_clamp_index(ID *id, int index)
+{
+  const int length = BKE_id_attributes_length(id, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
+  return min_ii(index, length - 1);
+}
+
+static const char *color_name_from_index(ID *id, int index)
+{
+  const CustomDataLayer *layer = BKE_id_attribute_from_index(
+      id, index, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
+  return layer ? layer->name : nullptr;
+}
+
 bool BKE_id_attribute_remove(ID *id, const char *name, ReportList *reports)
 {
+  using namespace blender;
   using namespace blender::bke;
   if (!name || name[0] == '\0') {
     BKE_report(reports, RPT_ERROR, "The attribute name must not be empty");
@@ -305,9 +410,43 @@ bool BKE_id_attribute_remove(ID *id, const char *name, ReportList *reports)
     if (BMEditMesh *em = mesh->edit_mesh) {
       for (const int domain : IndexRange(ATTR_DOMAIN_NUM)) {
         if (CustomData *data = info[domain].customdata) {
-          if (BM_data_layer_free_named(em->bm, data, name)) {
-            return true;
+          const std::string name_copy = name;
+          const int layer_index = CustomData_get_named_layer_index_notype(data, name_copy.c_str());
+          if (layer_index == -1) {
+            continue;
           }
+
+          const eCustomDataType type = eCustomDataType(data->layers[layer_index].type);
+          const bool is_active_color_attribute = name_copy.c_str() ==
+                                                 StringRef(mesh->active_color_attribute);
+          const bool is_default_color_attribute = name_copy.c_str() ==
+                                                  StringRef(mesh->default_color_attribute);
+          const int active_color_index = color_name_to_index(id, mesh->active_color_attribute);
+          const int default_color_index = color_name_to_index(id, mesh->default_color_attribute);
+
+          if (!BM_data_layer_free_named(em->bm, data, name_copy.c_str())) {
+            BLI_assert_unreachable();
+          }
+
+          if (is_active_color_attribute) {
+            BKE_id_attributes_active_color_set(
+                id, color_name_from_index(id, color_clamp_index(id, active_color_index)));
+          }
+          if (is_default_color_attribute) {
+            BKE_id_attributes_default_color_set(
+                id, color_name_from_index(id, color_clamp_index(id, default_color_index)));
+          }
+
+          if (type == CD_PROP_FLOAT2 && domain == ATTR_DOMAIN_CORNER) {
+            char buffer[MAX_CUSTOMDATA_LAYER_NAME];
+            BM_data_layer_free_named(
+                em->bm, data, BKE_uv_map_vert_select_name_get(name_copy.c_str(), buffer));
+            BM_data_layer_free_named(
+                em->bm, data, BKE_uv_map_edge_select_name_get(name_copy.c_str(), buffer));
+            BM_data_layer_free_named(
+                em->bm, data, BKE_uv_map_pin_name_get(name_copy.c_str(), buffer));
+          }
+          return true;
         }
       }
       return false;
@@ -319,14 +458,53 @@ bool BKE_id_attribute_remove(ID *id, const char *name, ReportList *reports)
     return false;
   }
 
+  if (GS(id->name) == ID_ME) {
+    const std::string name_copy = name;
+    std::optional<blender::bke::AttributeMetaData> metadata = attributes->lookup_meta_data(
+        name_copy);
+    if (!metadata) {
+      return false;
+    }
+    /* Update active and default color attributes. */
+    Mesh *mesh = reinterpret_cast<Mesh *>(id);
+    const bool is_active_color_attribute = name_copy == StringRef(mesh->active_color_attribute);
+    const bool is_default_color_attribute = name_copy == StringRef(mesh->default_color_attribute);
+    const int active_color_index = color_name_to_index(id, mesh->active_color_attribute);
+    const int default_color_index = color_name_to_index(id, mesh->default_color_attribute);
+
+    if (!attributes->remove(name_copy)) {
+      BLI_assert_unreachable();
+    }
+
+    if (is_active_color_attribute) {
+      BKE_id_attributes_active_color_set(
+          id, color_name_from_index(id, color_clamp_index(id, active_color_index)));
+    }
+    if (is_default_color_attribute) {
+      BKE_id_attributes_default_color_set(
+          id, color_name_from_index(id, color_clamp_index(id, default_color_index)));
+    }
+
+    if (metadata->data_type == CD_PROP_FLOAT2 && metadata->domain == ATTR_DOMAIN_CORNER) {
+      char buffer[MAX_CUSTOMDATA_LAYER_NAME];
+      attributes->remove(BKE_uv_map_vert_select_name_get(name_copy.c_str(), buffer));
+      attributes->remove(BKE_uv_map_edge_select_name_get(name_copy.c_str(), buffer));
+      attributes->remove(BKE_uv_map_pin_name_get(name_copy.c_str(), buffer));
+    }
+    return true;
+  }
+
   return attributes->remove(name);
 }
 
 CustomDataLayer *BKE_id_attribute_find(const ID *id,
                                        const char *name,
-                                       const int type,
+                                       const eCustomDataType type,
                                        const eAttrDomain domain)
 {
+  if (!name) {
+    return nullptr;
+  }
   DomainInfo info[ATTR_DOMAIN_NUM];
   get_domains(id, info);
 
@@ -350,11 +528,15 @@ CustomDataLayer *BKE_id_attribute_search(ID *id,
                                          const eCustomDataMask type_mask,
                                          const eAttrDomainMask domain_mask)
 {
+  if (!name) {
+    return nullptr;
+  }
   DomainInfo info[ATTR_DOMAIN_NUM];
   get_domains(id, info);
 
   for (eAttrDomain domain = ATTR_DOMAIN_POINT; domain < ATTR_DOMAIN_NUM;
-       domain = static_cast<eAttrDomain>(int(domain) + 1)) {
+       domain = static_cast<eAttrDomain>(int(domain) + 1))
+  {
     if (!(domain_mask & ATTR_DOMAIN_AS_MASK(domain))) {
       continue;
     }
@@ -456,6 +638,8 @@ bool BKE_id_attribute_required(const ID *id, const char *name)
       return BKE_pointcloud_attribute_required((const PointCloud *)id, name);
     case ID_CV:
       return BKE_curves_attribute_required((const Curves *)id, name);
+    case ID_ME:
+      return BKE_mesh_attribute_required(name);
     default:
       return false;
   }
@@ -495,29 +679,14 @@ CustomDataLayer *BKE_id_attributes_active_get(ID *id)
   return nullptr;
 }
 
-void BKE_id_attributes_active_set(ID *id, CustomDataLayer *active_layer)
+void BKE_id_attributes_active_set(ID *id, const char *name)
 {
-  DomainInfo info[ATTR_DOMAIN_NUM];
-  get_domains(id, info);
+  const CustomDataLayer *layer = BKE_id_attribute_search(
+      id, name, CD_MASK_PROP_ALL, ATTR_DOMAIN_MASK_ALL);
+  BLI_assert(layer != nullptr);
 
-  int index = 0;
-
-  for (const int domain : IndexRange(ATTR_DOMAIN_NUM)) {
-    const CustomData *customdata = info[domain].customdata;
-    if (customdata == nullptr) {
-      continue;
-    }
-    for (int i = 0; i < customdata->totlayer; i++) {
-      const CustomDataLayer *layer = &customdata->layers[i];
-      if (layer == active_layer) {
-        *BKE_id_attributes_active_index_p(id) = index;
-        return;
-      }
-      if (CD_MASK_PROP_ALL & CD_TYPE_AS_MASK(layer->type)) {
-        index++;
-      }
-    }
-  }
+  const int index = BKE_id_attribute_to_index(id, layer, ATTR_DOMAIN_MASK_ALL, CD_MASK_PROP_ALL);
+  *BKE_id_attributes_active_index_p(id) = index;
 }
 
 int *BKE_id_attributes_active_index_p(ID *id)
@@ -580,7 +749,8 @@ CustomDataLayer *BKE_id_attribute_from_index(ID *id,
 
     for (int i = 0; i < customdata->totlayer; i++) {
       if (!(layer_mask & CD_TYPE_AS_MASK(customdata->layers[i].type)) ||
-          (customdata->layers[i].flag & CD_FLAG_TEMPORARY)) {
+          (customdata->layers[i].flag & CD_FLAG_TEMPORARY))
+      {
         continue;
       }
 
@@ -605,7 +775,7 @@ static void get_domains_types(eAttrDomain domains[ATTR_DOMAIN_NUM])
   }
 
   /* Swap corner and face. */
-  SWAP(eAttrDomain, domains[ATTR_DOMAIN_FACE], domains[ATTR_DOMAIN_CORNER]);
+  std::swap(domains[ATTR_DOMAIN_FACE], domains[ATTR_DOMAIN_CORNER]);
 }
 
 int BKE_id_attribute_to_index(const ID *id,
@@ -648,164 +818,96 @@ int BKE_id_attribute_to_index(const ID *id,
   return -1;
 }
 
-CustomDataLayer *BKE_id_attribute_subset_active_get(const ID *id,
-                                                    int active_flag,
-                                                    eAttrDomainMask domain_mask,
-                                                    eCustomDataMask mask)
+const char *BKE_id_attributes_active_color_name(const ID *id)
 {
-  DomainInfo info[ATTR_DOMAIN_NUM];
-  eAttrDomain domains[ATTR_DOMAIN_NUM];
-
-  get_domains_types(domains);
-  get_domains(id, info);
-
-  CustomDataLayer *candidate = nullptr;
-  for (int i = 0; i < ARRAY_SIZE(domains); i++) {
-    if (!((1 << domains[i]) & domain_mask) || !info[domains[i]].customdata) {
-      continue;
-    }
-
-    CustomData *cdata = info[domains[i]].customdata;
-
-    for (int j = 0; j < cdata->totlayer; j++) {
-      CustomDataLayer *layer = cdata->layers + j;
-
-      if (!(CD_TYPE_AS_MASK(layer->type) & mask) || (layer->flag & CD_FLAG_TEMPORARY)) {
-        continue;
-      }
-
-      if (layer->flag & active_flag) {
-        return layer;
-      }
-
-      candidate = layer;
-    }
+  if (GS(id->name) == ID_ME) {
+    return reinterpret_cast<const Mesh *>(id)->active_color_attribute;
   }
-
-  return candidate;
+  return nullptr;
 }
 
-void BKE_id_attribute_subset_active_set(ID *id,
-                                        CustomDataLayer *layer,
-                                        int active_flag,
-                                        eAttrDomainMask domain_mask,
-                                        eCustomDataMask mask)
+const char *BKE_id_attributes_default_color_name(const ID *id)
 {
-  DomainInfo info[ATTR_DOMAIN_NUM];
-  eAttrDomain domains[ATTR_DOMAIN_NUM];
-
-  get_domains_types(domains);
-  get_domains(id, info);
-
-  for (int i = 0; i < ATTR_DOMAIN_NUM; i++) {
-    eAttrDomainMask domain_mask2 = (eAttrDomainMask)(1 << domains[i]);
-
-    if (!(domain_mask2 & domain_mask) || !info[domains[i]].customdata) {
-      continue;
-    }
-
-    CustomData *cdata = info[domains[i]].customdata;
-
-    for (int j = 0; j < cdata->totlayer; j++) {
-      CustomDataLayer *layer_iter = cdata->layers + j;
-
-      if (!(CD_TYPE_AS_MASK(layer_iter->type) & mask) || (layer_iter->flag & CD_FLAG_TEMPORARY)) {
-        continue;
-      }
-
-      layer_iter->flag &= ~active_flag;
-    }
+  if (GS(id->name) == ID_ME) {
+    return reinterpret_cast<const Mesh *>(id)->default_color_attribute;
   }
-
-  layer->flag |= active_flag;
+  return nullptr;
 }
 
-CustomDataLayer *BKE_id_attributes_active_color_get(const ID *id)
+void BKE_id_attributes_active_color_set(ID *id, const char *name)
 {
-  return BKE_id_attribute_subset_active_get(
-      id, CD_FLAG_COLOR_ACTIVE, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
-}
-
-void BKE_id_attributes_active_color_set(ID *id, CustomDataLayer *active_layer)
-{
-  BKE_id_attribute_subset_active_set(
-      id, active_layer, CD_FLAG_COLOR_ACTIVE, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
-}
-
-CustomDataLayer *BKE_id_attributes_render_color_get(const ID *id)
-{
-  return BKE_id_attribute_subset_active_get(
-      id, CD_FLAG_COLOR_RENDER, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
-}
-
-void BKE_id_attributes_render_color_set(ID *id, CustomDataLayer *active_layer)
-{
-  BKE_id_attribute_subset_active_set(
-      id, active_layer, CD_FLAG_COLOR_RENDER, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
-}
-
-CustomDataLayer *BKE_id_attributes_color_find(const ID *id, const char *name)
-{
-  CustomDataLayer *layer = BKE_id_attribute_find(id, name, CD_PROP_COLOR, ATTR_DOMAIN_POINT);
-  if (layer == nullptr) {
-    layer = BKE_id_attribute_find(id, name, CD_PROP_COLOR, ATTR_DOMAIN_CORNER);
-  }
-  if (layer == nullptr) {
-    layer = BKE_id_attribute_find(id, name, CD_PROP_BYTE_COLOR, ATTR_DOMAIN_POINT);
-  }
-  if (layer == nullptr) {
-    layer = BKE_id_attribute_find(id, name, CD_PROP_BYTE_COLOR, ATTR_DOMAIN_CORNER);
-  }
-
-  return layer;
-}
-
-void BKE_id_attribute_copy_domains_temp(short id_type,
-                                        const CustomData *vdata,
-                                        const CustomData *edata,
-                                        const CustomData *ldata,
-                                        const CustomData *pdata,
-                                        const CustomData *cdata,
-                                        ID *r_id)
-{
-  CustomData reset;
-
-  CustomData_reset(&reset);
-
-  switch (id_type) {
+  switch (GS(id->name)) {
     case ID_ME: {
-      Mesh *me = (Mesh *)r_id;
-      memset((void *)me, 0, sizeof(*me));
-
-      me->edit_mesh = nullptr;
-
-      me->vdata = vdata ? *vdata : reset;
-      me->edata = edata ? *edata : reset;
-      me->ldata = ldata ? *ldata : reset;
-      me->pdata = pdata ? *pdata : reset;
-
-      break;
-    }
-    case ID_PT: {
-      PointCloud *pointcloud = (PointCloud *)r_id;
-
-      memset((void *)pointcloud, 0, sizeof(*pointcloud));
-
-      pointcloud->pdata = vdata ? *vdata : reset;
-      break;
-    }
-    case ID_CV: {
-      Curves *curves = (Curves *)r_id;
-
-      memset((void *)curves, 0, sizeof(*curves));
-
-      curves->geometry.point_data = vdata ? *vdata : reset;
-      curves->geometry.curve_data = cdata ? *cdata : reset;
+      Mesh *mesh = reinterpret_cast<Mesh *>(id);
+      MEM_SAFE_FREE(mesh->active_color_attribute);
+      if (name) {
+        mesh->active_color_attribute = BLI_strdup(name);
+      }
       break;
     }
     default:
       break;
   }
+}
 
-  *((short *)r_id->name) = id_type;
+void BKE_id_attributes_default_color_set(ID *id, const char *name)
+{
+  switch (GS(id->name)) {
+    case ID_ME: {
+      Mesh *mesh = reinterpret_cast<Mesh *>(id);
+      MEM_SAFE_FREE(mesh->default_color_attribute);
+      if (name) {
+        mesh->default_color_attribute = BLI_strdup(name);
+      }
+      break;
+    }
+    default:
+      break;
+  }
+}
+
+CustomDataLayer *BKE_id_attributes_color_find(const ID *id, const char *name)
+{
+  if (CustomDataLayer *layer = BKE_id_attribute_find(id, name, CD_PROP_COLOR, ATTR_DOMAIN_POINT)) {
+    return layer;
+  }
+  if (CustomDataLayer *layer = BKE_id_attribute_find(id, name, CD_PROP_COLOR, ATTR_DOMAIN_CORNER))
+  {
+    return layer;
+  }
+  if (CustomDataLayer *layer = BKE_id_attribute_find(
+          id, name, CD_PROP_BYTE_COLOR, ATTR_DOMAIN_POINT))
+  {
+    return layer;
+  }
+  if (CustomDataLayer *layer = BKE_id_attribute_find(
+          id, name, CD_PROP_BYTE_COLOR, ATTR_DOMAIN_CORNER))
+  {
+    return layer;
+  }
+  return nullptr;
+}
+
+const char *BKE_uv_map_vert_select_name_get(const char *uv_map_name, char *buffer)
+{
+  BLI_assert(strlen(UV_VERTSEL_NAME) == 2);
+  BLI_assert(strlen(uv_map_name) < MAX_CUSTOMDATA_LAYER_NAME - 4);
+  BLI_snprintf(buffer, MAX_CUSTOMDATA_LAYER_NAME, ".%s.%s", UV_VERTSEL_NAME, uv_map_name);
+  return buffer;
+}
+
+const char *BKE_uv_map_edge_select_name_get(const char *uv_map_name, char *buffer)
+{
+  BLI_assert(strlen(UV_EDGESEL_NAME) == 2);
+  BLI_assert(strlen(uv_map_name) < MAX_CUSTOMDATA_LAYER_NAME - 4);
+  BLI_snprintf(buffer, MAX_CUSTOMDATA_LAYER_NAME, ".%s.%s", UV_EDGESEL_NAME, uv_map_name);
+  return buffer;
+}
+
+const char *BKE_uv_map_pin_name_get(const char *uv_map_name, char *buffer)
+{
+  BLI_assert(strlen(UV_PINNED_NAME) == 2);
+  BLI_assert(strlen(uv_map_name) < MAX_CUSTOMDATA_LAYER_NAME - 4);
+  BLI_snprintf(buffer, MAX_CUSTOMDATA_LAYER_NAME, ".%s.%s", UV_PINNED_NAME, uv_map_name);
+  return buffer;
 }
