@@ -1,8 +1,10 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
+#include "BLI_array_utils.hh"
+
 #include "BKE_attribute_math.hh"
 
-namespace blender::attribute_math {
+namespace blender::bke::attribute_math {
 
 ColorGeometry4fMixer::ColorGeometry4fMixer(MutableSpan<ColorGeometry4f> buffer,
                                            ColorGeometry4f default_color)
@@ -23,7 +25,6 @@ void ColorGeometry4fMixer::set(const int64_t index,
                                const ColorGeometry4f &color,
                                const float weight)
 {
-  BLI_assert(weight >= 0.0f);
   buffer_[index].r = color.r * weight;
   buffer_[index].g = color.g * weight;
   buffer_[index].b = color.b * weight;
@@ -35,7 +36,6 @@ void ColorGeometry4fMixer::mix_in(const int64_t index,
                                   const ColorGeometry4f &color,
                                   const float weight)
 {
-  BLI_assert(weight >= 0.0f);
   ColorGeometry4f &output_color = buffer_[index];
   output_color.r += color.r * weight;
   output_color.g += color.g * weight;
@@ -89,7 +89,6 @@ void ColorGeometry4bMixer::ColorGeometry4bMixer::set(int64_t index,
                                                      const ColorGeometry4b &color,
                                                      const float weight)
 {
-  BLI_assert(weight >= 0.0f);
   accumulation_buffer_[index][0] = color.r * weight;
   accumulation_buffer_[index][1] = color.g * weight;
   accumulation_buffer_[index][2] = color.b * weight;
@@ -99,7 +98,6 @@ void ColorGeometry4bMixer::ColorGeometry4bMixer::set(int64_t index,
 
 void ColorGeometry4bMixer::mix_in(int64_t index, const ColorGeometry4b &color, float weight)
 {
-  BLI_assert(weight >= 0.0f);
   float4 &accum_value = accumulation_buffer_[index];
   accum_value[0] += color.r * weight;
   accum_value[1] += color.g * weight;
@@ -132,4 +130,20 @@ void ColorGeometry4bMixer::finalize(const IndexMask mask)
   });
 }
 
-}  // namespace blender::attribute_math
+void gather(const GSpan src, const Span<int> map, GMutableSpan dst)
+{
+  attribute_math::convert_to_static_type(src.type(), [&](auto dummy) {
+    using T = decltype(dummy);
+    array_utils::gather(src.typed<T>(), map, dst.typed<T>());
+  });
+}
+
+void gather(const GVArray &src, const Span<int> map, GMutableSpan dst)
+{
+  attribute_math::convert_to_static_type(src.type(), [&](auto dummy) {
+    using T = decltype(dummy);
+    array_utils::gather(src.typed<T>(), map, dst.typed<T>());
+  });
+}
+
+}  // namespace blender::bke::attribute_math
