@@ -41,6 +41,7 @@
 #include "BKE_lib_query.h"
 #include "BKE_lib_remap.h"
 #include "BKE_main.h"
+#include "BKE_main_namemap.h"
 #include "BKE_material.h"
 #include "BKE_object.h"
 #include "BKE_report.h"
@@ -76,6 +77,10 @@ typedef struct BlendfileLinkAppendContextItem {
   /** Library ID from which the #new_id has been linked (NULL until it has been successfully
    * linked). */
   Library *source_library;
+  /** Liboverride of the linked ID (NULL until it has been successfully created or an existing one
+   * has been found). */
+  ID *liboverride_id;
+
   /** Opaque user data pointer. */
   void *userdata;
 } BlendfileLinkAppendContextItem;
@@ -118,9 +123,21 @@ typedef struct BlendfileLinkAppendContextCallBack {
   BlendfileLinkAppendContextItem *item;
   ReportList *reports;
 
+  /** Whether the currently evaluated usage is within some liboverride dependency context. Note
+   * that this include liboverride reference itself, but also e.g. if a linked Mesh is used by the
+   * reference of an overridden object.
+   *
+   * Mutually exclusive with #is_liboverride_dependency_only. */
+  bool is_liboverride_dependency;
+  /** Whether the currently evaluated usage is exclusively within some liboverride dependency
+   * context, i.e. the full all usages of this data so far have only been a part of liboverride
+   * references and their dependencies.
+   *
+   * Mutually exclusive with #is_liboverride_dependency. */
+  bool is_liboverride_dependency_only;
 } BlendfileLinkAppendContextCallBack;
 
-/* Actions to apply to an item (i.e. linked ID). */
+/** Actions to apply to an item (i.e. linked ID). */
 enum {
   LINK_APPEND_ACT_UNSET = 0,
   LINK_APPEND_ACT_KEEP_LINKED,
@@ -129,10 +146,23 @@ enum {
   LINK_APPEND_ACT_COPY_LOCAL,
 };
 
-/* Various status info about an item (i.e. linked ID). */
+/** Various status info about an item (i.e. linked ID). */
 enum {
-  /* An indirectly linked ID. */
+  /** An indirectly linked ID. */
   LINK_APPEND_TAG_INDIRECT = 1 << 0,
+  /** An ID also used as liboverride dependency (either directly, as a liboverride reference, or
+   * indirectly, as data used by a liboverride reference). It should never be directly made local.
+   *
+   * Mutually exclusive with #LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY.
+   */
+  LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY = 1 << 1,
+  /** An ID only used as liboverride dependency (either directly or indirectly, see
+   * #LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY for precisions). It should not be considered during
+   * the 'make local' process, and remain purely linked data.
+   *
+   * Mutually exclusive with #LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY.
+   */
+  LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY = 1 << 2,
 };
 
 static BlendHandle *link_append_context_library_blohandle_ensure(
@@ -189,8 +219,8 @@ void BKE_blendfile_link_append_context_free(BlendfileLinkAppendContext *lapp_con
     BLI_ghash_free(lapp_context->new_id_to_item, NULL, NULL);
   }
 
-  for (LinkNode *liblink = lapp_context->libraries.list; liblink != NULL;
-       liblink = liblink->next) {
+  for (LinkNode *liblink = lapp_context->libraries.list; liblink != NULL; liblink = liblink->next)
+  {
     BlendfileLinkAppendContextLibrary *lib_context = liblink->link;
     link_append_context_library_blohandle_release(lapp_context, lib_context);
   }
@@ -238,9 +268,9 @@ void BKE_blendfile_link_append_context_library_add(BlendfileLinkAppendContext *l
   BlendfileLinkAppendContextLibrary *lib_context = BLI_memarena_calloc(lapp_context->memarena,
                                                                        sizeof(*lib_context));
 
-  size_t len = strlen(libname) + 1;
-  char *libpath = BLI_memarena_alloc(lapp_context->memarena, len);
-  BLI_strncpy(libpath, libname, len);
+  const size_t libname_size = strlen(libname) + 1;
+  char *libpath = BLI_memarena_alloc(lapp_context->memarena, libname_size);
+  memcpy(libpath, libname, libname_size);
 
   lib_context->path = libpath;
   lib_context->blo_handle = blo_handle;
@@ -258,10 +288,10 @@ BlendfileLinkAppendContextItem *BKE_blendfile_link_append_context_item_add(
 {
   BlendfileLinkAppendContextItem *item = BLI_memarena_calloc(lapp_context->memarena,
                                                              sizeof(*item));
-  size_t len = strlen(idname) + 1;
+  const size_t idname_size = strlen(idname) + 1;
 
-  item->name = BLI_memarena_alloc(lapp_context->memarena, len);
-  BLI_strncpy(item->name, idname, len);
+  item->name = BLI_memarena_alloc(lapp_context->memarena, idname_size);
+  memcpy(item->name, idname, idname_size);
   item->idcode = idcode;
   item->libraries = BLI_BITMAP_NEW_MEMARENA(lapp_context->memarena, lapp_context->num_libraries);
 
@@ -298,8 +328,8 @@ int BKE_blendfile_link_append_context_item_idtypes_from_library_add(
 
   while ((id_code = BKE_idtype_idcode_iter_step(&id_code_iter))) {
     if (!BKE_idtype_idcode_is_linkable(id_code) ||
-        (id_types_filter != 0 &&
-         (BKE_idtype_idcode_to_idfilter(id_code) & id_types_filter) == 0)) {
+        (id_types_filter != 0 && (BKE_idtype_idcode_to_idfilter(id_code) & id_types_filter) == 0))
+    {
       continue;
     }
 
@@ -351,6 +381,12 @@ ID *BKE_blendfile_link_append_context_item_newid_get(
   return item->new_id;
 }
 
+ID *BKE_blendfile_link_append_context_item_liboverrideid_get(
+    BlendfileLinkAppendContext *UNUSED(lapp_context), BlendfileLinkAppendContextItem *item)
+{
+  return item->liboverride_id;
+}
+
 short BKE_blendfile_link_append_context_item_idcode_get(
     struct BlendfileLinkAppendContext *UNUSED(lapp_context),
     struct BlendfileLinkAppendContextItem *item)
@@ -368,11 +404,13 @@ void BKE_blendfile_link_append_context_item_foreach(
     BlendfileLinkAppendContextItem *item = itemlink->link;
 
     if ((flag & BKE_BLENDFILE_LINK_APPEND_FOREACH_ITEM_FLAG_DO_DIRECT) == 0 &&
-        (item->tag & LINK_APPEND_TAG_INDIRECT) == 0) {
+        (item->tag & LINK_APPEND_TAG_INDIRECT) == 0)
+    {
       continue;
     }
     if ((flag & BKE_BLENDFILE_LINK_APPEND_FOREACH_ITEM_FLAG_DO_INDIRECT) == 0 &&
-        (item->tag & LINK_APPEND_TAG_INDIRECT) != 0) {
+        (item->tag & LINK_APPEND_TAG_INDIRECT) != 0)
+    {
       continue;
     }
 
@@ -574,8 +612,10 @@ static void loose_data_instantiate_obdata_preprocess(
  * (return false). */
 static bool loose_data_instantiate_collection_parents_check_recursive(Collection *collection)
 {
-  for (CollectionParent *parent_collection = collection->parents.first; parent_collection != NULL;
-       parent_collection = parent_collection->next) {
+  for (CollectionParent *parent_collection = collection->runtime.parents.first;
+       parent_collection != NULL;
+       parent_collection = parent_collection->next)
+  {
     if ((parent_collection->collection->id.tag & LIB_TAG_DOIT) != 0) {
       return true;
     }
@@ -612,10 +652,10 @@ static void loose_data_instantiate_collection_process(
     }
 
     /* Forced instantiation of indirectly appended collections is not wanted. Users can now
-     * easily instantiate collections (and their objects) as needed by themselves. See T67032. */
+     * easily instantiate collections (and their objects) as needed by themselves. See #67032. */
     /* We need to check that objects in that collections are already instantiated in a scene.
      * Otherwise, it's better to add the collection to the scene's active collection, than to
-     * instantiate its objects in active scene's collection directly. See T61141.
+     * instantiate its objects in active scene's collection directly. See #61141.
      *
      * NOTE: We only check object directly into that collection, not recursively into its
      * children.
@@ -624,7 +664,7 @@ static void loose_data_instantiate_collection_process(
     /* The collection could be linked/appended together with an Empty object instantiating it,
      * better not instantiate the collection in the view-layer in that case.
      *
-     * Can easily happen when copy/pasting such instantiating empty, see T93839. */
+     * Can easily happen when copy/pasting such instantiating empty, see #93839. */
     const bool collection_is_instantiated = collection_instantiated_by_any_object(bmain,
                                                                                   collection);
     /* Always consider adding collections directly selected by the user. */
@@ -665,7 +705,8 @@ static void loose_data_instantiate_collection_process(
     /* When instantiated into view-layer, do not add collections if one of their parents is also
      * instantiated. */
     if (!do_instantiate_as_empty &&
-        loose_data_instantiate_collection_parents_check_recursive(collection)) {
+        loose_data_instantiate_collection_parents_check_recursive(collection))
+    {
       continue;
     }
     /* When instantiated as empty, do not add indirectly linked (i.e. non-user-selected)
@@ -750,7 +791,7 @@ static void loose_data_instantiate_object_process(LooseDataInstantiateContext *i
      * While this is not ideal (in theory no object should remain un-owned), in case of indirectly
      * linked objects, the other solution would be to add them to a local collection, which would
      * make them directly linked. Think for now keeping them indirectly linked is more important.
-     * Ref. T93757.
+     * Ref. #93757.
      */
     if (is_linking && (item->tag & LINK_APPEND_TAG_INDIRECT) != 0) {
       continue;
@@ -911,8 +952,9 @@ static int foreach_libblock_link_append_callback(LibraryIDLinkCallbackData *cb_d
 {
   /* NOTE: It is important to also skip liboverride references here, as those should never be made
    * local. */
-  if (cb_data->cb_flag & (IDWALK_CB_EMBEDDED | IDWALK_CB_INTERNAL | IDWALK_CB_LOOPBACK |
-                          IDWALK_CB_OVERRIDE_LIBRARY_REFERENCE)) {
+  if (cb_data->cb_flag & (IDWALK_CB_EMBEDDED | IDWALK_CB_EMBEDDED_NOT_OWNING | IDWALK_CB_INTERNAL |
+                          IDWALK_CB_LOOPBACK))
+  {
     return IDWALK_RET_NOP;
   }
 
@@ -934,8 +976,8 @@ static int foreach_libblock_link_append_callback(LibraryIDLinkCallbackData *cb_d
      * the dependency here. Indeed, either they are both linked in another way (through their own
      * meshes for shape keys e.g.), or this is an unsupported case (two shape-keys depending on
      * each-other need to be also 'linked' in by their respective meshes, independent shape-keys
-     * are not allowed). ref T96048. */
-    if (id != cb_data->id_self && BKE_idtype_idcode_is_linkable(GS(cb_data->id_self->name))) {
+     * are not allowed). ref #96048. */
+    if (id != cb_data->self_id && BKE_idtype_idcode_is_linkable(GS(cb_data->self_id->name))) {
       BKE_library_foreach_ID_link(
           cb_data->bmain, id, foreach_libblock_link_append_callback, data, IDWALK_NOP);
     }
@@ -955,7 +997,7 @@ static int foreach_libblock_link_append_callback(LibraryIDLinkCallbackData *cb_d
   const bool do_recursive = (data->lapp_context->params->flag & BLO_LIBLINK_APPEND_RECURSIVE) !=
                                 0 ||
                             do_link;
-  if (!do_recursive && cb_data->id_owner->lib != id->lib) {
+  if (!do_recursive && cb_data->owner_id->lib != id->lib) {
     return IDWALK_RET_NOP;
   }
 
@@ -965,7 +1007,7 @@ static int foreach_libblock_link_append_callback(LibraryIDLinkCallbackData *cb_d
         data->lapp_context, id->name, GS(id->name), NULL);
     item->new_id = id;
     item->source_library = id->lib;
-    /* Since we did not have an item for that ID yet, we know user did not selected it explicitly,
+    /* Since we did not have an item for that ID yet, we know user did not select it explicitly,
      * it was rather linked indirectly. This info is important for instantiation of collections. */
     item->tag |= LINK_APPEND_TAG_INDIRECT;
     /* In linking case we already know what we want to do with those items. */
@@ -973,6 +1015,54 @@ static int foreach_libblock_link_append_callback(LibraryIDLinkCallbackData *cb_d
       item->action = LINK_APPEND_ACT_KEEP_LINKED;
     }
     new_id_to_item_mapping_add(data->lapp_context, id, item);
+
+    if ((cb_data->cb_flag & IDWALK_CB_OVERRIDE_LIBRARY_REFERENCE) != 0 ||
+        data->is_liboverride_dependency_only)
+    {
+      /* New item, (currently) detected as only used as a liboverride linked dependency. */
+      item->tag |= LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY;
+    }
+    else if (data->is_liboverride_dependency) {
+      /* New item, (currently) detected as used as a liboverride linked dependency, among
+       * others. */
+      item->tag |= LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY;
+    }
+  }
+  else {
+    if ((cb_data->cb_flag & IDWALK_CB_OVERRIDE_LIBRARY_REFERENCE) != 0 ||
+        data->is_liboverride_dependency_only)
+    {
+      /* Existing item, here only used as a liboverride reference dependency. If it was not tagged
+       * as such before, it is also used by non-liboverride reference data. */
+      if ((item->tag & LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY) == 0) {
+        item->tag |= LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY;
+        /* In case that item was already processed, and marked to be directly made local, its
+         * action needs to be changed to copying instead, since no liboverride reference (or their
+         * dependencies) should ever be made local. */
+        if (item->action == LINK_APPEND_ACT_MAKE_LOCAL) {
+          CLOG_INFO(
+              &LOG,
+              3,
+              "Appended ID '%s' is also used as a liboverride linked dependency, duplicating it.",
+              id->name);
+          item->action = LINK_APPEND_ACT_COPY_LOCAL;
+        }
+      }
+    }
+    else if ((item->tag & LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY) != 0) {
+      /* Existing item, here used in a non-liboverride dependency context. If it was
+       * tagged as a liboverride dependency only, its tag and action need to be updated. */
+      item->tag |= LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY;
+      item->tag &= ~LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY;
+      if (item->action == LINK_APPEND_ACT_KEEP_LINKED) {
+        CLOG_INFO(
+            &LOG,
+            3,
+            "Appended ID '%s' is also used as a liboverride linked dependency, duplicating it.",
+            id->name);
+        item->action = LINK_APPEND_ACT_COPY_LOCAL;
+      }
+    }
   }
 
   /* NOTE: currently there is no need to do anything else here, but in the future this would be
@@ -994,16 +1084,20 @@ static void blendfile_link_append_proxies_convert(Main *bmain, ReportList *repor
   BlendFileReadReport bf_reports = {.reports = reports};
   BKE_lib_override_library_main_proxy_convert(bmain, &bf_reports);
 
+  /* Currently liboverride code can generate invalid namemap. This is a known issue, requires
+   * #107847 to be properly fixed. */
+  BKE_main_namemap_validate_and_fix(bmain);
+
   if (bf_reports.count.proxies_to_lib_overrides_success != 0 ||
-      bf_reports.count.proxies_to_lib_overrides_failures != 0) {
-    BKE_reportf(
-        bf_reports.reports,
-        RPT_WARNING,
-        "Proxies have been removed from Blender (%d proxies were automatically converted "
-        "to library overrides, %d proxies could not be converted and were cleared). "
-        "Please consider re-saving any library .blend file with the newest Blender version",
-        bf_reports.count.proxies_to_lib_overrides_success,
-        bf_reports.count.proxies_to_lib_overrides_failures);
+      bf_reports.count.proxies_to_lib_overrides_failures != 0)
+  {
+    BKE_reportf(bf_reports.reports,
+                RPT_WARNING,
+                "Proxies have been removed from Blender (%d proxies were automatically converted "
+                "to library overrides, %d proxies could not be converted and were cleared). "
+                "Consider re-saving any library .blend file with the newest Blender version",
+                bf_reports.count.proxies_to_lib_overrides_success,
+                bf_reports.count.proxies_to_lib_overrides_failures);
   }
 }
 
@@ -1022,11 +1116,16 @@ void BKE_blendfile_append(BlendfileLinkAppendContext *lapp_context, ReportList *
   const bool do_reuse_local_id = (lapp_context->params->flag &
                                   BLO_LIBLINK_APPEND_LOCAL_ID_REUSE) != 0;
 
-  const int make_local_common_flags = LIB_ID_MAKELOCAL_FULL_LIBRARY |
-                                      ((lapp_context->params->flag &
-                                        BLO_LIBLINK_APPEND_ASSET_DATA_CLEAR) != 0 ?
-                                           LIB_ID_MAKELOCAL_ASSET_DATA_CLEAR :
-                                           0);
+  const int make_local_common_flags =
+      LIB_ID_MAKELOCAL_FULL_LIBRARY |
+      ((lapp_context->params->flag & BLO_LIBLINK_APPEND_ASSET_DATA_CLEAR) != 0 ?
+           LIB_ID_MAKELOCAL_ASSET_DATA_CLEAR :
+           0) |
+      /* In recursive case (i.e. everything becomes local), clear liboverrides. Otherwise (i.e.
+       * only data from immediately linked libraries is made local), preserve liboverrides. */
+      ((lapp_context->params->flag & BLO_LIBLINK_APPEND_RECURSIVE) != 0 ?
+           LIB_ID_MAKELOCAL_LIBOVERRIDE_CLEAR :
+           0);
 
   LinkNode *itemlink;
 
@@ -1050,29 +1149,61 @@ void BKE_blendfile_append(BlendfileLinkAppendContext *lapp_context, ReportList *
                                     id->lib->filepath,
                                     id->name) :
                                 NULL;
+    const bool is_liboverride_dependency = (item->tag & LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY) !=
+                                           0;
+    const bool is_liboverride_dependency_only = (item->tag &
+                                                 LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY) != 0;
+    /* These two flags are mutually exclusive. */
+    BLI_assert(!(is_liboverride_dependency && is_liboverride_dependency_only));
 
     if (item->action != LINK_APPEND_ACT_UNSET) {
       /* Already set, pass. */
     }
+    else if (item->tag & LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY) {
+      CLOG_INFO(
+          &LOG,
+          3,
+          "Appended ID '%s' is only used as a liboverride linked dependency, keeping it linked.",
+          id->name);
+      item->action = LINK_APPEND_ACT_KEEP_LINKED;
+    }
     else if (do_reuse_local_id && existing_local_id != NULL) {
-      CLOG_INFO(&LOG, 3, "Appended ID '%s' as a matching local one, re-using it...", id->name);
+      CLOG_INFO(&LOG, 3, "Appended ID '%s' as a matching local one, re-using it.", id->name);
       item->action = LINK_APPEND_ACT_REUSE_LOCAL;
       item->userdata = existing_local_id;
     }
     else if (id->tag & LIB_TAG_PRE_EXISTING) {
-      CLOG_INFO(&LOG, 3, "Appended ID '%s' was already linked, need to copy it...", id->name);
+      CLOG_INFO(&LOG, 3, "Appended ID '%s' was already linked, duplicating it.", id->name);
+      item->action = LINK_APPEND_ACT_COPY_LOCAL;
+    }
+    else if (item->tag & LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY) {
+      CLOG_INFO(
+          &LOG,
+          3,
+          "Appended ID '%s' is also used as a liboverride linked dependency, duplicating it.",
+          id->name);
       item->action = LINK_APPEND_ACT_COPY_LOCAL;
     }
     else {
-      CLOG_INFO(&LOG, 3, "Appended ID '%s' will be made local...", id->name);
+      CLOG_INFO(&LOG, 3, "Appended ID '%s' will be made local.", id->name);
       item->action = LINK_APPEND_ACT_MAKE_LOCAL;
     }
 
-    /* Only check dependencies if we are not keeping linked data, nor re-using existing local data.
+    /* Only check dependencies if linked data are not kept, nor existing local data are re-used,
+     * and if the item is not used for liboverride dependencies.
+     *
+     * NOTE: liboverride dependencies require complete check of all usages of the ID, as this may
+     * change the status and action of its item (see #foreach_libblock_link_append_callback code).
      */
-    if (!ELEM(item->action, LINK_APPEND_ACT_KEEP_LINKED, LINK_APPEND_ACT_REUSE_LOCAL)) {
+    if (!ELEM(item->action, LINK_APPEND_ACT_KEEP_LINKED, LINK_APPEND_ACT_REUSE_LOCAL) ||
+        is_liboverride_dependency || is_liboverride_dependency_only)
+    {
       BlendfileLinkAppendContextCallBack cb_data = {
-          .lapp_context = lapp_context, .item = item, .reports = reports};
+          .lapp_context = lapp_context,
+          .item = item,
+          .reports = reports,
+          .is_liboverride_dependency = is_liboverride_dependency,
+          .is_liboverride_dependency_only = is_liboverride_dependency_only};
       BKE_library_foreach_ID_link(
           bmain, id, foreach_libblock_link_append_callback, &cb_data, IDWALK_NOP);
     }
@@ -1080,7 +1211,8 @@ void BKE_blendfile_append(BlendfileLinkAppendContext *lapp_context, ReportList *
     /* If we found a matching existing local id but are not re-using it, we need to properly clear
      * its weak reference to linked data. */
     if (existing_local_id != NULL &&
-        !ELEM(item->action, LINK_APPEND_ACT_KEEP_LINKED, LINK_APPEND_ACT_REUSE_LOCAL)) {
+        !ELEM(item->action, LINK_APPEND_ACT_KEEP_LINKED, LINK_APPEND_ACT_REUSE_LOCAL))
+    {
       BKE_main_library_weak_reference_remove_item(lapp_context->library_weak_reference_mapping,
                                                   id->lib->filepath,
                                                   id->name,
@@ -1098,9 +1230,9 @@ void BKE_blendfile_append(BlendfileLinkAppendContext *lapp_context, ReportList *
 
     ID *local_appended_new_id = NULL;
     char lib_filepath[FILE_MAX];
-    BLI_strncpy(lib_filepath, id->lib->filepath, sizeof(lib_filepath));
+    STRNCPY(lib_filepath, id->lib->filepath);
     char lib_id_name[MAX_ID_NAME];
-    BLI_strncpy(lib_id_name, id->name, sizeof(lib_id_name));
+    STRNCPY(lib_id_name, id->name);
 
     switch (item->action) {
       case LINK_APPEND_ACT_COPY_LOCAL:
@@ -1196,6 +1328,12 @@ void BKE_blendfile_append(BlendfileLinkAppendContext *lapp_context, ReportList *
     if (id->tag & LIB_TAG_PRE_EXISTING) {
       continue;
     }
+    /* Do NOT delete a linked data that is (also) used a liboverride dependency. */
+    if (item->tag &
+        (LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY | LINK_APPEND_TAG_LIBOVERRIDE_DEPENDENCY_ONLY))
+    {
+      continue;
+    }
 
     id->tag |= LIB_TAG_DOIT;
   }
@@ -1227,7 +1365,8 @@ void BKE_blendfile_link(BlendfileLinkAppendContext *lapp_context, ReportList *re
   int lib_idx, item_idx;
 
   for (lib_idx = 0, liblink = lapp_context->libraries.list; liblink;
-       lib_idx++, liblink = liblink->next) {
+       lib_idx++, liblink = liblink->next)
+  {
     BlendfileLinkAppendContextLibrary *lib_context = liblink->link;
     char *libname = lib_context->path;
     BlendHandle *blo_handle = link_append_context_library_blohandle_ensure(
@@ -1244,7 +1383,7 @@ void BKE_blendfile_link(BlendfileLinkAppendContext *lapp_context, ReportList *re
     mainl = BLO_library_link_begin(&blo_handle, libname, lapp_context->params);
     lib = mainl->curlib;
     BLI_assert(lib != NULL);
-    /* In case lib was already existing but not found originally, see T99820. */
+    /* In case lib was already existing but not found originally, see #99820. */
     lib->id.tag &= ~LIB_TAG_MISSING;
 
     if (mainl->versionfile < 250) {
@@ -1260,7 +1399,8 @@ void BKE_blendfile_link(BlendfileLinkAppendContext *lapp_context, ReportList *re
     /* For each lib file, we try to link all items belonging to that lib,
      * and tag those successful to not try to load them again with the other libraries. */
     for (item_idx = 0, itemlink = lapp_context->items.list; itemlink;
-         item_idx++, itemlink = itemlink->next) {
+         item_idx++, itemlink = itemlink->next)
+    {
       BlendfileLinkAppendContextItem *item = itemlink->link;
       ID *new_id;
 
@@ -1283,10 +1423,11 @@ void BKE_blendfile_link(BlendfileLinkAppendContext *lapp_context, ReportList *re
     BLO_library_link_end(mainl, &blo_handle, lapp_context->params);
     link_append_context_library_blohandle_release(lapp_context, lib_context);
   }
+  (void)item_idx; /* Quiet set-but-unused warning (may be removed). */
 
   /* Instantiate newly linked IDs as needed, if no append is scheduled. */
-  if ((lapp_context->params->flag & FILE_LINK) != 0 &&
-      lapp_context->params->context.scene != NULL) {
+  if ((lapp_context->params->flag & FILE_LINK) != 0 && lapp_context->params->context.scene != NULL)
+  {
     new_id_to_item_mapping_create(lapp_context);
     /* NOTE: Since we append items for IDs not already listed (i.e. implicitly linked indirect
      * dependencies), this list will grow and we will process those IDs later, leading to a flatten
@@ -1316,6 +1457,94 @@ void BKE_blendfile_link(BlendfileLinkAppendContext *lapp_context, ReportList *re
   if ((lapp_context->params->flag & FILE_LINK) != 0) {
     blendfile_link_append_proxies_convert(lapp_context->params->bmain, reports);
   }
+
+  BKE_main_namemap_clear(lapp_context->params->bmain);
+}
+
+void BKE_blendfile_override(BlendfileLinkAppendContext *lapp_context,
+                            const eBKELibLinkOverride flags,
+                            ReportList *UNUSED(reports))
+{
+  if (lapp_context->num_items == 0) {
+    /* Nothing to override. */
+    return;
+  }
+
+  Main *bmain = lapp_context->params->bmain;
+
+  /* Liboverride only makes sense if data was linked, not appended. */
+  BLI_assert((lapp_context->params->flag & FILE_LINK) != 0);
+
+  const bool set_runtime = (flags & BKE_LIBLINK_OVERRIDE_CREATE_RUNTIME) != 0;
+  const bool do_use_exisiting_liboverrides = (flags &
+                                              BKE_LIBLINK_OVERRIDE_USE_EXISTING_LIBOVERRIDES) != 0;
+
+  GHash *linked_ids_to_local_liboverrides = NULL;
+  if (do_use_exisiting_liboverrides) {
+    linked_ids_to_local_liboverrides = BLI_ghash_ptr_new(__func__);
+
+    ID *id_iter;
+    FOREACH_MAIN_ID_BEGIN (bmain, id_iter) {
+      if (ID_IS_LINKED(id_iter)) {
+        continue;
+      }
+      if (!ID_IS_OVERRIDE_LIBRARY_REAL(id_iter)) {
+        continue;
+      }
+      /* Do not consider regular liboverrides if runtime ones are requested, and vice-versa. */
+      if ((set_runtime && (id_iter->tag & LIB_TAG_RUNTIME) == 0) ||
+          (!set_runtime && (id_iter->tag & LIB_TAG_RUNTIME) != 0))
+      {
+        continue;
+      }
+
+      /* In case several liboverrides exist of the same data, only consider the first found one. */
+      ID **id_ptr;
+      if (BLI_ghash_ensure_p(linked_ids_to_local_liboverrides,
+                             id_iter->override_library->reference,
+                             (void ***)&id_ptr))
+      {
+        continue;
+      }
+      *id_ptr = id_iter;
+    }
+    FOREACH_MAIN_ID_END;
+  }
+
+  for (LinkNode *itemlink = lapp_context->items.list; itemlink; itemlink = itemlink->next) {
+    BlendfileLinkAppendContextItem *item = itemlink->link;
+    ID *id = item->new_id;
+    if (id == NULL) {
+      continue;
+    }
+    BLI_assert(item->userdata == NULL);
+
+    if (do_use_exisiting_liboverrides) {
+      item->liboverride_id = BLI_ghash_lookup(linked_ids_to_local_liboverrides, id);
+    }
+    if (item->liboverride_id == NULL) {
+      item->liboverride_id = BKE_lib_override_library_create_from_id(bmain, id, false);
+      if (set_runtime) {
+        item->liboverride_id->tag |= LIB_TAG_RUNTIME;
+        if ((id->tag & LIB_TAG_PRE_EXISTING) == 0) {
+          /* If the linked ID is newly linked, in case its override is runtime-only, assume its
+           * reference to be indirectly linked.
+           *
+           * This is more of an heuristic for 'as best as possible' user feedback in the UI
+           * (Outliner), which is expected to be valid in almost all practical use-cases. Direct or
+           * indirect linked status is properly checked before saving .blend file. */
+          id->tag &= ~LIB_TAG_EXTERN;
+          id->tag |= LIB_TAG_INDIRECT;
+        }
+      }
+    }
+  }
+
+  if (do_use_exisiting_liboverrides) {
+    BLI_ghash_free(linked_ids_to_local_liboverrides, NULL, NULL);
+  }
+
+  BKE_main_namemap_clear(bmain);
 }
 
 /** \} */
@@ -1328,7 +1557,7 @@ static void blendfile_library_relocate_remap(Main *bmain,
                                              ID *new_id,
                                              ReportList *reports,
                                              const bool do_reload,
-                                             const short remap_flags)
+                                             const int remap_flags)
 {
   BLI_assert(old_id);
   if (do_reload) {
@@ -1424,8 +1653,8 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
 
   /* All override rules need to be up to date, since there will be no do_version here, otherwise
    * older, now-invalid rules might be applied and likely fail, or some changes might be missing,
-   * etc. See T93353. */
-  BKE_lib_override_library_main_operations_create(bmain, true);
+   * etc. See #93353. */
+  BKE_lib_override_library_main_operations_create(bmain, true, NULL);
 
   /* Remove all IDs to be reloaded from Main. */
   lba_idx = set_listbasepointers(bmain, lbarray);
@@ -1476,7 +1705,8 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
    * We need to do this in a first, separated loop, otherwise some of those may not be handled by
    * ID remapping, which means they would still reference old data to be deleted... */
   for (item_idx = 0, itemlink = lapp_context->items.list; itemlink;
-       item_idx++, itemlink = itemlink->next) {
+       item_idx++, itemlink = itemlink->next)
+  {
     BlendfileLinkAppendContextItem *item = itemlink->link;
     ID *old_id = item->userdata;
 
@@ -1496,10 +1726,11 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
 
   BKE_layer_collection_resync_forbid();
   /* Note that in reload case, we also want to replace indirect usages. */
-  const short remap_flags = ID_REMAP_SKIP_NEVER_NULL_USAGE |
-                            (do_reload ? 0 : ID_REMAP_SKIP_INDIRECT_USAGE);
+  const int remap_flags = ID_REMAP_SKIP_NEVER_NULL_USAGE |
+                          (do_reload ? 0 : ID_REMAP_SKIP_INDIRECT_USAGE);
   for (item_idx = 0, itemlink = lapp_context->items.list; itemlink;
-       item_idx++, itemlink = itemlink->next) {
+       item_idx++, itemlink = itemlink->next)
+  {
     BlendfileLinkAppendContextItem *item = itemlink->link;
     ID *old_id = item->userdata;
     ID *new_id = item->new_id;
@@ -1543,7 +1774,8 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
 
     BKE_main_id_tag_all(bmain, LIB_TAG_DOIT, false);
     for (item_idx = 0, itemlink = lapp_context->items.list; itemlink;
-         item_idx++, itemlink = itemlink->next) {
+         item_idx++, itemlink = itemlink->next)
+    {
       BlendfileLinkAppendContextItem *item = itemlink->link;
       ID *old_id = item->userdata;
 
@@ -1559,6 +1791,24 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
         continue;
       }
 
+      /* In case the active scene was reloaded, the context pointers in
+       * `lapp_context->params->context` need to be updated before the old Scene ID is freed. */
+      if (old_id == &lapp_context->params->context.scene->id) {
+        BLI_assert(GS(old_id->name) == ID_SCE);
+        Scene *new_scene = (Scene *)item->new_id;
+        BLI_assert(new_scene != NULL);
+        lapp_context->params->context.scene = new_scene;
+        if (lapp_context->params->context.view_layer != NULL) {
+          ViewLayer *new_view_layer = BKE_view_layer_find(
+              new_scene, lapp_context->params->context.view_layer->name);
+          lapp_context->params->context.view_layer = (new_view_layer != NULL) ?
+                                                         new_view_layer :
+                                                         new_scene->view_layers.first;
+        }
+        /* lapp_context->params->context.v3d should never become invalid by newly linked data here.
+         */
+      }
+
       if (old_id->us == 0) {
         old_id->tag |= LIB_TAG_DOIT;
         item->userdata = NULL;
@@ -1573,6 +1823,7 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
     /* Should not be needed, all tagged IDs should have been deleted above, just 'in case'. */
     BKE_main_id_tag_all(bmain, LIB_TAG_DOIT, false);
   }
+  (void)item_idx; /* Quiet set-but-unused warning (may be removed). */
 
   /* Some datablocks can get reloaded/replaced 'silently' because they are not linkable
    * (shape keys e.g.), so we need another loop here to clear old ones if possible. */
@@ -1635,7 +1886,7 @@ void BKE_blendfile_library_relocate(BlendfileLinkAppendContext *lapp_context,
                                              .reports = reports,
                                          });
     /* We need to rebuild some of the deleted override rules (for UI feedback purpose). */
-    BKE_lib_override_library_main_operations_create(bmain, true);
+    BKE_lib_override_library_main_operations_create(bmain, true, NULL);
   }
 
   BKE_main_collection_sync(bmain);
