@@ -25,6 +25,8 @@
 #include "BKE_particle.h"
 #include "BLI_kdopbvh.h"
 
+#include "BLT_translation.h"
+
 #include "BKE_modifier.h"
 
 #include "RNA_enum_types.h"
@@ -110,7 +112,8 @@ static bool rule_goal_avoid(BoidRule *rule, BoidBrainData *bbd, BoidValues *val,
         /* skip current object */
       }
       else if (pd->forcefield == PFIELD_BOID && mul * pd->f_strength > 0.0f &&
-               get_effector_data(cur, &cur_efd, &epoint, 0)) {
+               get_effector_data(cur, &cur_efd, &epoint, 0))
+      {
         float temp = mul * pd->f_strength *
                      effector_falloff(cur, &cur_efd, &epoint, bbd->part->effector_weights);
 
@@ -144,10 +147,10 @@ static bool rule_goal_avoid(BoidRule *rule, BoidBrainData *bbd, BoidValues *val,
     priority = 1.0f;
   }
 
-  /* then use that effector */
-  if (priority > (rule->type == eBoidRuleType_Avoid ?
-                      gabr->fear_factor :
-                      0.0f)) { /* with avoid, factor is "fear factor" */
+  /* Then use that effector. */
+
+  /* With avoid, factor is "fear factor". */
+  if (priority > (rule->type == eBoidRuleType_Avoid ? gabr->fear_factor : 0.0f)) {
     Object *eob = eff->ob;
     PartDeflect *pd = eff->pd;
     float surface = (pd && pd->shape == PFIELD_SHAPE_SURFACE) ? 1.0f : 0.0f;
@@ -170,7 +173,8 @@ static bool rule_goal_avoid(BoidRule *rule, BoidBrainData *bbd, BoidValues *val,
       }
     }
     else if ((rule->type == eBoidRuleType_Avoid) && (bpa->data.mode == eBoidMode_Climbing) &&
-             (priority > 2.0f * gabr->fear_factor)) {
+             (priority > 2.0f * gabr->fear_factor))
+    {
       /* detach from surface and try to fly away from danger */
       negate_v3_v3(efd.vec_to_point, bpa->gravity);
     }
@@ -757,7 +761,8 @@ static bool rule_fight(BoidRule *rule, BoidBrainData *bbd, BoidValues *val, Part
     /* check if boid doesn't want to fight */
     bpa = pa->boid;
     if (bpa->data.health / bbd->part->boids->health * bbd->part->boids->aggression <
-        e_strength / f_strength) {
+        e_strength / f_strength)
+    {
       /* decide to flee */
       if (closest_dist < fbr->flee_distance * fbr->distance) {
         negate_v3(bbd->wanted_co);
@@ -936,8 +941,8 @@ static bool boid_rule_applies(ParticleData *pa, BoidSettings *UNUSED(boids), Boi
     return false;
   }
 
-  if (ELEM(bpa->data.mode, eBoidMode_OnLand, eBoidMode_Climbing) &&
-      rule->flag & BOIDRULE_ON_LAND) {
+  if (ELEM(bpa->data.mode, eBoidMode_OnLand, eBoidMode_Climbing) && rule->flag & BOIDRULE_ON_LAND)
+  {
     return true;
   }
 
@@ -1013,7 +1018,8 @@ static bool apply_boid_rule(
 
   if (fuzziness < 0.0f || compare_len_v3v3(bbd->wanted_co,
                                            pa->prev_state.vel,
-                                           fuzziness * len_v3(pa->prev_state.vel)) == 0) {
+                                           fuzziness * len_v3(pa->prev_state.vel)) == 0)
+  {
     return true;
   }
   return false;
@@ -1127,7 +1133,7 @@ void boid_brain(BoidBrainData *bbd, int p, ParticleData *pa)
 
   /* decide on jumping & liftoff */
   if (bpa->data.mode == eBoidMode_OnLand) {
-    /* fuzziness makes boids capable of misjudgement */
+    /* Fuzziness makes boids capable of misjudgment. */
     float mul = 1.0f + state->rule_fuzziness;
 
     if (boids->options & BOID_ALLOW_FLIGHT && bbd->wanted_co[2] > 0.0f) {
@@ -1229,7 +1235,8 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
   /* if boids can't fly they fall to the ground */
   if ((boids->options & BOID_ALLOW_FLIGHT) == 0 &&
       ELEM(bpa->data.mode, eBoidMode_OnLand, eBoidMode_Climbing) == 0 &&
-      psys_uses_gravity(bbd->sim)) {
+      psys_uses_gravity(bbd->sim))
+  {
     bpa->data.mode = eBoidMode_Falling;
   }
 
@@ -1417,7 +1424,8 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
 
       /* stick boid on goal when close enough */
       if (bbd->goal_ob && boid_goal_signed_dist(pa->state.co, bbd->goal_co, bbd->goal_nor) <=
-                              pa->size * boids->height) {
+                              pa->size * boids->height)
+      {
         bpa->data.mode = eBoidMode_Climbing;
         bpa->ground = bbd->goal_ob;
         boid_find_ground(bbd, pa, ground_co, ground_nor);
@@ -1452,7 +1460,8 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
       if (boids->options & BOID_ALLOW_LAND) {
         /* stick boid on goal when close enough */
         if (bbd->goal_ob && boid_goal_signed_dist(pa->state.co, bbd->goal_co, bbd->goal_nor) <=
-                                pa->size * boids->height) {
+                                pa->size * boids->height)
+        {
           bpa->data.mode = eBoidMode_Climbing;
           bpa->ground = bbd->goal_ob;
           boid_find_ground(bbd, pa, ground_co, ground_nor);
@@ -1497,7 +1506,8 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
     case eBoidMode_OnLand: {
       /* stick boid on goal when close enough */
       if (bbd->goal_ob && boid_goal_signed_dist(pa->state.co, bbd->goal_co, bbd->goal_nor) <=
-                              pa->size * boids->height) {
+                              pa->size * boids->height)
+      {
         bpa->data.mode = eBoidMode_Climbing;
         bpa->ground = bbd->goal_ob;
         boid_find_ground(bbd, pa, ground_co, ground_nor);
@@ -1607,7 +1617,7 @@ BoidRule *boid_new_rule(int type)
 
   rule->type = type;
   rule->flag |= BOIDRULE_IN_AIR | BOIDRULE_ON_LAND;
-  BLI_strncpy(rule->name, rna_enum_boidrule_type_items[type - 1].name, sizeof(rule->name));
+  STRNCPY_UTF8(rule->name, DATA_(rna_enum_boidrule_type_items[type - 1].name));
 
   return rule;
 }
@@ -1643,7 +1653,7 @@ BoidState *boid_new_state(BoidSettings *boids)
 
   state->id = boids->last_state_id++;
   if (state->id) {
-    BLI_snprintf(state->name, sizeof(state->name), "State %i", state->id);
+    SNPRINTF(state->name, "State %i", state->id);
   }
   else {
     strcpy(state->name, "State");
