@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2005 Blender Foundation. All rights reserved. */
+ * Copyright 2005 Blender Foundation */
 
 /** \file
  * \ingroup bke
@@ -24,7 +24,7 @@
 #include "BKE_editmesh.h"
 #include "BKE_geometry_set.hh"
 #include "BKE_lib_id.h"
-#include "BKE_mesh.h"
+#include "BKE_mesh.hh"
 #include "BKE_mesh_wrapper.h"
 #include "BKE_modifier.h"
 #include "BKE_multires.h"
@@ -142,7 +142,8 @@ void BKE_crazyspace_set_quats_editmesh(BMEditMesh *em,
     do {
       if (BM_elem_flag_test(l_iter->v, BM_ELEM_HIDDEN) ||
           BM_elem_flag_test(l_iter->v, BM_ELEM_TAG) ||
-          (use_select && !BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT))) {
+          (use_select && !BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)))
+      {
         continue;
       }
 
@@ -188,46 +189,46 @@ void BKE_crazyspace_set_quats_mesh(Mesh *me,
   BLI_bitmap *vert_tag = BLI_BITMAP_NEW(me->totvert, __func__);
 
   /* first store two sets of tangent vectors in vertices, we derive it just from the face-edges */
-  const Span<MVert> verts = me->verts();
-  const Span<MPoly> polys = me->polys();
-  const Span<MLoop> loops = me->loops();
+  const Span<float3> positions = me->vert_positions();
+  const OffsetIndices polys = me->polys();
+  const Span<int> corner_verts = me->corner_verts();
 
   for (int i = 0; i < me->totpoly; i++) {
-    const MPoly *poly = &polys[i];
-    const MLoop *ml_next = &loops[poly->loopstart];
-    const MLoop *ml_curr = &ml_next[poly->totloop - 1];
-    const MLoop *ml_prev = &ml_next[poly->totloop - 2];
+    const IndexRange poly = polys[i];
+    const int *corner_vert_next = &corner_verts[poly.start()];
+    const int *corner_vert_curr = &corner_vert_next[poly.size() - 1];
+    const int *corner_vert_prev = &corner_vert_next[poly.size() - 2];
 
-    for (int j = 0; j < poly->totloop; j++) {
-      if (!BLI_BITMAP_TEST(vert_tag, ml_curr->v)) {
+    for (int j = 0; j < poly.size(); j++) {
+      if (!BLI_BITMAP_TEST(vert_tag, *corner_vert_curr)) {
         const float *co_prev, *co_curr, *co_next; /* orig */
         const float *vd_prev, *vd_curr, *vd_next; /* deform */
 
         /* retrieve mapped coordinates */
-        vd_prev = mappedcos[ml_prev->v];
-        vd_curr = mappedcos[ml_curr->v];
-        vd_next = mappedcos[ml_next->v];
+        vd_prev = mappedcos[*corner_vert_prev];
+        vd_curr = mappedcos[*corner_vert_curr];
+        vd_next = mappedcos[*corner_vert_next];
 
         if (origcos) {
-          co_prev = origcos[ml_prev->v];
-          co_curr = origcos[ml_curr->v];
-          co_next = origcos[ml_next->v];
+          co_prev = origcos[*corner_vert_prev];
+          co_curr = origcos[*corner_vert_curr];
+          co_next = origcos[*corner_vert_next];
         }
         else {
-          co_prev = verts[ml_prev->v].co;
-          co_curr = verts[ml_curr->v].co;
-          co_next = verts[ml_next->v].co;
+          co_prev = positions[*corner_vert_prev];
+          co_curr = positions[*corner_vert_curr];
+          co_next = positions[*corner_vert_next];
         }
 
         set_crazy_vertex_quat(
-            quats[ml_curr->v], co_curr, co_next, co_prev, vd_curr, vd_next, vd_prev);
+            quats[*corner_vert_curr], co_curr, co_next, co_prev, vd_curr, vd_next, vd_prev);
 
-        BLI_BITMAP_ENABLE(vert_tag, ml_curr->v);
+        BLI_BITMAP_ENABLE(vert_tag, *corner_vert_curr);
       }
 
-      ml_prev = ml_curr;
-      ml_curr = ml_next;
-      ml_next++;
+      corner_vert_prev = corner_vert_curr;
+      corner_vert_curr = corner_vert_next;
+      corner_vert_next++;
     }
   }
 
@@ -291,7 +292,8 @@ int BKE_crazyspace_get_first_deform_matrices_editbmesh(struct Depsgraph *depsgra
 
   for (; md && i <= cageIndex; md = md->next, i++) {
     if (editbmesh_modifier_is_enabled(scene, ob, md, me != nullptr) &&
-        BKE_modifier_is_correctable_deformed(md)) {
+        BKE_modifier_is_correctable_deformed(md))
+    {
       modifiers_left_num++;
     }
   }
@@ -389,7 +391,7 @@ int BKE_sculpt_get_first_deform_matrices(struct Depsgraph *depsgraph,
       if (defmats == nullptr) {
         /* NOTE: Evaluated object is re-set to its original un-deformed state. */
         Mesh *me = static_cast<Mesh *>(object_eval.data);
-        me_eval = BKE_mesh_copy_for_eval(me, true);
+        me_eval = BKE_mesh_copy_for_eval(me);
         crazyspace_init_verts_and_matrices(me_eval, &defmats, &deformedVerts);
       }
 
@@ -398,7 +400,7 @@ int BKE_sculpt_get_first_deform_matrices(struct Depsgraph *depsgraph,
       }
       else {
         /* More complex handling will continue in BKE_crazyspace_build_sculpt.
-         * Exiting the loop on a non-deform modifier causes issues - T71213. */
+         * Exiting the loop on a non-deform modifier causes issues - #71213. */
         BLI_assert(crazyspace_modifier_supports_deform(md));
         break;
       }
@@ -470,7 +472,7 @@ void BKE_crazyspace_build_sculpt(struct Depsgraph *depsgraph,
         }
 
         if (mesh_eval == nullptr) {
-          mesh_eval = BKE_mesh_copy_for_eval(mesh, true);
+          mesh_eval = BKE_mesh_copy_for_eval(mesh);
         }
 
         mti->deformVerts(md, &mectx, mesh_eval, deformedVerts, mesh_eval->totvert);
@@ -522,7 +524,8 @@ void BKE_crazyspace_api_eval(Depsgraph *depsgraph,
                              struct ReportList *reports)
 {
   if (object->runtime.crazyspace_deform_imats != nullptr ||
-      object->runtime.crazyspace_deform_cos != nullptr) {
+      object->runtime.crazyspace_deform_cos != nullptr)
+  {
     return;
   }
 
@@ -571,7 +574,7 @@ void BKE_crazyspace_api_displacement_to_original(struct Object *object,
   if (vertex_index < 0 || vertex_index >= object->runtime.crazyspace_verts_num) {
     BKE_reportf(reports,
                 RPT_ERROR,
-                "Invalid vertex index %d (expected to be within 0 to %d range))",
+                "Invalid vertex index %d (expected to be within 0 to %d range)",
                 vertex_index,
                 object->runtime.crazyspace_verts_num);
     return;
@@ -596,19 +599,17 @@ void BKE_crazyspace_api_eval_clear(Object *object)
 
 namespace blender::bke::crazyspace {
 
-GeometryDeformation get_evaluated_curves_deformation(const Depsgraph &depsgraph,
-                                                     const Object &ob_orig)
+GeometryDeformation get_evaluated_curves_deformation(const Object *ob_eval, const Object &ob_orig)
 {
   BLI_assert(ob_orig.type == OB_CURVES);
   const Curves &curves_id_orig = *static_cast<const Curves *>(ob_orig.data);
-  const CurvesGeometry &curves_orig = CurvesGeometry::wrap(curves_id_orig.geometry);
+  const CurvesGeometry &curves_orig = curves_id_orig.geometry.wrap();
   const int points_num = curves_orig.points_num();
 
   GeometryDeformation deformation;
   /* Use the undeformed positions by default. */
   deformation.positions = curves_orig.positions();
 
-  const Object *ob_eval = DEG_get_evaluated_object(&depsgraph, const_cast<Object *>(&ob_orig));
   if (ob_eval == nullptr) {
     return deformation;
   }
@@ -643,7 +644,7 @@ GeometryDeformation get_evaluated_curves_deformation(const Depsgraph &depsgraph,
     if (curves_component_eval != nullptr) {
       const Curves *curves_id_eval = curves_component_eval->get_for_read();
       if (curves_id_eval != nullptr) {
-        const CurvesGeometry &curves_eval = CurvesGeometry::wrap(curves_id_eval->geometry);
+        const CurvesGeometry &curves_eval = curves_id_eval->geometry.wrap();
         if (curves_eval.points_num() == points_num) {
           deformation.positions = curves_eval.positions();
         }
@@ -653,4 +654,11 @@ GeometryDeformation get_evaluated_curves_deformation(const Depsgraph &depsgraph,
   return deformation;
 }
 
+GeometryDeformation get_evaluated_curves_deformation(const Depsgraph &depsgraph,
+                                                     const Object &ob_orig)
+{
+  const Object *ob_eval = DEG_get_evaluated_object(&depsgraph, const_cast<Object *>(&ob_orig));
+  return get_evaluated_curves_deformation(ob_eval, ob_orig);
+}
+
 }  // namespace blender::bke::crazyspace
