@@ -6,6 +6,8 @@
 
 #include <algorithm>
 
+#include "BLI_task.hh"
+
 #include "BKE_attribute_math.hh"
 #include "BKE_curves.hh"
 
@@ -33,20 +35,21 @@ void calculate_evaluated_offsets(const Span<int8_t> handle_types_left,
                                  MutableSpan<int> evaluated_offsets)
 {
   const int size = handle_types_left.size();
-  BLI_assert(evaluated_offsets.size() == size);
+  BLI_assert(evaluated_offsets.size() == size + 1);
 
+  evaluated_offsets.first() = 0;
   if (size == 1) {
-    evaluated_offsets.first() = 1;
+    evaluated_offsets.last() = 1;
     return;
   }
 
   int offset = 0;
-
   for (const int i : IndexRange(size - 1)) {
-    offset += segment_is_vector(handle_types_left, handle_types_right, i) ? 1 : resolution;
     evaluated_offsets[i] = offset;
+    offset += segment_is_vector(handle_types_left, handle_types_right, i) ? 1 : resolution;
   }
 
+  evaluated_offsets.last(1) = offset;
   if (cyclic) {
     offset += last_cyclic_segment_is_vector(handle_types_left, handle_types_right) ? 1 :
                                                                                      resolution;
@@ -233,12 +236,11 @@ void evaluate_segment(const float3 &point_0,
 void calculate_evaluated_positions(const Span<float3> positions,
                                    const Span<float3> handles_left,
                                    const Span<float3> handles_right,
-                                   const Span<int> evaluated_offsets,
+                                   const OffsetIndices<int> evaluated_offsets,
                                    MutableSpan<float3> evaluated_positions)
 {
-  BLI_assert(evaluated_offsets.last() == evaluated_positions.size());
-  BLI_assert(evaluated_offsets.size() == positions.size());
-  if (evaluated_offsets.last() == 1) {
+  BLI_assert(evaluated_offsets.total_size() == evaluated_positions.size());
+  if (evaluated_offsets.total_size() == 1) {
     evaluated_positions.first() = positions.first();
     return;
   }
@@ -248,29 +250,29 @@ void calculate_evaluated_positions(const Span<float3> positions,
                    handles_right.first(),
                    handles_left[1],
                    positions[1],
-                   evaluated_positions.take_front(evaluated_offsets.first()));
+                   evaluated_positions.slice(evaluated_offsets[0]));
 
   /* Give each task fewer segments as the resolution gets larger. */
   const int grain_size = std::max<int>(evaluated_positions.size() / positions.size() * 32, 1);
-  threading::parallel_for(
-      positions.index_range().drop_back(1).drop_front(1), grain_size, [&](IndexRange range) {
-        for (const int i : range) {
-          const IndexRange evaluated_range = offsets_to_range(evaluated_offsets, i - 1);
-          if (evaluated_range.size() == 1) {
-            evaluated_positions[evaluated_range.first()] = positions[i];
-          }
-          else {
-            evaluate_segment(positions[i],
-                             handles_right[i],
-                             handles_left[i + 1],
-                             positions[i + 1],
-                             evaluated_positions.slice(evaluated_range));
-          }
-        }
-      });
+  const IndexRange inner_segments = positions.index_range().drop_back(1).drop_front(1);
+  threading::parallel_for(inner_segments, grain_size, [&](IndexRange range) {
+    for (const int i : range) {
+      const IndexRange evaluated_range = evaluated_offsets[i];
+      if (evaluated_range.size() == 1) {
+        evaluated_positions[evaluated_range.first()] = positions[i];
+      }
+      else {
+        evaluate_segment(positions[i],
+                         handles_right[i],
+                         handles_left[i + 1],
+                         positions[i + 1],
+                         evaluated_positions.slice(evaluated_range));
+      }
+    }
+  });
 
   /* Evaluate the final cyclic segment if necessary. */
-  const IndexRange last_segment_points = offsets_to_range(evaluated_offsets, positions.size() - 2);
+  const IndexRange last_segment_points = evaluated_offsets[positions.index_range().last()];
   if (last_segment_points.size() == 1) {
     evaluated_positions.last() = positions.last();
   }
@@ -295,34 +297,34 @@ static inline void linear_interpolation(const T &a, const T &b, MutableSpan<T> d
 
 template<typename T>
 static void interpolate_to_evaluated(const Span<T> src,
-                                     const Span<int> evaluated_offsets,
+                                     const OffsetIndices<int> evaluated_offsets,
                                      MutableSpan<T> dst)
 {
   BLI_assert(!src.is_empty());
-  BLI_assert(evaluated_offsets.size() == src.size());
-  BLI_assert(evaluated_offsets.last() == dst.size());
+  BLI_assert(evaluated_offsets.total_size() == dst.size());
   if (src.size() == 1) {
     BLI_assert(dst.size() == 1);
     dst.first() = src.first();
     return;
   }
 
-  linear_interpolation(src.first(), src[1], dst.take_front(evaluated_offsets.first()));
+  linear_interpolation(src.first(), src[1], dst.slice(evaluated_offsets[0]));
 
   threading::parallel_for(
       src.index_range().drop_back(1).drop_front(1), 512, [&](IndexRange range) {
         for (const int i : range) {
-          const IndexRange segment_points = offsets_to_range(evaluated_offsets, i - 1);
-          linear_interpolation(src[i], src[i + 1], dst.slice(segment_points));
+          const IndexRange segment = evaluated_offsets[i];
+          linear_interpolation(src[i], src[i + 1], dst.slice(segment));
         }
       });
 
-  const IndexRange last_segment_points(evaluated_offsets.last(1),
-                                       evaluated_offsets.last() - evaluated_offsets.last(1));
-  linear_interpolation(src.last(), src.first(), dst.slice(last_segment_points));
+  const IndexRange last_segment = evaluated_offsets[src.index_range().last()];
+  linear_interpolation(src.last(), src.first(), dst.slice(last_segment));
 }
 
-void interpolate_to_evaluated(const GSpan src, const Span<int> evaluated_offsets, GMutableSpan dst)
+void interpolate_to_evaluated(const GSpan src,
+                              const OffsetIndices<int> evaluated_offsets,
+                              GMutableSpan dst)
 {
   attribute_math::convert_to_static_type(src.type(), [&](auto dummy) {
     using T = decltype(dummy);
