@@ -227,7 +227,7 @@ static AdrBit2Path ob_layer_bits[] = {
   } \
   (void)0
 
-/* This function checks if a Blocktype+Adrcode combo, returning a mapping table */
+/* This function checks if a `blocktype+adrcode` combination, returning a mapping table. */
 static AdrBit2Path *adrcode_bitmaps_to_paths(int blocktype, int adrcode, int *tot)
 {
   /* Object layers */
@@ -445,7 +445,7 @@ static char *shapekey_adrcodes_to_paths(ID *id, int adrcode, int *UNUSED(array_i
   /* block will be attached to ID_KE block... */
   if (adrcode == 0) {
     /* adrcode=0 was the misnamed "speed" curve (now "evaluation time") */
-    BLI_strncpy(buf, "eval_time", sizeof(buf));
+    STRNCPY(buf, "eval_time");
   }
   else {
     /* Find the name of the ShapeKey (i.e. KeyBlock) to look for */
@@ -457,11 +457,11 @@ static char *shapekey_adrcodes_to_paths(ID *id, int adrcode, int *UNUSED(array_i
       /* Use the keyblock name, escaped, so that path lookups for this will work */
       char kb_name_esc[sizeof(kb->name) * 2];
       BLI_str_escape(kb_name_esc, kb->name, sizeof(kb_name_esc));
-      BLI_snprintf(buf, sizeof(buf), "key_blocks[\"%s\"].value", kb_name_esc);
+      SNPRINTF(buf, "key_blocks[\"%s\"].value", kb_name_esc);
     }
     else {
       /* Fallback - Use the adrcode as index directly, so that this can be manually fixed */
-      BLI_snprintf(buf, sizeof(buf), "key_blocks[%d].value", adrcode);
+      SNPRINTF(buf, "key_blocks[%d].value", adrcode);
     }
   }
   return buf;
@@ -580,7 +580,7 @@ static const char *mtex_adrcodes_to_paths(int adrcode, int *UNUSED(array_index))
 
   /* only build and return path if there's a property */
   if (prop) {
-    BLI_snprintf(buf, 128, "%s.%s", base, prop);
+    SNPRINTF(buf, "%s.%s", base, prop);
     return buf;
   }
 
@@ -1154,8 +1154,7 @@ static char *get_rna_access(ID *id,
     char constname_esc[sizeof(((bConstraint *)NULL)->name) * 2];
     BLI_str_escape(actname_esc, actname, sizeof(actname_esc));
     BLI_str_escape(constname_esc, constname, sizeof(constname_esc));
-    BLI_snprintf(
-        buf, sizeof(buf), "pose.bones[\"%s\"].constraints[\"%s\"]", actname_esc, constname_esc);
+    SNPRINTF(buf, "pose.bones[\"%s\"].constraints[\"%s\"]", actname_esc, constname_esc);
   }
   else if (actname && actname[0]) {
     if ((blocktype == ID_OB) && STREQ(actname, "Object")) {
@@ -1171,20 +1170,20 @@ static char *get_rna_access(ID *id,
       /* Pose-Channel */
       char actname_esc[sizeof(((bActionChannel *)NULL)->name) * 2];
       BLI_str_escape(actname_esc, actname, sizeof(actname_esc));
-      BLI_snprintf(buf, sizeof(buf), "pose.bones[\"%s\"]", actname_esc);
+      SNPRINTF(buf, "pose.bones[\"%s\"]", actname_esc);
     }
   }
   else if (constname && constname[0]) {
     /* Constraint in Object */
     char constname_esc[sizeof(((bConstraint *)NULL)->name) * 2];
     BLI_str_escape(constname_esc, constname, sizeof(constname_esc));
-    BLI_snprintf(buf, sizeof(buf), "constraints[\"%s\"]", constname_esc);
+    SNPRINTF(buf, "constraints[\"%s\"]", constname_esc);
   }
   else if (seq) {
     /* Sequence names in Scene */
     char seq_name_esc[(sizeof(seq->name) - 2) * 2];
     BLI_str_escape(seq_name_esc, seq->name + 2, sizeof(seq_name_esc));
-    BLI_snprintf(buf, sizeof(buf), "sequence_editor.sequences_all[\"%s\"]", seq_name_esc);
+    SNPRINTF(buf, "sequence_editor.sequences_all[\"%s\"]", seq_name_esc);
   }
   else {
     buf[0] = '\0'; /* empty string */
@@ -1202,7 +1201,7 @@ static char *get_rna_access(ID *id,
 
   /* if there was no array index pointer provided, add it to the path */
   if (array_index == NULL) {
-    BLI_snprintf(buf, sizeof(buf), "[\"%d\"]", dummy_index);
+    SNPRINTF(buf, "[\"%d\"]", dummy_index);
     BLI_dynstr_append(path, buf);
   }
 
@@ -1261,7 +1260,7 @@ static ChannelDriver *idriver_to_cdriver(IpoDriver *idriver)
     /* FIXME: expression will be useless due to API changes, but at least not totally lost */
     cdriver->type = DRIVER_TYPE_PYTHON;
     if (idriver->name[0]) {
-      BLI_strncpy(cdriver->expression, idriver->name, sizeof(cdriver->expression));
+      STRNCPY(cdriver->expression, idriver->name);
     }
   }
   else {
@@ -1283,7 +1282,7 @@ static ChannelDriver *idriver_to_cdriver(IpoDriver *idriver)
         dtar->id = (ID *)idriver->ob;
         dtar->idtype = ID_OB;
         if (idriver->name[0]) {
-          BLI_strncpy(dtar->pchan_name, idriver->name, sizeof(dtar->pchan_name));
+          STRNCPY(dtar->pchan_name, idriver->name);
         }
 
         /* second bone target (name was stored in same var as the first one) */
@@ -1291,8 +1290,7 @@ static ChannelDriver *idriver_to_cdriver(IpoDriver *idriver)
         dtar->id = (ID *)idriver->ob;
         dtar->idtype = ID_OB;
         if (idriver->name[0]) { /* XXX: for safety. */
-          BLI_strncpy(
-              dtar->pchan_name, idriver->name + DRIVER_NAME_OFFS, sizeof(dtar->pchan_name));
+          STRNCPY(dtar->pchan_name, idriver->name + DRIVER_NAME_OFFS);
         }
       }
       else {
@@ -1305,7 +1303,7 @@ static ChannelDriver *idriver_to_cdriver(IpoDriver *idriver)
         dtar->id = (ID *)idriver->ob;
         dtar->idtype = ID_OB;
         if (idriver->name[0]) {
-          BLI_strncpy(dtar->pchan_name, idriver->name, sizeof(dtar->pchan_name));
+          STRNCPY(dtar->pchan_name, idriver->name);
         }
         dtar->transChan = adrcode_to_dtar_transchan(idriver->adrcode);
         dtar->flag |= DTAR_FLAG_LOCALSPACE; /* old drivers took local space */
@@ -1362,7 +1360,7 @@ static void fcurve_add_to_list(
         agrp->flag |= AGRP_MUTED;
       }
 
-      BLI_strncpy(agrp->name, grpname, sizeof(agrp->name));
+      STRNCPY(agrp->name, grpname);
 
       BLI_addtail(&tmp_act.groups, agrp);
       BLI_uniquename(&tmp_act.groups,
@@ -1614,7 +1612,8 @@ static void icu_to_fcurves(ID *id,
          * - we need radians for RNA to do the right thing
          */
         if (((icu->blocktype == ID_OB) && ELEM(icu->adrcode, OB_ROT_X, OB_ROT_Y, OB_ROT_Z)) ||
-            ((icu->blocktype == ID_PO) && ELEM(icu->adrcode, AC_EUL_X, AC_EUL_Y, AC_EUL_Z))) {
+            ((icu->blocktype == ID_PO) && ELEM(icu->adrcode, AC_EUL_X, AC_EUL_Y, AC_EUL_Z)))
+        {
           const float fac = (float)M_PI / 18.0f; /* 10.0f * M_PI/180.0f; */
 
           dst->vec[0][1] *= fac;
@@ -1627,7 +1626,8 @@ static void icu_to_fcurves(ID *id,
          * - we now need as 'frames'
          */
         if ((id) && (icu->blocktype == GS(id->name)) &&
-            (fcu->rna_path && STREQ(fcu->rna_path, "eval_time"))) {
+            (fcu->rna_path && STREQ(fcu->rna_path, "eval_time")))
+        {
           Curve *cu = (Curve *)id;
 
           dst->vec[0][1] *= cu->pathlen;
@@ -1644,10 +1644,8 @@ static void icu_to_fcurves(ID *id,
           DriverVar *dvar = fcu->driver->variables.first;
           DriverTarget *dtar = &dvar->targets[0];
 
-          if (ELEM(dtar->transChan,
-                   DTAR_TRANSCHAN_ROTX,
-                   DTAR_TRANSCHAN_ROTY,
-                   DTAR_TRANSCHAN_ROTZ)) {
+          if (ELEM(dtar->transChan, DTAR_TRANSCHAN_ROTX, DTAR_TRANSCHAN_ROTY, DTAR_TRANSCHAN_ROTZ))
+          {
             const float fac = (float)M_PI / 18.0f;
 
             dst->vec[0][0] *= fac;
@@ -1876,7 +1874,7 @@ static void ipo_to_animdata(
     if (adt->action == NULL) {
       char nameBuf[MAX_ID_NAME];
 
-      BLI_snprintf(nameBuf, sizeof(nameBuf), "CDA:%s", ipo->id.name + 2);
+      SNPRINTF(nameBuf, "CDA:%s", ipo->id.name + 2);
 
       adt->action = BKE_action_add(bmain, nameBuf);
       if (G.debug & G_DEBUG) {
@@ -2005,7 +2003,8 @@ static void nlastrips_to_animdata(ID *id, ListBase *strips)
         /* trying to add to the current failed (no space),
          * so add a new track to the stack, and add to that...
          */
-        nlt = BKE_nlatrack_add(adt, NULL, false);
+        nlt = BKE_nlatrack_new_tail(&adt->nla_tracks, false);
+        BKE_nlatrack_set_active(&adt->nla_tracks, nlt);
         BKE_nlatrack_add_strip(nlt, strip, false);
       }
 
