@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
-#include "BKE_node.h"
+#include "BKE_node.hh"
 #include "BKE_node_runtime.hh"
 
 #include "DNA_node_types.h"
@@ -22,38 +22,26 @@ void preprocess_geometry_node_tree_for_evaluation(bNodeTree &tree_cow)
   blender::nodes::ensure_geometry_nodes_lazy_function_graph(tree_cow);
 }
 
-static void double_checked_lock(std::mutex &mutex, bool &data_is_dirty, FunctionRef<void()> fn)
+static void update_interface_sockets(const bNodeTree &ntree)
 {
-  if (!data_is_dirty) {
-    return;
-  }
-  std::lock_guard lock{mutex};
-  if (!data_is_dirty) {
-    return;
-  }
-  fn();
-  data_is_dirty = false;
-}
-
-static void double_checked_lock_with_task_isolation(std::mutex &mutex,
-                                                    bool &data_is_dirty,
-                                                    FunctionRef<void()> fn)
-{
-  double_checked_lock(mutex, data_is_dirty, [&]() { threading::isolate_task(fn); });
+  bNodeTreeRuntime &tree_runtime = *ntree.runtime;
+  tree_runtime.interface_inputs = ntree.inputs;
+  tree_runtime.interface_outputs = ntree.outputs;
 }
 
 static void update_node_vector(const bNodeTree &ntree)
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
-  tree_runtime.nodes.clear();
+  const Span<bNode *> nodes = tree_runtime.nodes_by_id;
   tree_runtime.group_nodes.clear();
   tree_runtime.has_undefined_nodes_or_sockets = false;
-  LISTBASE_FOREACH (bNode *, node, &ntree.nodes) {
-    node->runtime->index_in_tree = tree_runtime.nodes.append_and_get_index(node);
-    node->runtime->owner_tree = const_cast<bNodeTree *>(&ntree);
-    tree_runtime.has_undefined_nodes_or_sockets |= node->typeinfo == &NodeTypeUndefined;
-    if (node->is_group()) {
-      tree_runtime.group_nodes.append(node);
+  for (const int i : nodes.index_range()) {
+    bNode &node = *nodes[i];
+    node.runtime->index_in_tree = i;
+    node.runtime->owner_tree = const_cast<bNodeTree *>(&ntree);
+    tree_runtime.has_undefined_nodes_or_sockets |= node.typeinfo == &bke::NodeTypeUndefined;
+    if (node.is_group()) {
+      tree_runtime.group_nodes.append(&node);
     }
   }
 }
@@ -63,22 +51,11 @@ static void update_link_vector(const bNodeTree &ntree)
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
   tree_runtime.links.clear();
   LISTBASE_FOREACH (bNodeLink *, link, &ntree.links) {
-    tree_runtime.links.append(link);
-  }
-}
+    /* Check that the link connects nodes within this tree. */
+    BLI_assert(tree_runtime.nodes_by_id.contains(link->fromnode));
+    BLI_assert(tree_runtime.nodes_by_id.contains(link->tonode));
 
-static void update_internal_links(const bNodeTree &ntree)
-{
-  bNodeTreeRuntime &tree_runtime = *ntree.runtime;
-  for (bNode *node : tree_runtime.nodes) {
-    node->runtime->internal_links.clear();
-    for (bNodeSocket *socket : node->runtime->outputs) {
-      socket->runtime->internal_link_input = nullptr;
-    }
-    LISTBASE_FOREACH (bNodeLink *, link, &node->internal_links) {
-      node->runtime->internal_links.append(link);
-      link->tosock->runtime->internal_link_input = link->fromsock;
-    }
+    tree_runtime.links.append(link);
   }
 }
 
@@ -88,7 +65,7 @@ static void update_socket_vectors_and_owner_node(const bNodeTree &ntree)
   tree_runtime.sockets.clear();
   tree_runtime.input_sockets.clear();
   tree_runtime.output_sockets.clear();
-  for (bNode *node : tree_runtime.nodes) {
+  for (bNode *node : tree_runtime.nodes_by_id) {
     bNodeRuntime &node_runtime = *node->runtime;
     node_runtime.inputs.clear();
     node_runtime.outputs.clear();
@@ -98,7 +75,8 @@ static void update_socket_vectors_and_owner_node(const bNodeTree &ntree)
       socket->runtime->index_in_inout_sockets = tree_runtime.input_sockets.append_and_get_index(
           socket);
       socket->runtime->owner_node = node;
-      tree_runtime.has_undefined_nodes_or_sockets |= socket->typeinfo == &NodeSocketTypeUndefined;
+      tree_runtime.has_undefined_nodes_or_sockets |= socket->typeinfo ==
+                                                     &bke::NodeSocketTypeUndefined;
     }
     LISTBASE_FOREACH (bNodeSocket *, socket, &node->outputs) {
       socket->runtime->index_in_node = node_runtime.outputs.append_and_get_index(socket);
@@ -106,7 +84,21 @@ static void update_socket_vectors_and_owner_node(const bNodeTree &ntree)
       socket->runtime->index_in_inout_sockets = tree_runtime.output_sockets.append_and_get_index(
           socket);
       socket->runtime->owner_node = node;
-      tree_runtime.has_undefined_nodes_or_sockets |= socket->typeinfo == &NodeSocketTypeUndefined;
+      tree_runtime.has_undefined_nodes_or_sockets |= socket->typeinfo ==
+                                                     &bke::NodeSocketTypeUndefined;
+    }
+  }
+}
+
+static void update_internal_link_inputs(const bNodeTree &ntree)
+{
+  bNodeTreeRuntime &tree_runtime = *ntree.runtime;
+  for (bNode *node : tree_runtime.nodes_by_id) {
+    for (bNodeSocket *socket : node->runtime->outputs) {
+      socket->runtime->internal_link_input = nullptr;
+    }
+    for (bNodeLink &link : node->runtime->internal_links) {
+      link.tosock->runtime->internal_link_input = link.fromsock;
     }
   }
 }
@@ -114,7 +106,7 @@ static void update_socket_vectors_and_owner_node(const bNodeTree &ntree)
 static void update_directly_linked_links_and_sockets(const bNodeTree &ntree)
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
-  for (bNode *node : tree_runtime.nodes) {
+  for (bNode *node : tree_runtime.nodes_by_id) {
     for (bNodeSocket *socket : node->runtime->inputs) {
       socket->runtime->directly_linked_links.clear();
       socket->runtime->directly_linked_sockets.clear();
@@ -209,9 +201,10 @@ static void find_logical_origins_for_socket_recursive(
 static void update_logical_origins(const bNodeTree &ntree)
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
-  threading::parallel_for(tree_runtime.nodes.index_range(), 128, [&](const IndexRange range) {
+  Span<bNode *> nodes = tree_runtime.nodes_by_id;
+  threading::parallel_for(nodes.index_range(), 128, [&](const IndexRange range) {
     for (const int i : range) {
-      bNode &node = *tree_runtime.nodes[i];
+      bNode &node = *nodes[i];
       for (bNodeSocket *socket : node.runtime->inputs) {
         Vector<bNodeSocket *, 16> sockets_in_current_chain;
         socket->runtime->logically_linked_sockets.clear();
@@ -231,7 +224,7 @@ static void update_nodes_by_type(const bNodeTree &ntree)
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
   tree_runtime.nodes_by_type.clear();
-  for (bNode *node : tree_runtime.nodes) {
+  for (bNode *node : tree_runtime.nodes_by_id) {
     tree_runtime.nodes_by_type.add(node->typeinfo, node);
   }
 }
@@ -239,8 +232,9 @@ static void update_nodes_by_type(const bNodeTree &ntree)
 static void update_sockets_by_identifier(const bNodeTree &ntree)
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
-  threading::parallel_for(tree_runtime.nodes.index_range(), 128, [&](const IndexRange range) {
-    for (bNode *node : tree_runtime.nodes.as_span().slice(range)) {
+  Span<bNode *> nodes = tree_runtime.nodes_by_id;
+  threading::parallel_for(nodes.index_range(), 128, [&](const IndexRange range) {
+    for (bNode *node : nodes.slice(range)) {
       node->runtime->inputs_by_identifier.clear();
       node->runtime->outputs_by_identifier.clear();
       for (bNodeSocket *socket : node->runtime->inputs) {
@@ -263,7 +257,37 @@ struct ToposortNodeState {
   bool is_in_stack = false;
 };
 
-static void toposort_from_start_node(const ToposortDirection direction,
+static Vector<const bNode *> get_implicit_origin_nodes(const bNodeTree &ntree, bNode &node)
+{
+  Vector<const bNode *> origin_nodes;
+  if (node.type == GEO_NODE_SIMULATION_OUTPUT) {
+    for (const bNode *sim_input_node :
+         ntree.runtime->nodes_by_type.lookup(nodeTypeFind("GeometryNodeSimulationInput")))
+    {
+      const auto &storage = *static_cast<const NodeGeometrySimulationInput *>(
+          sim_input_node->storage);
+      if (storage.output_node_id == node.identifier) {
+        origin_nodes.append(sim_input_node);
+      }
+    }
+  }
+  return origin_nodes;
+}
+
+static Vector<const bNode *> get_implicit_target_nodes(const bNodeTree &ntree, bNode &node)
+{
+  Vector<const bNode *> target_nodes;
+  if (node.type == GEO_NODE_SIMULATION_INPUT) {
+    const auto &storage = *static_cast<const NodeGeometrySimulationInput *>(node.storage);
+    if (const bNode *sim_output_node = ntree.node_by_id(storage.output_node_id)) {
+      target_nodes.append(sim_output_node);
+    }
+  }
+  return target_nodes;
+}
+
+static void toposort_from_start_node(const bNodeTree &ntree,
+                                     const ToposortDirection direction,
                                      bNode &start_node,
                                      MutableSpan<ToposortNodeState> node_states,
                                      Vector<bNode *> &r_sorted_nodes,
@@ -273,14 +297,34 @@ static void toposort_from_start_node(const ToposortDirection direction,
     bNode *node;
     int socket_index = 0;
     int link_index = 0;
+    int implicit_link_index = 0;
   };
 
   Stack<Item, 64> nodes_to_check;
   nodes_to_check.push({&start_node});
-  node_states[start_node.runtime->index_in_tree].is_in_stack = true;
+  node_states[start_node.index()].is_in_stack = true;
   while (!nodes_to_check.is_empty()) {
     Item &item = nodes_to_check.peek();
     bNode &node = *item.node;
+    bool pushed_node = false;
+
+    auto handle_linked_node = [&](bNode &linked_node) {
+      ToposortNodeState &linked_node_state = node_states[linked_node.index()];
+      if (linked_node_state.is_done) {
+        /* The linked node has already been visited. */
+        return true;
+      }
+      if (linked_node_state.is_in_stack) {
+        r_cycle_detected = true;
+      }
+      else {
+        nodes_to_check.push({&linked_node});
+        linked_node_state.is_in_stack = true;
+        pushed_node = true;
+      }
+      return false;
+    };
+
     const Span<bNodeSocket *> sockets = (direction == ToposortDirection::LeftToRight) ?
                                             node.runtime->inputs :
                                             node.runtime->outputs;
@@ -305,25 +349,39 @@ static void toposort_from_start_node(const ToposortDirection direction,
       }
       bNodeSocket &linked_socket = *socket.runtime->directly_linked_sockets[item.link_index];
       bNode &linked_node = *linked_socket.runtime->owner_node;
-      ToposortNodeState &linked_node_state = node_states[linked_node.runtime->index_in_tree];
-      if (linked_node_state.is_done) {
+      if (handle_linked_node(linked_node)) {
         /* The linked node has already been visited. */
         item.link_index++;
         continue;
       }
-      if (linked_node_state.is_in_stack) {
-        r_cycle_detected = true;
-      }
-      else {
-        nodes_to_check.push({&linked_node});
-        linked_node_state.is_in_stack = true;
-      }
       break;
     }
 
-    /* If no other element has been pushed, the current node can be pushed to the sorted list. */
-    if (&item == &nodes_to_check.peek()) {
-      ToposortNodeState &node_state = node_states[node.runtime->index_in_tree];
+    if (!pushed_node) {
+      /* Some nodes are internally linked without an explicit `bNodeLink`. The toposort should
+       * still order them correctly and find cycles. */
+      const Vector<const bNode *> implicitly_linked_nodes =
+          (direction == ToposortDirection::LeftToRight) ? get_implicit_origin_nodes(ntree, node) :
+                                                          get_implicit_target_nodes(ntree, node);
+      while (true) {
+        if (item.implicit_link_index == implicitly_linked_nodes.size()) {
+          /* All implicitly linked nodes have already been visited. */
+          break;
+        }
+        const bNode &linked_node = *implicitly_linked_nodes[item.implicit_link_index];
+        if (handle_linked_node(const_cast<bNode &>(linked_node))) {
+          /* The implicitly linked node has already been visited. */
+          item.implicit_link_index++;
+          continue;
+        }
+        break;
+      }
+    }
+
+    /* If no other element has been pushed, the current node can be pushed to the sorted list.
+     */
+    if (!pushed_node) {
+      ToposortNodeState &node_state = node_states[node.index()];
       node_state.is_done = true;
       node_state.is_in_stack = false;
       r_sorted_nodes.append(&node);
@@ -339,37 +397,70 @@ static void update_toposort(const bNodeTree &ntree,
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
   r_sorted_nodes.clear();
-  r_sorted_nodes.reserve(tree_runtime.nodes.size());
+  r_sorted_nodes.reserve(tree_runtime.nodes_by_id.size());
   r_cycle_detected = false;
 
-  Array<ToposortNodeState> node_states(tree_runtime.nodes.size());
-  for (bNode *node : tree_runtime.nodes) {
-    if (node_states[node->runtime->index_in_tree].is_done) {
+  Array<ToposortNodeState> node_states(tree_runtime.nodes_by_id.size());
+  for (bNode *node : tree_runtime.nodes_by_id) {
+    if (node_states[node->index()].is_done) {
       /* Ignore nodes that are done already. */
       continue;
     }
     if ((direction == ToposortDirection::LeftToRight) ?
             node->runtime->has_available_linked_outputs :
-            node->runtime->has_available_linked_inputs) {
+            node->runtime->has_available_linked_inputs)
+    {
       /* Ignore non-start nodes. */
       continue;
     }
-    toposort_from_start_node(direction, *node, node_states, r_sorted_nodes, r_cycle_detected);
+    toposort_from_start_node(
+        ntree, direction, *node, node_states, r_sorted_nodes, r_cycle_detected);
   }
 
-  if (r_sorted_nodes.size() < tree_runtime.nodes.size()) {
+  if (r_sorted_nodes.size() < tree_runtime.nodes_by_id.size()) {
     r_cycle_detected = true;
-    for (bNode *node : tree_runtime.nodes) {
-      if (node_states[node->runtime->index_in_tree].is_done) {
+    for (bNode *node : tree_runtime.nodes_by_id) {
+      if (node_states[node->index()].is_done) {
         /* Ignore nodes that are done already. */
         continue;
       }
       /* Start toposort at this node which is somewhere in the middle of a loop. */
-      toposort_from_start_node(direction, *node, node_states, r_sorted_nodes, r_cycle_detected);
+      toposort_from_start_node(
+          ntree, direction, *node, node_states, r_sorted_nodes, r_cycle_detected);
     }
   }
 
-  BLI_assert(tree_runtime.nodes.size() == r_sorted_nodes.size());
+  BLI_assert(tree_runtime.nodes_by_id.size() == r_sorted_nodes.size());
+}
+
+static void update_root_frames(const bNodeTree &ntree)
+{
+  bNodeTreeRuntime &tree_runtime = *ntree.runtime;
+  Span<bNode *> nodes = tree_runtime.nodes_by_id;
+
+  tree_runtime.root_frames.clear();
+
+  for (bNode *node : nodes) {
+    if (!node->parent && node->is_frame()) {
+      tree_runtime.root_frames.append(node);
+    }
+  }
+}
+
+static void update_direct_frames_childrens(const bNodeTree &ntree)
+{
+  bNodeTreeRuntime &tree_runtime = *ntree.runtime;
+  Span<bNode *> nodes = tree_runtime.nodes_by_id;
+
+  for (bNode *node : nodes) {
+    node->runtime->direct_children_in_frame.clear();
+  }
+
+  for (bNode *node : nodes) {
+    if (const bNode *frame = node->parent) {
+      frame->runtime->direct_children_in_frame.append(node);
+    }
+  }
 }
 
 static void update_group_output_node(const bNodeTree &ntree)
@@ -396,32 +487,34 @@ static void update_group_output_node(const bNodeTree &ntree)
 static void ensure_topology_cache(const bNodeTree &ntree)
 {
   bNodeTreeRuntime &tree_runtime = *ntree.runtime;
-  double_checked_lock_with_task_isolation(
-      tree_runtime.topology_cache_mutex, tree_runtime.topology_cache_is_dirty, [&]() {
-        update_node_vector(ntree);
-        update_link_vector(ntree);
-        update_socket_vectors_and_owner_node(ntree);
-        update_internal_links(ntree);
-        update_directly_linked_links_and_sockets(ntree);
-        threading::parallel_invoke([&]() { update_logical_origins(ntree); },
-                                   [&]() { update_nodes_by_type(ntree); },
-                                   [&]() { update_sockets_by_identifier(ntree); },
-                                   [&]() {
-                                     update_toposort(ntree,
-                                                     ToposortDirection::LeftToRight,
-                                                     tree_runtime.toposort_left_to_right,
-                                                     tree_runtime.has_available_link_cycle);
-                                   },
-                                   [&]() {
-                                     bool dummy;
-                                     update_toposort(ntree,
-                                                     ToposortDirection::RightToLeft,
-                                                     tree_runtime.toposort_right_to_left,
-                                                     dummy);
-                                   });
-        update_group_output_node(ntree);
-        tree_runtime.topology_cache_exists = true;
-      });
+  tree_runtime.topology_cache_mutex.ensure([&]() {
+    update_interface_sockets(ntree);
+    update_node_vector(ntree);
+    update_link_vector(ntree);
+    update_socket_vectors_and_owner_node(ntree);
+    update_internal_link_inputs(ntree);
+    update_directly_linked_links_and_sockets(ntree);
+    update_nodes_by_type(ntree);
+    threading::parallel_invoke(
+        tree_runtime.nodes_by_id.size() > 32,
+        [&]() { update_logical_origins(ntree); },
+        [&]() { update_sockets_by_identifier(ntree); },
+        [&]() {
+          update_toposort(ntree,
+                          ToposortDirection::LeftToRight,
+                          tree_runtime.toposort_left_to_right,
+                          tree_runtime.has_available_link_cycle);
+        },
+        [&]() {
+          bool dummy;
+          update_toposort(
+              ntree, ToposortDirection::RightToLeft, tree_runtime.toposort_right_to_left, dummy);
+        },
+        [&]() { update_root_frames(ntree); },
+        [&]() { update_direct_frames_childrens(ntree); });
+    update_group_output_node(ntree);
+    tree_runtime.topology_cache_exists = true;
+  });
 }
 
 }  // namespace blender::bke::node_tree_runtime
