@@ -174,16 +174,16 @@ PackedFile *BKE_packedfile_new_from_memory(void *mem, int memlen)
   return pf;
 }
 
-PackedFile *BKE_packedfile_new(ReportList *reports, const char *filepath, const char *basepath)
+PackedFile *BKE_packedfile_new(ReportList *reports, const char *filepath_rel, const char *basepath)
 {
   PackedFile *pf = NULL;
   int file, filelen;
-  char name[FILE_MAX];
+  char filepath[FILE_MAX];
   void *data;
 
   /* render result has no filepath and can be ignored
    * any other files with no name can be ignored too */
-  if (filepath[0] == '\0') {
+  if (filepath_rel[0] == '\0') {
     return pf;
   }
 
@@ -191,15 +191,15 @@ PackedFile *BKE_packedfile_new(ReportList *reports, const char *filepath, const
 
   /* convert relative filenames to absolute filenames */
 
-  BLI_strncpy(name, filepath, sizeof(name));
-  BLI_path_abs(name, basepath);
+  STRNCPY(filepath, filepath_rel);
+  BLI_path_abs(filepath, basepath);
 
   /* open the file
    * and create a PackedFile structure */
 
-  file = BLI_open(name, O_BINARY | O_RDONLY, 0);
+  file = BLI_open(filepath, O_BINARY | O_RDONLY, 0);
   if (file == -1) {
-    BKE_reportf(reports, RPT_ERROR, "Unable to pack file, source path '%s' not found", name);
+    BKE_reportf(reports, RPT_ERROR, "Unable to pack file, source path '%s' not found", filepath);
   }
   else {
     filelen = BLI_file_descriptor_size(file);
@@ -251,8 +251,8 @@ void BKE_packedfile_pack_all(Main *bmain, ReportList *reports, bool verbose)
   }
 
   for (vfont = bmain->fonts.first; vfont; vfont = vfont->id.next) {
-    if (vfont->packedfile == NULL && !ID_IS_LINKED(vfont) &&
-        BKE_vfont_is_builtin(vfont) == false) {
+    if (vfont->packedfile == NULL && !ID_IS_LINKED(vfont) && BKE_vfont_is_builtin(vfont) == false)
+    {
       vfont->packedfile = BKE_packedfile_new(
           reports, vfont->filepath, BKE_main_blendfile_path(bmain));
       tot++;
@@ -285,28 +285,28 @@ void BKE_packedfile_pack_all(Main *bmain, ReportList *reports, bool verbose)
 
 int BKE_packedfile_write_to_file(ReportList *reports,
                                  const char *ref_file_name,
-                                 const char *filepath,
+                                 const char *filepath_rel,
                                  PackedFile *pf,
                                  const bool guimode)
 {
   int file, number;
   int ret_value = RET_OK;
   bool remove_tmp = false;
-  char name[FILE_MAX];
-  char tempname[FILE_MAX];
+  char filepath[FILE_MAX];
+  char filepath_temp[FILE_MAX];
   /*      void *data; */
 
   if (guimode) {
   }  // XXX  waitcursor(1);
 
-  BLI_strncpy(name, filepath, sizeof(name));
-  BLI_path_abs(name, ref_file_name);
+  STRNCPY(filepath, filepath_rel);
+  BLI_path_abs(filepath, ref_file_name);
 
-  if (BLI_exists(name)) {
+  if (BLI_exists(filepath)) {
     for (number = 1; number <= 999; number++) {
-      BLI_snprintf(tempname, sizeof(tempname), "%s.%03d_", name, number);
-      if (!BLI_exists(tempname)) {
-        if (BLI_copy(name, tempname) == RET_OK) {
+      SNPRINTF(filepath_temp, "%s.%03d_", filepath, number);
+      if (!BLI_exists(filepath_temp)) {
+        if (BLI_copy(filepath, filepath_temp) == RET_OK) {
           remove_tmp = true;
         }
         break;
@@ -314,21 +314,20 @@ int BKE_packedfile_write_to_file(ReportList *reports,
     }
   }
 
-  /* make sure the path to the file exists... */
-  BLI_make_existing_file(name);
+  BLI_file_ensure_parent_dir_exists(filepath);
 
-  file = BLI_open(name, O_BINARY + O_WRONLY + O_CREAT + O_TRUNC, 0666);
+  file = BLI_open(filepath, O_BINARY + O_WRONLY + O_CREAT + O_TRUNC, 0666);
   if (file == -1) {
-    BKE_reportf(reports, RPT_ERROR, "Error creating file '%s'", name);
+    BKE_reportf(reports, RPT_ERROR, "Error creating file '%s'", filepath);
     ret_value = RET_ERROR;
   }
   else {
     if (write(file, pf->data, pf->size) != pf->size) {
-      BKE_reportf(reports, RPT_ERROR, "Error writing file '%s'", name);
+      BKE_reportf(reports, RPT_ERROR, "Error writing file '%s'", filepath);
       ret_value = RET_ERROR;
     }
     else {
-      BKE_reportf(reports, RPT_INFO, "Saved packed file to: %s", name);
+      BKE_reportf(reports, RPT_INFO, "Saved packed file to: %s", filepath);
     }
 
     close(file);
@@ -336,17 +335,17 @@ int BKE_packedfile_write_to_file(ReportList *reports,
 
   if (remove_tmp) {
     if (ret_value == RET_ERROR) {
-      if (BLI_rename(tempname, name) != 0) {
+      if (BLI_rename_overwrite(filepath_temp, filepath) != 0) {
         BKE_reportf(reports,
                     RPT_ERROR,
                     "Error restoring temp file (check files '%s' '%s')",
-                    tempname,
-                    name);
+                    filepath_temp,
+                    filepath);
       }
     }
     else {
-      if (BLI_delete(tempname, false, false) != 0) {
-        BKE_reportf(reports, RPT_ERROR, "Error deleting '%s' (ignored)", tempname);
+      if (BLI_delete(filepath_temp, false, false) != 0) {
+        BKE_reportf(reports, RPT_ERROR, "Error deleting '%s' (ignored)", filepath_temp);
       }
     }
   }
@@ -358,18 +357,18 @@ int BKE_packedfile_write_to_file(ReportList *reports,
 }
 
 enum ePF_FileCompare BKE_packedfile_compare_to_file(const char *ref_file_name,
-                                                    const char *filepath,
+                                                    const char *filepath_rel,
                                                     PackedFile *pf)
 {
   BLI_stat_t st;
   enum ePF_FileCompare ret_val;
   char buf[4096];
-  char name[FILE_MAX];
+  char filepath[FILE_MAX];
 
-  BLI_strncpy(name, filepath, sizeof(name));
-  BLI_path_abs(name, ref_file_name);
+  STRNCPY(filepath, filepath_rel);
+  BLI_path_abs(filepath, ref_file_name);
 
-  if (BLI_stat(name, &st) == -1) {
+  if (BLI_stat(filepath, &st) == -1) {
     ret_val = PF_CMP_NOFILE;
   }
   else if (st.st_size != pf->size) {
@@ -378,7 +377,7 @@ enum ePF_FileCompare BKE_packedfile_compare_to_file(const char *ref_file_name,
   else {
     /* we'll have to compare the two... */
 
-    const int file = BLI_open(name, O_BINARY | O_RDONLY, 0);
+    const int file = BLI_open(filepath, O_BINARY | O_RDONLY, 0);
     if (file == -1) {
       ret_val = PF_CMP_NOFILE;
     }
@@ -430,7 +429,7 @@ char *BKE_packedfile_unpack_to_file(ReportList *reports,
       case PF_USE_LOCAL: {
         char temp_abs[FILE_MAX];
 
-        BLI_strncpy(temp_abs, local_name, sizeof(temp_abs));
+        STRNCPY(temp_abs, local_name);
         BLI_path_abs(temp_abs, ref_file_name);
 
         /* if file exists use it */
@@ -449,7 +448,7 @@ char *BKE_packedfile_unpack_to_file(ReportList *reports,
       case PF_USE_ORIGINAL: {
         char temp_abs[FILE_MAX];
 
-        BLI_strncpy(temp_abs, abs_name, sizeof(temp_abs));
+        STRNCPY(temp_abs, abs_name);
         BLI_path_abs(temp_abs, ref_file_name);
 
         /* if file exists use it */
@@ -479,23 +478,24 @@ char *BKE_packedfile_unpack_to_file(ReportList *reports,
   return newname;
 }
 
-static void unpack_generate_paths(const char *name,
+static void unpack_generate_paths(const char *filepath,
                                   ID *id,
                                   char *r_abspath,
+                                  size_t abspath_maxncpy,
                                   char *r_relpath,
-                                  size_t abspathlen,
-                                  size_t relpathlen)
+                                  size_t relpath_maxncpy)
 {
   const short id_type = GS(id->name);
-  char tempname[FILE_MAX];
-  char tempdir[FILE_MAXDIR];
+  char temp_filename[FILE_MAX];
+  char temp_dirname[FILE_MAXDIR];
 
-  BLI_split_dirfile(name, tempdir, tempname, sizeof(tempdir), sizeof(tempname));
+  BLI_path_split_dir_file(
+      filepath, temp_dirname, sizeof(temp_dirname), temp_filename, sizeof(temp_filename));
 
-  if (tempname[0] == '\0') {
+  if (temp_filename[0] == '\0') {
     /* NOTE: we generally do not have any real way to re-create extension out of data. */
-    const size_t len = BLI_strncpy_rlen(tempname, id->name + 2, sizeof(tempname));
-    printf("%s\n", tempname);
+    const size_t len = STRNCPY_RLEN(temp_filename, id->name + 2);
+    printf("%s\n", temp_filename);
 
     /* For images ensure that the temporary filename contains tile number information as well as
      * a file extension based on the file magic. */
@@ -507,23 +507,23 @@ static void unpack_generate_paths(const char *name,
         enum eImbFileType ftype = IMB_ispic_type_from_memory((const uchar *)pf->data, pf->size);
         if (ima->source == IMA_SRC_TILED) {
           char tile_number[6];
-          BLI_snprintf(tile_number, sizeof(tile_number), ".%d", imapf->tile_number);
-          BLI_strncpy(tempname + len, tile_number, sizeof(tempname) - len);
+          SNPRINTF(tile_number, ".%d", imapf->tile_number);
+          BLI_strncpy(temp_filename + len, tile_number, sizeof(temp_filename) - len);
         }
         if (ftype != IMB_FTYPE_NONE) {
           const int imtype = BKE_ftype_to_imtype(ftype, NULL);
-          BKE_image_path_ensure_ext_from_imtype(tempname, imtype);
+          BKE_image_path_ext_from_imtype_ensure(temp_filename, sizeof(temp_filename), imtype);
         }
       }
     }
 
-    BLI_filename_make_safe(tempname);
-    printf("%s\n", tempname);
+    BLI_path_make_safe_filename(temp_filename);
+    printf("%s\n", temp_filename);
   }
 
-  if (tempdir[0] == '\0') {
+  if (temp_dirname[0] == '\0') {
     /* Fallback to relative dir. */
-    BLI_strncpy(tempdir, "//", sizeof(tempdir));
+    STRNCPY(temp_dirname, "//");
   }
 
   {
@@ -545,13 +545,13 @@ static void unpack_generate_paths(const char *name,
         break;
     }
     if (dir_name) {
-      BLI_path_join(r_relpath, relpathlen, "//", dir_name, tempname);
+      BLI_path_join(r_relpath, relpath_maxncpy, "//", dir_name, temp_filename);
     }
   }
 
   {
-    size_t len = BLI_strncpy_rlen(r_abspath, tempdir, abspathlen);
-    BLI_strncpy(r_abspath + len, tempname, abspathlen - len);
+    size_t len = BLI_strncpy_rlen(r_abspath, temp_dirname, abspath_maxncpy);
+    BLI_strncpy(r_abspath + len, temp_filename, abspath_maxncpy - len);
   }
 }
 
@@ -567,7 +567,7 @@ char *BKE_packedfile_unpack(Main *bmain,
 
   if (id != NULL) {
     unpack_generate_paths(
-        orig_file_path, id, absname, localname, sizeof(absname), sizeof(localname));
+        orig_file_path, id, absname, sizeof(absname), localname, sizeof(localname));
     new_name = BKE_packedfile_unpack_to_file(
         reports, BKE_main_blendfile_path(bmain), absname, localname, pf, how);
   }
@@ -589,7 +589,7 @@ int BKE_packedfile_unpack_vfont(Main *bmain,
       ret_value = RET_OK;
       BKE_packedfile_free(vfont->packedfile);
       vfont->packedfile = NULL;
-      BLI_strncpy(vfont->filepath, new_file_path, sizeof(vfont->filepath));
+      STRNCPY(vfont->filepath, new_file_path);
       MEM_freeN(new_file_path);
     }
   }
@@ -608,7 +608,7 @@ int BKE_packedfile_unpack_sound(Main *bmain,
     char *new_file_path = BKE_packedfile_unpack(
         bmain, reports, (ID *)sound, sound->filepath, sound->packedfile, how);
     if (new_file_path != NULL) {
-      BLI_strncpy(sound->filepath, new_file_path, sizeof(sound->filepath));
+      STRNCPY(sound->filepath, new_file_path);
       MEM_freeN(new_file_path);
 
       BKE_packedfile_free(sound->packedfile);
@@ -646,16 +646,15 @@ int BKE_packedfile_unpack_image(Main *bmain,
         /* update the new corresponding view filepath */
         iv = BLI_findstring(&ima->views, imapf->filepath, offsetof(ImageView, filepath));
         if (iv) {
-          BLI_strncpy(iv->filepath, new_file_path, sizeof(imapf->filepath));
+          STRNCPY(iv->filepath, new_file_path);
         }
 
         /* keep the new name in the image for non-pack specific reasons */
         if (how != PF_REMOVE) {
-          BLI_strncpy(ima->filepath, new_file_path, sizeof(imapf->filepath));
+          STRNCPY(ima->filepath, new_file_path);
           if (ima->source == IMA_SRC_TILED) {
             /* Ensure that the Image filepath is kept in a tokenized format. */
-            char *filename = (char *)BLI_path_basename(ima->filepath);
-            BKE_image_ensure_tile_token(filename);
+            BKE_image_ensure_tile_token(ima->filepath, sizeof(ima->filepath));
           }
         }
         MEM_freeN(new_file_path);
@@ -687,7 +686,7 @@ int BKE_packedfile_unpack_volume(Main *bmain,
     char *new_file_path = BKE_packedfile_unpack(
         bmain, reports, (ID *)volume, volume->filepath, volume->packedfile, how);
     if (new_file_path != NULL) {
-      BLI_strncpy(volume->filepath, new_file_path, sizeof(volume->filepath));
+      STRNCPY(volume->filepath, new_file_path);
       MEM_freeN(new_file_path);
 
       BKE_packedfile_free(volume->packedfile);
@@ -878,7 +877,7 @@ void BKE_packedfile_blend_read(BlendDataReader *reader, PackedFile **pf_p)
   BLO_read_packed_address(reader, &pf->data);
   if (pf->data == NULL) {
     /* We cannot allow a PackedFile with a NULL data field,
-     * the whole code assumes this is not possible. See T70315. */
+     * the whole code assumes this is not possible. See #70315. */
     printf("%s: NULL packedfile data, cleaning up...\n", __func__);
     MEM_SAFE_FREE(pf);
   }
