@@ -14,7 +14,7 @@
 #include "BLI_bounds.hh"
 #include "BLI_index_range.hh"
 #include "BLI_listbase.h"
-#include "BLI_math_vec_types.hh"
+#include "BLI_math_vector.hh"
 #include "BLI_rand.h"
 #include "BLI_span.hh"
 #include "BLI_string.h"
@@ -64,24 +64,27 @@ static void pointcloud_init_data(ID *id)
   CustomData_reset(&pointcloud->pdata);
   CustomData_add_layer_named(&pointcloud->pdata,
                              CD_PROP_FLOAT3,
-                             CD_SET_DEFAULT,
-                             nullptr,
+                             CD_CONSTRUCT,
                              pointcloud->totpoint,
                              POINTCLOUD_ATTR_POSITION);
+
+  pointcloud->runtime = new blender::bke::PointCloudRuntime();
 }
 
-static void pointcloud_copy_data(Main * /*bmain*/, ID *id_dst, const ID *id_src, const int flag)
+static void pointcloud_copy_data(Main * /*bmain*/,
+                                 ID *id_dst,
+                                 const ID *id_src,
+                                 const int /*flag*/)
 {
   PointCloud *pointcloud_dst = (PointCloud *)id_dst;
   const PointCloud *pointcloud_src = (const PointCloud *)id_src;
   pointcloud_dst->mat = static_cast<Material **>(MEM_dupallocN(pointcloud_src->mat));
 
-  const eCDAllocType alloc_type = (flag & LIB_ID_COPY_CD_REFERENCE) ? CD_REFERENCE : CD_DUPLICATE;
-  CustomData_copy(&pointcloud_src->pdata,
-                  &pointcloud_dst->pdata,
-                  CD_MASK_ALL,
-                  alloc_type,
-                  pointcloud_dst->totpoint);
+  CustomData_copy(
+      &pointcloud_src->pdata, &pointcloud_dst->pdata, CD_MASK_ALL, pointcloud_dst->totpoint);
+
+  pointcloud_dst->runtime = new blender::bke::PointCloudRuntime();
+  pointcloud_dst->runtime->bounds_cache = pointcloud_src->runtime->bounds_cache;
 
   pointcloud_dst->batch_cache = nullptr;
 }
@@ -93,6 +96,7 @@ static void pointcloud_free_data(ID *id)
   BKE_pointcloud_batch_cache_free(pointcloud);
   CustomData_free(&pointcloud->pdata, pointcloud->totpoint);
   MEM_SAFE_FREE(pointcloud->mat);
+  delete pointcloud->runtime;
 }
 
 static void pointcloud_foreach_id(ID *id, LibraryForeachIDData *data)
@@ -139,6 +143,8 @@ static void pointcloud_blend_read_data(BlendDataReader *reader, ID *id)
 
   /* Materials */
   BLO_read_pointer_array(reader, (void **)&pointcloud->mat);
+
+  pointcloud->runtime = new blender::bke::PointCloudRuntime();
 }
 
 static void pointcloud_blend_read_lib(BlendLibReader *reader, ID *id)
@@ -158,33 +164,33 @@ static void pointcloud_blend_read_expand(BlendExpander *expander, ID *id)
 }
 
 IDTypeInfo IDType_ID_PT = {
-    /* id_code */ ID_PT,
-    /* id_filter */ FILTER_ID_PT,
-    /* main_listbase_index */ INDEX_ID_PT,
-    /* struct_size */ sizeof(PointCloud),
-    /* name */ "PointCloud",
-    /* name_plural */ "pointclouds",
-    /* translation_context */ BLT_I18NCONTEXT_ID_POINTCLOUD,
-    /* flags */ IDTYPE_FLAGS_APPEND_IS_REUSABLE,
-    /* asset_type_info */ nullptr,
+    /*id_code*/ ID_PT,
+    /*id_filter*/ FILTER_ID_PT,
+    /*main_listbase_index*/ INDEX_ID_PT,
+    /*struct_size*/ sizeof(PointCloud),
+    /*name*/ "PointCloud",
+    /*name_plural*/ "pointclouds",
+    /*translation_context*/ BLT_I18NCONTEXT_ID_POINTCLOUD,
+    /*flags*/ IDTYPE_FLAGS_APPEND_IS_REUSABLE,
+    /*asset_type_info*/ nullptr,
 
-    /* init_data */ pointcloud_init_data,
-    /* copy_data */ pointcloud_copy_data,
-    /* free_data */ pointcloud_free_data,
-    /* make_local */ nullptr,
-    /* foreach_id */ pointcloud_foreach_id,
-    /* foreach_cache */ nullptr,
-    /* foreach_path */ nullptr,
-    /* owner_pointer_get */ nullptr,
+    /*init_data*/ pointcloud_init_data,
+    /*copy_data*/ pointcloud_copy_data,
+    /*free_data*/ pointcloud_free_data,
+    /*make_local*/ nullptr,
+    /*foreach_id*/ pointcloud_foreach_id,
+    /*foreach_cache*/ nullptr,
+    /*foreach_path*/ nullptr,
+    /*owner_pointer_get*/ nullptr,
 
-    /* blend_write */ pointcloud_blend_write,
-    /* blend_read_data */ pointcloud_blend_read_data,
-    /* blend_read_lib */ pointcloud_blend_read_lib,
-    /* blend_read_expand */ pointcloud_blend_read_expand,
+    /*blend_write*/ pointcloud_blend_write,
+    /*blend_read_data*/ pointcloud_blend_read_data,
+    /*blend_read_lib*/ pointcloud_blend_read_lib,
+    /*blend_read_expand*/ pointcloud_blend_read_expand,
 
-    /* blend_read_undo_preserve */ nullptr,
+    /*blend_read_undo_preserve*/ nullptr,
 
-    /* lib_override_apply_post */ nullptr,
+    /*lib_override_apply_post*/ nullptr,
 };
 
 static void pointcloud_random(PointCloud *pointcloud)
@@ -196,21 +202,18 @@ static void pointcloud_random(PointCloud *pointcloud)
   RNG *rng = BLI_rng_new(0);
 
   blender::bke::MutableAttributeAccessor attributes = pointcloud->attributes_for_write();
-  blender::bke::SpanAttributeWriter positions =
-      attributes.lookup_or_add_for_write_only_span<float3>(POINTCLOUD_ATTR_POSITION,
-                                                           ATTR_DOMAIN_POINT);
+  blender::MutableSpan<float3> positions = pointcloud->positions_for_write();
   blender::bke::SpanAttributeWriter<float> radii =
       attributes.lookup_or_add_for_write_only_span<float>(POINTCLOUD_ATTR_RADIUS,
                                                           ATTR_DOMAIN_POINT);
 
-  for (const int i : positions.span.index_range()) {
-    positions.span[i] =
-        float3(BLI_rng_get_float(rng), BLI_rng_get_float(rng), BLI_rng_get_float(rng)) * 2.0f -
-        1.0f;
+  for (const int i : positions.index_range()) {
+    positions[i] = float3(BLI_rng_get_float(rng), BLI_rng_get_float(rng), BLI_rng_get_float(rng)) *
+                       2.0f -
+                   1.0f;
     radii.span[i] = 0.05f * BLI_rng_get_float(rng);
   }
 
-  positions.finish();
   radii.finish();
 
   BLI_rng_free(rng);
@@ -246,64 +249,39 @@ PointCloud *BKE_pointcloud_new_nomain(const int totpoint)
   return pointcloud;
 }
 
-void BKE_pointcloud_nomain_to_pointcloud(PointCloud *pointcloud_src,
-                                         PointCloud *pointcloud_dst,
-                                         bool take_ownership)
+void BKE_pointcloud_nomain_to_pointcloud(PointCloud *pointcloud_src, PointCloud *pointcloud_dst)
 {
   BLI_assert(pointcloud_src->id.tag & LIB_TAG_NO_MAIN);
 
-  eCDAllocType alloctype = CD_DUPLICATE;
-
-  if (take_ownership) {
-    bool has_any_referenced_layers = CustomData_has_referenced(&pointcloud_src->pdata);
-
-    if (!has_any_referenced_layers) {
-      alloctype = CD_ASSIGN;
-    }
-  }
-
   CustomData_free(&pointcloud_dst->pdata, pointcloud_dst->totpoint);
 
   const int totpoint = pointcloud_dst->totpoint = pointcloud_src->totpoint;
-  CustomData_copy(
-      &pointcloud_src->pdata, &pointcloud_dst->pdata, CD_MASK_ALL, alloctype, totpoint);
+  CustomData_copy(&pointcloud_src->pdata, &pointcloud_dst->pdata, CD_MASK_ALL, totpoint);
 
-  if (take_ownership) {
-    if (alloctype == CD_ASSIGN) {
-      /* Free the CustomData but keep the layers. */
-      CustomData_free_typemask(&pointcloud_src->pdata, pointcloud_src->totpoint, 0);
-    }
-    BKE_id_free(nullptr, pointcloud_src);
-  }
+  BKE_id_free(nullptr, pointcloud_src);
 }
 
-static std::optional<blender::bounds::MinMaxResult<float3>> point_cloud_bounds(
-    const PointCloud &pointcloud)
-{
-  blender::bke::AttributeAccessor attributes = pointcloud.attributes();
-  blender::VArraySpan<float3> positions = attributes.lookup_or_default<float3>(
-      POINTCLOUD_ATTR_POSITION, ATTR_DOMAIN_POINT, float3(0));
-  blender::VArray<float> radii = attributes.lookup_or_default<float>(
-      POINTCLOUD_ATTR_RADIUS, ATTR_DOMAIN_POINT, 0.0f);
-
-  if (!(radii.is_single() && radii.get_internal_single() == 0.0f)) {
-    return blender::bounds::min_max_with_radii(positions, radii.get_internal_span());
-  }
-  return blender::bounds::min_max(positions);
-}
-
-bool BKE_pointcloud_minmax(const PointCloud *pointcloud, float r_min[3], float r_max[3])
+bool PointCloud::bounds_min_max(blender::float3 &min, blender::float3 &max) const
 {
   using namespace blender;
-
-  const std::optional<bounds::MinMaxResult<float3>> min_max = point_cloud_bounds(*pointcloud);
-  if (!min_max) {
+  using namespace blender::bke;
+  if (this->totpoint == 0) {
     return false;
   }
-
-  copy_v3_v3(r_min, math::min(min_max->min, float3(r_min)));
-  copy_v3_v3(r_max, math::max(min_max->max, float3(r_max)));
-
+  this->runtime->bounds_cache.ensure([&](Bounds<float3> &r_bounds) {
+    const AttributeAccessor attributes = this->attributes();
+    const Span<float3> positions = this->positions();
+    if (attributes.contains(POINTCLOUD_ATTR_RADIUS)) {
+      const VArraySpan radii = *attributes.lookup<float>(POINTCLOUD_ATTR_RADIUS);
+      r_bounds = *bounds::min_max_with_radii(positions, radii);
+    }
+    else {
+      r_bounds = *bounds::min_max(positions);
+    }
+  });
+  const Bounds<float3> &bounds = this->runtime->bounds_cache.data();
+  min = math::min(bounds.min, min);
+  max = math::max(bounds.max, max);
   return true;
 }
 
@@ -326,7 +304,7 @@ BoundBox *BKE_pointcloud_boundbox_get(Object *ob)
   }
   else {
     const PointCloud *pointcloud = static_cast<PointCloud *>(ob->data);
-    BKE_pointcloud_minmax(pointcloud, min, max);
+    pointcloud->bounds_min_max(min, max);
   }
   BKE_boundbox_init_from_minmax(ob->runtime.bb, min, max);
 
@@ -340,16 +318,10 @@ bool BKE_pointcloud_attribute_required(const PointCloud * /*pointcloud*/, const
 
 /* Dependency Graph */
 
-PointCloud *BKE_pointcloud_copy_for_eval(struct PointCloud *pointcloud_src, bool reference)
+PointCloud *BKE_pointcloud_copy_for_eval(struct PointCloud *pointcloud_src)
 {
-  int flags = LIB_ID_COPY_LOCALIZE;
-
-  if (reference) {
-    flags |= LIB_ID_COPY_CD_REFERENCE;
-  }
-
-  PointCloud *result = (PointCloud *)BKE_id_copy_ex(nullptr, &pointcloud_src->id, nullptr, flags);
-  return result;
+  return reinterpret_cast<PointCloud *>(
+      BKE_id_copy_ex(nullptr, &pointcloud_src->id, nullptr, LIB_ID_COPY_LOCALIZE));
 }
 
 static void pointcloud_evaluate_modifiers(struct Depsgraph *depsgraph,
@@ -378,6 +350,8 @@ static void pointcloud_evaluate_modifiers(struct Depsgraph *depsgraph,
       continue;
     }
 
+    blender::bke::ScopedModifierTimer modifier_timer{*md};
+
     if (mti->modifyGeometrySet) {
       mti->modifyGeometrySet(md, &mectx, &geometry_set);
     }
@@ -427,6 +401,16 @@ void BKE_pointcloud_data_update(struct Depsgraph *depsgraph, struct Scene *scene
   object->runtime.geometry_set_eval = new GeometrySet(std::move(geometry_set));
 }
 
+void PointCloud::tag_positions_changed()
+{
+  this->runtime->bounds_cache.tag_dirty();
+}
+
+void PointCloud::tag_radii_changed()
+{
+  this->runtime->bounds_cache.tag_dirty();
+}
+
 /* Draw Cache */
 
 void (*BKE_pointcloud_batch_cache_dirty_tag_cb)(PointCloud *pointcloud, int mode) = nullptr;
