@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2013 Blender Foundation. All rights reserved. */
+ * Copyright 2013 Blender Foundation */
 
 /** \file
  * \ingroup bke
@@ -66,24 +66,12 @@ static CLG_LogRef LOG = {"bke.rigidbody"};
 static void rigidbody_update_ob_array(RigidBodyWorld *rbw);
 
 #else
-static void RB_dworld_remove_constraint(void *UNUSED(world), void *UNUSED(con))
-{
-}
-static void RB_dworld_remove_body(void *UNUSED(world), void *UNUSED(body))
-{
-}
-static void RB_dworld_delete(void *UNUSED(world))
-{
-}
-static void RB_body_delete(void *UNUSED(body))
-{
-}
-static void RB_shape_delete(void *UNUSED(shape))
-{
-}
-static void RB_constraint_delete(void *UNUSED(con))
-{
-}
+static void RB_dworld_remove_constraint(void *UNUSED(world), void *UNUSED(con)) {}
+static void RB_dworld_remove_body(void *UNUSED(world), void *UNUSED(body)) {}
+static void RB_dworld_delete(void *UNUSED(world)) {}
+static void RB_body_delete(void *UNUSED(body)) {}
+static void RB_shape_delete(void *UNUSED(shape)) {}
+static void RB_constraint_delete(void *UNUSED(con)) {}
 
 #endif
 
@@ -220,7 +208,8 @@ bool BKE_rigidbody_is_affected_by_simulation(Object *ob)
 
   RigidBodyOb *rbo = ob->rigidbody_object;
   if (rbo == NULL || rbo->flag & RBO_FLAG_KINEMATIC || rbo->type == RBO_TYPE_PASSIVE ||
-      obCompoundParent) {
+      obCompoundParent)
+  {
     return false;
   }
 
@@ -359,12 +348,12 @@ static rbCollisionShape *rigidbody_get_shape_convexhull_from_mesh(Object *ob,
 {
   rbCollisionShape *shape = NULL;
   Mesh *mesh = NULL;
-  MVert *mvert = NULL;
+  float(*positions)[3] = NULL;
   int totvert = 0;
 
   if (ob->type == OB_MESH && ob->data) {
     mesh = rigidbody_get_mesh(ob);
-    mvert = (mesh) ? BKE_mesh_verts_for_write(mesh) : NULL;
+    positions = (mesh) ? BKE_mesh_vert_positions_for_write(mesh) : NULL;
     totvert = (mesh) ? mesh->totvert : 0;
   }
   else {
@@ -372,7 +361,8 @@ static rbCollisionShape *rigidbody_get_shape_convexhull_from_mesh(Object *ob,
   }
 
   if (totvert) {
-    shape = RB_shape_new_convex_hull((float *)mvert, sizeof(MVert), totvert, margin, can_embed);
+    shape = RB_shape_new_convex_hull(
+        (float *)positions, sizeof(float[3]), totvert, margin, can_embed);
   }
   else {
     CLOG_ERROR(&LOG, "no vertices to define Convex Hull collision shape with");
@@ -401,11 +391,11 @@ static rbCollisionShape *rigidbody_get_shape_trimesh_from_mesh(Object *ob)
       return NULL;
     }
 
-    const MVert *mvert = BKE_mesh_verts(mesh);
+    const float(*positions)[3] = BKE_mesh_vert_positions(mesh);
     totvert = mesh->totvert;
     looptri = BKE_mesh_runtime_looptri_ensure(mesh);
     tottri = BKE_mesh_runtime_looptri_len(mesh);
-    const MLoop *mloop = BKE_mesh_loops(mesh);
+    const int *corner_verts = BKE_mesh_corner_verts(mesh);
 
     /* sanity checking - potential case when no data will be present */
     if ((totvert == 0) || (tottri == 0)) {
@@ -419,20 +409,20 @@ static rbCollisionShape *rigidbody_get_shape_trimesh_from_mesh(Object *ob)
       /* init mesh data for collision shape */
       mdata = RB_trimesh_data_new(tottri, totvert);
 
-      RB_trimesh_add_vertices(mdata, (float *)mvert, totvert, sizeof(MVert));
+      RB_trimesh_add_vertices(mdata, (float *)positions, totvert, sizeof(float[3]));
 
       /* loop over all faces, adding them as triangles to the collision shape
        * (so for some faces, more than triangle will get added)
        */
-      if (mvert && looptri) {
+      if (positions && looptri) {
         for (i = 0; i < tottri; i++) {
           /* add first triangle - verts 1,2,3 */
           const MLoopTri *lt = &looptri[i];
           int vtri[3];
 
-          vtri[0] = mloop[lt->tri[0]].v;
-          vtri[1] = mloop[lt->tri[1]].v;
-          vtri[2] = mloop[lt->tri[2]].v;
+          vtri[0] = corner_verts[lt->tri[0]];
+          vtri[1] = corner_verts[lt->tri[1]];
+          vtri[2] = corner_verts[lt->tri[2]];
 
           RB_trimesh_add_triangle_indices(mdata, i, UNPACK3(vtri));
         }
@@ -598,7 +588,8 @@ static void rigidbody_validate_sim_shape(RigidBodyWorld *rbw, Object *ob, bool r
 
   /* Also don't create a shape if this object is parent of a compound shape */
   if (ob->parent != NULL && ob->parent->rigidbody_object != NULL &&
-      ob->parent->rigidbody_object->shape == RB_SHAPE_COMPOUND) {
+      ob->parent->rigidbody_object->shape == RB_SHAPE_COMPOUND)
+  {
     return;
   }
 
@@ -676,14 +667,14 @@ void BKE_rigidbody_calc_volume(Object *ob, float *r_vol)
           return;
         }
 
-        const MVert *mvert = BKE_mesh_verts(mesh);
+        const float(*positions)[3] = BKE_mesh_vert_positions(mesh);
         totvert = mesh->totvert;
         lt = BKE_mesh_runtime_looptri_ensure(mesh);
         tottri = BKE_mesh_runtime_looptri_len(mesh);
-        const MLoop *mloop = BKE_mesh_loops(mesh);
+        const int *corner_verts = BKE_mesh_corner_verts(mesh);
 
         if (totvert > 0 && tottri > 0) {
-          BKE_mesh_calc_volume(mvert, totvert, lt, tottri, mloop, &volume, NULL);
+          BKE_mesh_calc_volume(positions, totvert, lt, tottri, corner_verts, &volume, NULL);
           const float volume_scale = mat4_to_volume_scale(ob->object_to_world);
           volume *= fabsf(volume_scale);
         }
@@ -750,14 +741,14 @@ void BKE_rigidbody_calc_center_of_mass(Object *ob, float r_center[3])
           return;
         }
 
-        const MVert *mvert = BKE_mesh_verts(mesh);
+        const float(*positions)[3] = BKE_mesh_vert_positions(mesh);
         totvert = mesh->totvert;
         looptri = BKE_mesh_runtime_looptri_ensure(mesh);
         tottri = BKE_mesh_runtime_looptri_len(mesh);
-        const MLoop *mloop = BKE_mesh_loops(mesh);
 
         if (totvert > 0 && tottri > 0) {
-          BKE_mesh_calc_volume(mvert, totvert, looptri, tottri, mloop, NULL, r_center);
+          BKE_mesh_calc_volume(
+              positions, totvert, looptri, tottri, BKE_mesh_corner_verts(mesh), NULL, r_center);
         }
       }
       break;
@@ -805,7 +796,8 @@ static void rigidbody_validate_sim_object(RigidBodyWorld *rbw, Object *ob, bool
     }
     /* Don't create rigid body object if the parent is a compound shape */
     if (ob->parent != NULL && ob->parent->rigidbody_object != NULL &&
-        ob->parent->rigidbody_object->shape == RB_SHAPE_COMPOUND) {
+        ob->parent->rigidbody_object->shape == RB_SHAPE_COMPOUND)
+    {
       return;
     }
 
@@ -1346,7 +1338,7 @@ RigidBodyCon *BKE_rigidbody_create_constraint(Scene *scene, Object *ob, short ty
   return rbc;
 }
 
-void BKE_rigidbody_objects_collection_validate(Scene *scene, RigidBodyWorld *rbw)
+void BKE_rigidbody_objects_collection_validate(Main *bmain, Scene *scene, RigidBodyWorld *rbw)
 {
   if (rbw->group != NULL) {
     FOREACH_COLLECTION_OBJECT_RECURSIVE_BEGIN (rbw->group, object) {
@@ -1354,8 +1346,10 @@ void BKE_rigidbody_objects_collection_validate(Scene *scene, RigidBodyWorld *rbw
         continue;
       }
       object->rigidbody_object = BKE_rigidbody_create_object(scene, object, RBO_TYPE_ACTIVE);
+      DEG_id_tag_update(&object->id, ID_RECALC_TRANSFORM);
     }
     FOREACH_COLLECTION_OBJECT_RECURSIVE_END;
+    DEG_relations_tag_update(bmain);
   }
 }
 
@@ -1613,7 +1607,8 @@ static void rigidbody_update_ob_array(RigidBodyWorld *rbw)
     (void)object;
     /* Ignore if this object is the direct child of an object with a compound shape */
     if (object->parent == NULL || object->parent->rigidbody_object == NULL ||
-        object->parent->rigidbody_object->shape != RB_SHAPE_COMPOUND) {
+        object->parent->rigidbody_object->shape != RB_SHAPE_COMPOUND)
+    {
       n++;
     }
   }
@@ -1628,7 +1623,8 @@ static void rigidbody_update_ob_array(RigidBodyWorld *rbw)
   FOREACH_COLLECTION_OBJECT_RECURSIVE_BEGIN (rbw->group, object) {
     /* Ignore if this object is the direct child of an object with a compound shape */
     if (object->parent == NULL || object->parent->rigidbody_object == NULL ||
-        object->parent->rigidbody_object->shape != RB_SHAPE_COMPOUND) {
+        object->parent->rigidbody_object->shape != RB_SHAPE_COMPOUND)
+    {
       rbw->objects[i] = object;
       i++;
     }
@@ -1673,14 +1669,14 @@ static void rigidbody_update_sim_ob(Depsgraph *depsgraph, Object *ob, RigidBodyO
   if (rbo->shape == RB_SHAPE_TRIMESH && rbo->flag & RBO_FLAG_USE_DEFORM) {
     Mesh *mesh = ob->runtime.mesh_deform_eval;
     if (mesh) {
-      MVert *mvert = BKE_mesh_verts_for_write(mesh);
+      float(*positions)[3] = BKE_mesh_vert_positions_for_write(mesh);
       int totvert = mesh->totvert;
       const BoundBox *bb = BKE_object_boundbox_get(ob);
 
       RB_shape_trimesh_update(rbo->shared->physics_shape,
-                              (float *)mvert,
+                              (float *)positions,
                               totvert,
-                              sizeof(MVert),
+                              sizeof(float[3]),
                               bb->vec[0],
                               bb->vec[6]);
     }
@@ -1703,7 +1699,7 @@ static void rigidbody_update_sim_ob(Depsgraph *depsgraph, Object *ob, RigidBodyO
     }
   }
 
-  /* Make transformed objects temporarily kinmatic
+  /* Make transformed objects temporarily kinematic
    * so that they can be moved by the user during simulation. */
   if (is_selected && (G.moving & G_TRANSFORM_OBJ)) {
     RB_body_set_kinematic_state(rbo->shared->physics_object, true);
@@ -1727,7 +1723,7 @@ static void rigidbody_update_simulation(Depsgraph *depsgraph,
 {
   /* update world */
   /* Note physics_world can get NULL when undoing the deletion of the last object in it (see
-   * T70667). */
+   * #70667). */
   if (rebuild || rbw->shared->physics_world == NULL) {
     BKE_rigidbody_validate_sim_world(scene, rbw, rebuild);
     /* We have rebuilt the world so we need to make sure the rest is rebuilt as well. */
@@ -1949,8 +1945,8 @@ static void rigidbody_update_external_forces(Depsgraph *depsgraph,
 
     /* update influence of effectors - but don't do it on an effector */
     /* only dynamic bodies need effector update */
-    if (rbo->type == RBO_TYPE_ACTIVE &&
-        ((ob->pd == NULL) || (ob->pd->forcefield == PFIELD_NULL))) {
+    if (rbo->type == RBO_TYPE_ACTIVE && ((ob->pd == NULL) || (ob->pd->forcefield == PFIELD_NULL)))
+    {
       EffectorWeights *effector_weights = rbw->effector_weights;
       EffectedPoint epoint;
       ListBase *effectors;
@@ -2019,7 +2015,8 @@ static void rigidbody_update_simulation_post_step(Depsgraph *depsgraph, RigidBod
     RigidBodyOb *rbo = ob->rigidbody_object;
     /* Reset kinematic state for transformed objects. */
     if (rbo && base && (base->flag & BASE_SELECTED) && (G.moving & G_TRANSFORM_OBJ) &&
-        rbo->shared->physics_object) {
+        rbo->shared->physics_object)
+    {
       RB_body_set_kinematic_state(rbo->shared->physics_object,
                                   rbo->flag & RBO_FLAG_KINEMATIC || rbo->flag & RBO_FLAG_DISABLED);
       RB_body_set_mass(rbo->shared->physics_object, RBO_GET_MASS(rbo));
@@ -2048,7 +2045,8 @@ void BKE_rigidbody_sync_transforms(RigidBodyWorld *rbw, Object *ob, float ctime)
 
   /* use rigid body transform after cache start frame if objects is not being transformed */
   if (BKE_rigidbody_check_sim_running(rbw, ctime) &&
-      !(ob->base_flag & BASE_SELECTED && G.moving & G_TRANSFORM_OBJ)) {
+      !(ob->base_flag & BASE_SELECTED && G.moving & G_TRANSFORM_OBJ))
+  {
     float mat[4][4], size_mat[4][4], size[3];
 
     normalize_qt(rbo->orn); /* RB_TODO investigate why quaternion isn't normalized at this point */
@@ -2163,7 +2161,8 @@ void BKE_rigidbody_rebuild_world(Depsgraph *depsgraph, Scene *scene, float ctime
     (void)object;
     /* Ignore if this object is the direct child of an object with a compound shape */
     if (object->parent == NULL || object->parent->rigidbody_object == NULL ||
-        object->parent->rigidbody_object->shape != RB_SHAPE_COMPOUND) {
+        object->parent->rigidbody_object->shape != RB_SHAPE_COMPOUND)
+    {
       n++;
     }
   }
@@ -2274,14 +2273,15 @@ void BKE_rigidbody_do_simulation(Depsgraph *depsgraph, Scene *scene, float ctime
 #  if defined(__GNUC__) || defined(__clang__)
 #    pragma GCC diagnostic push
 #    pragma GCC diagnostic ignored "-Wunused-parameter"
+#  elif defined(_MSC_VER)
+/* Suppress unreferenced formal parameter warning. */
+#    pragma warning(disable : 4100)
 #  endif
 
 void BKE_rigidbody_object_copy(Main *bmain, Object *ob_dst, const Object *ob_src, const int flag)
 {
 }
-void BKE_rigidbody_validate_sim_world(Scene *scene, RigidBodyWorld *rbw, bool rebuild)
-{
-}
+void BKE_rigidbody_validate_sim_world(Scene *scene, RigidBodyWorld *rbw, bool rebuild) {}
 
 void BKE_rigidbody_calc_volume(Object *ob, float *r_vol)
 {
@@ -2301,9 +2301,7 @@ struct RigidBodyWorld *BKE_rigidbody_world_copy(RigidBodyWorld *rbw, const int f
 {
   return NULL;
 }
-void BKE_rigidbody_world_groups_relink(struct RigidBodyWorld *rbw)
-{
-}
+void BKE_rigidbody_world_groups_relink(struct RigidBodyWorld *rbw) {}
 void BKE_rigidbody_world_id_loop(struct RigidBodyWorld *rbw,
                                  RigidbodyWorldIDFunc func,
                                  void *userdata)
@@ -2322,9 +2320,7 @@ struct RigidBodyWorld *BKE_rigidbody_get_world(Scene *scene)
   return NULL;
 }
 
-void BKE_rigidbody_ensure_local_object(Main *bmain, Object *ob)
-{
-}
+void BKE_rigidbody_ensure_local_object(Main *bmain, Object *ob) {}
 
 bool BKE_rigidbody_add_object(Main *bmain, Scene *scene, Object *ob, int type, ReportList *reports)
 {
@@ -2335,12 +2331,8 @@ bool BKE_rigidbody_add_object(Main *bmain, Scene *scene, Object *ob, int type, R
 void BKE_rigidbody_remove_object(struct Main *bmain, Scene *scene, Object *ob, const bool free_us)
 {
 }
-void BKE_rigidbody_remove_constraint(Main *bmain, Scene *scene, Object *ob, const bool free_us)
-{
-}
-void BKE_rigidbody_sync_transforms(RigidBodyWorld *rbw, Object *ob, float ctime)
-{
-}
+void BKE_rigidbody_remove_constraint(Main *bmain, Scene *scene, Object *ob, const bool free_us) {}
+void BKE_rigidbody_sync_transforms(RigidBodyWorld *rbw, Object *ob, float ctime) {}
 void BKE_rigidbody_aftertrans_update(
     Object *ob, float loc[3], float rot[3], float quat[4], float rotAxis[3], float rotAngle)
 {
@@ -2349,21 +2341,11 @@ bool BKE_rigidbody_check_sim_running(RigidBodyWorld *rbw, float ctime)
 {
   return false;
 }
-void BKE_rigidbody_cache_reset(RigidBodyWorld *rbw)
-{
-}
-void BKE_rigidbody_rebuild_world(Depsgraph *depsgraph, Scene *scene, float ctime)
-{
-}
-void BKE_rigidbody_do_simulation(Depsgraph *depsgraph, Scene *scene, float ctime)
-{
-}
-void BKE_rigidbody_objects_collection_validate(Scene *scene, RigidBodyWorld *rbw)
-{
-}
-void BKE_rigidbody_constraints_collection_validate(Scene *scene, RigidBodyWorld *rbw)
-{
-}
+void BKE_rigidbody_cache_reset(RigidBodyWorld *rbw) {}
+void BKE_rigidbody_rebuild_world(Depsgraph *depsgraph, Scene *scene, float ctime) {}
+void BKE_rigidbody_do_simulation(Depsgraph *depsgraph, Scene *scene, float ctime) {}
+void BKE_rigidbody_objects_collection_validate(Main *bmain, Scene *scene, RigidBodyWorld *rbw) {}
+void BKE_rigidbody_constraints_collection_validate(Scene *scene, RigidBodyWorld *rbw) {}
 void BKE_rigidbody_main_collection_object_add(Main *bmain, Collection *collection, Object *object)
 {
 }
