@@ -258,11 +258,11 @@ bSound *BKE_sound_new_file(Main *bmain, const char *filepath)
   const char *blendfile_path = BKE_main_blendfile_path(bmain);
   char str[FILE_MAX];
 
-  BLI_strncpy(str, filepath, sizeof(str));
+  STRNCPY(str, filepath);
   BLI_path_abs(str, blendfile_path);
 
   sound = BKE_libblock_alloc(bmain, ID_SO, BLI_path_basename(filepath), 0);
-  BLI_strncpy(sound->filepath, filepath, FILE_MAX);
+  STRNCPY(sound->filepath, filepath);
   /* sound->type = SOUND_TYPE_FILE; */ /* XXX unused currently */
 
   /* Extract sound specs for bSound */
@@ -286,12 +286,12 @@ bSound *BKE_sound_new_file_exists_ex(Main *bmain, const char *filepath, bool *r_
   bSound *sound;
   char str[FILE_MAX], strtest[FILE_MAX];
 
-  BLI_strncpy(str, filepath, sizeof(str));
+  STRNCPY(str, filepath);
   BLI_path_abs(str, BKE_main_blendfile_path(bmain));
 
   /* first search an identical filepath */
   for (sound = bmain->sounds.first; sound; sound = sound->id.next) {
-    BLI_strncpy(strtest, sound->filepath, sizeof(sound->filepath));
+    STRNCPY(strtest, sound->filepath);
     BLI_path_abs(strtest, ID_BLEND_PATH(bmain, &sound->id));
 
     if (BLI_path_cmp(strtest, str) == 0) {
@@ -551,7 +551,7 @@ static void sound_load_audio(Main *bmain, bSound *sound, bool free_waveform)
     PackedFile *pf = sound->packedfile;
 
     /* don't modify soundact->sound->filepath, only change a copy */
-    BLI_strncpy(fullpath, sound->filepath, sizeof(fullpath));
+    STRNCPY(fullpath, sound->filepath);
     BLI_path_abs(fullpath, ID_BLEND_PATH(bmain, &sound->id));
 
     /* but we need a packed file then */
@@ -649,12 +649,12 @@ void BKE_sound_destroy_scene(Scene *scene)
   }
 }
 
-void BKE_sound_lock()
+void BKE_sound_lock(void)
 {
   AUD_Device_lock(sound_device);
 }
 
-void BKE_sound_unlock()
+void BKE_sound_unlock(void)
 {
   AUD_Device_unlock(sound_device);
 }
@@ -854,7 +854,7 @@ static double get_cur_time(Scene *scene)
 {
   /* We divide by the current framelen to take into account time remapping.
    * Otherwise we will get the wrong starting time which will break A/V sync.
-   * See T74111 for further details. */
+   * See #74111 for further details. */
   return FRA2TIME((scene->r.cfra + scene->r.subframe) / (double)scene->r.framelen);
 }
 
@@ -944,7 +944,8 @@ void BKE_sound_seek_scene(Main *bmain, Scene *scene)
     }
     AUD_Handle_resume(scene->playback_handle);
     if (scene->sound_scrub_handle &&
-        AUD_Handle_getStatus(scene->sound_scrub_handle) != AUD_STATUS_INVALID) {
+        AUD_Handle_getStatus(scene->sound_scrub_handle) != AUD_STATUS_INVALID)
+    {
       AUD_Handle_setPosition(scene->sound_scrub_handle, 0);
     }
     else {
@@ -1025,7 +1026,7 @@ void BKE_sound_free_waveform(bSound *sound)
   sound->tags &= ~SOUND_TAGS_WAVEFORM_NO_RELOAD;
 }
 
-void BKE_sound_read_waveform(Main *bmain, bSound *sound, short *stop)
+void BKE_sound_read_waveform(Main *bmain, bSound *sound, bool *stop)
 {
   bool need_close_audio_handles = false;
   if (sound->playback_handle == NULL) {
@@ -1041,8 +1042,11 @@ void BKE_sound_read_waveform(Main *bmain, bSound *sound, short *stop)
     int length = info.length * SOUND_WAVE_SAMPLES_PER_SECOND;
 
     waveform->data = MEM_mallocN(sizeof(float[3]) * length, "SoundWaveform.samples");
+    /* Ideally this would take a boolean argument. */
+    short stop_i16 = *stop;
     waveform->length = AUD_readSound(
-        sound->playback_handle, waveform->data, length, SOUND_WAVE_SAMPLES_PER_SECOND, stop);
+        sound->playback_handle, waveform->data, length, SOUND_WAVE_SAMPLES_PER_SECOND, &stop_i16);
+    *stop = stop_i16 != 0;
   }
   else {
     /* Create an empty waveform here if the sound couldn't be
@@ -1227,7 +1231,7 @@ bool BKE_sound_info_get(struct Main *main, struct bSound *sound, SoundInfo *soun
   }
   /* TODO(sergey): Make it fully independent audio handle. */
   /* Don't free waveforms during non-destructive queries.
-   * This causes unnecessary recalculation - see T69921 */
+   * This causes unnecessary recalculation - see #69921 */
   sound_load_audio(main, sound, false);
   const bool result = sound_info_from_playback_handle(sound->playback_handle, sound_info);
   sound_free_audio(sound);
@@ -1245,7 +1249,7 @@ bool BKE_sound_stream_info_get(struct Main *main,
   AUD_StreamInfo *stream_infos;
   int stream_count;
 
-  BLI_strncpy(str, filepath, sizeof(str));
+  STRNCPY(str, filepath);
   BLI_path_abs(str, blendfile_path);
 
   sound = AUD_Sound_file(str);
@@ -1278,48 +1282,20 @@ bool BKE_sound_stream_info_get(struct Main *main,
 
 #  include "BLI_utildefines.h"
 
-void BKE_sound_force_device(const char *UNUSED(device))
-{
-}
-void BKE_sound_init_once(void)
-{
-}
-void BKE_sound_init(Main *UNUSED(bmain))
-{
-}
-void BKE_sound_exit(void)
-{
-}
-void BKE_sound_exit_once(void)
-{
-}
-void BKE_sound_cache(bSound *UNUSED(sound))
-{
-}
-void BKE_sound_delete_cache(bSound *UNUSED(sound))
-{
-}
-void BKE_sound_load(Main *UNUSED(bmain), bSound *UNUSED(sound))
-{
-}
-void BKE_sound_create_scene(Scene *UNUSED(scene))
-{
-}
-void BKE_sound_destroy_scene(Scene *UNUSED(scene))
-{
-}
-void BKE_sound_lock(void)
-{
-}
-void BKE_sound_unlock(void)
-{
-}
-void BKE_sound_reset_scene_specs(Scene *UNUSED(scene))
-{
-}
-void BKE_sound_mute_scene(Scene *UNUSED(scene), int UNUSED(muted))
-{
-}
+void BKE_sound_force_device(const char *UNUSED(device)) {}
+void BKE_sound_init_once(void) {}
+void BKE_sound_init(Main *UNUSED(bmain)) {}
+void BKE_sound_exit(void) {}
+void BKE_sound_exit_once(void) {}
+void BKE_sound_cache(bSound *UNUSED(sound)) {}
+void BKE_sound_delete_cache(bSound *UNUSED(sound)) {}
+void BKE_sound_load(Main *UNUSED(bmain), bSound *UNUSED(sound)) {}
+void BKE_sound_create_scene(Scene *UNUSED(scene)) {}
+void BKE_sound_destroy_scene(Scene *UNUSED(scene)) {}
+void BKE_sound_lock(void) {}
+void BKE_sound_unlock(void) {}
+void BKE_sound_reset_scene_specs(Scene *UNUSED(scene)) {}
+void BKE_sound_mute_scene(Scene *UNUSED(scene), int UNUSED(muted)) {}
 void *BKE_sound_scene_add_scene_sound(Scene *UNUSED(scene),
                                       Sequence *UNUSED(sequence),
                                       int UNUSED(startframe),
@@ -1344,12 +1320,8 @@ void *BKE_sound_add_scene_sound_defaults(Scene *UNUSED(scene), Sequence *UNUSED(
 {
   return NULL;
 }
-void BKE_sound_remove_scene_sound(Scene *UNUSED(scene), void *UNUSED(handle))
-{
-}
-void BKE_sound_mute_scene_sound(void *UNUSED(handle), bool UNUSED(mute))
-{
-}
+void BKE_sound_remove_scene_sound(Scene *UNUSED(scene), void *UNUSED(handle)) {}
+void BKE_sound_mute_scene_sound(void *UNUSED(handle), bool UNUSED(mute)) {}
 void BKE_sound_move_scene_sound(const Scene *UNUSED(scene),
                                 void *UNUSED(handle),
                                 int UNUSED(startframe),
@@ -1358,18 +1330,10 @@ void BKE_sound_move_scene_sound(const Scene *UNUSED(scene),
                                 double UNUSED(audio_offset))
 {
 }
-void BKE_sound_move_scene_sound_defaults(Scene *UNUSED(scene), Sequence *UNUSED(sequence))
-{
-}
-void BKE_sound_play_scene(Scene *UNUSED(scene))
-{
-}
-void BKE_sound_stop_scene(Scene *UNUSED(scene))
-{
-}
-void BKE_sound_seek_scene(Main *UNUSED(bmain), Scene *UNUSED(scene))
-{
-}
+void BKE_sound_move_scene_sound_defaults(Scene *UNUSED(scene), Sequence *UNUSED(sequence)) {}
+void BKE_sound_play_scene(Scene *UNUSED(scene)) {}
+void BKE_sound_stop_scene(Scene *UNUSED(scene)) {}
+void BKE_sound_seek_scene(Main *UNUSED(bmain), Scene *UNUSED(scene)) {}
 double BKE_sound_sync_scene(Scene *UNUSED(scene))
 {
   return NAN_FLT;
@@ -1381,31 +1345,17 @@ int BKE_sound_scene_playing(Scene *UNUSED(scene))
 void BKE_sound_read_waveform(Main *bmain,
                              bSound *sound,
                              /* NOLINTNEXTLINE: readability-non-const-parameter. */
-                             short *stop)
+                             bool *stop)
 {
   UNUSED_VARS(sound, stop, bmain);
 }
-void BKE_sound_init_main(Main *UNUSED(bmain))
-{
-}
-void BKE_sound_set_cfra(int UNUSED(cfra))
-{
-}
-void BKE_sound_update_sequencer(Main *UNUSED(main), bSound *UNUSED(sound))
-{
-}
-void BKE_sound_update_scene(Depsgraph *UNUSED(depsgraph), Scene *UNUSED(scene))
-{
-}
-void BKE_sound_update_scene_sound(void *UNUSED(handle), bSound *UNUSED(sound))
-{
-}
-void BKE_sound_update_scene_listener(Scene *UNUSED(scene))
-{
-}
-void BKE_sound_update_fps(Main *UNUSED(bmain), Scene *UNUSED(scene))
-{
-}
+void BKE_sound_init_main(Main *UNUSED(bmain)) {}
+void BKE_sound_set_cfra(int UNUSED(cfra)) {}
+void BKE_sound_update_sequencer(Main *UNUSED(main), bSound *UNUSED(sound)) {}
+void BKE_sound_update_scene(Depsgraph *UNUSED(depsgraph), Scene *UNUSED(scene)) {}
+void BKE_sound_update_scene_sound(void *UNUSED(handle), bSound *UNUSED(sound)) {}
+void BKE_sound_update_scene_listener(Scene *UNUSED(scene)) {}
+void BKE_sound_update_fps(Main *UNUSED(bmain), Scene *UNUSED(scene)) {}
 void BKE_sound_set_scene_sound_volume(void *UNUSED(handle),
                                       float UNUSED(volume),
                                       char UNUSED(animated))
@@ -1414,9 +1364,7 @@ void BKE_sound_set_scene_sound_volume(void *UNUSED(handle),
 void BKE_sound_set_scene_sound_pan(void *UNUSED(handle), float UNUSED(pan), char UNUSED(animated))
 {
 }
-void BKE_sound_set_scene_volume(Scene *UNUSED(scene), float UNUSED(volume))
-{
-}
+void BKE_sound_set_scene_volume(Scene *UNUSED(scene), float UNUSED(volume)) {}
 void BKE_sound_set_scene_sound_pitch(void *UNUSED(handle),
                                      float UNUSED(pitch),
                                      char UNUSED(animated))
@@ -1432,9 +1380,7 @@ char **BKE_sound_get_device_names(void)
   return names;
 }
 
-void BKE_sound_free_waveform(bSound *UNUSED(sound))
-{
-}
+void BKE_sound_free_waveform(bSound *UNUSED(sound)) {}
 
 bool BKE_sound_info_get(struct Main *UNUSED(main),
                         struct bSound *UNUSED(sound),
