@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2018 Blender Foundation. All rights reserved. */
+ * Copyright 2018 Blender Foundation */
 
 /** \file
  * \ingroup bke
@@ -91,9 +91,3 @@ BLI_INLINE float BKE_subdiv_crease_to_sharpness_f(float edge_crease)
 {
   return edge_crease * edge_crease * 10.0f;
 }
-
-BLI_INLINE float BKE_subdiv_crease_to_sharpness_char(char edge_crease)
-{
-  const float edge_crease_f = edge_crease / 255.0f;
-  return BKE_subdiv_crease_to_sharpness_f(edge_crease_f);
-}
