@@ -17,7 +17,8 @@
 #include "BLI_compiler_attrs.h"
 #include "BLI_utildefines.h"
 
-#if defined(_MSC_VER) || defined(__APPLE__) || defined(__HAIKU__) || defined(__NetBSD__)
+#if defined(_MSC_VER) || defined(__APPLE__) || defined(__HAIKU__) || defined(__NetBSD__) || \
+    defined(__OpenBSD__)
 typedef int64_t off64_t;
 #endif
 
