@@ -4,13 +4,23 @@
 
 namespace blender::array_utils {
 
+void copy(const GVArray &src, GMutableSpan dst, const int64_t grain_size)
+{
+  BLI_assert(src.type() == dst.type());
+  BLI_assert(src.size() == dst.size());
+  threading::parallel_for(src.index_range(), grain_size, [&](const IndexRange range) {
+    src.materialize_to_uninitialized(range, dst.data());
+  });
+}
+
 void copy(const GVArray &src,
           const IndexMask selection,
           GMutableSpan dst,
           const int64_t grain_size)
 {
   BLI_assert(src.type() == dst.type());
-  BLI_assert(src.size() == dst.size());
+  BLI_assert(src.size() >= selection.min_array_size());
+  BLI_assert(dst.size() >= selection.min_array_size());
   threading::parallel_for(selection.index_range(), grain_size, [&](const IndexRange range) {
     src.materialize_to_uninitialized(selection.slice(range), dst.data());
   });
@@ -33,4 +43,13 @@ void gather(const GSpan src, const IndexMask indices, GMutableSpan dst, const in
   gather(GVArray::ForSpan(src), indices, dst, grain_size);
 }
 
+void invert_booleans(MutableSpan<bool> span)
+{
+  threading::parallel_for(span.index_range(), 4096, [&](IndexRange range) {
+    for (const int i : range) {
+      span[i] = !span[i];
+    }
+  });
+}
+
 }  // namespace blender::array_utils
