@@ -251,7 +251,7 @@ static float BLI_convexhull_aabb_fit_hull_2d(const float (*points_hull)[2], int
     i_prev = i;
   }
 
-  return (area_best != FLT_MAX) ? atan2f(dvec_best[0], dvec_best[1]) : 0.0f;
+  return (area_best != FLT_MAX) ? (float)atan2(dvec_best[0], dvec_best[1]) : 0.0f;
 }
 
 float BLI_convexhull_aabb_fit_points_2d(const float (*points)[2], int n)
