@@ -244,6 +244,9 @@ void NodePort::to_dot_string(std::stringstream &ss) const
   if (port_name_.has_value()) {
     ss << ":" << *port_name_;
   }
+  if (port_position_.has_value()) {
+    ss << ":" << *port_position_;
+  }
 }
 
 std::string color_attr_from_hsv(float h, float s, float v)
@@ -253,11 +256,7 @@ std::string color_attr_from_hsv(float h, float s, float v)
   return ss.str();
 }
 
-NodeWithSocketsRef::NodeWithSocketsRef(Node &node,
-                                       StringRef name,
-                                       Span<std::string> input_names,
-                                       Span<std::string> output_names)
-    : node_(&node)
+NodeWithSocketsRef::NodeWithSocketsRef(Node &node, const NodeWithSockets &data) : node_(&node)
 {
   std::stringstream ss;
 
@@ -265,33 +264,39 @@ NodeWithSocketsRef::NodeWithSocketsRef(Node &node,
 
   /* Header */
   ss << R"(<tr><td colspan="3" align="center"><b>)";
-  ss << ((name.size() == 0) ? "No Name" : name);
+  ss << (data.node_name.empty() ? "No Name" : data.node_name);
   ss << "</b></td></tr>";
 
   /* Sockets */
-  int socket_max_amount = std::max(input_names.size(), output_names.size());
+  int socket_max_amount = std::max(data.inputs.size(), data.outputs.size());
   for (int i = 0; i < socket_max_amount; i++) {
     ss << "<tr>";
-    if (i < input_names.size()) {
-      StringRef name = input_names[i];
-      if (name.size() == 0) {
-        name = "No Name";
-      }
+    if (i < data.inputs.size()) {
+      const NodeWithSockets::Input &input = data.inputs[i];
       ss << R"(<td align="left" port="in)" << i << "\">";
-      ss << name;
+      if (input.fontcolor) {
+        ss << R"(<font color=")" << *input.fontcolor << "\">";
+      }
+      ss << (input.name.empty() ? "No Name" : input.name);
+      if (input.fontcolor) {
+        ss << "</font>";
+      }
       ss << "</td>";
     }
     else {
       ss << "<td></td>";
     }
     ss << "<td></td>";
-    if (i < output_names.size()) {
-      StringRef name = output_names[i];
-      if (name.size() == 0) {
-        name = "No Name";
-      }
+    if (i < data.outputs.size()) {
+      const NodeWithSockets::Output &output = data.outputs[i];
       ss << R"(<td align="right" port="out)" << i << "\">";
-      ss << name;
+      if (output.fontcolor) {
+        ss << R"(<font color=")" << *output.fontcolor << "\">";
+      }
+      ss << (output.name.empty() ? "No Name" : output.name);
+      if (output.fontcolor) {
+        ss << "</font>";
+      }
       ss << "</td>";
     }
     else {
