@@ -12,7 +12,7 @@ namespace blender::lazy_threading {
  * deadlocks.
  */
 using HintReceivers = RawStack<RawVector<FunctionRef<void()>, 0>, 0>;
-thread_local HintReceivers hint_receivers = []() {
+static thread_local HintReceivers hint_receivers = []() {
   HintReceivers receivers;
   /* Make sure there is always at least one vector. */
   receivers.push_as();
