@@ -362,9 +362,6 @@ MINLINE int divide_floor_i(int a, int b)
   return r ? d - ((a < 0) ^ (b < 0)) : d;
 }
 
-/**
- * Integer division that returns the ceiling, instead of flooring like normal C division.
- */
 MINLINE uint divide_ceil_u(uint a, uint b)
 {
   return (a + b - 1) / b;
@@ -375,9 +372,6 @@ MINLINE uint64_t divide_ceil_ul(uint64_t a, uint64_t b)
   return (a + b - 1) / b;
 }
 
-/**
- * Returns \a a if it is a multiple of \a b or the next multiple or \a b after \b a .
- */
 MINLINE uint ceil_to_multiple_u(uint a, uint b)
 {
   return divide_ceil_u(a, b) * b;
@@ -393,6 +387,16 @@ MINLINE int mod_i(int i, int n)
   return (i % n + n) % n;
 }
 
+MINLINE float mod_f_positive(const float f, const float n)
+{
+  const float modulo = fmodf(f, n);
+  if (modulo < 0) {
+    /* fmodf returns a value in the interval (-n, n). */
+    return modulo + n;
+  }
+  return modulo;
+}
+
 MINLINE float fractf(float a)
 {
   return a - floorf(a);
@@ -551,6 +555,15 @@ MINLINE unsigned long long max_ulul(unsigned long long a, unsigned long long b)
   return (b < a) ? a : b;
 }
 
+MINLINE double min_ddd(double a, double b, double c)
+{
+  return min_dd(min_dd(a, b), c);
+}
+MINLINE double max_ddd(double a, double b, double c)
+{
+  return max_dd(max_dd(a, b), c);
+}
+
 MINLINE float min_fff(float a, float b, float c)
 {
   return min_ff(min_ff(a, b), c);
@@ -833,7 +846,7 @@ MINLINE unsigned char unit_float_to_uchar_clamp(float val)
       (val <= 0.0f) ? 0 : ((val > (1.0f - 0.5f / 255.0f)) ? 255 : ((255.0f * val) + 0.5f))));
 }
 #define unit_float_to_uchar_clamp(val) \
-  ((CHECK_TYPE_INLINE(val, float)), unit_float_to_uchar_clamp(val))
+  ((CHECK_TYPE_INLINE_NONCONST((val), float)), unit_float_to_uchar_clamp(val))
 
 MINLINE unsigned short unit_float_to_ushort_clamp(float val)
 {
@@ -842,14 +855,14 @@ MINLINE unsigned short unit_float_to_ushort_clamp(float val)
                                                          (val * 65535.0f + 0.5f));
 }
 #define unit_float_to_ushort_clamp(val) \
-  ((CHECK_TYPE_INLINE(val, float)), unit_float_to_ushort_clamp(val))
+  ((CHECK_TYPE_INLINE_NONCONST(val, float)), unit_float_to_ushort_clamp(val))
 
 MINLINE unsigned char unit_ushort_to_uchar(unsigned short val)
 {
   return (unsigned char)(((val) >= 65535 - 128) ? 255 : ((val) + 128) >> 8);
 }
 #define unit_ushort_to_uchar(val) \
-  ((CHECK_TYPE_INLINE(val, unsigned short)), unit_ushort_to_uchar(val))
+  ((CHECK_TYPE_INLINE_NONCONST(val, unsigned short)), unit_ushort_to_uchar(val))
 
 #define unit_float_to_uchar_clamp_v3(v1, v2) \
   { \
