@@ -13,7 +13,7 @@
 
 #include <sys/stat.h>
 
-#if defined(__NetBSD__) || defined(__DragonFly__) || defined(__HAIKU__)
+#if defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__) || defined(__HAIKU__)
 /* Other modern unix OS's should probably use this also. */
 #  include <sys/statvfs.h>
 #  define USE_STATFS_STATVFS
@@ -54,11 +54,36 @@
 #include "BLI_linklist.h"
 #include "BLI_path_util.h"
 #include "BLI_string.h"
+#include "BLI_threads.h"
 #include "BLI_utildefines.h"
 
+/* NOTE: The implementation for Apple lives in storage_apple.mm. */
+#if !defined(__APPLE__)
+bool BLI_change_working_dir(const char *dir)
+{
+  BLI_assert(BLI_thread_is_main());
+
+  if (!BLI_is_dir(dir)) {
+    return false;
+  }
+#  if defined(WIN32)
+  wchar_t wdir[FILE_MAX];
+  if (conv_utf_8_to_16(dir, wdir, ARRAY_SIZE(wdir)) != 0) {
+    return false;
+  }
+  return _wchdir(wdir) == 0;
+#  else
+  int result = chdir(dir);
+  if (result == 0) {
+    BLI_setenv("PWD", dir);
+  }
+  return result == 0;
+#  endif
+}
+
 char *BLI_current_working_dir(char *dir, const size_t maxncpy)
 {
-#if defined(WIN32)
+#  if defined(WIN32)
   wchar_t path[MAX_PATH];
   if (_wgetcwd(path, MAX_PATH)) {
     if (BLI_strncpy_wchar_as_utf8(dir, path, maxncpy) != maxncpy) {
@@ -66,7 +91,7 @@ char *BLI_current_working_dir(char *dir, const size_t maxncpy)
     }
   }
   return NULL;
-#else
+#  else
   const char *pwd = BLI_getenv("PWD");
   if (pwd) {
     size_t srclen = BLI_strnlen(pwd, maxncpy);
@@ -77,8 +102,9 @@ char *BLI_current_working_dir(char *dir, const size_t maxncpy)
     return NULL;
   }
   return getcwd(dir, maxncpy);
-#endif
+#  endif
 }
+#endif /* !defined (__APPLE__) */
 
 double BLI_dir_free_space(const char *dir)
 {
@@ -110,7 +136,7 @@ double BLI_dir_free_space(const char *dir)
   struct statfs disk;
 #  endif
 
-  char name[FILE_MAXDIR], *slash;
+  char dirname[FILE_MAXDIR], *slash;
   int len = strlen(dir);
 
   if (len >= FILE_MAXDIR) {
@@ -118,28 +144,28 @@ double BLI_dir_free_space(const char *dir)
     return -1;
   }
 
-  strcpy(name, dir);
+  strcpy(dirname, dir);
 
   if (len) {
-    slash = strrchr(name, '/');
+    slash = strrchr(dirname, '/');
     if (slash) {
       slash[1] = 0;
     }
   }
   else {
-    strcpy(name, "/");
+    strcpy(dirname, "/");
   }
 
 #  if defined(USE_STATFS_STATVFS)
-  if (statvfs(name, &disk)) {
+  if (statvfs(dirname, &disk)) {
     return -1;
   }
 #  elif defined(USE_STATFS_4ARGS)
-  if (statfs(name, &disk, sizeof(struct statfs), 0)) {
+  if (statfs(dirname, &disk, sizeof(struct statfs), 0)) {
     return -1;
   }
 #  else
-  if (statfs(name, &disk)) {
+  if (statfs(dirname, &disk)) {
     return -1;
   }
 #  endif
@@ -209,7 +235,14 @@ eFileAttributes BLI_file_attributes(const char *path)
   if (conv_utf_8_to_16(path, wline, ARRAY_SIZE(wline)) != 0) {
     return ret;
   }
+
   DWORD attr = GetFileAttributesW(wline);
+  if (attr == INVALID_FILE_ATTRIBUTES) {
+    BLI_assert_msg(GetLastError() != ERROR_FILE_NOT_FOUND,
+                   "BLI_file_attributes should only be called on existing files.");
+    return ret;
+  }
+
   if (attr & FILE_ATTRIBUTE_READONLY) {
     ret |= FILE_ATTR_READONLY;
   }
@@ -235,7 +268,8 @@ eFileAttributes BLI_file_attributes(const char *path)
     ret |= FILE_ATTR_SPARSE_FILE;
   }
   if (attr & FILE_ATTRIBUTE_OFFLINE || attr & FILE_ATTRIBUTE_RECALL_ON_OPEN ||
-      attr & FILE_ATTRIBUTE_RECALL_ON_DATA_ACCESS) {
+      attr & FILE_ATTRIBUTE_RECALL_ON_DATA_ACCESS)
+  {
     ret |= FILE_ATTR_OFFLINE;
   }
   if (attr & FILE_ATTRIBUTE_REPARSE_POINT) {
@@ -568,30 +602,12 @@ void BLI_file_free_lines(LinkNode *lines)
 
 bool BLI_file_older(const char *file1, const char *file2)
 {
-#ifdef WIN32
-  struct _stat st1, st2;
-
-  UTF16_ENCODE(file1);
-  UTF16_ENCODE(file2);
-
-  if (_wstat(file1_16, &st1)) {
+  BLI_stat_t st1, st2;
+  if (BLI_stat(file1, &st1)) {
     return false;
   }
-  if (_wstat(file2_16, &st2)) {
+  if (BLI_stat(file2, &st2)) {
     return false;
   }
-
-  UTF16_UN_ENCODE(file2);
-  UTF16_UN_ENCODE(file1);
-#else
-  struct stat st1, st2;
-
-  if (stat(file1, &st1)) {
-    return false;
-  }
-  if (stat(file2, &st2)) {
-    return false;
-  }
-#endif
   return (st1.st_mtime < st2.st_mtime);
 }
