@@ -24,8 +24,6 @@
 #  pragma GCC diagnostic error "-Wsign-conversion"
 #endif
 
-// #define DEBUG_STRSIZE
-
 /* -------------------------------------------------------------------- */
 /** \name String Duplicate/Copy
  * \{ */
@@ -61,14 +59,12 @@ char *BLI_strdupcat(const char *__restrict str1, const char *__restrict str2)
   return str;
 }
 
-char *BLI_strncpy(char *__restrict dst, const char *__restrict src, const size_t maxncpy)
+char *BLI_strncpy(char *__restrict dst, const char *__restrict src, const size_t dst_maxncpy)
 {
-  size_t srclen = BLI_strnlen(src, maxncpy - 1);
-  BLI_assert(maxncpy != 0);
+  BLI_string_debug_size(dst, dst_maxncpy);
 
-#ifdef DEBUG_STRSIZE
-  memset(dst, 0xff, sizeof(*dst) * maxncpy);
-#endif
+  BLI_assert(dst_maxncpy != 0);
+  size_t srclen = BLI_strnlen(src, dst_maxncpy - 1);
 
   memcpy(dst, src, srclen);
   dst[srclen] = '\0';
@@ -78,13 +74,10 @@ char *BLI_strncpy(char *__restrict dst, const char *__restrict src, const size_t
 char *BLI_strncpy_ensure_pad(char *__restrict dst,
                              const char *__restrict src,
                              const char pad,
-                             size_t maxncpy)
+                             size_t dst_maxncpy)
 {
-  BLI_assert(maxncpy != 0);
-
-#ifdef DEBUG_STRSIZE
-  memset(dst, 0xff, sizeof(*dst) * maxncpy);
-#endif
+  BLI_string_debug_size(dst, dst_maxncpy);
+  BLI_assert(dst_maxncpy != 0);
 
   if (src[0] == '\0') {
     dst[0] = '\0';
@@ -96,12 +89,12 @@ char *BLI_strncpy_ensure_pad(char *__restrict dst,
 
     if (src[idx] != pad) {
       dst[idx++] = pad;
-      maxncpy--;
+      dst_maxncpy--;
     }
-    maxncpy--; /* trailing '\0' */
+    dst_maxncpy--; /* trailing '\0' */
 
-    srclen = BLI_strnlen(src, maxncpy);
-    if ((src[srclen - 1] != pad) && (srclen == maxncpy)) {
+    srclen = BLI_strnlen(src, dst_maxncpy);
+    if ((src[srclen - 1] != pad) && (srclen == dst_maxncpy)) {
       srclen--;
     }
 
@@ -117,14 +110,12 @@ char *BLI_strncpy_ensure_pad(char *__restrict dst,
   return dst;
 }
 
-size_t BLI_strncpy_rlen(char *__restrict dst, const char *__restrict src, const size_t maxncpy)
+size_t BLI_strncpy_rlen(char *__restrict dst, const char *__restrict src, const size_t dst_maxncpy)
 {
-  size_t srclen = BLI_strnlen(src, maxncpy - 1);
-  BLI_assert(maxncpy != 0);
+  BLI_string_debug_size(dst, dst_maxncpy);
 
-#ifdef DEBUG_STRSIZE
-  memset(dst, 0xff, sizeof(*dst) * maxncpy);
-#endif
+  size_t srclen = BLI_strnlen(src, dst_maxncpy - 1);
+  BLI_assert(dst_maxncpy != 0);
 
   memcpy(dst, src, srclen);
   dst[srclen] = '\0';
@@ -138,86 +129,104 @@ size_t BLI_strcpy_rlen(char *__restrict dst, const char *__restrict src)
   return srclen;
 }
 
+/* -------------------------------------------------------------------- */
+/** \name String Append
+ * \{ */
+
+char *BLI_strncat(char *__restrict dst, const char *__restrict src, const size_t dst_maxncpy)
+{
+  BLI_string_debug_size(dst, dst_maxncpy);
+
+  size_t len = BLI_strnlen(dst, dst_maxncpy);
+  if (len < dst_maxncpy) {
+    BLI_strncpy(dst + len, src, dst_maxncpy - len);
+  }
+  return dst;
+}
+
 /** \} */
 
 /* -------------------------------------------------------------------- */
 /** \name String Printing
  * \{ */
 
-size_t BLI_vsnprintf(char *__restrict buffer,
-                     size_t maxncpy,
+size_t BLI_vsnprintf(char *__restrict dst,
+                     size_t dst_maxncpy,
                      const char *__restrict format,
                      va_list arg)
 {
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   size_t n;
 
-  BLI_assert(buffer != NULL);
-  BLI_assert(maxncpy > 0);
+  BLI_assert(dst != NULL);
+  BLI_assert(dst_maxncpy > 0);
   BLI_assert(format != NULL);
 
-  n = (size_t)vsnprintf(buffer, maxncpy, format, arg);
+  n = (size_t)vsnprintf(dst, dst_maxncpy, format, arg);
 
-  if (n != -1 && n < maxncpy) {
-    buffer[n] = '\0';
+  if (n != -1 && n < dst_maxncpy) {
+    dst[n] = '\0';
   }
   else {
-    buffer[maxncpy - 1] = '\0';
+    dst[dst_maxncpy - 1] = '\0';
   }
 
   return n;
 }
 
-size_t BLI_vsnprintf_rlen(char *__restrict buffer,
-                          size_t maxncpy,
+size_t BLI_vsnprintf_rlen(char *__restrict dst,
+                          size_t dst_maxncpy,
                           const char *__restrict format,
                           va_list arg)
 {
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   size_t n;
 
-  BLI_assert(buffer != NULL);
-  BLI_assert(maxncpy > 0);
+  BLI_assert(dst != NULL);
+  BLI_assert(dst_maxncpy > 0);
   BLI_assert(format != NULL);
 
-  n = (size_t)vsnprintf(buffer, maxncpy, format, arg);
+  n = (size_t)vsnprintf(dst, dst_maxncpy, format, arg);
 
-  if (n != -1 && n < maxncpy) {
+  if (n != -1 && n < dst_maxncpy) {
     /* pass */
   }
   else {
-    n = maxncpy - 1;
+    n = dst_maxncpy - 1;
   }
-  buffer[n] = '\0';
+  dst[n] = '\0';
 
   return n;
 }
 
-size_t BLI_snprintf(char *__restrict dst, size_t maxncpy, const char *__restrict format, ...)
+size_t BLI_snprintf(char *__restrict dst, size_t dst_maxncpy, const char *__restrict format, ...)
 {
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   size_t n;
   va_list arg;
 
-#ifdef DEBUG_STRSIZE
-  memset(dst, 0xff, sizeof(*dst) * maxncpy);
-#endif
-
   va_start(arg, format);
-  n = BLI_vsnprintf(dst, maxncpy, format, arg);
+  n = BLI_vsnprintf(dst, dst_maxncpy, format, arg);
   va_end(arg);
 
   return n;
 }
 
-size_t BLI_snprintf_rlen(char *__restrict dst, size_t maxncpy, const char *__restrict format, ...)
+size_t BLI_snprintf_rlen(char *__restrict dst,
+                         size_t dst_maxncpy,
+                         const char *__restrict format,
+                         ...)
 {
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   size_t n;
   va_list arg;
 
-#ifdef DEBUG_STRSIZE
-  memset(dst, 0xff, sizeof(*dst) * maxncpy);
-#endif
-
   va_start(arg, format);
-  n = BLI_vsnprintf_rlen(dst, maxncpy, format, arg);
+  n = BLI_vsnprintf_rlen(dst, dst_maxncpy, format, arg);
   va_end(arg);
 
   return n;
@@ -260,8 +269,8 @@ int BLI_sprintf(char *__restrict str, const char *__restrict format, ...)
 
 size_t BLI_str_escape(char *__restrict dst, const char *__restrict src, const size_t dst_maxncpy)
 {
-
   BLI_assert(dst_maxncpy != 0);
+  BLI_string_debug_size(dst, dst_maxncpy);
 
   size_t len = 0;
   for (; (len < dst_maxncpy) && (*src != '\0'); dst++, src++, len++) {
@@ -316,6 +325,8 @@ size_t BLI_str_unescape_ex(char *__restrict dst,
                            const size_t dst_maxncpy,
                            bool *r_is_complete)
 {
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   size_t len = 0;
   bool is_complete = true;
   const size_t max_strlen = dst_maxncpy - 1; /* Account for trailing zero byte. */
@@ -337,6 +348,8 @@ size_t BLI_str_unescape_ex(char *__restrict dst,
 
 size_t BLI_str_unescape(char *__restrict dst, const char *__restrict src, const size_t src_maxncpy)
 {
+  BLI_string_debug_size(dst, src_maxncpy); /* `dst` must be at least as big as `src`. */
+
   size_t len = 0;
   for (const char *src_end = src + src_maxncpy; (src < src_end) && *src; src++) {
     char c = *src;
@@ -402,7 +415,7 @@ bool BLI_str_quoted_substr_range(const char *__restrict str,
   return true;
 }
 
-/* NOTE(@campbellbarton): in principal it should be possible to access a quoted string
+/* NOTE(@ideasman42): in principal it should be possible to access a quoted string
  * with an arbitrary size, currently all callers for this functionality
  * happened to use a fixed size buffer, so only #BLI_str_quoted_substr is needed. */
 #if 0
@@ -435,15 +448,17 @@ char *BLI_str_quoted_substrN(const char *__restrict str, const char *__restrict
 bool BLI_str_quoted_substr(const char *__restrict str,
                            const char *__restrict prefix,
                            char *result,
-                           size_t result_maxlen)
+                           size_t result_maxncpy)
 {
+  BLI_string_debug_size(result, result_maxncpy);
+
   int start_match_ofs, end_match_ofs;
   if (!BLI_str_quoted_substr_range(str, prefix, &start_match_ofs, &end_match_ofs)) {
     return false;
   }
   const size_t escaped_len = (size_t)(end_match_ofs - start_match_ofs);
   bool is_complete;
-  BLI_str_unescape_ex(result, str + start_match_ofs, escaped_len, result_maxlen, &is_complete);
+  BLI_str_unescape_ex(result, str + start_match_ofs, escaped_len, result_maxncpy, &is_complete);
   if (is_complete == false) {
     *result = '\0';
   }
@@ -522,19 +537,70 @@ void BLI_str_replace_char(char *str, char src, char dst)
 }
 
 bool BLI_str_replace_table_exact(char *string,
-                                 const size_t string_len,
+                                 const size_t string_maxncpy,
                                  const char *replace_table[][2],
                                  int replace_table_len)
 {
+  BLI_string_debug_size_after_nil(string, string_maxncpy);
+
   for (int i = 0; i < replace_table_len; i++) {
     if (STREQ(string, replace_table[i][0])) {
-      BLI_strncpy(string, replace_table[i][1], string_len);
+      BLI_strncpy(string, replace_table[i][1], string_maxncpy);
       return true;
     }
   }
   return false;
 }
 
+size_t BLI_str_replace_range(
+    char *string, size_t string_maxncpy, int src_beg, int src_end, const char *dst)
+{
+  int string_len = strlen(string);
+  BLI_assert(src_beg <= src_end);
+  BLI_assert(src_end <= string_len);
+  const int src_len = src_end - src_beg;
+  int dst_len = strlen(dst);
+
+  if (src_len < dst_len) {
+    /* Grow, first handle special cases. */
+
+    /* Special case, the src_end is entirely clipped. */
+    if (UNLIKELY(string_maxncpy <= src_beg + dst_len)) {
+      /* There is only room for the destination. */
+      dst_len = ((int)string_maxncpy - src_beg) - 1;
+      string_len = src_end;
+      string[string_len] = '\0';
+    }
+
+    const int ofs = dst_len - src_len;
+    /* Clip the string when inserting the destination string exceeds `string_maxncpy`. */
+    if (string_len + ofs >= string_maxncpy) {
+      string_len = ((int)string_maxncpy - ofs) - 1;
+      string[string_len] = '\0';
+      BLI_assert(src_end <= string_len);
+    }
+
+    /* Grow. */
+    memmove(string + (src_end + ofs), string + src_end, (size_t)(string_len - src_end) + 1);
+    string_len += ofs;
+  }
+  else if (src_len > dst_len) {
+    /* Shrink. */
+    const int ofs = src_len - dst_len;
+    memmove(string + (src_end - ofs), string + src_end, (size_t)(string_len - src_end) + 1);
+    string_len -= ofs;
+  }
+  else { /* Simple case, no resizing. */
+    BLI_assert(src_len == dst_len);
+  }
+
+  if (dst_len > 0) {
+    memcpy(string + src_beg, dst, (size_t)dst_len);
+  }
+  BLI_assert(string[string_len] == '\0');
+  return (size_t)string_len;
+}
+
 /** \} */
 
 /* -------------------------------------------------------------------- */
@@ -1058,19 +1124,19 @@ size_t BLI_str_partition_ex(const char *str,
 }
 
 int BLI_string_find_split_words(
-    const char *str, const size_t len, const char delim, int r_words[][2], int words_max)
+    const char *str, const size_t str_maxlen, const char delim, int r_words[][2], int words_max)
 {
   int n = 0, i;
   bool charsearch = true;
 
   /* Skip leading spaces */
-  for (i = 0; (i < len) && (str[i] != '\0'); i++) {
+  for (i = 0; (i < str_maxlen) && (str[i] != '\0'); i++) {
     if (str[i] != delim) {
       break;
     }
   }
 
-  for (; (i < len) && (str[i] != '\0') && (n < words_max); i++) {
+  for (; (i < str_maxlen) && (str[i] != '\0') && (n < words_max); i++) {
     if ((str[i] != delim) && (charsearch == true)) {
       r_words[n][0] = i;
       charsearch = false;
@@ -1098,7 +1164,7 @@ int BLI_string_find_split_words(
 /** \name String Formatting (Numeric)
  * \{ */
 
-static size_t BLI_str_format_int_grouped_ex(char src[16], char dst[16], int num_len)
+static size_t BLI_str_format_int_grouped_ex(char *src, char *dst, int num_len)
 {
   char *p_src = src;
   char *p_dst = dst;
@@ -1122,26 +1188,37 @@ static size_t BLI_str_format_int_grouped_ex(char src[16], char dst[16], int num_
   return (size_t)(p_dst - dst);
 }
 
-size_t BLI_str_format_int_grouped(char dst[16], int num)
+size_t BLI_str_format_int_grouped(char dst[BLI_STR_FORMAT_INT32_GROUPED_SIZE], int num)
 {
-  char src[16];
-  const int num_len = BLI_snprintf(src, sizeof(src), "%d", num);
+  const size_t dst_maxncpy = BLI_STR_FORMAT_INT32_GROUPED_SIZE;
+  BLI_string_debug_size(dst, dst_maxncpy);
+  UNUSED_VARS_NDEBUG(dst_maxncpy);
+
+  char src[BLI_STR_FORMAT_INT32_GROUPED_SIZE];
+  const int num_len = SNPRINTF(src, "%d", num);
 
   return BLI_str_format_int_grouped_ex(src, dst, num_len);
 }
 
-size_t BLI_str_format_uint64_grouped(char dst[16], uint64_t num)
+size_t BLI_str_format_uint64_grouped(char dst[BLI_STR_FORMAT_UINT64_GROUPED_SIZE], uint64_t num)
 {
-  /* NOTE: Buffer to hold maximum `uint64`, which is 1.8e+19. but
-   * we also need space for commas and null-terminator. */
-  char src[27];
-  const int num_len = BLI_snprintf(src, sizeof(src), "%" PRIu64 "", num);
+  const size_t dst_maxncpy = BLI_STR_FORMAT_UINT64_GROUPED_SIZE;
+  BLI_string_debug_size(dst, dst_maxncpy);
+  UNUSED_VARS_NDEBUG(dst_maxncpy);
+
+  char src[BLI_STR_FORMAT_UINT64_GROUPED_SIZE];
+  const int num_len = SNPRINTF(src, "%" PRIu64 "", num);
 
   return BLI_str_format_int_grouped_ex(src, dst, num_len);
 }
 
-void BLI_str_format_byte_unit(char dst[15], long long int bytes, const bool base_10)
+void BLI_str_format_byte_unit(char dst[BLI_STR_FORMAT_INT64_BYTE_UNIT_SIZE],
+                              long long int bytes,
+                              const bool base_10)
 {
+  const size_t dst_maxncpy = BLI_STR_FORMAT_INT64_BYTE_UNIT_SIZE;
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   double bytes_converted = bytes;
   int order = 0;
   int decimals;
@@ -1159,15 +1236,17 @@ void BLI_str_format_byte_unit(char dst[15], long long int bytes, const bool base
   decimals = MAX2(order - 1, 0);
 
   /* Format value first, stripping away floating zeroes. */
-  const size_t dst_len = 15;
-  size_t len = BLI_snprintf_rlen(dst, dst_len, "%.*f", decimals, bytes_converted);
+  size_t len = BLI_snprintf_rlen(dst, dst_maxncpy, "%.*f", decimals, bytes_converted);
   len -= (size_t)BLI_str_rstrip_float_zero(dst, '\0');
   dst[len++] = ' ';
-  BLI_strncpy(dst + len, base_10 ? units_base_10[order] : units_base_2[order], dst_len - len);
+  BLI_strncpy(dst + len, base_10 ? units_base_10[order] : units_base_2[order], dst_maxncpy - len);
 }
 
-void BLI_str_format_decimal_unit(char dst[7], int number_to_format)
+void BLI_str_format_decimal_unit(char dst[BLI_STR_FORMAT_INT32_DECIMAL_UNIT_SIZE],
+                                 int number_to_format)
 {
+  BLI_string_debug_size(dst, BLI_STR_FORMAT_INT32_DECIMAL_UNIT_SIZE);
+
   float number_to_format_converted = number_to_format;
   int order = 0;
   const float base = 1000;
@@ -1179,16 +1258,20 @@ void BLI_str_format_decimal_unit(char dst[7], int number_to_format)
     order++;
   }
 
-  const size_t dst_len = 7;
+  const size_t dst_maxncpy = BLI_STR_FORMAT_INT32_DECIMAL_UNIT_SIZE;
   int decimals = 0;
   if ((order > 0) && fabsf(number_to_format_converted) < 100.0f) {
     decimals = 1;
   }
-  BLI_snprintf(dst, dst_len, "%.*f%s", decimals, number_to_format_converted, units[order]);
+  BLI_snprintf(dst, dst_maxncpy, "%.*f%s", decimals, number_to_format_converted, units[order]);
 }
 
-void BLI_str_format_integer_unit(char dst[5], const int number_to_format)
+void BLI_str_format_integer_unit(char dst[BLI_STR_FORMAT_INT32_INTEGER_UNIT_SIZE],
+                                 const int number_to_format)
 {
+  const size_t dst_maxncpy = BLI_STR_FORMAT_INT32_INTEGER_UNIT_SIZE;
+  BLI_string_debug_size(dst, dst_maxncpy);
+
   float number_to_format_converted = number_to_format;
   int order = 0;
   const float base = 1000;
@@ -1207,9 +1290,8 @@ void BLI_str_format_integer_unit(char dst[5], const int number_to_format)
     order++;
   }
 
-  const size_t dst_len = 5;
   BLI_snprintf(dst,
-               dst_len,
+               dst_maxncpy,
                "%s%s%d%s",
                number_to_format < 0 ? "-" : "",
                add_dot ? "." : "",
@@ -1218,3 +1300,21 @@ void BLI_str_format_integer_unit(char dst[5], const int number_to_format)
 }
 
 /** \} */
+
+/* -------------------------------------------------------------------- */
+/** \name String Debugging
+ * \{ */
+
+#ifdef WITH_STRSIZE_DEBUG
+void BLI_string_debug_size_after_nil(char *str, size_t str_maxncpy)
+{
+  /* Step over the nil, into the character afterwards. */
+  size_t str_tail = BLI_strnlen(str, str_maxncpy) + 2;
+  if (str_tail < str_maxncpy) {
+    BLI_string_debug_size(str + str_tail, str_maxncpy - str_tail);
+  }
+}
+
+#endif /* WITH_STRSIZE_DEBUG */
+
+/** \} */
