@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2011 Blender Foundation. All rights reserved. */
+ * Copyright 2011 Blender Foundation */
 
 /** \file
  * \ingroup bli
@@ -17,6 +17,18 @@
 #  pragma GCC diagnostic error "-Wsign-conversion"
 #endif
 
+/**
+ * The category of character as returned by #cursor_delim_type_unicode.
+ *
+ * \note Don't compare with any values besides #STRCUR_DELIM_NONE as cursor motion
+ * should only delimit on changes, not treat some groups differently.
+ *
+ * For range calculation the order prioritizes expansion direction,
+ * when the cursor is between two different categories, "hug" the smaller values.
+ * Where white-space gets lowest priority. See #BLI_str_cursor_step_bounds_utf8.
+ * This is done so expanding the range at a word boundary always chooses the word instead
+ * of the white-space before or after it.
+ */
 typedef enum eStrCursorDelimType {
   STRCUR_DELIM_NONE,
   STRCUR_DELIM_ALPHANUMERIC,
@@ -24,8 +36,8 @@ typedef enum eStrCursorDelimType {
   STRCUR_DELIM_BRACE,
   STRCUR_DELIM_OPERATOR,
   STRCUR_DELIM_QUOTE,
-  STRCUR_DELIM_WHITESPACE,
   STRCUR_DELIM_OTHER,
+  STRCUR_DELIM_WHITESPACE,
 } eStrCursorDelimType;
 
 static eStrCursorDelimType cursor_delim_type_unicode(const uint uch)
@@ -96,30 +108,32 @@ static eStrCursorDelimType cursor_delim_type_utf8(const char *ch_utf8,
   return cursor_delim_type_unicode(uch);
 }
 
-/* Keep in sync with BLI_str_cursor_step_next_utf32. */
-bool BLI_str_cursor_step_next_utf8(const char *str, size_t maxlen, int *pos)
+bool BLI_str_cursor_step_next_utf8(const char *str, size_t str_maxlen, int *pos)
 {
-  if ((*pos) >= (int)maxlen) {
+  /* NOTE: Keep in sync with #BLI_str_cursor_step_next_utf32. */
+
+  if ((*pos) >= (int)str_maxlen) {
     return false;
   }
-  const char *str_end = str + (maxlen + 1);
+  const char *str_end = str + (str_maxlen + 1);
   const char *str_pos = str + (*pos);
   const char *str_next = str_pos;
   do {
     str_next = BLI_str_find_next_char_utf8(str_next, str_end);
   } while (str_next < str_end && str_next[0] != 0 && BLI_str_utf8_char_width(str_next) < 1);
   (*pos) += (str_next - str_pos);
-  if ((*pos) > (int)maxlen) {
-    (*pos) = (int)maxlen;
+  if ((*pos) > (int)str_maxlen) {
+    (*pos) = (int)str_maxlen;
   }
 
   return true;
 }
 
-/* Keep in sync with BLI_str_cursor_step_prev_utf32. */
-bool BLI_str_cursor_step_prev_utf8(const char *str, size_t maxlen, int *pos)
+bool BLI_str_cursor_step_prev_utf8(const char *str, size_t str_maxlen, int *pos)
 {
-  if ((*pos) > 0 && (*pos) <= maxlen) {
+  /* NOTE: Keep in sync with #BLI_str_cursor_step_prev_utf32. */
+
+  if ((*pos) > 0 && (*pos) <= str_maxlen) {
     const char *str_pos = str + (*pos);
     const char *str_prev = str_pos;
     do {
@@ -133,7 +147,7 @@ bool BLI_str_cursor_step_prev_utf8(const char *str, size_t maxlen, int *pos)
 }
 
 void BLI_str_cursor_step_utf8(const char *str,
-                              size_t maxlen,
+                              size_t str_maxlen,
                               int *pos,
                               eStrCursorJumpDirection direction,
                               eStrCursorJumpType jump,
@@ -143,26 +157,26 @@ void BLI_str_cursor_step_utf8(const char *str,
 
   if (direction == STRCUR_DIR_NEXT) {
     if (use_init_step) {
-      BLI_str_cursor_step_next_utf8(str, maxlen, pos);
+      BLI_str_cursor_step_next_utf8(str, str_maxlen, pos);
     }
     else {
       BLI_assert(jump == STRCUR_JUMP_DELIM);
     }
 
     if (jump != STRCUR_JUMP_NONE) {
-      const eStrCursorDelimType delim_type = (*pos) < maxlen ?
-                                                 cursor_delim_type_utf8(str, maxlen, *pos) :
+      const eStrCursorDelimType delim_type = (*pos) < str_maxlen ?
+                                                 cursor_delim_type_utf8(str, str_maxlen, *pos) :
                                                  STRCUR_DELIM_NONE;
       /* jump between special characters (/,\,_,-, etc.),
        * look at function cursor_delim_type() for complete
        * list of special character, ctr -> */
-      while ((*pos) < maxlen) {
-        if (BLI_str_cursor_step_next_utf8(str, maxlen, pos)) {
-          if (*pos == maxlen) {
+      while ((*pos) < str_maxlen) {
+        if (BLI_str_cursor_step_next_utf8(str, str_maxlen, pos)) {
+          if (*pos == str_maxlen) {
             break;
           }
           if ((jump != STRCUR_JUMP_ALL) &&
-              (delim_type != cursor_delim_type_utf8(str, maxlen, *pos))) {
+              (delim_type != cursor_delim_type_utf8(str, str_maxlen, *pos))) {
             break;
           }
         }
@@ -174,24 +188,24 @@ void BLI_str_cursor_step_utf8(const char *str,
   }
   else if (direction == STRCUR_DIR_PREV) {
     if (use_init_step) {
-      BLI_str_cursor_step_prev_utf8(str, maxlen, pos);
+      BLI_str_cursor_step_prev_utf8(str, str_maxlen, pos);
     }
     else {
       BLI_assert(jump == STRCUR_JUMP_DELIM);
     }
 
     if (jump != STRCUR_JUMP_NONE) {
-      const eStrCursorDelimType delim_type = (*pos) > 0 ?
-                                                 cursor_delim_type_utf8(str, maxlen, *pos - 1) :
-                                                 STRCUR_DELIM_NONE;
+      const eStrCursorDelimType delim_type = (*pos) > 0 ? cursor_delim_type_utf8(
+                                                              str, str_maxlen, *pos - 1) :
+                                                          STRCUR_DELIM_NONE;
       /* jump between special characters (/,\,_,-, etc.),
        * look at function cursor_delim_type() for complete
        * list of special character, ctr -> */
       while ((*pos) > 0) {
         const int pos_prev = *pos;
-        if (BLI_str_cursor_step_prev_utf8(str, maxlen, pos)) {
+        if (BLI_str_cursor_step_prev_utf8(str, str_maxlen, pos)) {
           if ((jump != STRCUR_JUMP_ALL) &&
-              (delim_type != cursor_delim_type_utf8(str, maxlen, *pos))) {
+              (delim_type != cursor_delim_type_utf8(str, str_maxlen, *pos))) {
             /* left only: compensate for index/change in direction */
             if ((pos_orig - (*pos)) >= 1) {
               *pos = pos_prev;
@@ -210,22 +224,24 @@ void BLI_str_cursor_step_utf8(const char *str,
   }
 }
 
-/* Keep in sync with BLI_str_cursor_step_next_utf8. */
-bool BLI_str_cursor_step_next_utf32(const char32_t *str, size_t maxlen, int *pos)
+bool BLI_str_cursor_step_next_utf32(const char32_t *str, size_t str_maxlen, int *pos)
 {
-  if ((*pos) >= (int)maxlen) {
+  /* NOTE: Keep in sync with #BLI_str_cursor_step_next_utf8. */
+
+  if ((*pos) >= (int)str_maxlen) {
     return false;
   }
   do {
     (*pos)++;
-  } while (*pos < (int)maxlen && str[*pos] != 0 && BLI_wcwidth(str[*pos]) == 0);
+  } while (*pos < (int)str_maxlen && str[*pos] != 0 && BLI_wcwidth(str[*pos]) == 0);
 
   return true;
 }
 
-/* Keep in sync with BLI_str_cursor_step_prev_utf8. */
-bool BLI_str_cursor_step_prev_utf32(const char32_t *str, size_t UNUSED(maxlen), int *pos)
+bool BLI_str_cursor_step_prev_utf32(const char32_t *str, size_t UNUSED(str_maxlen), int *pos)
 {
+  /* NOTE: Keep in sync with #BLI_str_cursor_step_prev_utf8. */
+
   if ((*pos) <= 0) {
     return false;
   }
@@ -237,7 +253,7 @@ bool BLI_str_cursor_step_prev_utf32(const char32_t *str, size_t UNUSED(maxlen),
 }
 
 void BLI_str_cursor_step_utf32(const char32_t *str,
-                               size_t maxlen,
+                               size_t str_maxlen,
                                int *pos,
                                eStrCursorJumpDirection direction,
                                eStrCursorJumpType jump,
@@ -247,21 +263,21 @@ void BLI_str_cursor_step_utf32(const char32_t *str,
 
   if (direction == STRCUR_DIR_NEXT) {
     if (use_init_step) {
-      BLI_str_cursor_step_next_utf32(str, maxlen, pos);
+      BLI_str_cursor_step_next_utf32(str, str_maxlen, pos);
     }
     else {
       BLI_assert(jump == STRCUR_JUMP_DELIM);
     }
 
     if (jump != STRCUR_JUMP_NONE) {
-      const eStrCursorDelimType delim_type = (*pos) < maxlen ?
+      const eStrCursorDelimType delim_type = (*pos) < str_maxlen ?
                                                  cursor_delim_type_unicode((uint)str[*pos]) :
                                                  STRCUR_DELIM_NONE;
       /* jump between special characters (/,\,_,-, etc.),
        * look at function cursor_delim_type_unicode() for complete
        * list of special character, ctr -> */
-      while ((*pos) < maxlen) {
-        if (BLI_str_cursor_step_next_utf32(str, maxlen, pos)) {
+      while ((*pos) < str_maxlen) {
+        if (BLI_str_cursor_step_next_utf32(str, str_maxlen, pos)) {
           if ((jump != STRCUR_JUMP_ALL) &&
               (delim_type != cursor_delim_type_unicode((uint)str[*pos]))) {
             break;
@@ -275,7 +291,7 @@ void BLI_str_cursor_step_utf32(const char32_t *str,
   }
   else if (direction == STRCUR_DIR_PREV) {
     if (use_init_step) {
-      BLI_str_cursor_step_prev_utf32(str, maxlen, pos);
+      BLI_str_cursor_step_prev_utf32(str, str_maxlen, pos);
     }
     else {
       BLI_assert(jump == STRCUR_JUMP_DELIM);
@@ -290,7 +306,7 @@ void BLI_str_cursor_step_utf32(const char32_t *str,
        * list of special character, ctr -> */
       while ((*pos) > 0) {
         const int pos_prev = *pos;
-        if (BLI_str_cursor_step_prev_utf32(str, maxlen, pos)) {
+        if (BLI_str_cursor_step_prev_utf32(str, str_maxlen, pos)) {
           if ((jump != STRCUR_JUMP_ALL) &&
               (delim_type != cursor_delim_type_unicode((uint)str[*pos]))) {
             /* left only: compensate for index/change in direction */
@@ -310,3 +326,46 @@ void BLI_str_cursor_step_utf32(const char32_t *str,
     BLI_assert_unreachable();
   }
 }
+
+void BLI_str_cursor_step_bounds_utf8(
+    const char *str, const size_t str_maxlen, const int pos, int *r_start, int *r_end)
+{
+  /* Identify the type of characters are on either side of the current cursor position. */
+  const eStrCursorDelimType prev = (pos > 0) ? cursor_delim_type_utf8(str, str_maxlen, pos - 1) :
+                                               STRCUR_DELIM_NONE;
+  const eStrCursorDelimType next = (pos < str_maxlen) ?
+                                       cursor_delim_type_utf8(str, str_maxlen, pos) :
+                                       STRCUR_DELIM_NONE;
+  *r_start = pos;
+  *r_end = pos;
+
+  if ((prev <= next) && (prev != STRCUR_DELIM_NONE)) {
+    /* Expand backward if we are between similar content. */
+    BLI_str_cursor_step_utf8(str, str_maxlen, r_start, STRCUR_DIR_PREV, STRCUR_JUMP_DELIM, false);
+  }
+  if ((prev >= next) && (next != STRCUR_DELIM_NONE)) {
+    /* Expand forward if we are between similar content. */
+    BLI_str_cursor_step_utf8(str, str_maxlen, r_end, STRCUR_DIR_NEXT, STRCUR_JUMP_DELIM, false);
+  }
+}
+
+void BLI_str_cursor_step_bounds_utf32(
+    const char32_t *str, const size_t str_maxlen, const int pos, int *r_start, int *r_end)
+{
+  /* Identify the type of characters are on either side of the current cursor position. */
+  const eStrCursorDelimType prev = (pos > 0) ? cursor_delim_type_unicode(str[pos - 1]) :
+                                               STRCUR_DELIM_NONE;
+  const eStrCursorDelimType next = (pos < str_maxlen) ? cursor_delim_type_unicode(str[pos]) :
+                                                        STRCUR_DELIM_NONE;
+  *r_start = pos;
+  *r_end = pos;
+
+  if ((prev <= next) && (prev != STRCUR_DELIM_NONE)) {
+    /* Expand backward if we are between similar content. */
+    BLI_str_cursor_step_utf32(str, str_maxlen, r_start, STRCUR_DIR_PREV, STRCUR_JUMP_DELIM, false);
+  }
+  if ((prev >= next) && (next != STRCUR_DELIM_NONE)) {
+    /* Expand forward if we are between similar content. */
+    BLI_str_cursor_step_utf32(str, str_maxlen, r_end, STRCUR_DIR_NEXT, STRCUR_JUMP_DELIM, false);
+  }
+}
