@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
- * Copyright 2017 Blender Foundation. All rights reserved. */
+ * Copyright 2017 Blender Foundation */
 
 /** \file
  * \ingroup bli
@@ -22,23 +22,26 @@
 #  pragma GCC diagnostic error "-Wsign-conversion"
 #endif
 
-size_t BLI_split_name_num(char *left, int *nr, const char *name, const char delim)
+size_t BLI_string_split_name_number(const char *name,
+                                    const char delim,
+                                    char *r_name_left,
+                                    int *r_number)
 {
   const size_t name_len = strlen(name);
 
-  *nr = 0;
-  memcpy(left, name, (name_len + 1) * sizeof(char));
+  *r_number = 0;
+  memcpy(r_name_left, name, (name_len + 1) * sizeof(char));
 
   /* name doesn't end with a delimiter "foo." */
   if ((name_len > 1 && name[name_len - 1] == delim) == 0) {
     size_t a = name_len;
     while (a--) {
       if (name[a] == delim) {
-        left[a] = '\0'; /* truncate left part here */
-        *nr = atol(name + a + 1);
+        r_name_left[a] = '\0'; /* truncate left part here */
+        *r_number = atol(name + a + 1);
         /* casting down to an int, can overflow for large numbers */
-        if (*nr < 0) {
-          *nr = 0;
+        if (*r_number < 0) {
+          *r_number = 0;
         }
         return a;
       }
@@ -72,9 +75,15 @@ static bool is_char_sep(const char c)
   return ELEM(c, '.', ' ', '-', '_');
 }
 
-void BLI_string_split_suffix(const char *string, char *r_body, char *r_suf, const size_t str_len)
+void BLI_string_split_suffix(const char *string,
+                             const size_t string_maxlen,
+                             char *r_body,
+                             char *r_suf)
 {
-  size_t len = BLI_strnlen(string, str_len);
+  BLI_string_debug_size(r_body, string_maxlen);
+  BLI_string_debug_size(r_suf, string_maxlen);
+
+  size_t len = BLI_strnlen(string, string_maxlen);
   size_t i;
 
   r_body[0] = r_suf[0] = '\0';
@@ -90,9 +99,15 @@ void BLI_string_split_suffix(const char *string, char *r_body, char *r_suf, cons
   memcpy(r_body, string, len + 1);
 }
 
-void BLI_string_split_prefix(const char *string, char *r_pre, char *r_body, const size_t str_len)
+void BLI_string_split_prefix(const char *string,
+                             const size_t string_maxlen,
+                             char *r_pre,
+                             char *r_body)
 {
-  size_t len = BLI_strnlen(string, str_len);
+  BLI_string_debug_size(r_pre, string_maxlen);
+  BLI_string_debug_size(r_body, string_maxlen);
+
+  size_t len = BLI_strnlen(string, string_maxlen);
   size_t i;
 
   r_body[0] = r_pre[0] = '\0';
@@ -109,46 +124,48 @@ void BLI_string_split_prefix(const char *string, char *r_pre, char *r_body, cons
   BLI_strncpy(r_body, string, len);
 }
 
-size_t BLI_string_flip_side_name(char *r_name,
-                                 const char *from_name,
+size_t BLI_string_flip_side_name(char *name_dst,
+                                 const char *name_src,
                                  const bool strip_number,
-                                 const size_t name_len)
+                                 const size_t name_dst_maxncpy)
 {
+  BLI_string_debug_size(name_dst, name_dst_maxncpy);
+
   size_t len;
-  char *prefix = alloca(name_len);  /* The part before the facing */
-  char *suffix = alloca(name_len);  /* The part after the facing */
-  char *replace = alloca(name_len); /* The replacement string */
-  char *number = alloca(name_len);  /* The number extension string */
+  char *prefix = alloca(name_dst_maxncpy);  /* The part before the facing */
+  char *suffix = alloca(name_dst_maxncpy);  /* The part after the facing */
+  char *replace = alloca(name_dst_maxncpy); /* The replacement string */
+  char *number = alloca(name_dst_maxncpy);  /* The number extension string */
   char *index = NULL;
   bool is_set = false;
 
   *prefix = *suffix = *replace = *number = '\0';
 
   /* always copy the name, since this can be called with an uninitialized string */
-  len = BLI_strncpy_rlen(r_name, from_name, name_len);
+  len = BLI_strncpy_rlen(name_dst, name_src, name_dst_maxncpy);
   if (len < 3) {
     /* we don't do names like .R or .L */
     return len;
   }
 
   /* We first check the case with a .### extension, let's find the last period */
-  if (isdigit(r_name[len - 1])) {
-    index = strrchr(r_name, '.');     /* Last occurrence. */
+  if (isdigit(name_dst[len - 1])) {
+    index = strrchr(name_dst, '.');   /* Last occurrence. */
     if (index && isdigit(index[1])) { /* Doesn't handle case `bone.1abc2` correct..., whatever! */
       if (strip_number == false) {
-        BLI_strncpy(number, index, name_len);
+        BLI_strncpy(number, index, name_dst_maxncpy);
       }
       *index = 0;
-      len = BLI_strnlen(r_name, name_len);
+      len = BLI_strnlen(name_dst, name_dst_maxncpy);
     }
   }
 
-  BLI_strncpy(prefix, r_name, name_len);
+  BLI_strncpy(prefix, name_dst, name_dst_maxncpy);
 
   /* First case; separator (`.` or `_`) with extensions in `r R l L`. */
-  if ((len > 1) && is_char_sep(r_name[len - 2])) {
+  if ((len > 1) && is_char_sep(name_dst[len - 2])) {
     is_set = true;
-    switch (r_name[len - 1]) {
+    switch (name_dst[len - 1]) {
       case 'l':
         prefix[len - 1] = 0;
         strcpy(replace, "r");
@@ -171,27 +188,27 @@ size_t BLI_string_flip_side_name(char *r_name,
   }
 
   /* case; beginning with r R l L, with separator after it */
-  if (!is_set && is_char_sep(r_name[1])) {
+  if (!is_set && is_char_sep(name_dst[1])) {
     is_set = true;
-    switch (r_name[0]) {
+    switch (name_dst[0]) {
       case 'l':
         strcpy(replace, "r");
-        BLI_strncpy(suffix, r_name + 1, name_len);
+        BLI_strncpy(suffix, name_dst + 1, name_dst_maxncpy);
         prefix[0] = 0;
         break;
       case 'r':
         strcpy(replace, "l");
-        BLI_strncpy(suffix, r_name + 1, name_len);
+        BLI_strncpy(suffix, name_dst + 1, name_dst_maxncpy);
         prefix[0] = 0;
         break;
       case 'L':
         strcpy(replace, "R");
-        BLI_strncpy(suffix, r_name + 1, name_len);
+        BLI_strncpy(suffix, name_dst + 1, name_dst_maxncpy);
         prefix[0] = 0;
         break;
       case 'R':
         strcpy(replace, "L");
-        BLI_strncpy(suffix, r_name + 1, name_len);
+        BLI_strncpy(suffix, name_dst + 1, name_dst_maxncpy);
         prefix[0] = 0;
         break;
       default:
@@ -210,7 +227,7 @@ size_t BLI_string_flip_side_name(char *r_name,
         strcpy(replace, (index[1] == 'I') ? "LEFT" : "Left");
       }
       *index = 0;
-      BLI_strncpy(suffix, index + 5, name_len);
+      BLI_strncpy(suffix, index + 5, name_dst_maxncpy);
     }
     else if (((index = BLI_strcasestr(prefix, "left")) == prefix) || (index == prefix + len - 4)) {
       is_set = true;
@@ -221,11 +238,12 @@ size_t BLI_string_flip_side_name(char *r_name,
         strcpy(replace, (index[1] == 'E') ? "RIGHT" : "Right");
       }
       *index = 0;
-      BLI_strncpy(suffix, index + 4, name_len);
+      BLI_strncpy(suffix, index + 4, name_dst_maxncpy);
     }
   }
 
-  return BLI_snprintf_rlen(r_name, name_len, "%s%s%s%s", prefix, replace, suffix, number);
+  return BLI_snprintf_rlen(
+      name_dst, name_dst_maxncpy, "%s%s%s%s", prefix, replace, suffix, number);
 }
 
 /* Unique name utils. */
@@ -235,36 +253,38 @@ bool BLI_uniquename_cb(UniquenameCheckCallback unique_check,
                        const char *defname,
                        char delim,
                        char *name,
-                       size_t name_len)
+                       size_t name_maxncpy)
 {
+  BLI_string_debug_size_after_nil(name, name_maxncpy);
+
   if (name[0] == '\0') {
-    BLI_strncpy(name, defname, name_len);
+    BLI_strncpy(name, defname, name_maxncpy);
   }
 
   if (unique_check(arg, name)) {
     char numstr[16];
-    char *tempname = alloca(name_len);
-    char *left = alloca(name_len);
+    char *tempname = alloca(name_maxncpy);
+    char *left = alloca(name_maxncpy);
     int number;
-    size_t len = BLI_split_name_num(left, &number, name, delim);
+    size_t len = BLI_string_split_name_number(name, delim, left, &number);
     do {
       /* add 1 to account for \0 */
-      const size_t numlen = BLI_snprintf(numstr, sizeof(numstr), "%c%03d", delim, ++number) + 1;
+      const size_t numlen = SNPRINTF(numstr, "%c%03d", delim, ++number) + 1;
 
       /* highly unlikely the string only has enough room for the number
        * but support anyway */
-      if ((len == 0) || (numlen >= name_len)) {
+      if ((len == 0) || (numlen >= name_maxncpy)) {
         /* number is know not to be utf-8 */
-        BLI_strncpy(tempname, numstr, name_len);
+        BLI_strncpy(tempname, numstr, name_maxncpy);
       }
       else {
         char *tempname_buf;
-        tempname_buf = tempname + BLI_strncpy_utf8_rlen(tempname, left, name_len - numlen);
+        tempname_buf = tempname + BLI_strncpy_utf8_rlen(tempname, left, name_maxncpy - numlen);
         memcpy(tempname_buf, numstr, numlen);
       }
     } while (unique_check(arg, tempname));
 
-    BLI_strncpy(name, tempname, name_len);
+    BLI_strncpy(name, tempname, name_maxncpy);
 
     return true;
   }
@@ -306,8 +326,12 @@ static bool uniquename_unique_check(void *arg, const char *name)
   return uniquename_find_dupe(data->lb, data->vlink, name, data->name_offset);
 }
 
-bool BLI_uniquename(
-    ListBase *list, void *vlink, const char *defname, char delim, int name_offset, size_t name_len)
+bool BLI_uniquename(ListBase *list,
+                    void *vlink,
+                    const char *defname,
+                    char delim,
+                    int name_offset,
+                    size_t name_maxncpy)
 {
   struct {
     ListBase *lb;
@@ -318,10 +342,10 @@ bool BLI_uniquename(
   data.vlink = vlink;
   data.name_offset = name_offset;
 
-  BLI_assert(name_len > 1);
+  BLI_assert(name_maxncpy > 1);
 
   /* See if we are given an empty string */
-  if (ELEM(NULL, vlink, defname)) {
+  if (ELEM(NULL, vlink)) {
     return false;
   }
 
@@ -330,7 +354,7 @@ bool BLI_uniquename(
                            defname,
                            delim,
                            POINTER_OFFSET(vlink, name_offset),
-                           name_len);
+                           name_maxncpy);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -345,13 +369,15 @@ bool BLI_uniquename(
  * \{ */
 
 size_t BLI_string_join_array(char *result,
-                             size_t result_len,
+                             size_t result_maxncpy,
                              const char *strings[],
-                             uint strings_len)
+                             uint strings_num)
 {
+  BLI_string_debug_size(result, result_maxncpy);
+
   char *c = result;
-  char *c_end = &result[result_len - 1];
-  for (uint i = 0; i < strings_len; i++) {
+  char *c_end = &result[result_maxncpy - 1];
+  for (uint i = 0; i < strings_num; i++) {
     const char *p = strings[i];
     while (*p && (c < c_end)) {
       *c++ = *p++;
@@ -362,11 +388,13 @@ size_t BLI_string_join_array(char *result,
 }
 
 size_t BLI_string_join_array_by_sep_char(
-    char *result, size_t result_len, char sep, const char *strings[], uint strings_len)
+    char *result, size_t result_maxncpy, char sep, const char *strings[], uint strings_num)
 {
+  BLI_string_debug_size(result, result_maxncpy);
+
   char *c = result;
-  char *c_end = &result[result_len - 1];
-  for (uint i = 0; i < strings_len; i++) {
+  char *c_end = &result[result_maxncpy - 1];
+  for (uint i = 0; i < strings_num; i++) {
     if (i != 0) {
       if (c < c_end) {
         *c++ = sep;
@@ -381,26 +409,26 @@ size_t BLI_string_join_array_by_sep_char(
   return (size_t)(c - result);
 }
 
-char *BLI_string_join_arrayN(const char *strings[], uint strings_len)
+char *BLI_string_join_arrayN(const char *strings[], uint strings_num)
 {
   uint total_len = 1;
-  for (uint i = 0; i < strings_len; i++) {
+  for (uint i = 0; i < strings_num; i++) {
     total_len += strlen(strings[i]);
   }
   char *result = MEM_mallocN(sizeof(char) * total_len, __func__);
   char *c = result;
-  for (uint i = 0; i < strings_len; i++) {
+  for (uint i = 0; i < strings_num; i++) {
     c += BLI_strcpy_rlen(c, strings[i]);
   }
-  /* Only needed when `strings_len == 0`. */
+  /* Only needed when `strings_num == 0`. */
   *c = '\0';
   return result;
 }
 
-char *BLI_string_join_array_by_sep_charN(char sep, const char *strings[], uint strings_len)
+char *BLI_string_join_array_by_sep_charN(char sep, const char *strings[], uint strings_num)
 {
   uint total_len = 0;
-  for (uint i = 0; i < strings_len; i++) {
+  for (uint i = 0; i < strings_num; i++) {
     total_len += strlen(strings[i]) + 1;
   }
   if (total_len == 0) {
@@ -409,8 +437,8 @@ char *BLI_string_join_array_by_sep_charN(char sep, const char *strings[], uint s
 
   char *result = MEM_mallocN(sizeof(char) * total_len, __func__);
   char *c = result;
-  if (strings_len != 0) {
-    for (uint i = 0; i < strings_len; i++) {
+  if (strings_num != 0) {
+    for (uint i = 0; i < strings_num; i++) {
       c += BLI_strcpy_rlen(c, strings[i]);
       *c = sep;
       c++;
@@ -424,10 +452,10 @@ char *BLI_string_join_array_by_sep_charN(char sep, const char *strings[], uint s
 char *BLI_string_join_array_by_sep_char_with_tableN(char sep,
                                                     char *table[],
                                                     const char *strings[],
-                                                    uint strings_len)
+                                                    uint strings_num)
 {
   uint total_len = 0;
-  for (uint i = 0; i < strings_len; i++) {
+  for (uint i = 0; i < strings_num; i++) {
     total_len += strlen(strings[i]) + 1;
   }
   if (total_len == 0) {
@@ -436,8 +464,8 @@ char *BLI_string_join_array_by_sep_char_with_tableN(char sep,
 
   char *result = MEM_mallocN(sizeof(char) * total_len, __func__);
   char *c = result;
-  if (strings_len != 0) {
-    for (uint i = 0; i < strings_len; i++) {
+  if (strings_num != 0) {
+    for (uint i = 0; i < strings_num; i++) {
       table[i] = c; /* <-- only difference to BLI_string_join_array_by_sep_charN. */
       c += BLI_strcpy_rlen(c, strings[i]);
       *c = sep;
