@@ -240,7 +240,7 @@ TEST(map, MutableItemToItemConversion)
   map.add(2, 1);
 
   Vector<int> keys, values;
-  for (Map<int, int>::Item item : map.items()) {
+  for (MapItem<int, int> item : map.items()) {
     keys.append(item.key);
     values.append(item.value);
   }
@@ -628,7 +628,7 @@ TEST(map, RemoveDuringIteration)
   Iter begin = map.items().begin();
   Iter end = map.items().end();
   for (Iter iter = begin; iter != end; ++iter) {
-    Map<int, int>::MutableItem item = *iter;
+    MutableMapItem<int, int> item = *iter;
     if (item.value == 2) {
       map.remove(iter);
     }
@@ -646,8 +646,8 @@ TEST(map, RemoveIf)
   for (const int64_t i : IndexRange(100)) {
     map.add(i * i, i);
   }
-  map.remove_if([](auto item) { return item.key > 100; });
-  EXPECT_EQ(map.size(), 11);
+  const int64_t removed = map.remove_if([](auto item) { return item.key > 100; });
+  EXPECT_EQ(map.size() + removed, 100);
   for (const int64_t i : IndexRange(100)) {
     if (i <= 10) {
       EXPECT_EQ(map.lookup(i * i), i);
@@ -671,6 +671,24 @@ TEST(map, LookupKey)
   EXPECT_EQ(map.lookup_key_ptr("a"), map.lookup_key_ptr_as("a"));
 }
 
+TEST(map, VectorKey)
+{
+  Map<Vector<int>, int> map;
+  map.add({1, 2, 3}, 100);
+  map.add({3, 2, 1}, 200);
+
+  EXPECT_EQ(map.size(), 2);
+  EXPECT_EQ(map.lookup({1, 2, 3}), 100);
+  EXPECT_EQ(map.lookup({3, 2, 1}), 200);
+  EXPECT_FALSE(map.contains({1, 2}));
+
+  std::array<int, 3> array = {1, 2, 3};
+  EXPECT_EQ(map.lookup_as(array), 100);
+
+  map.remove_as(Vector<int>({1, 2, 3}).as_mutable_span());
+  EXPECT_EQ(map.size(), 1);
+}
+
 /**
  * Set this to 1 to activate the benchmark. It is disabled by default, because it prints a lot.
  */
