@@ -20,7 +20,7 @@
 #include "DNA_cloth_types.h"
 #include "DNA_constraint_types.h"
 #include "DNA_fluid_types.h"
-#include "DNA_gpencil_types.h"
+#include "DNA_gpencil_legacy_types.h"
 #include "DNA_light_types.h"
 #include "DNA_linestyle_types.h"
 #include "DNA_mask_types.h"
@@ -96,7 +96,7 @@ static bGPDpalette *BKE_gpencil_palette_addnew(bGPdata *gpd, const char *name)
 
   /* set basic settings */
   /* auto-name */
-  BLI_strncpy(palette->info, name, sizeof(palette->info));
+  STRNCPY(palette->info, name);
   BLI_uniquename(&gpd->palettes,
                  palette,
                  DATA_("GP_Palette"),
@@ -129,7 +129,7 @@ static bGPDpalettecolor *BKE_gpencil_palettecolor_addnew(bGPDpalette *palette, c
   ARRAY_SET_ITEMS(palcolor->fill, 1.0f, 1.0f, 1.0f);
 
   /* auto-name */
-  BLI_strncpy(palcolor->info, name, sizeof(palcolor->info));
+  STRNCPY(palcolor->info, name);
   BLI_uniquename(&palette->colors,
                  palcolor,
                  DATA_("Color"),
@@ -149,13 +149,13 @@ static bGPDpalettecolor *BKE_gpencil_palettecolor_addnew(bGPDpalette *palette, c
  */
 static void migrate_single_rot_stabilization_track_settings(MovieTrackingStabilization *stab)
 {
-  if (stab->rot_track) {
-    if (!(stab->rot_track->flag & TRACK_USE_2D_STAB_ROT)) {
+  if (stab->rot_track_legacy) {
+    if (!(stab->rot_track_legacy->flag & TRACK_USE_2D_STAB_ROT)) {
       stab->tot_rot_track++;
-      stab->rot_track->flag |= TRACK_USE_2D_STAB_ROT;
+      stab->rot_track_legacy->flag |= TRACK_USE_2D_STAB_ROT;
     }
   }
-  stab->rot_track = NULL; /* this field is now ignored */
+  stab->rot_track_legacy = NULL; /* this field is now ignored */
 }
 
 static void do_version_constraints_radians_degrees_270_1(ListBase *lb)
@@ -276,13 +276,22 @@ static void do_version_hue_sat_node(bNodeTree *ntree, bNode *node)
     return;
   }
 
-  /* Make sure new sockets are properly created. */
-  node_verify_sockets(ntree, node, false);
   /* Convert value from old storage to new sockets. */
   NodeHueSat *nhs = node->storage;
-  bNodeSocket *hue = nodeFindSocket(node, SOCK_IN, "Hue"),
-              *saturation = nodeFindSocket(node, SOCK_IN, "Saturation"),
-              *value = nodeFindSocket(node, SOCK_IN, "Value");
+  bNodeSocket *hue = nodeFindSocket(node, SOCK_IN, "Hue");
+  bNodeSocket *saturation = nodeFindSocket(node, SOCK_IN, "Saturation");
+  bNodeSocket *value = nodeFindSocket(node, SOCK_IN, "Value");
+  if (hue == NULL) {
+    hue = nodeAddStaticSocket(ntree, node, SOCK_IN, SOCK_FLOAT, PROP_FACTOR, "Hue", "Hue");
+  }
+  if (saturation == NULL) {
+    saturation = nodeAddStaticSocket(
+        ntree, node, SOCK_IN, SOCK_FLOAT, PROP_FACTOR, "Saturation", "Saturation");
+  }
+  if (value == NULL) {
+    value = nodeAddStaticSocket(ntree, node, SOCK_IN, SOCK_FLOAT, PROP_FACTOR, "Value", "Value");
+  }
+
   ((bNodeSocketValueFloat *)hue->default_value)->value = nhs->hue;
   ((bNodeSocketValueFloat *)saturation->default_value)->value = nhs->sat;
   ((bNodeSocketValueFloat *)value->default_value)->value = nhs->val;
@@ -311,13 +320,12 @@ static void do_versions_compositor_render_passes_storage(bNode *node)
   int pass_index = 0;
   const char *sockname;
   for (bNodeSocket *sock = node->outputs.first; sock && pass_index < 31;
-       sock = sock->next, pass_index++) {
+       sock = sock->next, pass_index++)
+  {
     if (sock->storage == NULL) {
       NodeImageLayer *sockdata = MEM_callocN(sizeof(NodeImageLayer), "node image layer");
       sock->storage = sockdata;
-      BLI_strncpy(sockdata->pass_name,
-                  node_cmp_rlayers_sock_to_pass(pass_index),
-                  sizeof(sockdata->pass_name));
+      STRNCPY(sockdata->pass_name, node_cmp_rlayers_sock_to_pass(pass_index));
 
       if (pass_index == 0) {
         sockname = "Image";
@@ -328,7 +336,7 @@ static void do_versions_compositor_render_passes_storage(bNode *node)
       else {
         sockname = node_cmp_rlayers_sock_to_pass(pass_index);
       }
-      BLI_strncpy(sock->name, sockname, sizeof(sock->name));
+      STRNCPY(sock->name, sockname);
     }
   }
 }
@@ -400,7 +408,7 @@ static void do_version_bbone_easing_fcurve_fix(ID *UNUSED(id),
         FMod_Stepped *data = fcm->data;
 
         /* Modifier doesn't work if the modifier's copy of start/end frame are both 0
-         * as those were only getting written to the fcm->data copy (T52009)
+         * as those were only getting written to the fcm->data copy (#52009)
          */
         if ((fcm->sfra == fcm->efra) && (fcm->sfra == 0)) {
           fcm->sfra = data->start_frame;
@@ -548,7 +556,7 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
 
   if (!MAIN_VERSION_ATLEAST(bmain, 270, 4)) {
     /* ui_previews were not handled correctly when copying areas,
-     * leading to corrupted files (see T39847).
+     * leading to corrupted files (see #39847).
      * This will always reset situation to a valid state.
      */
     bScreen *screen;
@@ -600,7 +608,7 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
         sce->r.bake.normal_swizzle[0] = R_BAKE_POSX;
         sce->r.bake.normal_swizzle[1] = R_BAKE_POSY;
         sce->r.bake.normal_swizzle[2] = R_BAKE_POSZ;
-        BLI_strncpy(sce->r.bake.filepath, U.renderdir, sizeof(sce->r.bake.filepath));
+        STRNCPY(sce->r.bake.filepath, U.renderdir);
 
         sce->r.bake.im_format.planes = R_IMF_PLANES_RGBA;
         sce->r.bake.im_format.imtype = R_IMF_IMTYPE_PNG;
@@ -681,7 +689,8 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
     Brush *br;
     for (br = bmain->brushes.first; br; br = br->id.next) {
       if ((br->ob_mode & OB_MODE_SCULPT) &&
-          ELEM(br->sculpt_tool, SCULPT_TOOL_GRAB, SCULPT_TOOL_SNAKE_HOOK)) {
+          ELEM(br->sculpt_tool, SCULPT_TOOL_GRAB, SCULPT_TOOL_SNAKE_HOOK))
+      {
         br->alpha = 1.0f;
       }
     }
@@ -919,11 +928,11 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
     for (scene = bmain->scenes.first; scene; scene = scene->id.next) {
       BKE_scene_add_render_view(scene, STEREO_LEFT_NAME);
       srv = scene->r.views.first;
-      BLI_strncpy(srv->suffix, STEREO_LEFT_SUFFIX, sizeof(srv->suffix));
+      STRNCPY(srv->suffix, STEREO_LEFT_SUFFIX);
 
       BKE_scene_add_render_view(scene, STEREO_RIGHT_NAME);
       srv = scene->r.views.last;
-      BLI_strncpy(srv->suffix, STEREO_RIGHT_SUFFIX, sizeof(srv->suffix));
+      STRNCPY(srv->suffix, STEREO_RIGHT_SUFFIX);
 
       if (scene->ed) {
         SEQ_for_each_callback(&scene->ed->seqbase, seq_update_proxy_cb, NULL);
@@ -968,7 +977,7 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
         BLI_addtail(&ima->packedfiles, imapf);
 
         imapf->packedfile = ima->packedfile;
-        BLI_strncpy(imapf->filepath, ima->filepath, FILE_MAX);
+        STRNCPY(imapf->filepath, ima->filepath);
         ima->packedfile = NULL;
       }
     }
@@ -1361,7 +1370,7 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
               LISTBASE_FOREACH (bGPDframe *, gpf, &gpl->frames) {
                 LISTBASE_FOREACH (bGPDstroke *, gps, &gpf->strokes) {
                   /* set stroke to palette and force recalculation */
-                  BLI_strncpy(gps->colorname, gpl->info, sizeof(gps->colorname));
+                  STRNCPY(gps->colorname, gpl->info);
                   gps->thickness = gpl->thickness;
 
                   /* set alpha strength to 1 */
@@ -1380,26 +1389,24 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
 
   if (!MAIN_VERSION_ATLEAST(bmain, 278, 0)) {
     if (!DNA_struct_elem_find(fd->filesdna, "MovieTrackingTrack", "float", "weight_stab")) {
-      MovieClip *clip;
-      for (clip = bmain->movieclips.first; clip; clip = clip->id.next) {
-        MovieTracking *tracking = &clip->tracking;
-        MovieTrackingObject *tracking_object;
-        for (tracking_object = tracking->objects.first; tracking_object != NULL;
-             tracking_object = tracking_object->next) {
-          ListBase *tracksbase = BKE_tracking_object_get_tracks(tracking, tracking_object);
-          MovieTrackingTrack *track;
-          for (track = tracksbase->first; track != NULL; track = track->next) {
+      LISTBASE_FOREACH (MovieClip *, clip, &bmain->movieclips) {
+        const MovieTracking *tracking = &clip->tracking;
+        LISTBASE_FOREACH (MovieTrackingObject *, tracking_object, &tracking->objects) {
+          const ListBase *tracksbase = (tracking_object->flag & TRACKING_OBJECT_CAMERA) ?
+                                           &tracking->tracks_legacy :
+                                           &tracking_object->tracks;
+          LISTBASE_FOREACH (MovieTrackingTrack *, track, tracksbase) {
             track->weight_stab = track->weight;
           }
         }
       }
     }
 
-    if (!DNA_struct_elem_find(
-            fd->filesdna, "MovieTrackingStabilization", "int", "tot_rot_track")) {
+    if (!DNA_struct_elem_find(fd->filesdna, "MovieTrackingStabilization", "int", "tot_rot_track"))
+    {
       MovieClip *clip;
       for (clip = bmain->movieclips.first; clip != NULL; clip = clip->id.next) {
-        if (clip->tracking.stabilization.rot_track) {
+        if (clip->tracking.stabilization.rot_track_legacy) {
           migrate_single_rot_stabilization_track_settings(&clip->tracking.stabilization);
         }
         if (clip->tracking.stabilization.scale == 0.0f) {
@@ -1413,8 +1420,6 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
         clip->tracking.stabilization.anchor_frame = 1;
         /* by default show the track lists expanded, to improve "discoverability" */
         clip->tracking.stabilization.flag |= TRACKING_SHOW_STAB_TRACKS;
-        /* deprecated, not used anymore */
-        clip->tracking.stabilization.ok = false;
       }
     }
   }
@@ -1534,8 +1539,8 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
       LISTBASE_FOREACH (MaskLayer *, mlayer, &mask->masklayers) {
         LISTBASE_FOREACH (MaskSpline *, mspline, &mlayer->splines) {
           int i = 0;
-          for (MaskSplinePoint *mspoint = mspline->points; i < mspline->tot_point;
-               mspoint++, i++) {
+          for (MaskSplinePoint *mspoint = mspline->points; i < mspline->tot_point; mspoint++, i++)
+          {
             if (mspoint->parent.id_type == 0) {
               BKE_mask_parent_init(&mspoint->parent);
             }
@@ -1544,7 +1549,7 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
       }
     }
 
-    /* Fix for T50736, Glare comp node using same var for two different things. */
+    /* Fix for #50736, Glare comp node using same var for two different things. */
     if (!DNA_struct_elem_find(fd->filesdna, "NodeGlare", "char", "star_45")) {
       FOREACH_NODETREE_BEGIN (bmain, ntree, id) {
         if (ntree->type == NTREE_COMPOSIT) {
@@ -1595,7 +1600,7 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
       }
     }
 
-    /* Fix related to VGroup modifiers creating named defgroup CD layers! See T51520. */
+    /* Fix related to VGroup modifiers creating named defgroup CD layers! See #51520. */
     for (Mesh *me = bmain->meshes.first; me; me = me->id.next) {
       CustomData_set_layer_name(&me->vdata, CD_MDEFORMVERT, 0, "");
     }
@@ -1680,7 +1685,8 @@ void blo_do_versions_270(FileData *fd, Library *UNUSED(lib), Main *bmain)
     }
 
     if (!DNA_struct_elem_find(
-            fd->filesdna, "ParticleInstanceModifierData", "float", "particle_amount")) {
+            fd->filesdna, "ParticleInstanceModifierData", "float", "particle_amount"))
+    {
       for (Object *ob = bmain->objects.first; ob; ob = ob->id.next) {
         LISTBASE_FOREACH (ModifierData *, md, &ob->modifiers) {
           if (md->type == eModifierType_ParticleInstance) {
