@@ -6,6 +6,10 @@
 
 #pragma once
 
+#ifdef __cplusplus
+#  include "BLI_map.hh"
+#endif
+
 struct ARegion;
 struct ListBase;
 struct Main;
@@ -15,10 +19,24 @@ struct bNodeTree;
 extern "C" {
 #endif
 
+/**
+ * Check if a region of type \a region_type exists in \a regionbase. Otherwise add it after the
+ * first region of type \a link_after_region_type.
+ * \returns null if a region of the given type already existed, otherwise the newly added region.
+ */
 struct ARegion *do_versions_add_region_if_not_found(struct ListBase *regionbase,
                                                     int region_type,
-                                                    const char *name,
+                                                    const char *allocname,
                                                     int link_after_region_type);
+/**
+ * Check if a region of type \a region_type exists in \a regionbase. Otherwise add it after the
+ * first region of type \a link_after_region_type.
+ * \returns either a new, or already existing region.
+ */
+ARegion *do_versions_ensure_region(ListBase *regionbase,
+                                   int region_type,
+                                   const char *allocname,
+                                   int link_after_region_type);
 
 /**
  * Rename if the ID doesn't exist.
@@ -84,12 +102,21 @@ struct bNodeSocket *version_node_add_socket_if_not_exist(struct bNodeTree *ntree
                                                          const char *name);
 
 /**
- * The versioning code generally expects `SOCK_IN_USE` to be set correctly. This function updates
- * the flag on all sockets after changes to the node tree.
+ * The versioning code generally expects `SOCK_IS_LINKED` to be set correctly. This function
+ * updates the flag on all sockets after changes to the node tree.
  */
 void version_socket_update_is_used(bNodeTree *ntree);
 ARegion *do_versions_add_region(int regiontype, const char *name);
 
+void sequencer_init_preview_region(ARegion *region);
+
 #ifdef __cplusplus
 }
 #endif
+
+#ifdef __cplusplus
+void node_tree_relink_with_socket_id_map(bNodeTree &ntree,
+                                         bNode &old_node,
+                                         bNode &new_node,
+                                         const blender::Map<std::string, std::string> &map);
+#endif
