@@ -102,8 +102,9 @@
 #include "BKE_layer.h"
 #include "BKE_lib_id.h"
 #include "BKE_lib_override.h"
+#include "BKE_lib_query.h"
 #include "BKE_main.h"
-#include "BKE_node.h"
+#include "BKE_node.hh"
 #include "BKE_packedFile.h"
 #include "BKE_report.h"
 #include "BKE_workspace.h"
@@ -623,7 +624,8 @@ static void mywrite_id_begin(WriteData *wd, ID *id)
     if (wd->mem.id_session_uuid_mapping != nullptr &&
         (curr_memchunk == nullptr || curr_memchunk->id_session_uuid != id->session_uuid ||
          (prev_memchunk != nullptr &&
-          (prev_memchunk->id_session_uuid == curr_memchunk->id_session_uuid)))) {
+          (prev_memchunk->id_session_uuid == curr_memchunk->id_session_uuid))))
+    {
       void *ref = BLI_ghash_lookup(wd->mem.id_session_uuid_mapping,
                                    POINTER_FROM_UINT(id->session_uuid));
       if (ref != nullptr) {
@@ -787,7 +789,7 @@ static void current_screen_compat(Main *mainvar,
 
   if (wm) {
     if (use_active_win) {
-      /* write the active window into the file, needed for multi-window undo T43424 */
+      /* write the active window into the file, needed for multi-window undo #43424 */
       for (window = static_cast<wmWindow *>(wm->windows.first); window; window = window->next) {
         if (window->active) {
           break;
@@ -820,7 +822,7 @@ struct RenderInfo {
  * This was originally added for the historic render-daemon feature,
  * now write because it can be easily extracted without reading the whole blend file.
  *
- * See: `release/scripts/modules/blend_render_info.py`
+ * See: `scripts/modules/blend_render_info.py`
  */
 static void write_renderinfo(WriteData *wd, Main *mainvar)
 {
@@ -838,9 +840,9 @@ static void write_renderinfo(WriteData *wd, Main *mainvar)
       data.efra = sce->r.efra;
       memset(data.scene_name, 0, sizeof(data.scene_name));
 
-      BLI_strncpy(data.scene_name, sce->id.name + 2, sizeof(data.scene_name));
+      STRNCPY(data.scene_name, sce->id.name + 2);
 
-      writedata(wd, REND, sizeof(data), &data);
+      writedata(wd, BLO_CODE_REND, sizeof(data), &data);
     }
   }
 }
@@ -855,7 +857,7 @@ static void write_keymapitem(BlendWriter *writer, const wmKeyMapItem *kmi)
 
 static void write_userdef(BlendWriter *writer, const UserDef *userdef)
 {
-  writestruct(writer->wd, USER, UserDef, 1, userdef);
+  writestruct(writer->wd, BLO_CODE_USER, UserDef, 1, userdef);
 
   LISTBASE_FOREACH (const bTheme *, btheme, &userdef->themes) {
     BLO_write_struct(writer, bTheme, btheme);
@@ -921,6 +923,10 @@ static void write_userdef(BlendWriter *writer, const UserDef *userdef)
     BLO_write_struct(writer, bPathCompare, path_cmp);
   }
 
+  LISTBASE_FOREACH (const bUserScriptDirectory *, script_dir, &userdef->script_directories) {
+    BLO_write_struct(writer, bUserScriptDirectory, script_dir);
+  }
+
   LISTBASE_FOREACH (const bUserAssetLibrary *, asset_library_ref, &userdef->asset_libraries) {
     BLO_write_struct(writer, bUserAssetLibrary, asset_library_ref);
   }
@@ -954,9 +960,9 @@ static void write_libraries(WriteData *wd, Main *main)
       found_one = false;
       while (!found_one && tot--) {
         for (id = static_cast<ID *>(lbarray[tot]->first); id; id = static_cast<ID *>(id->next)) {
-          if (id->us > 0 &&
-              ((id->tag & LIB_TAG_EXTERN) ||
-               ((id->tag & LIB_TAG_INDIRECT) && (id->flag & LIB_INDIRECT_WEAK_LINK)))) {
+          if (id->us > 0 && ((id->tag & LIB_TAG_EXTERN) || ((id->tag & LIB_TAG_INDIRECT) &&
+                                                            (id->flag & LIB_INDIRECT_WEAK_LINK))))
+          {
             found_one = true;
             break;
           }
@@ -990,9 +996,9 @@ static void write_libraries(WriteData *wd, Main *main)
       /* Write link placeholders for all direct linked IDs. */
       while (a--) {
         for (id = static_cast<ID *>(lbarray[a]->first); id; id = static_cast<ID *>(id->next)) {
-          if (id->us > 0 &&
-              ((id->tag & LIB_TAG_EXTERN) ||
-               ((id->tag & LIB_TAG_INDIRECT) && (id->flag & LIB_INDIRECT_WEAK_LINK)))) {
+          if (id->us > 0 && ((id->tag & LIB_TAG_EXTERN) || ((id->tag & LIB_TAG_INDIRECT) &&
+                                                            (id->flag & LIB_INDIRECT_WEAK_LINK))))
+          {
             if (!BKE_idtype_idcode_is_linkable(GS(id->name))) {
               CLOG_ERROR(&LOG,
                          "Data-block '%s' from lib '%s' is not linkable, but is flagged as "
@@ -1048,7 +1054,7 @@ static void write_global(WriteData *wd, int fileflags, Main *mainvar)
   if (fileflags & G_FILE_RECOVER_WRITE) {
     STRNCPY(fg.filepath, mainvar->filepath);
   }
-  BLI_snprintf(subvstr, sizeof(subvstr), "%4d", BLENDER_FILE_SUBVERSION);
+  SNPRINTF(subvstr, "%4d", BLENDER_FILE_SUBVERSION);
   memcpy(fg.subvstr, subvstr, 4);
 
   fg.subversion = BLENDER_FILE_SUBVERSION;
@@ -1057,12 +1063,12 @@ static void write_global(WriteData *wd, int fileflags, Main *mainvar)
 #ifdef WITH_BUILDINFO
   /* TODO(sergey): Add branch name to file as well? */
   fg.build_commit_timestamp = build_commit_timestamp;
-  BLI_strncpy(fg.build_hash, build_hash, sizeof(fg.build_hash));
+  STRNCPY(fg.build_hash, build_hash);
 #else
   fg.build_commit_timestamp = 0;
-  BLI_strncpy(fg.build_hash, "unknown", sizeof(fg.build_hash));
+  STRNCPY(fg.build_hash, "unknown");
 #endif
-  writestruct(wd, GLOB, FileGlobal, 1, &fg);
+  writestruct(wd, BLO_CODE_GLOB, FileGlobal, 1, &fg);
 }
 
 /**
@@ -1073,7 +1079,7 @@ static void write_global(WriteData *wd, int fileflags, Main *mainvar)
 static void write_thumb(WriteData *wd, const BlendThumbnail *thumb)
 {
   if (thumb) {
-    writedata(wd, TEST, BLEN_THUMB_MEMSIZE_FILE(thumb->width, thumb->height), thumb);
+    writedata(wd, BLO_CODE_TEST, BLEN_THUMB_MEMSIZE_FILE(thumb->width, thumb->height), thumb);
   }
 }
 
@@ -1083,6 +1089,106 @@ static void write_thumb(WriteData *wd, const BlendThumbnail *thumb)
 /** \name File Writing (Private)
  * \{ */
 
+#define ID_BUFFER_STATIC_SIZE 8192
+
+typedef struct BLO_Write_IDBuffer {
+  const struct IDTypeInfo *id_type;
+  ID *temp_id;
+  char id_buffer_static[ID_BUFFER_STATIC_SIZE];
+} BLO_Write_IDBuffer;
+
+static void id_buffer_init_for_id_type(BLO_Write_IDBuffer *id_buffer, const IDTypeInfo *id_type)
+{
+  if (id_type != id_buffer->id_type) {
+    const size_t idtype_struct_size = id_type->struct_size;
+    if (idtype_struct_size > ID_BUFFER_STATIC_SIZE) {
+      CLOG_ERROR(&LOG,
+                 "ID maximum buffer size (%d bytes) is not big enough to fit IDs of type %s, "
+                 "which needs %lu bytes",
+                 ID_BUFFER_STATIC_SIZE,
+                 id_type->name,
+                 idtype_struct_size);
+      id_buffer->temp_id = static_cast<ID *>(MEM_mallocN(idtype_struct_size, __func__));
+    }
+    else {
+      if (static_cast<void *>(id_buffer->temp_id) != id_buffer->id_buffer_static) {
+        MEM_SAFE_FREE(id_buffer->temp_id);
+      }
+      id_buffer->temp_id = reinterpret_cast<ID *>(id_buffer->id_buffer_static);
+    }
+    id_buffer->id_type = id_type;
+  }
+}
+
+static void id_buffer_init_from_id(BLO_Write_IDBuffer *id_buffer, ID *id, const bool is_undo)
+{
+  BLI_assert(id_buffer->id_type == BKE_idtype_get_info_from_id(id));
+
+  if (is_undo) {
+    /* Record the changes that happened up to this undo push in
+     * recalc_up_to_undo_push, and clear `recalc_after_undo_push` again
+     * to start accumulating for the next undo push. */
+    id->recalc_up_to_undo_push = id->recalc_after_undo_push;
+    id->recalc_after_undo_push = 0;
+  }
+
+  /* Copy ID data itself into buffer, to be able to freely modify it. */
+  const size_t idtype_struct_size = id_buffer->id_type->struct_size;
+  ID *temp_id = id_buffer->temp_id;
+  memcpy(temp_id, id, idtype_struct_size);
+
+  /* Clear runtime data to reduce false detection of changed data in undo/redo context. */
+  if (is_undo) {
+    temp_id->tag &= LIB_TAG_KEEP_ON_UNDO;
+  }
+  else {
+    temp_id->tag = 0;
+  }
+  temp_id->us = 0;
+  temp_id->icon_id = 0;
+  /* Those listbase data change every time we add/remove an ID, and also often when
+   * renaming one (due to re-sorting). This avoids generating a lot of false 'is changed'
+   * detections between undo steps. */
+  temp_id->prev = nullptr;
+  temp_id->next = nullptr;
+  /* Those runtime pointers should never be set during writing stage, but just in case clear
+   * them too. */
+  temp_id->orig_id = nullptr;
+  temp_id->newid = nullptr;
+  /* Even though in theory we could be able to preserve this python instance across undo even
+   * when we need to re-read the ID into its original address, this is currently cleared in
+   * #direct_link_id_common in `readfile.c` anyway. */
+  temp_id->py_instance = nullptr;
+}
+
+/* Helper callback for checking linked IDs used by given ID (assumed local), to ensure directly
+ * linked data is tagged accordingly. */
+static int write_id_direct_linked_data_process_cb(LibraryIDLinkCallbackData *cb_data)
+{
+  ID *self_id = cb_data->self_id;
+  ID *id = *cb_data->id_pointer;
+  const int cb_flag = cb_data->cb_flag;
+
+  if (id == nullptr || !ID_IS_LINKED(id)) {
+    return IDWALK_RET_NOP;
+  }
+  BLI_assert(!ID_IS_LINKED(self_id));
+  BLI_assert((cb_flag & IDWALK_CB_INDIRECT_USAGE) == 0);
+
+  if (self_id->tag & LIB_TAG_RUNTIME) {
+    return IDWALK_RET_NOP;
+  }
+
+  if (cb_flag & IDWALK_CB_DIRECT_WEAK_LINK) {
+    id_lib_indirect_weak_link(id);
+  }
+  else {
+    id_lib_extern(id);
+  }
+
+  return IDWALK_RET_NOP;
+}
+
 /* if MemFile * there's filesave to memory */
 static bool write_file_handle(Main *mainvar,
                               WriteWrap *ww,
@@ -1097,17 +1203,47 @@ static bool write_file_handle(Main *mainvar,
   char buf[16];
   WriteData *wd;
 
-  blo_split_main(&mainlist, mainvar);
-
   wd = mywrite_begin(ww, compare, current);
   BlendWriter writer = {wd};
 
-  BLI_snprintf(buf,
-               sizeof(buf),
-               "BLENDER%c%c%.3d",
-               (sizeof(void *) == 8) ? '-' : '_',
-               (ENDIAN_ORDER == B_ENDIAN) ? 'V' : 'v',
-               BLENDER_FILE_VERSION);
+  /* Clear 'directly linked' flag for all linked data, these are not necessarily valid/up-to-date
+   * info, they will be re-generated while write code is processing local IDs below. */
+  if (!wd->use_memfile) {
+    ID *id_iter;
+    FOREACH_MAIN_ID_BEGIN (mainvar, id_iter) {
+      if (ID_IS_LINKED(id_iter) && BKE_idtype_idcode_is_linkable(GS(id_iter->name))) {
+        if (USER_EXPERIMENTAL_TEST(&U, use_all_linked_data_direct)) {
+          /* Forces all linked data to be considered as directly linked.
+           * FIXME: Workaround some BAT tool limitations for Heist production, should be removed
+           * asap afterward. */
+          id_lib_extern(id_iter);
+        }
+        else if (ID_FAKE_USERS(id_iter) > 0 && id_iter->asset_data == nullptr) {
+          /* Even though fake user is not directly editable by the user on linked data, it is a
+           * common 'work-around' to set it in library files on data-blocks that need to be linked
+           * but typically do not have an actual real user (e.g. texts, etc.).
+           * See e.g. #105687 and #103867.
+           *
+           * Would be good to find a better solution, but for now consider these as directly linked
+           * as well. */
+          id_lib_extern(id_iter);
+        }
+        else {
+          id_iter->tag |= LIB_TAG_INDIRECT;
+          id_iter->tag &= ~LIB_TAG_EXTERN;
+        }
+      }
+    }
+    FOREACH_MAIN_ID_END;
+  }
+
+  blo_split_main(&mainlist, mainvar);
+
+  SNPRINTF(buf,
+           "BLENDER%c%c%.3d",
+           (sizeof(void *) == 8) ? '-' : '_',
+           (ENDIAN_ORDER == B_ENDIAN) ? 'V' : 'v',
+           BLENDER_FILE_VERSION);
 
   mywrite(wd, buf, 12);
 
@@ -1123,11 +1259,11 @@ static bool write_file_handle(Main *mainvar,
                                                  nullptr :
                                                  BKE_lib_override_library_operations_store_init();
 
-#define ID_BUFFER_STATIC_SIZE 8192
   /* This outer loop allows to save first data-blocks from real mainvar,
    * then the temp ones from override process,
    * if needed, without duplicating whole code. */
   Main *bmain = mainvar;
+  BLO_Write_IDBuffer *id_buffer = BLO_write_allocate_id_buffer();
   do {
     ListBase *lbarray[INDEX_ID_MAX];
     int a = set_listbasepointers(bmain, lbarray);
@@ -1138,19 +1274,8 @@ static bool write_file_handle(Main *mainvar,
         continue; /* Libraries are handled separately below. */
       }
 
-      char id_buffer_static[ID_BUFFER_STATIC_SIZE];
-      void *id_buffer = id_buffer_static;
       const IDTypeInfo *id_type = BKE_idtype_get_info_from_id(id);
-      const size_t idtype_struct_size = id_type->struct_size;
-      if (idtype_struct_size > ID_BUFFER_STATIC_SIZE) {
-        CLOG_ERROR(&LOG,
-                   "ID maximum buffer size (%d bytes) is not big enough to fit IDs of type %s, "
-                   "which needs %lu bytes",
-                   ID_BUFFER_STATIC_SIZE,
-                   id_type->name,
-                   id_type->struct_size);
-        id_buffer = MEM_mallocN(idtype_struct_size, __func__);
-      }
+      id_buffer_init_for_id_type(id_buffer, id_type);
 
       for (; id; id = static_cast<ID *>(id->next)) {
         /* We should never attempt to write non-regular IDs
@@ -1160,65 +1285,37 @@ static bool write_file_handle(Main *mainvar,
 
         /* We only write unused IDs in undo case.
          * NOTE: All Scenes, WindowManagers and WorkSpaces should always be written to disk, so
-         * their user-count should never be nullptr currently. */
+         * their user-count should never be zero currently. */
         if (id->us == 0 && !wd->use_memfile) {
           BLI_assert(!ELEM(GS(id->name), ID_SCE, ID_WM, ID_WS));
           continue;
         }
 
+        if ((id->tag & LIB_TAG_RUNTIME) != 0 && !wd->use_memfile) {
+          /* Runtime IDs are never written to .blend files, and they should not influence
+           * (in)direct status of linked IDs they may use. */
+          continue;
+        }
+
         const bool do_override = !ELEM(override_storage, nullptr, bmain) &&
                                  ID_IS_OVERRIDE_LIBRARY_REAL(id);
 
+        /* If not writing undo data, properly set directly linked IDs as `LIB_TAG_EXTERN`. */
+        if (!wd->use_memfile) {
+          BKE_library_foreach_ID_link(
+              bmain, id, write_id_direct_linked_data_process_cb, nullptr, IDWALK_READONLY);
+        }
+
         if (do_override) {
           BKE_lib_override_library_operations_store_start(bmain, override_storage, id);
         }
 
-        if (wd->use_memfile) {
-          /* Record the changes that happened up to this undo push in
-           * recalc_up_to_undo_push, and clear `recalc_after_undo_push` again
-           * to start accumulating for the next undo push. */
-          id->recalc_up_to_undo_push = id->recalc_after_undo_push;
-          id->recalc_after_undo_push = 0;
-
-          bNodeTree *nodetree = ntreeFromID(id);
-          if (nodetree != nullptr) {
-            nodetree->id.recalc_up_to_undo_push = nodetree->id.recalc_after_undo_push;
-            nodetree->id.recalc_after_undo_push = 0;
-          }
-          if (GS(id->name) == ID_SCE) {
-            Scene *scene = (Scene *)id;
-            if (scene->master_collection != nullptr) {
-              scene->master_collection->id.recalc_up_to_undo_push =
-                  scene->master_collection->id.recalc_after_undo_push;
-              scene->master_collection->id.recalc_after_undo_push = 0;
-            }
-          }
-        }
-
         mywrite_id_begin(wd, id);
 
-        memcpy(id_buffer, id, idtype_struct_size);
-
-        /* Clear runtime data to reduce false detection of changed data in undo/redo context. */
-        ((ID *)id_buffer)->tag = 0;
-        ((ID *)id_buffer)->us = 0;
-        ((ID *)id_buffer)->icon_id = 0;
-        /* Those listbase data change every time we add/remove an ID, and also often when
-         * renaming one (due to re-sorting). This avoids generating a lot of false 'is changed'
-         * detections between undo steps. */
-        ((ID *)id_buffer)->prev = nullptr;
-        ((ID *)id_buffer)->next = nullptr;
-        /* Those runtime pointers should never be set during writing stage, but just in case clear
-         * them too. */
-        ((ID *)id_buffer)->orig_id = nullptr;
-        ((ID *)id_buffer)->newid = nullptr;
-        /* Even though in theory we could be able to preserve this python instance across undo even
-         * when we need to re-read the ID into its original address, this is currently cleared in
-         * #direct_link_id_common in `readfile.c` anyway, */
-        ((ID *)id_buffer)->py_instance = nullptr;
+        id_buffer_init_from_id(id_buffer, id, wd->use_memfile);
 
         if (id_type->blend_write != nullptr) {
-          id_type->blend_write(&writer, (ID *)id_buffer, id);
+          id_type->blend_write(&writer, static_cast<ID *>(id_buffer->temp_id), id);
         }
 
         if (do_override) {
@@ -1228,14 +1325,12 @@ static bool write_file_handle(Main *mainvar,
         mywrite_id_end(wd, id);
       }
 
-      if (id_buffer != id_buffer_static) {
-        MEM_SAFE_FREE(id_buffer);
-      }
-
       mywrite_flush(wd);
     }
   } while ((bmain != override_storage) && (bmain = override_storage));
 
+  BLO_write_destroy_id_buffer(&id_buffer);
+
   if (override_storage) {
     BKE_lib_override_library_operations_store_finalize(override_storage);
     override_storage = nullptr;
@@ -1255,11 +1350,11 @@ static bool write_file_handle(Main *mainvar,
    *
    * Note that we *borrow* the pointer to 'DNAstr',
    * so writing each time uses the same address and doesn't cause unnecessary undo overhead. */
-  writedata(wd, DNA1, size_t(wd->sdna->data_len), wd->sdna->data);
+  writedata(wd, BLO_CODE_DNA1, size_t(wd->sdna->data_len), wd->sdna->data);
 
   /* end of file */
   memset(&bhead, 0, sizeof(BHead));
-  bhead.code = ENDB;
+  bhead.code = BLO_CODE_ENDB;
   mywrite(wd, &bhead, sizeof(BHead));
 
   blo_join_main(&mainlist);
@@ -1284,11 +1379,11 @@ static bool do_history(const char *name, ReportList *reports)
   }
 
   while (hisnr > 1) {
-    BLI_snprintf(tempname1, sizeof(tempname1), "%s%d", name, hisnr - 1);
+    SNPRINTF(tempname1, "%s%d", name, hisnr - 1);
     if (BLI_exists(tempname1)) {
-      BLI_snprintf(tempname2, sizeof(tempname2), "%s%d", name, hisnr);
+      SNPRINTF(tempname2, "%s%d", name, hisnr);
 
-      if (BLI_rename(tempname1, tempname2)) {
+      if (BLI_rename_overwrite(tempname1, tempname2)) {
         BKE_report(reports, RPT_ERROR, "Unable to make version backup");
         return true;
       }
@@ -1298,9 +1393,9 @@ static bool do_history(const char *name, ReportList *reports)
 
   /* is needed when hisnr==1 */
   if (BLI_exists(name)) {
-    BLI_snprintf(tempname1, sizeof(tempname1), "%s%d", name, hisnr);
+    SNPRINTF(tempname1, "%s%d", name, hisnr);
 
-    if (BLI_rename(name, tempname1)) {
+    if (BLI_rename_overwrite(name, tempname1)) {
       BKE_report(reports, RPT_ERROR, "Unable to make version backup");
       return true;
     }
@@ -1346,7 +1441,7 @@ bool BLO_write_file(Main *mainvar,
   }
 
   /* open temporary file, so we preserve the original in case we crash */
-  BLI_snprintf(tempname, sizeof(tempname), "%s@", filepath);
+  SNPRINTF(tempname, "%s@", filepath);
 
   ww_handle_init((write_flags & G_FILE_COMPRESS) ? WW_WRAP_ZSTD : WW_WRAP_NONE, &ww);
 
@@ -1378,14 +1473,14 @@ bool BLO_write_file(Main *mainvar,
 
     /* Normalize the paths in case there is some subtle difference (so they can be compared). */
     if (relbase_valid) {
-      BLI_split_dir_part(mainvar->filepath, dir_src, sizeof(dir_src));
-      BLI_path_normalize(nullptr, dir_src);
+      BLI_path_split_dir_part(mainvar->filepath, dir_src, sizeof(dir_src));
+      BLI_path_normalize(dir_src);
     }
     else {
       dir_src[0] = '\0';
     }
-    BLI_split_dir_part(filepath, dir_dst, sizeof(dir_dst));
-    BLI_path_normalize(nullptr, dir_dst);
+    BLI_path_split_dir_part(filepath, dir_dst, sizeof(dir_dst));
+    BLI_path_normalize(dir_dst);
 
     /* Only for relative, not relative-all, as this means making existing paths relative. */
     if (remap_mode == BLO_WRITE_PATH_REMAP_RELATIVE) {
@@ -1405,7 +1500,7 @@ bool BLO_write_file(Main *mainvar,
     if (remap_mode != BLO_WRITE_PATH_REMAP_NONE) {
       /* Some path processing (e.g. with libraries) may use the current `main->filepath`, if this
        * is not matching the path currently used for saving, unexpected paths corruptions can
-       * happen. See T98201. */
+       * happen. See #98201. */
       char mainvar_filepath_orig[FILE_MAX];
       STRNCPY(mainvar_filepath_orig, mainvar->filepath);
       STRNCPY(mainvar->filepath, filepath);
@@ -1467,7 +1562,7 @@ bool BLO_write_file(Main *mainvar,
     }
   }
 
-  if (BLI_rename(tempname, filepath) != 0) {
+  if (BLI_rename_overwrite(tempname, filepath) != 0) {
     BKE_report(reports, RPT_ERROR, "Cannot change old file (file saved with @)");
     return false;
   }
@@ -1490,9 +1585,42 @@ bool BLO_write_file_mem(Main *mainvar, MemFile *compare, MemFile *current, int w
   return (err == 0);
 }
 
+/*
+ * API to handle writing IDs while clearing some of their runtime data.
+ */
+
+BLO_Write_IDBuffer *BLO_write_allocate_id_buffer()
+{
+  return MEM_cnew<BLO_Write_IDBuffer>(__func__);
+}
+
+void BLO_write_init_id_buffer_from_id(BLO_Write_IDBuffer *id_buffer, ID *id, const bool is_undo)
+{
+  const IDTypeInfo *id_type = BKE_idtype_get_info_from_id(id);
+  id_buffer_init_for_id_type(id_buffer, id_type);
+  id_buffer_init_from_id(id_buffer, id, is_undo);
+}
+
+ID *BLO_write_get_id_buffer_temp_id(BLO_Write_IDBuffer *id_buffer)
+{
+  return id_buffer->temp_id;
+}
+
+void BLO_write_destroy_id_buffer(BLO_Write_IDBuffer **id_buffer)
+{
+  if (static_cast<void *>((*id_buffer)->temp_id) != (*id_buffer)->id_buffer_static) {
+    MEM_SAFE_FREE((*id_buffer)->temp_id);
+  }
+  MEM_SAFE_FREE(*id_buffer);
+}
+
+/*
+ * API to write chunks of data.
+ */
+
 void BLO_write_raw(BlendWriter *writer, size_t size_in_bytes, const void *data_ptr)
 {
-  writedata(writer->wd, DATA, size_in_bytes, data_ptr);
+  writedata(writer->wd, BLO_CODE_DATA, size_in_bytes, data_ptr);
 }
 
 void BLO_write_struct_by_name(BlendWriter *writer, const char *struct_name, const void *data_ptr)
@@ -1515,7 +1643,7 @@ void BLO_write_struct_array_by_name(BlendWriter *writer,
 
 void BLO_write_struct_by_id(BlendWriter *writer, int struct_id, const void *data_ptr)
 {
-  writestruct_nr(writer->wd, DATA, struct_id, 1, data_ptr);
+  writestruct_nr(writer->wd, BLO_CODE_DATA, struct_id, 1, data_ptr);
 }
 
 void BLO_write_struct_at_address_by_id(BlendWriter *writer,
@@ -1523,7 +1651,8 @@ void BLO_write_struct_at_address_by_id(BlendWriter *writer,
                                        const void *address,
                                        const void *data_ptr)
 {
-  BLO_write_struct_at_address_by_id_with_filecode(writer, DATA, struct_id, address, data_ptr);
+  BLO_write_struct_at_address_by_id_with_filecode(
+      writer, BLO_CODE_DATA, struct_id, address, data_ptr);
 }
 
 void BLO_write_struct_at_address_by_id_with_filecode(
@@ -1537,18 +1666,18 @@ void BLO_write_struct_array_by_id(BlendWriter *writer,
                                   int array_size,
                                   const void *data_ptr)
 {
-  writestruct_nr(writer->wd, DATA, struct_id, array_size, data_ptr);
+  writestruct_nr(writer->wd, BLO_CODE_DATA, struct_id, array_size, data_ptr);
 }
 
 void BLO_write_struct_array_at_address_by_id(
     BlendWriter *writer, int struct_id, int array_size, const void *address, const void *data_ptr)
 {
-  writestruct_at_address_nr(writer->wd, DATA, struct_id, array_size, address, data_ptr);
+  writestruct_at_address_nr(writer->wd, BLO_CODE_DATA, struct_id, array_size, address, data_ptr);
 }
 
 void BLO_write_struct_list_by_id(BlendWriter *writer, int struct_id, ListBase *list)
 {
-  writelist_nr(writer->wd, DATA, struct_id, list);
+  writelist_nr(writer->wd, BLO_CODE_DATA, struct_id, list);
 }
 
 void BLO_write_struct_list_by_name(BlendWriter *writer, const char *struct_name, ListBase *list)
@@ -1572,6 +1701,11 @@ int BLO_get_struct_id_by_name(BlendWriter *writer, const char *struct_name)
   return struct_id;
 }
 
+void BLO_write_int8_array(BlendWriter *writer, uint num, const int8_t *data_ptr)
+{
+  BLO_write_raw(writer, sizeof(int8_t) * size_t(num), data_ptr);
+}
+
 void BLO_write_int32_array(BlendWriter *writer, uint num, const int32_t *data_ptr)
 {
   BLO_write_raw(writer, sizeof(int32_t) * size_t(num), data_ptr);
