@@ -15,7 +15,7 @@
 #include "BKE_main.h"
 #include "BKE_mball_tessellate.h"
 #include "BKE_modifier.h"
-#include "BKE_node.h"
+#include "BKE_node.hh"
 #include "BKE_scene.h"
 #include "BKE_vfont.h"
 
@@ -39,7 +39,7 @@
 #include "WM_api.h"
 #include "wm.h"
 
-#include "GHOST_Path-api.h"
+#include "GHOST_Path-api.hh"
 
 #include "CLG_log.h"
 
@@ -81,7 +81,7 @@ void BlendfileLoadingBaseTest::TearDownTestCase()
     G.main->wm.first = nullptr;
   }
 
-  /* Copied from WM_exit_ex() in wm_init_exit.c, and cherry-picked those lines that match the
+  /* Copied from WM_exit_ex() in wm_init_exit.cc, and cherry-picked those lines that match the
    * allocation/initialization done in SetUpTestCase(). */
   BKE_blender_free();
   RNA_exit();
@@ -103,8 +103,8 @@ void BlendfileLoadingBaseTest::TearDownTestCase()
 void BlendfileLoadingBaseTest::TearDown()
 {
   BKE_mball_cubeTable_free();
-  depsgraph_free();
   blendfile_free();
+  depsgraph_free();
 
   testing::Test::TearDown();
 }
@@ -116,7 +116,7 @@ bool BlendfileLoadingBaseTest::blendfile_load(const char *filepath)
     return false;
   }
 
-  char abspath[FILENAME_MAX];
+  char abspath[FILE_MAX];
   BLI_path_join(abspath, sizeof(abspath), test_assets_dir.c_str(), filepath);
 
   BlendFileReadReport bf_reports = {nullptr};
