@@ -6,6 +6,10 @@
  * \ingroup bmesh
  */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * Geometry must be completely isolated.
  */
@@ -17,3 +21,7 @@ void BM_mesh_copy_arrays(BMesh *bm_src,
                          uint edges_src_len,
                          BMFace **faces_src,
                          uint faces_src_len);
+
+#ifdef __cplusplus
+}
+#endif
