@@ -13,23 +13,23 @@
  * A word on slot names:
  *
  * For geometry input slots, the following are valid names:
- * - verts
- * - edges
- * - faces
- * - edgefacein
- * - vertfacein
- * - vertedgein
- * - vertfacein
- * - geom
+ * - `verts`
+ * - `edges`
+ * - `faces`
+ * - `edge_face.in`
+ * - `vert_face.in`
+ * - `vert_edge.in`
+ * - `vert_face.in`
+ * - `geom`
  *
  * The basic rules are, for single-type geometry slots, use the plural of the
  * type name (e.g. edges).  for double-type slots, use the two type names plus
- * "in" (e.g. edgefacein).  for three-type slots, use geom.
+ * "in" (e.g. `edge_face.in`).  for three-type slots, use geom.
  *
  * for output slots, for single-type geometry slots, use the type name plus "out",
- * (e.g. verts.out), for double-type slots, use the two type names plus "out",
- * (e.g. vertfaces.out), for three-type slots, use geom.  note that you can also
- * use more esoteric names (e.g. geom_skirt.out) so long as the comment next to the
+ * (e.g. `verts.out`), for double-type slots, use the two type names plus "out",
+ * (e.g. `vert_faces.out`), for three-type slots, use geom.  note that you can also
+ * use more esoteric names (e.g. `geom_skirt.out`) so long as the comment next to the
  * slot definition tells you what types of elements are in it.
  */
 
@@ -270,6 +270,23 @@ static BMOpDefine bmo_reverse_faces_def = {
    BMO_OPTYPE_FLAG_NORMALS_CALC),
 };
 
+/*
+ * Flip Quad Tessellation
+ *
+ * Flip the tessellation direction of the selected quads.
+ */
+static BMOpDefine bmo_flip_quad_tessellation_def = {
+  "flip_quad_tessellation",
+  /* slot_in */
+  {
+    {"faces", BMO_OP_SLOT_ELEMENT_BUF, {BM_FACE}},
+    {{'\0'}}
+  },
+  {{{'\0'}}}, /* no output */
+  bmo_flip_quad_tessellation_exec,
+  (BMO_OPTYPE_FLAG_UNTAN_MULTIRES | BMO_OPTYPE_FLAG_NORMALS_CALC),
+};
+
 /*
  * Edge Bisect.
  *
@@ -447,7 +464,7 @@ static BMOpDefine bmo_pointmerge_def = {
 };
 
 /*
- * Collapse Connected UV's.
+ * Collapse Connected UVs.
  *
  * Collapses connected UV vertices.
  */
@@ -1373,7 +1390,7 @@ static BMOpDefine bmo_duplicate_def = {
   "duplicate",
   /* slots_in */
   {{"geom", BMO_OP_SLOT_ELEMENT_BUF, {BM_VERT | BM_EDGE | BM_FACE}}, /* input geometry */
-   {"dest", BMO_OP_SLOT_PTR, {(int)BMO_OP_SLOT_SUBTYPE_PTR_BMESH}}, /* destination bmesh, if NULL will use current on */
+   {"dest", BMO_OP_SLOT_PTR, {(int)BMO_OP_SLOT_SUBTYPE_PTR_BMESH}}, /* destination bmesh, if None will use current on */
    {"use_select_history", BMO_OP_SLOT_BOOL},
    {"use_edge_flip_from_face", BMO_OP_SLOT_BOOL},
    {{'\0'}},
@@ -1405,7 +1422,7 @@ static BMOpDefine bmo_split_def = {
   "split",
   /* slots_in */
   {{"geom", BMO_OP_SLOT_ELEMENT_BUF, {BM_VERT | BM_EDGE | BM_FACE}}, /* input geometry */
-   {"dest", BMO_OP_SLOT_PTR, {(int)BMO_OP_SLOT_SUBTYPE_PTR_BMESH}}, /* destination bmesh, if NULL will use current one */
+   {"dest", BMO_OP_SLOT_PTR, {(int)BMO_OP_SLOT_SUBTYPE_PTR_BMESH}}, /* destination bmesh, if None will use current one */
    {"use_only_faces", BMO_OP_SLOT_BOOL},  /* when enabled. don't duplicate loose verts/edges */
    {{'\0'}},
   },
@@ -1453,7 +1470,7 @@ static BMOpDefine bmo_spin_def = {
 /*
  * UV Rotation.
  *
- * Cycle the loop UV's
+ * Cycle the loop UVs
  */
 static BMOpDefine bmo_rotate_uvs_def = {
   "rotate_uvs",
@@ -1470,7 +1487,7 @@ static BMOpDefine bmo_rotate_uvs_def = {
 /*
  * UV Reverse.
  *
- * Reverse the UV's
+ * Reverse the UVs
  */
 static BMOpDefine bmo_reverse_uvs_def = {
   "reverse_uvs",
@@ -1780,7 +1797,7 @@ static BMOpDefine bmo_bevel_def = {
     bmo_enum_bevel_miter_type},           /* outer miter kind */
    {"spread", BMO_OP_SLOT_FLT},           /* amount to offset beveled edge */
    {"smoothresh", BMO_OP_SLOT_FLT},       /* for passing mesh's smoothresh, used in hardening */
-   {"custom_profile", BMO_OP_SLOT_PTR, {(int)BMO_OP_SLOT_SUBTYPE_PTR_STRUCT}}, /* CurveProfile */
+   {"custom_profile", BMO_OP_SLOT_PTR, {(int)BMO_OP_SLOT_SUBTYPE_PTR_STRUCT}}, /* CurveProfile, if None ignored */
    {"vmesh_method", BMO_OP_SLOT_INT, {(int)BMO_OP_SLOT_SUBTYPE_INT_ENUM},
     bmo_enum_bevel_vmesh_method},         /* The method to use to create meshes at intersections. */
    {{'\0'}},
@@ -2128,6 +2145,7 @@ const BMOpDefine *bmo_opdefines[] = {
     &bmo_extrude_face_region_def,
     &bmo_extrude_vert_indiv_def,
     &bmo_find_doubles_def,
+    &bmo_flip_quad_tessellation_def,
     &bmo_grid_fill_def,
     &bmo_inset_individual_def,
     &bmo_inset_region_def,
