@@ -225,20 +225,26 @@ float BM_face_calc_area_with_mat3(const BMFace *f, const float mat3[3][3])
   return len_v3(n) * 0.5f;
 }
 
-float BM_face_calc_area_uv(const BMFace *f, int cd_loop_uv_offset)
+float BM_face_calc_area_uv_signed(const BMFace *f, int cd_loop_uv_offset)
 {
   /* inline 'area_poly_v2' logic, avoid creating a temp array */
   const BMLoop *l_iter, *l_first;
 
   l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-  /* The Trapezium Area Rule */
+  /* Green's theorem applied to area of a polygon.
+   * TODO: `cross` should be of type `double` to reduce rounding error. */
   float cross = 0.0f;
   do {
-    const MLoopUV *luv = BM_ELEM_CD_GET_VOID_P(l_iter, cd_loop_uv_offset);
-    const MLoopUV *luv_next = BM_ELEM_CD_GET_VOID_P(l_iter->next, cd_loop_uv_offset);
-    cross += (luv_next->uv[0] - luv->uv[0]) * (luv_next->uv[1] + luv->uv[1]);
+    const float *luv = BM_ELEM_CD_GET_FLOAT_P(l_iter, cd_loop_uv_offset);
+    const float *luv_next = BM_ELEM_CD_GET_FLOAT_P(l_iter->next, cd_loop_uv_offset);
+    cross += (luv_next[0] - luv[0]) * (luv_next[1] + luv[1]);
   } while ((l_iter = l_iter->next) != l_first);
-  return fabsf(cross * 0.5f);
+  return cross * 0.5f;
+}
+
+float BM_face_calc_area_uv(const BMFace *f, int cd_loop_uv_offset)
+{
+  return fabsf(BM_face_calc_area_uv_signed(f, cd_loop_uv_offset));
 }
 
 float BM_face_calc_perimeter(const BMFace *f)
@@ -1229,7 +1235,7 @@ void BM_face_splits_check_legal(BMesh *bm, BMFace *f, BMLoop *(*loops)[2], int l
   }
 
 #define EDGE_SHARE_VERT(e1, e2) \
-  ((ELEM((e1)[0], (e2)[0], (e2)[1])) || (ELEM((e1)[1], (e2)[0], (e2)[1])))
+  (ELEM((e1)[0], (e2)[0], (e2)[1]) || ELEM((e1)[1], (e2)[0], (e2)[1]))
 
   /* do line crossing tests */
   for (i = 0, i_prev = f->len - 1; i < f->len; i_prev = i++) {
@@ -1268,7 +1274,8 @@ void BM_face_splits_check_optimal(BMFace *f, BMLoop *(*loops)[2], int len)
   for (i = 0; i < len; i++) {
     BMLoop *l_a_dummy, *l_b_dummy;
     if (f != BM_vert_pair_share_face_by_angle(
-                 loops[i][0]->v, loops[i][1]->v, &l_a_dummy, &l_b_dummy, false)) {
+                 loops[i][0]->v, loops[i][1]->v, &l_a_dummy, &l_b_dummy, false))
+    {
       loops[i][0] = NULL;
     }
   }
