@@ -1,18 +1,4 @@
-/*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /** \file
  * \ingroup bmesh
@@ -205,7 +191,8 @@ void bmo_extrude_edge_only_exec(BMesh *bm, BMOperator *op)
   }
 
   for (e = BMO_iter_new(&siter, dupeop.slots_out, "boundary_map.out", 0); e;
-       e = BMO_iter_step(&siter)) {
+       e = BMO_iter_step(&siter))
+  {
     BMVert *f_verts[4];
     e_new = BMO_iter_map_value_ptr(&siter);
 
@@ -381,7 +368,7 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 
   /* calculate verts to delete */
   BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
-    if (v->e) { /* only deal with verts attached to geometry T33651. */
+    if (v->e) { /* only deal with verts attached to geometry #33651. */
       found = false;
 
       BM_ITER_ELEM (e, &viter, v, BM_EDGES_OF_VERT) {
@@ -469,7 +456,8 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 
   slot_edges_exclude = BMO_slot_get(op->slots_in, "edges_exclude");
   for (e = BMO_iter_new(&siter, dupeop.slots_out, "boundary_map.out", 0); e;
-       e = BMO_iter_step(&siter)) {
+       e = BMO_iter_step(&siter))
+  {
     BMVert *f_verts[4];
 #ifdef USE_EDGE_REGION_FLAGS
     BMEdge *f_edges[4];
@@ -480,10 +468,10 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
       BMVert *v1 = e->v1, *v2 = e->v2;
 
       /* The original edge was excluded,
-       * this would result in a standalone wire edge - see T30399. */
+       * this would result in a standalone wire edge - see #30399. */
       BM_edge_kill(bm, e);
 
-      /* kill standalone vertices from this edge - see T32341. */
+      /* kill standalone vertices from this edge - see #32341. */
       if (!v1->e) {
         BM_vert_kill(bm, v1);
       }
@@ -494,7 +482,7 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
       continue;
     }
 
-    /* skip creating face for excluded edges see T35503. */
+    /* skip creating face for excluded edges see #35503. */
     if (BMO_slot_map_contains(slot_edges_exclude, e)) {
       /* simply skip creating the face */
       continue;
@@ -595,7 +583,8 @@ void bmo_extrude_face_region_exec(BMesh *bm, BMOperator *op)
 
   /* link isolated vert */
   for (v = BMO_iter_new(&siter, dupeop.slots_out, "isovert_map.out", 0); v;
-       v = BMO_iter_step(&siter)) {
+       v = BMO_iter_step(&siter))
+  {
     BMVert *v2 = BMO_iter_map_value_ptr(&siter);
 
     /* not essential, but ensures face normals from extruded edges are contiguous */
