@@ -602,7 +602,7 @@ void bmo_grid_fill_exec(BMesh *bm, BMOperator *op)
   if (count != 2) {
     /* Note that this error message has been adjusted to make sense when called
      * from the operator 'MESH_OT_fill_grid' which has a 'prepare' pass which can
-     * extract two 'rail' loops from a single edge loop, see T72075. */
+     * extract two 'rail' loops from a single edge loop, see #72075. */
     BMO_error_raise(bm,
                     op,
                     BMO_ERROR_CANCEL,
@@ -633,8 +633,8 @@ void bmo_grid_fill_exec(BMesh *bm, BMOperator *op)
 
   if (BM_mesh_edgeloops_find_path(
           bm, &eloops_rail, bm_edge_test_rail_cb, bm, v_a_first, v_b_first) &&
-      BM_mesh_edgeloops_find_path(
-          bm, &eloops_rail, bm_edge_test_rail_cb, bm, v_a_last, v_b_last)) {
+      BM_mesh_edgeloops_find_path(bm, &eloops_rail, bm_edge_test_rail_cb, bm, v_a_last, v_b_last))
+  {
     estore_rail_a = eloops_rail.first;
     estore_rail_b = eloops_rail.last;
   }
@@ -644,7 +644,8 @@ void bmo_grid_fill_exec(BMesh *bm, BMOperator *op)
     if (BM_mesh_edgeloops_find_path(
             bm, &eloops_rail, bm_edge_test_rail_cb, bm, v_a_first, v_b_last) &&
         BM_mesh_edgeloops_find_path(
-            bm, &eloops_rail, bm_edge_test_rail_cb, bm, v_a_last, v_b_first)) {
+            bm, &eloops_rail, bm_edge_test_rail_cb, bm, v_a_last, v_b_first))
+    {
       estore_rail_a = eloops_rail.first;
       estore_rail_b = eloops_rail.last;
       BM_edgeloop_flip(bm, estore_b);
