@@ -40,7 +40,8 @@ static BMFace *bm_face_split_walk_back(BMesh *bm, BMLoop *l_src, BMLoop **r_l)
   int num, i;
 
   for (l_dst = l_src->prev, num = 0; BM_elem_index_get(l_dst->prev->v) != -1;
-       l_dst = l_dst->prev, num++) {
+       l_dst = l_dst->prev, num++)
+  {
     /* pass */
   }
 
