@@ -147,6 +147,22 @@ void bmo_reverse_faces_exec(BMesh *bm, BMOperator *op)
 #define SEL_FLAG 1
 #define SEL_ORIG 2
 
+void bmo_flip_quad_tessellation_exec(BMesh *bm, BMOperator *op)
+{
+  BMOIter siter;
+  BMFace *f;
+  bool changed = false;
+  BMO_ITER (f, &siter, op->slots_in, "faces", BM_FACE) {
+    if (f->len == 4) {
+      f->l_first = f->l_first->next;
+      changed = true;
+    }
+  }
+  if (changed) {
+    bm->elem_index_dirty |= BM_LOOP;
+  }
+}
+
 static void bmo_face_flag_set_flush(BMesh *bm, BMFace *f, const short oflag, const bool value)
 {
   BMLoop *l_iter;
@@ -213,8 +229,8 @@ static void bmo_region_extend_expand(BMesh *bm,
             BMEdge *e;
             BM_ITER_ELEM (e, &eiter, v, BM_EDGES_OF_VERT) {
               if (BM_edge_is_wire(e)) {
-                if (!BMO_edge_flag_test(bm, e, SEL_FLAG) &&
-                    !BM_elem_flag_test(e, BM_ELEM_HIDDEN)) {
+                if (!BMO_edge_flag_test(bm, e, SEL_FLAG) && !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+                {
                   BMO_edge_flag_enable(bm, e, SEL_FLAG);
                   BMO_vert_flag_enable(bm, BM_edge_other_vert(e, v), SEL_FLAG);
                 }
@@ -239,7 +255,8 @@ static void bmo_region_extend_expand(BMesh *bm,
 
           BM_ITER_ELEM (f_other, &fiter, l->e, BM_FACES_OF_EDGE) {
             if (!BMO_face_flag_test(bm, f_other, SEL_ORIG | SEL_FLAG) &&
-                !BM_elem_flag_test(f_other, BM_ELEM_HIDDEN)) {
+                !BM_elem_flag_test(f_other, BM_ELEM_HIDDEN))
+            {
               BMO_face_flag_enable(bm, f_other, SEL_FLAG);
             }
           }
@@ -250,7 +267,8 @@ static void bmo_region_extend_expand(BMesh *bm,
 
           BM_ITER_ELEM (f_other, &fiter, l->v, BM_FACES_OF_VERT) {
             if (!BMO_face_flag_test(bm, f_other, SEL_ORIG | SEL_FLAG) &&
-                !BM_elem_flag_test(f_other, BM_ELEM_HIDDEN)) {
+                !BM_elem_flag_test(f_other, BM_ELEM_HIDDEN))
+            {
               BMO_face_flag_enable(bm, f_other, SEL_FLAG);
             }
           }
@@ -462,56 +480,56 @@ void bmo_rotate_uvs_exec(BMesh *bm, BMOperator *op)
   BMIter l_iter;   /* iteration loop */
 
   const bool use_ccw = BMO_slot_bool_get(op->slots_in, "use_ccw");
-  const int cd_loop_uv_offset = CustomData_get_offset(&bm->ldata, CD_MLOOPUV);
+  const int cd_loop_uv_offset = CustomData_get_offset(&bm->ldata, CD_PROP_FLOAT2);
 
   if (cd_loop_uv_offset != -1) {
     BMO_ITER (fs, &fs_iter, op->slots_in, "faces", BM_FACE) {
       if (use_ccw == false) { /* same loops direction */
         BMLoop *lf;           /* current face loops */
-        MLoopUV *f_luv;       /* first face loop uv */
+        float *f_luv;         /* first face loop uv */
         float p_uv[2];        /* previous uvs */
-        float t_uv[2];        /* tmp uvs */
+        float t_uv[2];        /* temp uvs */
 
         int n = 0;
         BM_ITER_ELEM (lf, &l_iter, fs, BM_LOOPS_OF_FACE) {
           /* current loop uv is the previous loop uv */
-          MLoopUV *luv = BM_ELEM_CD_GET_VOID_P(lf, cd_loop_uv_offset);
+          float *luv = BM_ELEM_CD_GET_FLOAT_P(lf, cd_loop_uv_offset);
           if (n == 0) {
             f_luv = luv;
-            copy_v2_v2(p_uv, luv->uv);
+            copy_v2_v2(p_uv, luv);
           }
           else {
-            copy_v2_v2(t_uv, luv->uv);
-            copy_v2_v2(luv->uv, p_uv);
+            copy_v2_v2(t_uv, luv);
+            copy_v2_v2(luv, p_uv);
             copy_v2_v2(p_uv, t_uv);
           }
           n++;
         }
 
-        copy_v2_v2(f_luv->uv, p_uv);
+        copy_v2_v2(f_luv, p_uv);
       }
-      else {            /* counter loop direction */
-        BMLoop *lf;     /* current face loops */
-        MLoopUV *p_luv; /* previous loop uv */
-        MLoopUV *luv;
+      else {          /* counter loop direction */
+        BMLoop *lf;   /* current face loops */
+        float *p_luv; /* previous loop uv */
+        float *luv;
         float t_uv[2]; /* current uvs */
 
         int n = 0;
         BM_ITER_ELEM (lf, &l_iter, fs, BM_LOOPS_OF_FACE) {
           /* previous loop uv is the current loop uv */
-          luv = BM_ELEM_CD_GET_VOID_P(lf, cd_loop_uv_offset);
+          luv = BM_ELEM_CD_GET_FLOAT_P(lf, cd_loop_uv_offset);
           if (n == 0) {
             p_luv = luv;
-            copy_v2_v2(t_uv, luv->uv);
+            copy_v2_v2(t_uv, luv);
           }
           else {
-            copy_v2_v2(p_luv->uv, luv->uv);
+            copy_v2_v2(p_luv, luv);
             p_luv = luv;
           }
           n++;
         }
 
-        copy_v2_v2(luv->uv, t_uv);
+        copy_v2_v2(luv, t_uv);
       }
     }
   }
@@ -530,22 +548,22 @@ static void bm_face_reverse_uvs(BMFace *f, const int cd_loop_uv_offset)
   float(*uvs)[2] = BLI_array_alloca(uvs, f->len);
 
   BM_ITER_ELEM_INDEX (l, &iter, f, BM_LOOPS_OF_FACE, i) {
-    MLoopUV *luv = BM_ELEM_CD_GET_VOID_P(l, cd_loop_uv_offset);
-    copy_v2_v2(uvs[i], luv->uv);
+    float *luv = BM_ELEM_CD_GET_FLOAT_P(l, cd_loop_uv_offset);
+    copy_v2_v2(uvs[i], luv);
   }
 
   /* now that we have the uvs in the array, reverse! */
   BM_ITER_ELEM_INDEX (l, &iter, f, BM_LOOPS_OF_FACE, i) {
     /* current loop uv is the previous loop uv */
-    MLoopUV *luv = BM_ELEM_CD_GET_VOID_P(l, cd_loop_uv_offset);
-    copy_v2_v2(luv->uv, uvs[(f->len - i - 1)]);
+    float *luv = BM_ELEM_CD_GET_FLOAT_P(l, cd_loop_uv_offset);
+    copy_v2_v2(luv, uvs[(f->len - i - 1)]);
   }
 }
 void bmo_reverse_uvs_exec(BMesh *bm, BMOperator *op)
 {
   BMOIter iter;
   BMFace *f;
-  const int cd_loop_uv_offset = CustomData_get_offset(&bm->ldata, CD_MLOOPUV);
+  const int cd_loop_uv_offset = CustomData_get_offset(&bm->ldata, CD_PROP_FLOAT2);
 
   if (cd_loop_uv_offset != -1) {
     BMO_ITER (f, &iter, op->slots_in, "faces", BM_FACE) {
@@ -557,27 +575,28 @@ void bmo_reverse_uvs_exec(BMesh *bm, BMOperator *op)
 /**************************************************************************** *
  * Cycle colors for a face
  **************************************************************************** */
+
 static void bmo_get_loop_color_ref(BMesh *bm,
                                    int index,
                                    int *r_cd_color_offset,
                                    int *r_cd_color_type)
 {
   Mesh me_query;
-
-  BKE_id_attribute_copy_domains_temp(
-      ID_ME, &bm->vdata, NULL, &bm->ldata, NULL, NULL, &me_query.id);
+  memset(&me_query, 0, sizeof(Mesh));
+  CustomData_reset(&me_query.vdata);
+  CustomData_reset(&me_query.edata);
+  CustomData_reset(&me_query.pdata);
+  me_query.ldata = bm->ldata;
+  *((short *)me_query.id.name) = ID_ME;
 
   CustomDataLayer *layer = BKE_id_attribute_from_index(
-      &me_query.id, index, ATTR_DOMAIN_MASK_COLOR, CD_MASK_COLOR_ALL);
-
-  if (!layer || BKE_id_attribute_domain(&me_query.id, layer) != ATTR_DOMAIN_CORNER) {
+      &me_query.id, index, ATTR_DOMAIN_MASK_CORNER, CD_MASK_COLOR_ALL);
+  if (!layer) {
     *r_cd_color_offset = -1;
     return;
   }
 
-  int layer_i = CustomData_get_layer_index(&bm->ldata, layer->type);
-
-  *r_cd_color_offset = bm->ldata.layers[layer_i].offset;
+  *r_cd_color_offset = layer->offset;
   *r_cd_color_type = layer->type;
 }
 
@@ -603,7 +622,7 @@ void bmo_rotate_colors_exec(BMesh *bm, BMOperator *op)
 
   const size_t size = cd_loop_color_type == CD_PROP_COLOR ? sizeof(MPropCol) : sizeof(MLoopCol);
   void *p_col;                /* previous color */
-  void *t_col = alloca(size); /* tmp color */
+  void *t_col = alloca(size); /* Temp color. */
 
   BMO_ITER (fs, &fs_iter, op->slots_in, "faces", BM_FACE) {
     if (use_ccw == false) { /* same loops direction */
