@@ -73,7 +73,8 @@ static bool bm_edge_is_contiguous_loop_cd_all(const BMEdge *e,
   int cd_loop_offset;
   for (cd_loop_offset = delimit_data->cd_loop_offset;
        cd_loop_offset < delimit_data->cd_loop_offset_end;
-       cd_loop_offset += delimit_data->cd_loop_size) {
+       cd_loop_offset += delimit_data->cd_loop_size)
+  {
     if (BM_edge_is_contiguous_loop_cd(e, delimit_data->cd_loop_type, cd_loop_offset) == false) {
       return false;
     }
@@ -199,7 +200,8 @@ static bool bm_loop_collapse_is_degenerate(BMLoop *l_ear)
     if (!BM_vert_is_edge_pair(l_ear->prev->v)) {
       mul_v2_m3v3_center(adjacent_2d, axis_mat, l_ear->prev->prev->v->co, center);
       if (signum_i(cross_tri_v2(adjacent_2d, tri_2d[0], tri_2d[1])) !=
-          signum_i(cross_tri_v2(adjacent_2d, tri_2d[0], tri_2d[2]))) {
+          signum_i(cross_tri_v2(adjacent_2d, tri_2d[0], tri_2d[2])))
+      {
         return true;
       }
     }
@@ -207,7 +209,8 @@ static bool bm_loop_collapse_is_degenerate(BMLoop *l_ear)
     if (!BM_vert_is_edge_pair(l_ear->next->v)) {
       mul_v2_m3v3_center(adjacent_2d, axis_mat, l_ear->next->next->v->co, center);
       if (signum_i(cross_tri_v2(adjacent_2d, tri_2d[2], tri_2d[1])) !=
-          signum_i(cross_tri_v2(adjacent_2d, tri_2d[2], tri_2d[0]))) {
+          signum_i(cross_tri_v2(adjacent_2d, tri_2d[2], tri_2d[0])))
+      {
         return true;
       }
     }
@@ -289,14 +292,14 @@ void BM_mesh_decimate_dissolve_ex(BMesh *bm,
   int i;
 
   if (delimit & BMO_DELIM_UV) {
-    const int layer_len = CustomData_number_of_layers(&bm->ldata, CD_MLOOPUV);
+    const int layer_len = CustomData_number_of_layers(&bm->ldata, CD_PROP_FLOAT2);
     if (layer_len == 0) {
       delimit &= ~BMO_DELIM_UV;
     }
     else {
-      delimit_data.cd_loop_type = CD_MLOOPUV;
+      delimit_data.cd_loop_type = CD_PROP_FLOAT2;
       delimit_data.cd_loop_size = CustomData_sizeof(delimit_data.cd_loop_type);
-      delimit_data.cd_loop_offset = CustomData_get_n_offset(&bm->ldata, CD_MLOOPUV, 0);
+      delimit_data.cd_loop_offset = CustomData_get_n_offset(&bm->ldata, CD_PROP_FLOAT2, 0);
       delimit_data.cd_loop_offset_end = delimit_data.cd_loop_size * layer_len;
     }
   }
@@ -330,7 +333,8 @@ void BM_mesh_decimate_dissolve_ex(BMesh *bm,
     }
 
     while ((BLI_heap_is_empty(eheap) == false) &&
-           (BLI_heap_node_value(enode_top = BLI_heap_top(eheap)) < angle_limit_cos_neg)) {
+           (BLI_heap_node_value(enode_top = BLI_heap_top(eheap)) < angle_limit_cos_neg))
+    {
       BMFace *f_new = NULL;
       BMEdge *e;
 
@@ -451,7 +455,8 @@ void BM_mesh_decimate_dissolve_ex(BMesh *bm,
     }
 
     while ((BLI_heap_is_empty(vheap) == false) &&
-           (BLI_heap_node_value(vnode_top = BLI_heap_top(vheap)) < angle_limit)) {
+           (BLI_heap_node_value(vnode_top = BLI_heap_top(vheap)) < angle_limit))
+    {
       BMEdge *e_new = NULL;
       BMVert *v;
 
@@ -464,7 +469,8 @@ void BM_mesh_decimate_dissolve_ex(BMesh *bm,
 #else
           BM_vert_is_edge_pair(v)
 #endif
-      ) {
+      )
+      {
         e_new = BM_vert_collapse_edge(bm, v->e, v, true, true, true); /* join edges */
 
         if (e_new) {
