@@ -8,6 +8,10 @@
 
 #pragma once
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \param defgrp_index: Vertex group index, -1 for no vertex groups.
  *
@@ -29,3 +33,7 @@ void BM_mesh_wireframe(BMesh *bm,
                        short mat_offset,
                        short mat_max,
                        bool use_tag);
+
+#ifdef __cplusplus
+}
+#endif
